package ant;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import ant.exception.AntComponentRegisterException;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class AbstractAntKit {
	private static final Log log = LogFactory.getLog(AbstractAntKit.class);

	protected List<AntComponent> components;
	protected Map<Class<? extends AntComponent>, AntComponent> cmap;

	protected AgentContext context;
	protected StandardWorldModel world;

	public AbstractAntKit() {
		components = new ArrayList<AntComponent>();
		cmap = new HashMap<Class<? extends AntComponent>, AntComponent>();
	}

	public AntComponent getAntComponent(Class<? extends AntComponent> clazz) {
		AntComponent ret = cmap.get(clazz);
		if (ret == null) {
			log.error("Not found class:" + clazz);
		}
		return ret;
	}

	public void registerComponent(Class<? extends AntComponent> clazz)
			throws AntComponentRegisterException {
		try {
			if (cmap.containsKey(clazz)) {
				log.warn(clazz.toString() + " already registered");
				return;
			}

			AntComponent comp = clazz.newInstance();
			components.add(comp);
			cmap.put(clazz, comp);

		} catch (InstantiationException e) {
			throw new AntComponentRegisterException(e);
		} catch (IllegalAccessException e) {
			throw new AntComponentRegisterException(e);
		}

	}

	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		for (AntComponent comp : components) {
			long _time_st = System.currentTimeMillis();
			// TODO remove

			long _st_usable = Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory()
					+ Runtime.getRuntime().freeMemory();
			comp.initial(context, world);
			long _ed_usable = Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory()
					+ Runtime.getRuntime().freeMemory();

			long _time_ed = System.currentTimeMillis();
			log.info("Component " + comp.getComponentName() + " Initial cost: "
					+ (_time_ed - _time_st) + " milliseconds , cost " + (_st_usable - _ed_usable)
					/ 1000000.0 + "MB memory");
		}
	}

	public void update(AgentContext context) {
		for (AntComponent comp : components) {
			long _time_st = System.currentTimeMillis();
			comp.update();
			
			long _time_ed = System.currentTimeMillis();
			if ((_time_ed - _time_st) > 500) {
				log.error("@" + context.getTimestamp() + " " + comp.getComponentName()
						+ " update cost: " + (_time_ed - _time_st) + " milliseconds");
			} else if ((_time_ed - _time_st) > 100) {
				log.warn("@" + context.getTimestamp() + " " + comp.getComponentName()
						+ " update cost: " + (_time_ed - _time_st) + " milliseconds");
			}
			// log.debug("@" +context.getTimestamp() + comp.getComponentName() +
			// " updated");
		}
	}

}
