package ant;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import ant.agent.command.AntCommand;
import ant.agent.command.AntRest;
import ant.agent.job.AbstractJob;
import ant.agent.job.universal.NeverJob;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class AbstractJobBasedAntAgent<E extends Human> extends AbstractAntAgent<E> {
	private static final Log log = LogFactory.getLog(AbstractJobBasedAntAgent.class);

	protected List<AbstractJob> jobs = null;

	@Override
	public final AntCommand think() {
		boolean executed = false;
		AntCommand ret = null;

		if (jobs == null) {
			jobs = produceJobs();
		}

		if (context.getTimestamp() < ConfigKey.getIgnoreTime(context.getConfig())) {
			for (AbstractJob job : jobs) {
				if (job instanceof NeverJob) {
					job.shouldExecute();
				}
			}
			return new AntRest(context.getTimestamp());
		}

		for (AbstractJob job : jobs) {
			long sTime = System.currentTimeMillis();
			boolean shouldExecute = false;
			if (executed) {
				job.overtaken();
			} else {
				shouldExecute = job.shouldExecute();
				if (!shouldExecute) {
					job.overleap();
					// if (!job.getJobName().contains("Sample")) {
					// log.debug("@" + getTime() + " pass job:" +
					// job.getJobName());
					// }
				} else {
					ret = job.execute();
					if (ret != null) {
						executed = true;
						this.context.setState(job.getState());
						this.context.putBundleKV("Executed Job", job.getJobName());

						if (!job.getJobName().contains("Sample")) {
							// TODO
							log.debug("@" + getTime() + " execute job:" + job.getJobName() + " state:" + context.getState());
						}
					} else {
						log.fatal("@" + getTime() + " job: " + job + " return null");
					}
				}
			}
			long dTime = System.currentTimeMillis() - sTime;
			if (dTime > 300) {
				log.error("@" + context.getTimestamp() + " job " + job.getJobName() + " cost:" + dTime + " ms, with shouldExecute:" + shouldExecute
						+ ",ret:" + ret + ",jobState:" + job.getState());
			} else if (dTime > 200) {
				log.warn("@" + context.getTimestamp() + " job " + job.getJobName() + " cost:" + dTime + " ms, with shouldExecute:" + shouldExecute
						+ ",ret:" + ret + ",jobState:" + job.getState());
			}
		}
		
		sendJobMessages();
		
		if (executed == false) {
			log.warn("No job executed @" + context.getTimestamp());
			return new AntRest(context.getTimestamp());
		} else {
			return ret;
		}
	}

	protected abstract List<AbstractJob> produceJobs();
	
	private void sendJobMessages() {
		
		AbstractAntCommunicationKit ack = context.getAntCommunicationKit();
		if (ack == null) {
			log.warn("Not found CommunicationKit!");
			return;
		}

		for (AbstractJob<?> job : jobs) {
			for (IMessage msg : job.pollRadioMessages()) {
				ack.sendRadio(msg);
			}
			for (IMessage msg : job.pollVoiceMessages()) {
				ack.sendVoice(msg);
			}
		}
		
	}
}
