package ant;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.config.Config;
import rescuecore2.messages.Command;
import rescuecore2.messages.control.KASense;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import ant.agent.CommonAgent;
import ant.agent.behavior.ATBehavior;
import ant.agent.behavior.AntBehavior;
import ant.agent.behavior.FBBehavior;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.agent.job.JobState;
import ant.utils.LazyMap;

/**
 * 
 * @author Ivan
 * 
 */
public final class AgentContext {
	public static final Log log = LogFactory.getLog(AgentContext.class);
	private final EntityID agentID;
	private final StandardEntityURN agentType;
	private final Human human;
	private final Config config;
	private final StandardWorldModel world;
	private final AbstractAntCommunicationKit cmu;
	private final AntComponentKit kit;
	private final CommonAgent agent;
	private final AntBehavior<? extends Human> behavior;
	private final Map<Integer, AntCommand> commands;

	private final Map<String, String> kvs;

	private JobState state;
	private int time;
	private ChangeSet changeSet;
	private Collection<Command> hearing;

	public AgentContext(CommonAgent agent, StandardWorldModel model,
			AbstractAntCommunicationKit cmu, AntComponentKit kit, Config config) {

		this.kvs = new LazyMap<String, String>() {
			@Override
			public String createValue() {
				return "";
			}
		};

		this.agent = agent;
		this.agentID = agent.getID();
		this.agentType = agent.getAgentType();
		this.human = agent.getHuman();
		this.world = model;
		this.cmu = cmu;
		this.kit = kit;
		this.config = config;
		this.commands = new HashMap<Integer, AntCommand>();

		if (agentType.equals(StandardEntityURN.AMBULANCE_TEAM)) {
			this.behavior = new ATBehavior(this);
		} else if (agentType.equals(StandardEntityURN.FIRE_BRIGADE)) {
			this.behavior = new FBBehavior(this);
		} else if (agentType.equals(StandardEntityURN.POLICE_FORCE)) {
			this.behavior = new PFBehavior(this);
		} else {
			if (!agentType.equals(StandardEntityURN.CIVILIAN)) {
				log.error("Cannot find suitable behavior");
			}
			this.behavior = new AntBehavior<Human>(this);
		}

		// TODO remove
		registerBundle();
	}

	public void setSense(KASense sense) {
		this.time = sense.getTime();
		this.changeSet = sense.getChangeSet();
		this.hearing = sense.getHearing();
	}

	public Collection<Command> getNaiveHearing() {
		return this.hearing;
	}

	/**
	 * only view set timestamp
	 */
	public void setTimestamp(int time) {
		this.time = time;
	}

	public void setChangeSet(ChangeSet changeSet) {
		this.changeSet = changeSet;
	}

	public AntComponentKit getAntComponentKit() {
		return kit;
	}

	public AbstractAntCommunicationKit getAntCommunicationKit() {
		return this.cmu;
	}

	public Config getConfig() {
		return this.config;
	}

	public EntityID getAgentID() {
		return this.agentID;
	}

	public StandardEntityURN getAgentType() {
		return this.agentType;
	}

	public int getTimestamp() {
		return this.time;
	}

	public Human getHuman() {
		return this.human;
	}

	public ChangeSet getChangeSet() {
		return changeSet;
	}

	public StandardWorldModel getWorld() {
		return this.world;
	}

	public void setState(JobState state) {
		this.state = state;
	}

	public JobState getState() {
		return this.state;
	}

	public AntBehavior<? extends Human> getBehavior() {
		return this.behavior;
	}

	public Random getUniformRandom() {
		long tmp = AntProperties.getRandomSeed();
		return new Random(tmp);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((agentID == null) ? 0 : agentID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AgentContext other = (AgentContext) obj;
		if (agentID == null) {
			if (other.agentID != null)
				return false;
		} else if (!agentID.equals(other.agentID))
			return false;
		return true;
	}

	public Map<Integer, AntCommand> getCommands() {
		return commands;
	}

	// TODO remove
	void registerBundle() {
//		Bundle.getInstance().register(this);
	}

	// TODO remove
	public void putBundleKV(String key, String val) {
		this.kvs.put(key, "@" + this.time + ":" + val);
	}

	public Map<String, String> getBundleKVs() {
		return this.kvs;
	}
}
