package ant;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardEntityURN;
import ant.component.at.AmbulanceKernelComponent;
import ant.component.at.AmbulanceMemComponent;
import ant.component.at.ExplorationRecordComponent;
import ant.component.fb.ActiveFireBrigadeComponent;
import ant.component.fb.EstimationFireConvexHullComponent;
import ant.component.fb.EstimationFireSplitComponent;
import ant.component.fb.ExtinguishRecordComponent;
import ant.component.fb.FireConfirmComponent;
import ant.component.fb.FireConvexHullComponent;
import ant.component.fb.FireConvexHullPickComponent;
import ant.component.fb.FireSimulatorComponent;
import ant.component.fb.FireSplitComponent;
import ant.component.fb.FireTraceComponent;
import ant.component.fb.RealTimeFireTraceBuildingManagerComponent;
import ant.component.pf.ObjectiveJigsawDecisionComponent;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveJigsawSystemComponent;
import ant.component.pf.ObjectiveMiscComponent;
import ant.component.pf.ObjectiveRaceComponent;
import ant.component.pf.adv.ObjectiveAdvProbeComponent;
import ant.component.pf.adv.ObjectiveWantedComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.AgentPartitionAssignComponent;
import ant.component.universal.BeliefCivVoiceComponent;
import ant.component.universal.BeliefClearComponent;
import ant.component.universal.BeliefTrailComponent;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingWithNoCivilianComponent;
import ant.component.universal.ChangeSetDetailComponent;
import ant.component.universal.HumanInfoComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.LineOfSightPerceptionComponent;
import ant.component.universal.LocationMapComponent;
import ant.component.universal.RoadInfoCacheComponent;
import ant.component.universal.RoutineMessageComponent;
import ant.component.universal.TrafficStepSimComponent;
import ant.exception.AntComponentRegisterException;
import ant.liaison.message.IMessage;
import ant.map.component.AbstractRouteComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.RoadPassObjectsComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.map.component.WorldMaintainComponent;
import ant.map.jigsaw.JigsawMainComponent;
import ant.map.jigsaw.JigsawMarkerComponent;
import ant.map.partition.PartitionAreaComponent;
import ant.map.search.astar.TrailsAdvancedAStarRouteComponent;

/**
 * 
 * @author Ivan
 * 
 */
public class AntComponentKit extends AbstractAntKit {
	private static final Log log = LogFactory.getLog(AntComponentKit.class);

	private StandardEntityURN agentType;

	public AntComponentKit(StandardEntityURN agentType) throws AntComponentRegisterException {
		super();
		this.agentType = agentType;
		if (agentType != StandardEntityURN.CIVILIAN && agentType != null) {
			registerComponents();
		}
	}

	@Override
	public void update(AgentContext context) {
		super.update(context);

		AbstractAntCommunicationKit ack = context.getAntCommunicationKit();
		if (ack == null) {
			log.warn("Not found CommunicationKit!");
			return;
		}

		for (AntComponent comp : components) {
			if (comp instanceof MessageProducer) {
				for (IMessage msg : ((MessageProducer) comp).pollRadioMessages()) {
					ack.sendRadio(msg);
				}
				for (IMessage msg : ((MessageProducer) comp).pollVoiceMessages()) {
					ack.sendVoice(msg);
				}
			}
		}

	}

	private void registerComponents() throws AntComponentRegisterException {
		// common component
		registerComponent(WorldMaintainComponent.class);
		registerComponent(AntMapBasisComponent.class);
		registerComponent(ChangeSetDetailComponent.class);
		registerComponent(LineOfSightPerceptionComponent.class);
		// registerComponent(SampleComponent.class);

		// common dependent component
		registerComponent(PartitionAreaComponent.class);
		registerComponent(RoadInfoCacheComponent.class);
		registerComponent(StaticCityGraphComponent.class);
		registerComponent(HumanInfoComponent.class);
		registerComponent(AgentLocationComponent.class);
		registerComponent(JamComponent.class);
		registerComponent(BuildingConditionComponent.class);
		registerComponent(BuildingWithNoCivilianComponent.class);
		registerComponent(RoutineMessageComponent.class);
		registerComponent(LocationMapComponent.class);
		registerComponent(BeliefClearComponent.class);
		registerComponent(JigsawMarkerComponent.class);
		registerComponent(BeliefCivVoiceComponent.class);
		
		for (Class<? extends AntComponent> clazz : AntProperties.getExternalCommonComponents()) {
			registerComponent(clazz);
		}

		// particular component
		if (agentType.equals(StandardEntityURN.AMBULANCE_TEAM)) {
			registerComponent(RoadPassObjectsComponent.class);
			registerComponent(BeliefTrailComponent.class);
			registerComponent(TrailsAdvancedAStarRouteComponent.class);

			registerComponent(RealTimeFireTraceBuildingManagerComponent.class);
			registerComponent(FireTraceComponent.class);
			
			registerComponent(AgentPartitionAssignComponent.class);
			registerComponent(AmbulanceMemComponent.class);
			registerComponent(AmbulanceKernelComponent.class);
			registerComponent(ExplorationRecordComponent.class);

			for (Class<? extends AntComponent> clazz : AntProperties.getExternalATComponents()) {
				registerComponent(clazz);
			}

		} else if (agentType.equals(StandardEntityURN.FIRE_BRIGADE)) {
			registerComponent(RoadPassObjectsComponent.class);
			registerComponent(BeliefTrailComponent.class);
			registerComponent(TrailsAdvancedAStarRouteComponent.class);

			registerComponent(ActiveFireBrigadeComponent.class);
			registerComponent(ExtinguishRecordComponent.class);
			registerComponent(FireSimulatorComponent.class);
			registerComponent(FireSplitComponent.class);
			registerComponent(FireConvexHullComponent.class);
			registerComponent(AgentPartitionAssignComponent.class);
			registerComponent(FireTraceComponent.class);
			registerComponent(FireConfirmComponent.class);
			registerComponent(EstimationFireSplitComponent.class);
			registerComponent(EstimationFireConvexHullComponent.class);
			registerComponent(FireConvexHullPickComponent.class);

			for (Class<? extends AntComponent> clazz : AntProperties.getExternalFBComponents()) {
				registerComponent(clazz);
			}

		} else if (agentType.equals(StandardEntityURN.POLICE_FORCE)) {
			// registerComponent(BeliefIdiotAgentActiveComponent.class);
			// registerComponent(IdiotLowComponent.class);
			// registerComponent(IdiotHighComponent.class);
			// registerComponent(IdiotMidComponent.class);
			// registerComponent(NaiveLaneComponent.class);

			// registerComponent(BeliefCivVoiceComponent.class);
			
			registerComponent(JigsawMainComponent.class);
			registerComponent(TrafficStepSimComponent.class);
			
			// Maxico Version
			registerComponent(BeliefTrailComponent.class);
			registerComponent(ObjectiveMiscComponent.class);
			registerComponent(ObjectiveJigsawMetaComponent.class);
			registerComponent(ObjectiveRaceComponent.class);
			registerComponent(ObjectiveJigsawSystemComponent.class);
			registerComponent(ObjectiveJigsawDecisionComponent.class);
			registerComponent(ObjectiveWantedComponent.class);
			registerComponent(ObjectiveAdvProbeComponent.class);
			// FOR OLD registerComponent(ObjectiveProbeComponent.class);
			
			// Iran Version
//			registerComponent(BeliefInactiveAgentComponent.class);
//			registerComponent(ResponseComponent.class);
//			registerComponent(OrdinaryPrimaryComponent.class);
//			registerComponent(OrdinaryHighComponent.class);
//			registerComponent(OrdinaryMidComponent.class);

			for (Class<? extends AntComponent> clazz : AntProperties.getExternalPFComponents()) {
				registerComponent(clazz);
			}

		} else if (agentType.equals(StandardEntityURN.AMBULANCE_CENTRE)) {
		} else if (agentType.equals(StandardEntityURN.FIRE_STATION)) {
		} else if (agentType.equals(StandardEntityURN.POLICE_OFFICE)) {
		}
		
	}

	public AntMapBasisComponent getAntMapBasisComponent() {
		return (AntMapBasisComponent) getAntComponent(AntMapBasisComponent.class);
	}

	public ChangeSetDetailComponent getChangeSetDetailComponent() {
		return (ChangeSetDetailComponent) getAntComponent(ChangeSetDetailComponent.class);
	}

	public AbstractRouteComponent getAbstractRouteComponent() {
		return (AbstractRouteComponent) getAntComponent(TrailsAdvancedAStarRouteComponent.class);
	}

	public StaticCityGraphComponent getStaticCityGraphComponent() {
		return (StaticCityGraphComponent) getAntComponent(StaticCityGraphComponent.class);
	}

	public RoadPassObjectsComponent getRoadPassObjectsComponent() {
		return (RoadPassObjectsComponent) getAntComponent(RoadPassObjectsComponent.class);
	}

	public AgentLocationComponent getAgentLocationComponent() {
		return (AgentLocationComponent) getAntComponent(AgentLocationComponent.class);
	}

	public PartitionAreaComponent getPartitionAreaComponent() {
		return (PartitionAreaComponent) getAntComponent(PartitionAreaComponent.class);
	}

	public BeliefClearComponent getBeliefClearComponent() {
		return (BeliefClearComponent) getAntComponent(BeliefClearComponent.class);
	}

	public BuildingWithNoCivilianComponent getBuildingWithNoCivilianComponent() {
		return (BuildingWithNoCivilianComponent) getAntComponent(BuildingWithNoCivilianComponent.class);
	}

	public LocationMapComponent getLocationMapComponent() {
		return (LocationMapComponent) getAntComponent(LocationMapComponent.class);
	}

	public HumanInfoComponent getHumanInfoComponent() {
		return (HumanInfoComponent) getAntComponent(HumanInfoComponent.class);
	}

	public RoadInfoCacheComponent getRoadInfoCacheComponent() {
		return (RoadInfoCacheComponent) getAntComponent(RoadInfoCacheComponent.class);
	}

	public BuildingConditionComponent getBuildingConditionComponent() {
		return (BuildingConditionComponent) getAntComponent(BuildingConditionComponent.class);
	}

}
