package ant;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author Ivan
 *
 */
public class AntProperties {
	private static final Log log = LogFactory.getLog(AntProperties.class);
	
	private static final String CONFIG_FILE = "ant.properties";

	private static final String HOST_KEY = "antlaunch.host";
	private static final String HOST_DEFAULT = "127.0.0.1";
	
	private static final String RANDOM_SEED_KEY = "ant.random_seed";
	private static final long RANDOM_SEED_DEFAULT = 65991;
	
	private static final String EXTERNAL_COMMON_COMPONENTS_KEY = "component.common_list";
	private static final List<Class<? extends AntComponent>> EXTERNAL_COMMON_COMPONENTS_DEFAULT = Collections.EMPTY_LIST;
	
	private static final String EXTERNAL_AT_COMPONENTS_KEY = "component.at_list";
	private static final List<Class<? extends AntComponent>> EXTERNAL_AT_COMPONENTS_DEFAULT = Collections.EMPTY_LIST;
	
	private static final String EXTERNAL_PF_COMPONENTS_KEY = "component.pf_list";
	private static final List<Class<? extends AntComponent>> EXTERNAL_PF_COMPONENTS_DEFAULT = Collections.EMPTY_LIST;
	
	private static final String EXTERNAL_FB_COMPONENTS_KEY = "component.fb_list";
	private static final List<Class<? extends AntComponent>> EXTERNAL_FB_COMPONENTS_DEFAULT = Collections.EMPTY_LIST;
	
	private static final String COMMKIT_SIMPLE_KEY = "commkit.simple";
	private static final boolean COMMKIT_SIMPLE_DEFAULT = false;
	
	private static Properties property = null;

//	static {
//		property = new Properties();
//		try {
//			FileInputStream is = new FileInputStream(CONFIG_FILE);
//			property.load(is);
//			is.close();
//		} catch (Exception e) {
//			property = null;
//		}
//	}

	public static String getHost() {
		if (property == null) {
			return HOST_DEFAULT;
		} else {
			if (property.containsKey(HOST_KEY)) {
				return property.getProperty(HOST_KEY);
			} else {
				return HOST_DEFAULT;
			}
		}
	}
	
	private static List<Class<? extends AntComponent>> getExternalComponents(String key, List<Class<? extends AntComponent>> defaultList) {
		if (property == null) {
			return defaultList;
		} else {
			if (property.containsKey(key)) {
				List<Class<? extends AntComponent>> ret = new ArrayList<Class<? extends AntComponent>>();

				String tmp = property.getProperty(key);
				for (String s : tmp.split(",")) {
					String c = s.trim();
					if (c == null || c.equals(""))
						continue;
						
					try {
						Class<?> clazz = Class.forName(c);
						if (isComponent(clazz))
							ret.add((Class<? extends AntComponent>) clazz);
						else
							throw new Exception("class:" + clazz.getName() + " isn't AntComponent");
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

				return ret;
			} else {
				return defaultList;
			}
		}
	}
	
	public static List<Class<? extends AntComponent>> getExternalCommonComponents() {
		return getExternalComponents(EXTERNAL_COMMON_COMPONENTS_KEY, EXTERNAL_COMMON_COMPONENTS_DEFAULT);
	}
	
	public static List<Class<? extends AntComponent>> getExternalPFComponents() {
		return getExternalComponents(EXTERNAL_PF_COMPONENTS_KEY, EXTERNAL_PF_COMPONENTS_DEFAULT);
	}
	
	public static List<Class<? extends AntComponent>> getExternalATComponents() {
		return getExternalComponents(EXTERNAL_AT_COMPONENTS_KEY, EXTERNAL_AT_COMPONENTS_DEFAULT);
	}
	
	public static List<Class<? extends AntComponent>> getExternalFBComponents() {
		return getExternalComponents(EXTERNAL_FB_COMPONENTS_KEY, EXTERNAL_FB_COMPONENTS_DEFAULT);
	}
	
	public static long getRandomSeed() {
		if (property == null) {
			return RANDOM_SEED_DEFAULT;
		} else {
			if (property.containsKey(RANDOM_SEED_KEY)) {
				try {
					return Long.parseLong(property.getProperty(RANDOM_SEED_KEY));
				} catch (Exception e) {
					log.error(e);
					return RANDOM_SEED_DEFAULT; 
				}
			} else {
				return RANDOM_SEED_DEFAULT;
			}
		}
	}
	
	public static boolean isCommKitSimple() {
		if (property == null) {
			return COMMKIT_SIMPLE_DEFAULT;
		} else {
			if (property.containsKey(COMMKIT_SIMPLE_KEY)) {
				try {
					return Boolean.parseBoolean(property.getProperty(COMMKIT_SIMPLE_KEY));
				} catch (Exception e) {
					log.error(e);
					return COMMKIT_SIMPLE_DEFAULT; 
				}
			} else {
				return COMMKIT_SIMPLE_DEFAULT;
			}
		}
	}
	
	private static boolean isComponent(Class<?> clazz) {
		// for (Class<?> c = clazz.getSuperclass(); c != null; c =
		// c.getSuperclass()) {
		// if (c == AntComponent.class) {
		// return true;
		// }
		// }
		for (Class<?> c : clazz.getInterfaces()) {
			if (c == AntComponent.class) {
				return true;
			}
		}
		return false;
	}
}
