package ant.agent;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractAntAgent;

/**
 * 
 * @author Ivan
 * 
 */
public final class CommonAmbulanceTeam extends CommonAgent<AmbulanceTeam> {
	private Collection<EntityID> unexploredBuildings;

	public CommonAmbulanceTeam(AbstractAntAgent<AmbulanceTeam> ant) {
		super(ant);
	}

	@Override
	protected final EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.AMBULANCE_TEAM);
	}

	@Override
	public final StandardEntityURN getAgentType() {
		return StandardEntityURN.AMBULANCE_TEAM;
	}

	@Override
	protected void exceptionInitial() {
		super.exceptionInitial();
		unexploredBuildings = new HashSet<EntityID>();
	}

	@Override
	protected void exceptionThink(int time, ChangeSet changed, Collection<Command> heard) {
		if (time == 3) {// config.getIntValue(kernel.KernelConstants.IGNORE_AGENT_COMMANDS_KEY))
						// {
			// Subscribe to channel 1
			sendSubscribe(time, 1);
		}

		updateUnexploredBuildings(changed);
		// Am I transporting a civilian to a refuge?
		if (someoneOnBoard()) {
			// Am I at a refuge?
			if (location() instanceof Refuge) {
				// Unload!
				sendUnload(time);
				return;
			} else {
				// Move to a refuge
				List<EntityID> path = search.breadthFirstSearch(me().getPosition(), refugeIDs);
				if (path != null) {
					sendMove(time, path);
					return;
				}
				// What do I do now? Might as well carry on and see if we can
				// dig someone else out.
			}
		}
		// Go through targets (sorted by distance) and check for things we can
		// do
		for (Human next : getTargets()) {
			if (next.getPosition().equals(location().getID())) {
				// Targets in the same place might need rescueing or loading
				if ((next instanceof Civilian) && next.getBuriedness() == 0
						&& !(location() instanceof Refuge)) {
					// Load
					sendLoad(time, next.getID());
					return;
				}
				if (next.getBuriedness() > 0) {
					// Rescue
					sendRescue(time, next.getID());
					return;
				}
			} else {
				// Try to move to the target
				List<EntityID> path = search.breadthFirstSearch(me().getPosition(), next
						.getPosition());
				if (path != null) {
					sendMove(time, path);
					return;
				}
			}
		}
		// Nothing to do
		List<EntityID> path = search.breadthFirstSearch(me().getPosition(), unexploredBuildings);
		if (path != null) {
			sendMove(time, path);
			return;
		}
		sendMove(time, randomWalk());

	}

	private boolean someoneOnBoard() {
		for (StandardEntity next : model.getEntitiesOfType(StandardEntityURN.CIVILIAN)) {
			if (((Human) next).getPosition().equals(getID())) {
				return true;
			}
		}
		return false;
	}

	private List<Human> getTargets() {
		List<Human> targets = new ArrayList<Human>();
		for (StandardEntity next : model.getEntitiesOfType(StandardEntityURN.CIVILIAN,
				StandardEntityURN.FIRE_BRIGADE, StandardEntityURN.POLICE_FORCE,
				StandardEntityURN.AMBULANCE_TEAM)) {
			Human h = (Human) next;
			if (h == me()) {
				continue;
			}
			if (h.isHPDefined() && h.isBuriednessDefined() && h.isDamageDefined()
					&& h.isPositionDefined() && h.getHP() > 0
					&& (h.getBuriedness() > 0 || h.getDamage() > 0)) {
				targets.add(h);
			}
		}
		Collections.sort(targets, new DistanceSorter(location(), model));
		return targets;
	}

	private void updateUnexploredBuildings(ChangeSet changed) {
		for (EntityID next : changed.getChangedEntities()) {
			unexploredBuildings.remove(next);
		}
	}
}
