package ant.agent;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.PoliceForce;
import ant.AbstractJobBasedAntAgent;
import ant.agent.job.AbstractJob;
import ant.agent.job.at.IsStuckOrBuriedCallHelpJob;
import ant.agent.job.at.SaveCivilianJobAdv;
import ant.agent.job.at.SaveHumanJob;
import ant.agent.job.fb.ExtinguishEntityJobWithFSC;
import ant.agent.job.fb.GetWaterJob;
import ant.agent.job.fb.GotoPartitionJob;
import ant.agent.job.fb.StuckInBuildingJobForFB;
import ant.agent.job.fb.TraceFireJob;
import ant.agent.job.pf.PFSaveMySelfJob;
import ant.agent.job.pf.PFStuckInBuildingJob;
import ant.agent.job.pf.PFStuckInRoadJob;
import ant.agent.job.pf.adv.PFPartitionalObjectiveJobAdv;
import ant.agent.job.pf.adv.PFPartitionalProbeJobAdv;
import ant.agent.job.pf.adv.PFPartitionalWantedJobAdv;
import ant.agent.job.universal.AdvancedStuckInBuildingJob;
import ant.agent.job.universal.AgentBuriedJob;
import ant.agent.job.universal.AntSaveMyselfJob;
import ant.agent.job.universal.IgnorationJob;
import ant.agent.job.universal.SearchJobForATAndFB;
import ant.agent.job.universal.StuckInRoadJob;
import ant.utils.agent.AntStandardLaunchTool;

public class ServerLauncher {

	public static class ServerLauncherAT extends AbstractJobBasedAntAgent<AmbulanceTeam> {
		@Override
		protected List<AbstractJob> produceJobs() {
			List<AbstractJob> list = new ArrayList<AbstractJob>();
			list.add(new AdvancedStuckInBuildingJob<AmbulanceTeam>(context));
			list.add(new StuckInRoadJob<AmbulanceTeam>(context));
			list.add(new IsStuckOrBuriedCallHelpJob(context));
			list.add(new AntSaveMyselfJob(context));
			list.add(new SaveHumanJob(context, true));
			list.add(new SaveCivilianJobAdv(context));
			list.add(new TraceFireJob<AmbulanceTeam>(context));
			list.add(new SearchJobForATAndFB<AmbulanceTeam>(context));
			return list;
		}

	}

	public static class ServerLauncherPF extends AbstractJobBasedAntAgent<PoliceForce> {
		@Override
		protected List<AbstractJob> produceJobs() {
			List<AbstractJob> list = new ArrayList<AbstractJob>();
			list.add(new IgnorationJob(context));
			list.add(new PFStuckInBuildingJob(context));
			list.add(new PFStuckInRoadJob(context));
			list.add(new PFSaveMySelfJob(context));
			list.add(new PFPartitionalObjectiveJobAdv(context));
			list.add(new PFPartitionalWantedJobAdv(context));
			list.add(new PFPartitionalProbeJobAdv(context));
			return list;
		}

	}

	public static class ServerLauncherFB extends AbstractJobBasedAntAgent<FireBrigade> {
		@Override
		protected List<AbstractJob> produceJobs() {
			List<AbstractJob> list = new ArrayList<AbstractJob>();
			list.add(new AgentBuriedJob<FireBrigade>(context));
			list.add(new StuckInRoadJob<FireBrigade>(context));
			list.add(new StuckInBuildingJobForFB(context));
			list.add(new AntSaveMyselfJob(context));
			list.add(new GetWaterJob(context));
			list.add(new TraceFireJob<FireBrigade>(context));
			list.add(new ExtinguishEntityJobWithFSC(context));
			list.add(new GotoPartitionJob<FireBrigade>(context));
			list.add(new SearchJobForATAndFB<FireBrigade>(context));
			return list;
		}

	}

	public static void main(String[] args) throws InstantiationException, IllegalAccessException {
		AntStandardLaunchTool tool = new AntStandardLaunchTool();
		tool.setNoAgent(false);
		tool.setPoliceForceAnt(ServerLauncherPF.class);
		tool.setAmbulanceTeamAnt(ServerLauncherAT.class);
		tool.setFireBrigadeAnt(ServerLauncherFB.class);
		tool.launch(args);
	}
}
