package ant.agent.behavior;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.at.AmbulanceMemComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.HumanInfoComponent;
import ant.utils.civilian.CivilianInfo;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Fun
 * 
 */
public class ATBehavior extends AntBehavior<AmbulanceTeam> {
	private static Log log = LogFactory.getLog(ATBehavior.class);
	
	public static final int AT_TASK_READD_INTERVAL = 20;
	public static final int AT_MIN_RESCUE_COST = 4;
	
	public ATBehavior(AgentContext context) {
		super(context);
	}

	public boolean isCivilian(EntityID human) {
		if (human == null) {
			return false;
		}
		StandardEntity man = context.getWorld().getEntity(human);
		if (man instanceof Civilian) {
			return true;
		} else {
			return false;
		}
	}

	public EntityID getLoadedCivilian() {
		Collection<StandardEntity> civilians;
		civilians = context.getWorld().getEntitiesOfType(
				StandardEntityURN.CIVILIAN);
		for (StandardEntity entity : civilians) {
			if (((Human) entity).getPosition().equals(context.getAgentID())) {
				return entity.getID();
			}
		}
		return null;
	}

	public boolean canLoadCivilian(EntityID civilian) {
		if (!canSee(civilian)) {
			return false;
		}

		StandardEntity se = context.getWorld().getEntity(civilian);

		if (se == null || !(se instanceof Civilian)) {
			return false;
		}

		Civilian c = (Civilian) se;

		if (!c.isPositionDefined()) {
			return false;
		}

		if (!c.getPosition().equals(context.getHuman().getPosition())) {
			return false;
		}

		StandardEntity position = context.getWorld().getEntity(c.getPosition());

		if (position instanceof Refuge) {
			return false;
		}

		if (!(position instanceof Building)) {
			return false;
		}

		if (!c.isHPDefined()) {
			return false;
		}
		if (c.getHP() <= 0) {
			return false;
		}
		
		if (c.isBuriednessDefined()) {
			if(c.getBuriedness() > 0){
				return false;
			}
		}else{
			if(!c.isDamageDefined()){
				return false;
			}
			if(c.getDamage()<=0&&c.getHP()==10000){
				return false;
			}else{
				log.error("c.getDamage()==0 && c.getHP()!=10000");
			}
		}
		
		return true;
	}

	public boolean canRescueHuman(EntityID hid) {
		if (!canSee(hid)) {
			return false;
		}
		
		StandardEntity se = context.getWorld().getEntity(hid);
		if (!(se instanceof Human)) {
			return false;
		}
		Human human = (Human) se;
		if (!human.isPositionDefined()) {
			return false;
		}
		if (!human.getPosition().equals(context.getHuman().getPosition())) {
			return false;
		}
		StandardEntity position = context.getWorld().getEntity(
				human.getPosition());
		if (!(position instanceof Building)) {
			return false;
		}
		if (position instanceof Refuge) {
			return false;
		}
		if (!human.isBuriednessDefined()) {
			return false;
		}
		if (human.getBuriedness() <= 0) {
			return false;
		}
		if (!human.isHPDefined()) {
			return false;
		}
		if (human.getHP() <= 0) {
			return false;
		}
		
		return true;
	}
	
	public boolean isATEnough(CivilianInfo ci){
		HumanInfoComponent hic = (HumanInfoComponent) context.getAntComponentKit().getAntComponent(HumanInfoComponent.class);
		AmbulanceMemComponent amc = (AmbulanceMemComponent) context.getAntComponentKit().getAntComponent(AmbulanceMemComponent.class);
		StandardEntity se = context.getWorld().getEntity(ci.getPosition());
		if(se != null && se instanceof Building){
			Set<EntityID> set = amc.getATSetInBuilding((Building)se);
			if(set == null || set.size() == 0){
				return false;
			}
			
			Set<EntityID> civSet = hic.getBuildingCivilianMap().get(se.getID());
			
			if(civSet == null|| civSet.size() == 0){
				log.error("BuildingCivilianMap error!");
				return true;
			}
			
			int timeneeded = ci.getTimeCostToRefuge() + ci.getBuriedness()*(civSet.size()<=3?civSet.size():3)/set.size();
			
			int savedTime = amc.getCurrentTime()+timeneeded;
			
			if(savedTime < (ci.getTimeToDie() - 10)){
				return true;
			}else{
				return false;
			}
			
		}else{
			return false;
		}
	}
	
	public int[] getAreaXY(EntityID id){
		int [] ret = new int[2];
		ret[0] = Integer.MAX_VALUE;
		ret[1] = Integer.MAX_VALUE; 
		StandardEntity se = context.getWorld().getEntity(id);
		if(se instanceof Area){
			ret[0] = ((Area) se).getX();
			ret[1] = ((Area) se).getY();
		}
		return ret;
	}
	
	/**
	 * 
	 * @param id
	 * @return area or null
	 */
	public Area getArea(EntityID id){
		Area ret = null;
		StandardEntity se = context.getWorld().getEntity(id);
		if(se instanceof Area){
			ret = (Area) se;
		}
		return ret;
	}
	/**
	 * 
	 * @param areaid
	 * @return Integer.MAX_VALUE if no such area
	 */
	public long manhattanDistanceToArea(EntityID areaid){
		long ret = Integer.MAX_VALUE;
		StandardEntity se  = context.getWorld().getEntity(areaid);
		if(se instanceof Area){ 
			ret = (long)GeoUtils.manhattanDistance(context.getHuman().getX(),context.getHuman().getY() , ((Area) se).getX(),((Area) se).getY()); // TODO 静态路
		}
		return ret;
	}

	public List<Civilian> getNearbyCivilian() {
		List<Civilian> ret = new ArrayList<Civilian>();
		Collection<StandardEntity> result = context.getAntComponentKit()
				.getChangeSetDetailComponent()
				.getEntitiesInSightMap(StandardEntityURN.CIVILIAN).values();
		for (StandardEntity standardEntity : result) {
			ret.add((Civilian) standardEntity);
		}
		return ret;
	}

	public boolean canBeSavedNaive(int hp, int dmg, int buriedness, int timestamp) {
		int deathTime = getDeathTimeNaive(hp, dmg, timestamp);
		if (timestamp + buriedness > deathTime) {
			return false;
		}
		return true;
	}

	public boolean isAgentRescued(EntityID agentID){
		AgentLocationComponent alc = context.getAntComponentKit().getAgentLocationComponent();
		EntityID initLocation = alc.getAgentInitialLocation(agentID);
		EntityID lastLocation = alc.getAgentLastLocation(agentID);
		if(!initLocation.equals(lastLocation)){
			return true;
		}else{
			return false;
		}
	}
}
