package ant.agent.behavior;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.universal.ChangeSetDetailComponent;
import ant.map.objects.RouteResult;
import ant.utils.Pair;
import ant.utils.blockade.Locator;
import ant.utils.geo.GeoUtils;
import ant.utils.geo.Line;
import ant.utils.geo.Point;

/**
 * 比任务更小粒度的行为
 * 
 * @author Fun
 */
public class AntBehavior<E extends Human> {
	private static Log log = LogFactory.getLog(AntBehavior.class);

	private static final int RANDOM_WALK_LENGTH = 25;
	public static final int AVERAGE_MOVE_DISTANCE = 41000;

	private static final int POSITION_IN_ROAD_TO_MOVE_MIN_DISTANCE = 500;
	private static final int POSITION_IN_ROAD_TO_MOVE_DEFAULT_DISTANCE = 5000;

	protected AgentContext context;
	private Random random;
	protected StandardWorldModel world;

	public AntBehavior(AgentContext context) {
		this.context = context;
		this.world = context.getWorld();
		this.random = new Random(context.getAgentID().getValue() + 1);
	}

	private int MoveTryTime = 0;

	/**
	 * 判断是否能移动到目的地并返回认为可达的路径
	 * 
	 * @param target
	 *            目的地ID
	 * @return 返回到达目的地的路径，如果无法到达目标则返回null
	 */
	// public List<EntityID> canMoveTo(EntityID target) {
	// List<EntityID> path = null;
	// // TODO
	// return path;
	// }

	/**
	 * Agent从卡住区域移动出来
	 */
	// public void moveOutFromTrap() {
	// // TODO 与Agent移动历史有关，判断是否被卡的尝试次数根据地图设置
	// }

	public boolean isStuck(boolean isArea) {
		Blockade blocakde = getStuckBlockade(isArea);
		if (blocakde == null) {
			return false;
		} else {
			return true;
		}
	}

	public boolean isBuried() {
		Human me = context.getHuman();
		if (me.isBuriednessDefined()) {
			if (me.getBuriedness() > 0) {
				return true;
			}
		}
		return false;
	}

	public boolean isInRefuge() {
		boolean ret;
		StandardEntity location;

		location = context.getHuman().getPosition(context.getWorld());
		ret = location instanceof Refuge;
		return ret;
	}

	public boolean canSee(EntityID id) {
		Set<EntityID> ids;
		boolean ret;

		ids = context.getChangeSet().getChangedEntities();
		ret = ids.contains(id);
		return ret;
	}

	public Area getMyPosition() {
		if (context.getHuman().getPosition(context.getWorld()) == null) {
			log.fatal("getPosition() == null");
		}
		return (Area) context.getHuman().getPosition(context.getWorld());
	}

	public int x() {
		return context.getHuman().getX();
	}

	public int y() {
		return context.getHuman().getY();
	}

	public Road getEntranceRoad(Building building) {
		for (EntityID next : building.getNeighbours()) {
			StandardEntity entity = context.getWorld().getEntity(next);
			if (entity instanceof Road) {
				return (Road) entity;
			}
		}
		return null;
	}

	@Deprecated
	public Blockade getStuckBlockade(boolean isArea) {
		StandardEntity location = this.context.getHuman().getPosition(context.getWorld());
		if (location instanceof Road) {
			Road road = (Road) location;
			if (road.isBlockadesDefined()) {
				List<EntityID> blockades = road.getBlockades();
				for (EntityID blockadeID : blockades) {
					Blockade blockade = (Blockade) context.getWorld().getEntity(blockadeID);
					if (isArea) {
						if (blockade.getShape().intersects(
								Locator.getBounds(context.getHuman(), context.getWorld()))) {
							return blockade;
						}
					} else {
						if (blockade.getShape()
								.contains(
										Locator.getPosition(context.getHuman().getID(),
												context.getWorld()))) {
							return blockade;
						}
					}
				}
			}
		}
		return null;
	}

	public List<Human> getNearbyTeammates() {
		return getNearbyAgents(context.getAgentType());
	}

	public List<Human> getNearbyAgents(StandardEntityURN urn) {
		Collection<StandardEntity> agents;
		List<Human> nearbyagents;

		nearbyagents = new ArrayList<Human>();
		if (!(urn == StandardEntityURN.AMBULANCE_TEAM || urn == StandardEntityURN.FIRE_BRIGADE || urn == StandardEntityURN.POLICE_FORCE)) {
			return nearbyagents;
		}

		Map<EntityID, StandardEntity> result = context.getAntComponentKit()
				.getChangeSetDetailComponent().getEntitiesInSightMap(urn);

		for (StandardEntity next : result.values()) {
			if (next.getID().equals(context.getAgentID())) {
				continue;
			}
			nearbyagents.add((Human) next);
		}
		return nearbyagents;
	}

	public List<Human> getNearbyAgents() {
		List<Human> nearbyagents = new ArrayList<Human>();
		Map<EntityID, StandardEntity> result = context.getAntComponentKit()
				.getChangeSetDetailComponent()
				.getEntitiesInSightMap(StandardEntityURN.AMBULANCE_TEAM);
		for (StandardEntity next : result.values()) {
			if (next.getID().equals(context.getAgentID())) {
				continue;
			}
			nearbyagents.add((Human) next);
		}
		result = context.getAntComponentKit().getChangeSetDetailComponent()
				.getEntitiesInSightMap(StandardEntityURN.FIRE_BRIGADE);
		for (StandardEntity next : result.values()) {
			if (next.getID().equals(context.getAgentID())) {
				continue;
			}
			nearbyagents.add((Human) next);
		}
		result = context.getAntComponentKit().getChangeSetDetailComponent()
				.getEntitiesInSightMap(StandardEntityURN.POLICE_FORCE);
		for (StandardEntity next : result.values()) {
			if (next.getID().equals(context.getAgentID())) {
				continue;
			}
			nearbyagents.add((Human) next);
		}
		return nearbyagents;
	}

	/**
	 * 
	 * @param area
	 * @return true if there is any civilian seen in this area, dead civilian is
	 *         ignored
	 */
	public boolean seeCivilianIn(Area area) {
		ChangeSetDetailComponent csd = context.getAntComponentKit().getChangeSetDetailComponent();
		for (StandardEntity entity : csd.getEntitiesInSight(StandardEntityURN.CIVILIAN)) {
			Civilian c = (Civilian) entity;
			if (c.isHPDefined() && c.getHP() > 0 && c.getPosition().equals(area.getID())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return the nearest refuge if there is one
	 */
	public Refuge getNearestRefuge() {
		Human agent = context.getHuman();
		EntityID here = agent.getPosition();
		double minDistance = Double.MAX_VALUE;
		Refuge nearestRefuge = null;
		for (Refuge refuge : context.getAntComponentKit().getAntMapBasisComponent().getRefugeSet()) {
			RouteResult route = context.getAntComponentKit().getAbstractRouteComponent()
					.search(here, agent.getX(), agent.getY(), refuge.getID());
			if (!route.hasAcceptableRoute()) {
				// can't reach this refuge
			} else {
				if (route.getDistance() < minDistance) {
					minDistance = route.getDistance();
					nearestRefuge = refuge;
				}
			}
		}
		return nearestRefuge;
	}

	public int getTimeCostMoveToAreaNaive(EntityID areaid) {
		StandardEntity se = context.getWorld().getEntity(areaid);
		long dist = Integer.MAX_VALUE;
		if (se instanceof Area) {
			int x = ((Area) se).getX();
			int y = ((Area) se).getY();
			dist = (long) GeoUtils.manhattanDistance(context.getHuman().getX(), context.getHuman()
					.getY(), x, y); // TODO 静态路
		}
		return ((int) dist / AVERAGE_MOVE_DISTANCE);
	}

	public int getTimeCostArea2AreaNaive(EntityID areaid1, EntityID areaid2) {
		StandardEntity se1 = context.getWorld().getEntity(areaid1);
		StandardEntity se2 = context.getWorld().getEntity(areaid2);
		long dist = Integer.MAX_VALUE;
		if ((se1 instanceof Area) && (se2 instanceof Area)) {
			Area a1 = (Area) se1;
			Area a2 = (Area) se2;
			dist = (long) GeoUtils.manhattanDistance(a1, a2);
		}
		return (int) (dist / AVERAGE_MOVE_DISTANCE);
	}

	public boolean isInBuildingHumanNeedHelp(EntityID hid) {
		StandardEntity se = context.getWorld().getEntity(hid);
		if (!(se instanceof Human)) {
			return false;
		}
		Human human = (Human) se;
		if (!human.isPositionDefined()) {
			return false;
		}

		StandardEntity position = context.getWorld().getEntity(human.getPosition());

		if (!(position instanceof Building)) {
			return false;
		}
		if (position instanceof Refuge) {
			return false;
		}
		if (!human.isBuriednessDefined()) {
			return false;
		}
		if (human.getBuriedness() <= 0) {
			return false;
		}
		if (!human.isHPDefined()) {
			return false;
		}
		if (human.getHP() <= 0) {
			return false;
		}
		return true;
	}

	public List<EntityID> getRandomWalk() {
		List<EntityID> path = new ArrayList<EntityID>();
		Set<EntityID> seen = new HashSet<EntityID>();
		Area current = getMyPosition();
		for (int i = 0; i < RANDOM_WALK_LENGTH; i++) {
			path.add(current.getID());
			seen.add(current.getID());
			List<EntityID> neighbours = current.getNeighbours();
			Collections.shuffle(neighbours, random);
			boolean found = false;
			for (EntityID next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				StandardEntity ar = context.getWorld().getEntity(next);
				if (ar instanceof Building) {
					continue;
				}
				current = (Area) ar;
				found = true;
				break;
			}
			if (!found) {
				break;
			}

		}

		if (path.size() == 1)
			return getRandomExWalk();
		return path;
	}

	private List<EntityID> getRandomExWalk() {
		List<EntityID> path = new ArrayList<EntityID>();
		Set<EntityID> seen = new HashSet<EntityID>();
		Area current = getMyPosition();
		for (int i = 0; i < RANDOM_WALK_LENGTH; i++) {
			path.add(current.getID());
			seen.add(current.getID());
			List<EntityID> neighbours = current.getNeighbours();
			Collections.shuffle(neighbours, random);
			boolean found = false;
			for (EntityID next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				StandardEntity ar = context.getWorld().getEntity(next);
				current = (Area) ar;
				found = true;
				break;
			}
			if (!found) {
				break;
			}
		}
		return path;
	}
	
//	private List<EntityID> getRandomExWalkWithRandomLength(int maxstep) {
//		List<EntityID> path = new ArrayList<EntityID>();
//		Set<EntityID> seen = new HashSet<EntityID>();
//		Area current = getMyPosition();
//		int lenstep = random.nextInt(maxstep - 1) + 2;
//		for (int i = 0; i < lenstep; i++) {
//			path.add(current.getID());
//			seen.add(current.getID());
//			List<EntityID> neighbours = current.getNeighbours();
//			Collections.shuffle(neighbours, random);
//			boolean found = false;
//			for (EntityID next : neighbours) {
//				if (seen.contains(next)) {
//					continue;
//				}
//				StandardEntity ar = context.getWorld().getEntity(next);
//				current = (Area) ar;
//				found = true;
//				break;
//			}
//			if (!found) {
//				break;
//			}
//		}
//		return path;
//	}
	

	public static boolean checkMoveRoute(StandardWorldModel world, List<EntityID> moveRoute) {
		if (moveRoute == null || moveRoute.size() == 0) {
			log.fatal("Illegal moveroute, Null Pointer");
			return false;
		}

		// Check area
		for (EntityID id : moveRoute) {
			if (world.getEntity(id) instanceof Area) {
			} else {
				log.fatal("Illegal moveroute, ID:" + id + "," + world.getEntity(id));
				return false;
			}
		}

		// Check connectivity
		for (int i = 0; i < moveRoute.size() - 1; i++) {
			Area from = (Area) world.getEntity(moveRoute.get(i));
			Area to = (Area) world.getEntity(moveRoute.get(i + 1));
			if (from.getEdgeTo(to.getID()) == null) {
				log.fatal("Illegal moveroute, from:" + from + ", to:" + to);
				return false;
			}
		}

		return true;
	}

	public int getDeathTimeNaive(int hp, int dmg, int timestamp) {
		int deathTime;
		if (hp >= 0) {
			if (dmg != 0) {
				deathTime = (int) (hp / (dmg * 1.05));
				deathTime += timestamp;
			} else {
				deathTime = 500;
			}
		} else {
			deathTime = -1;
		}
		return deathTime;
	}

	public Pair<Point, Road> getPositionInRoadToMove(Building b) {
		Edge edge = null;
		List<Road> roads = new ArrayList<Road>();
		for (Edge e : b.getEdges()) {
			if (e.isPassable()) {
				edge = e;
				roads = getRelatedRoad(edge);
				if (roads.isEmpty()) {
					continue;
				}

				Point pos = null;
				boolean hasPos = false;
				Road targetRoad = null;
				int distance = POSITION_IN_ROAD_TO_MOVE_DEFAULT_DISTANCE;
				while (true) {
					pos = GeoUtils.getPerpendicular(
							new Line(new Point(edge.getStartX(), edge.getStartY()), new Point(edge
									.getEndX(), edge.getEndY())), distance,
							new Point(b.getX(), b.getY()), false);
					if (pos == null) {
						log.error("getPerpendicular failed!");
						break;
					}
					for (Road r : roads) {
						if (GeoUtils.inPolygon(r.getApexList(), pos.getX(), pos.getY())) {
							hasPos = true;
							targetRoad = r;
							break;
						}
					}
					if (hasPos) {
						break;
					} else {
						distance /= 2;
						if (distance <= POSITION_IN_ROAD_TO_MOVE_MIN_DISTANCE) {
							pos = null;
							break;
						}
					}
				}
				if (hasPos) {
					return new Pair<Point, Road>(pos, targetRoad);
				}
			}
		}
		return null;
	}

	private List<Road> getRelatedRoad(Edge e) {
		List<Road> rv = new ArrayList<Road>();
		HashSet<Road> roads = new HashSet<Road>();
		EntityID neighbourID = e.getNeighbour();
		if (neighbourID == null) {
			log.error("no neighbour found!");
			return rv;
		}

		StandardEntity neighbour = world.getEntity(neighbourID);
		if (neighbour instanceof Road) {
			Road neighbourRoad = (Road) neighbour;
			rv.add(neighbourRoad);
			roads.add(neighbourRoad);
			for (EntityID id : neighbourRoad.getNeighbours()) {
				StandardEntity entity = world.getEntity(id);
				if (entity instanceof Road) {
					Road r = (Road) entity;
					rv.add(r);
					roads.add(r);
				}
			}
			for (Road r : roads) {
				for (EntityID id : r.getNeighbours()) {
					StandardEntity entity = world.getEntity(id);
					if (entity instanceof Road && !roads.contains(entity)) {
						rv.add((Road) entity);
					}
				}
			}
		}
		return rv;
	}
}
