package ant.agent.behavior;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.component.universal.BuildingWithNoCivilianComponent;
import ant.component.universal.ChangeSetDetailComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.RandomSelector;
import ant.utils.blockade.CleanType;
import ant.utils.blockade.RoadUtils;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ChangeSetDetailComponent} {@link AntMapBasisComponent} {@link StaticCityGraphComponent}
 *         {@link BuildingWithNoCivilianComponent}
 */

// FIXME!!! 检查所有ISblockadeDefined, 如果是building就要格外注意 因为moveroute和loca都有可能是Building

public class PFBehavior extends AntBehavior<PoliceForce> {
	private static final Log log = LogFactory.getLog(PFBehavior.class);

	private static final int CLOST_BLOCK_DISTANCE = 1000;

	public static enum CleanLevel {
		Fastest, Normal, Almost, All,
	}

	private int maxClearDistance;
	private Human human;
	private ChangeSetDetailComponent chs;
	private StandardWorldModel world;
	private Random rand;
	private StaticCityGraphComponent city;
	private AntMapBasisComponent amb;
	// private NaiveLaneComponent laneComp;
	private BuildingWithNoCivilianComponent bnc;

	public PFBehavior(AgentContext context) {
		super(context);
		this.maxClearDistance = ConfigKey.getClearRepairDistance(context.getConfig());
		this.human = context.getHuman();
		this.world = context.getWorld();
		this.rand = new Random(context.getHuman().getID().getValue() + 1);

		this.amb = context.getAntComponentKit().getAntMapBasisComponent();
		this.city = context.getAntComponentKit().getStaticCityGraphComponent();
		this.chs = context.getAntComponentKit().getChangeSetDetailComponent();
		this.bnc = context.getAntComponentKit().getBuildingWithNoCivilianComponent();
		// this.laneComp = (NaiveLaneComponent) context.getAntComponentKit().getAntComponent(NaiveLaneComponent.class);
	}

	public Random getMyRandom() {
		return rand;
	}

	public int getMaxClearDistance() {
		return maxClearDistance;
	}

	/**
	 * may be return null
	 */
	@Override
	public Refuge getNearestRefuge() {
		StandardEntity here = context.getHuman().getPosition(this.world);
		double min = Double.MAX_VALUE;
		Refuge nearest = null;
		for (Refuge rr : amb.getRefugeSet()) {
			double tmp = city.getDistanceToRefuge((Area) here, rr);
			if (tmp < min) {
				min = tmp;
				nearest = rr;
			}
		}
		return nearest;
	}

	// 无关紧要的
	public static boolean isBlockPeddling(Blockade b, Road r) {
		if (!r.isBlockadesDefined()) {
			log.error("Abnormal block state");
			return false;
		}
		// if (r.getBlockades().size() > 2)
		// return false;

		List<Point2D> tmp = GeometryTools2D.vertexArrayToPoints(b.getApexes());
		double a = GeometryTools2D.computeArea(tmp);
		if (a < 500 * 500 * 3.14 * 0.5) {
			// Road Center in Blockade
			if (GeoUtils.inPolygon(b.getApexes(), r.getX(), r.getY()))
				return false;
			else
				return true;
		} else {
			return false;
		}

	}

	public boolean isInClearRange(Blockade b) {
		int x = human.getX();
		int y = human.getY();
		List<Line2D> lines = GeometryTools2D.pointsToLines(GeometryTools2D.vertexArrayToPoints(b.getApexes()), true);
		double best = Double.MAX_VALUE;
		Point2D origin = new Point2D(x, y);
		for (Line2D next : lines) {
			Point2D closest = GeometryTools2D.getClosestPointOnSegment(next, origin);
			double d = GeometryTools2D.getDistance(origin, closest);
			if (d < best) {
				best = d;
			}
		}
		int intBest = (int) best;
		if (maxClearDistance < 0 || intBest < maxClearDistance) {
			return true;
		} else {
			return false;
		}
	}

	public double getMinDistanceToBlockade(int x, int y, Blockade b) {
		List<Line2D> lines = GeometryTools2D.pointsToLines(GeometryTools2D.vertexArrayToPoints(b.getApexes()), true);
		double best = Double.MAX_VALUE;
		Point2D origin = new Point2D(x, y);
		for (Line2D next : lines) {
			Point2D closest = GeometryTools2D.getClosestPointOnSegment(next, origin);
			double d = GeometryTools2D.getDistance(origin, closest);
			if (d < best) {
				best = d;
			}
		}
		int intBest = (int) best;
		return intBest;
	}

	/**
	 * TODO 优化 for BeliefClearComponent without regard to agent
	 */
	public static boolean isRoadClean(StandardWorldModel world, Road r) {
		if (!r.isBlockadesDefined())
			return false;
		else {
			if (r.getBlockades().isEmpty()) {
				return true;
			} else {
				for (EntityID id : r.getBlockades()) {
					if (!isBlockPeddling((Blockade) world.getEntity(id), r)) {
						return false;
					}
				}
				return true;
			}
		}
	}

	public Set<Blockade> getOnMyPathInsightInRangeBlockadeSubset(List<EntityID> moveRoute) {
		if (moveRoute == null || moveRoute.size() == 0) {
			// throw new IllegalArgumentException();
			log.fatal("Illegal Argument, Null Pointer");
			return Collections.emptySet();
		}

		Set<Blockade> blocks = new HashSet<Blockade>();
		Area b = (Area) world.getEntity(human.getPosition());
		if (b.isBlockadesDefined()) {
			for (EntityID bid : b.getBlockades()) {
				Blockade bb = (Blockade) world.getEntity(bid);
				if (canSee(bid) && isInClearRange(bb)) {
					blocks.add(bb);
				}
			}
		}
		for (int i = 0; i < 7; i++) {
			if (moveRoute.size() <= i)
				break;
			Area a = (Area) world.getEntity(moveRoute.get(i));
			if (a.isBlockadesDefined()) {
				for (EntityID bid : a.getBlockades()) {
					Blockade bb = (Blockade) world.getEntity(bid);
					if (canSee(bid) && isInClearRange(bb)) {
						blocks.add(bb);
					}
				}
			}
		}
		return blocks;
	}

	public Set<Blockade> getInsightInRangeBlockades() {
		Set<Blockade> ret = new HashSet<Blockade>();
		for (StandardEntity se : chs.getEntitiesInSight(StandardEntityURN.BLOCKADE)) {
			Blockade block = (Blockade) se;
			if (isInClearRange(block)) {
				ret.add(block);
			}
		}
		return ret;
	}

	public boolean hasInsightInRangeBlockade() {
		for (StandardEntity se : chs.getEntitiesInSight(StandardEntityURN.BLOCKADE)) {
			Blockade block = (Blockade) se;
			if (isInClearRange(block)) {
				return true;
			}
		}
		return false;
	}

	public Blockade getRandomOneOfInsightInRangeBlockade() {
		Set<Blockade> ret = new HashSet<Blockade>();
		for (StandardEntity se : chs.getEntitiesInSight(StandardEntityURN.BLOCKADE)) {
			Blockade block = (Blockade) se;
			if (isInClearRange(block)) {
				ret.add(block);
			}
		}
		
		if (ret.isEmpty())
			return null;
		return RandomSelector.randomSelect(ret, 1, getMyRandom()).iterator().next();
	}

	private void removePadddling(Set<Blockade> input) {
		Iterator<Blockade> iter = input.iterator();
		while (iter.hasNext()) {
			Blockade b = iter.next();
			// world.getEntity(b.getPosition())
			Area a = (Area) world.getEntity(b.getPosition());
			if (a instanceof Road) {
				if (isBlockPeddling(b, (Road) a)) {
					iter.remove();
				}
			}
		}
	}

	private void removeCannotCleanReach(Set<Blockade> input) {
		Iterator<Blockade> iter = input.iterator();
		while (iter.hasNext()) {
			if (!isInClearRange(iter.next())) {
				iter.remove();
			}
		}
	}

	private Set<Blockade> getAvailableRoadBlockadeToCivBuilding() {
		Set<Blockade> blks = new HashSet<Blockade>();
		Map<EntityID, StandardEntity> map = chs.getEntitiesInSightMap(StandardEntityURN.ROAD);
		for (StandardEntity se : chs.getEntitiesInSight(StandardEntityURN.BUILDING)) {
			Building b = (Building) se;
			if (!bnc.isBuildingWithNoCivilian(b.getID())) {
				// 有Civ 或者 没有看到中心
				for (EntityID nei : b.getNeighbours()) {
					Area tmp = (Area) world.getEntity(nei);
					if (tmp instanceof Road) {
						Road r = (Road) tmp;
						// Road in sight
						if (map.containsKey(r.getID())) {
							if (r.isBlockadesDefined()) {
								for (EntityID id : r.getBlockades()) {
									blks.add((Blockade) world.getEntity(id));
								}
							}
						}
					}
				}
			}
		}
		removeCannotCleanReach(blks);
		return blks;
	}

	private Set<Blockade> getAvailableRoadBlockadeToTruthCivBuilding(Set<EntityID> sightRoads) {
		Set<Blockade> blks = new HashSet<Blockade>();
		List<StandardEntity> list = chs.getEntitiesInSight(StandardEntityURN.CIVILIAN);
		for (StandardEntity se : list) {
			if (se instanceof Civilian) {
				Civilian c = (Civilian) se;
				// log.info("Seen:" + c);

				if (!isInBuildingHumanNeedHelp(c.getID()))
					continue;

				Building where = (Building) c.getPosition(world);

				// log.info("Seen " + c + " In " + where);
				EntityID pickRoadId = null;
				for (EntityID id : where.getNeighbours()) {
					if (sightRoads.contains(id)) {
						pickRoadId = id;
						break;
					}
				}

				if (pickRoadId != null) {
					Area tmp = (Area) world.getEntity(pickRoadId);
					if (!(tmp instanceof Road)) {
						continue;
					}
					Road r = (Road) tmp;
					if (!r.isBlockadesDefined()) {
						continue;
					}
					for (EntityID id : r.getBlockades()) {
						Blockade tempblk = (Blockade) world.getEntity(id);
						if (isInClearRange(tempblk))
							blks.add(tempblk);
					}
				}
			}
		}

		return blks;
	}

	public Blockade getPathFastestBlockades(List<EntityID> moveRoute) {
		EntityID blkid = RoadUtils.getBlockadeOnPath(context, CleanType.Partial, moveRoute);
		if (blkid == null)
			return null;
		return (Blockade) world.getEntity(blkid);
	}

	// TODO
	public Set<Blockade> getJamBlockades() {
		return Collections.emptySet();
	}

	// TODO!!!!
	// private Set<Blockade> getNormalPathBlockade(List<EntityID> moveRoute) {
	//
	// if (moveRoute == null || moveRoute.size() == 0) {
	// // throw new IllegalArgumentException();
	// log.fatal("Illegal Argument, Null Pointer");
	// return Collections.emptySet();
	// }
	//
	// Set<Blockade> retblocks = new HashSet<Blockade>();
	// Area mypos = (Area) world.getEntity(human.getPosition());
	// Area fstpos = (Area) world.getEntity(moveRoute.get(0));
	// int x1, y1, x2, y2;
	// if (!fstpos.getID().equals(mypos.getID())) {
	// if (mypos.isBlockadesDefined()) {
	// Edge edge = mypos.getEdgeTo(fstpos.getID());
	// if (edge != null) {
	// x1 = human.getX();
	// y1 = human.getY();
	// x2 = (edge.getStartX() + edge.getEndX()) / 2;
	// y2 = (edge.getStartY() + edge.getEndY()) / 2;
	// for (EntityID blkid : mypos.getBlockades()) {
	// Blockade tempblk = (Blockade) world.getEntity(blkid);
	// if (tempblk != null) {
	// if (RoadUtils.isOnSegment(tempblk, x1, y1, x2, y2)) {
	// retblocks.add(tempblk);
	// }
	// }
	// }
	// }
	// }
	// moveRoute.add(0, mypos.getID());
	// }
	//
	// for (int i = 1; i < 7; i++) {
	// if (moveRoute.size() <= i + 1) {
	// break;
	// }
	//
	// Area a = (Area) world.getEntity(moveRoute.get(i));
	// Area b = (Area) world.getEntity(moveRoute.get(i + 1));
	// if (a.isBlockadesDefined()) {
	// for (EntityID bid : a.getBlockades()) {
	// Blockade bb = (Blockade) world.getEntity(bid);
	// if (canSee(bid) && isInClearRange(bb)) {
	// blocks.add(bb);
	// }
	// }
	// }
	// }
	//
	// if (b.isBlockadesDefined()) {
	// for (EntityID bid : b.getBlockades()) {
	// Blockade bb = (Blockade) world.getEntity(bid);
	// if (canSee(bid) && isInClearRange(bb)) {
	// blocks.add(bb);
	// }
	// }
	// }
	//
	// return blocks;
	//
	// }

	private static Set<EntityID> getRoadIDsInSight(AgentContext context) {
		ChangeSetDetailComponent csdc = context.getAntComponentKit().getChangeSetDetailComponent();
		Set<EntityID> roadsInSight = new HashSet<EntityID>();
		for (StandardEntity se : csdc.getEntitiesInSight(StandardEntityURN.ROAD)) {
			roadsInSight.add(se.getID());
		}
		return roadsInSight;
	}

	private Set<Blockade> getOnPathNormalBlockades(List<EntityID> moveRoute, Set<EntityID> sightRoads) {

		if (moveRoute == null || moveRoute.size() == 0) {
			return Collections.emptySet();
		}

		Set<Blockade> retblocks = new HashSet<Blockade>();
		Area mypos = (Area) world.getEntity(human.getPosition());
		Area fstpos = (Area) world.getEntity(moveRoute.get(0));

		if (!fstpos.getID().equals(mypos.getID())) {
			if (mypos.isBlockadesDefined()) {
				Edge edge = mypos.getEdgeTo(fstpos.getID());
				if (edge != null) {
					int x1, y1, x2, y2;
					x1 = human.getX();
					y1 = human.getY();
					x2 = (edge.getStartX() + edge.getEndX()) / 2;
					y2 = (edge.getStartY() + edge.getEndY()) / 2;
					for (EntityID blkid : mypos.getBlockades()) {
						Blockade tempblk = (Blockade) world.getEntity(blkid);
						if (tempblk != null) {
							if (RoadUtils.isOnSegment(tempblk, x1, y1, x2, y2)) {
								if (isInClearRange(tempblk))
									retblocks.add(tempblk);
							}
						}
					}
				} else {
					log.error("Not Connect: " + mypos + " to " + fstpos + " " + moveRoute);
				}
			}
			// moveRoute.add(0, mypos.getID());
		}

		// for (int i = 2; i < moveRoute.size() && i < 5; i++) {
		// Area a = (Area) world.getEntity(moveRoute.get(i));
		// Area b = (Area) world.getEntity(moveRoute.get(i - 1));
		// Area c = (Area) world.getEntity(moveRoute.get(i - 2));
		//
		// if (a == null || b == null || c == null) {
		// log.error("NULL:" + moveRoute.get(i) + "," + moveRoute.get(i - 1) + "," + moveRoute.get(i - 2));
		// continue;
		// }
		//
		// if (a.getEdgeTo(b.getID()) == null || b.getEdgeTo(c.getID()) == null) {
		// log.error("A,B,C not connected:" + a + "," + b + "," + c + " " + moveRoute);
		// continue;
		// }
		//
		// if (a.getID().equals(c.getID())) {
		// log.error("A,C same:" + a + "," + c + " " + moveRoute);
		// continue;
		// }
		//
		// if (b.isBlockadesDefined()) {
		// continue;
		// }
		//
		// if (!sightRoads.contains(b.getID())) {
		// continue;
		// }
		//
		// Edge fromEdge = b.getEdgeTo(a.getID());
		// Edge toEdge = b.getEdgeTo(c.getID());
		// List<EntityID> bids = b.getBlockades();
		// for (EntityID blkid : bids) {
		// Blockade blockade = (Blockade) world.getEntity(blkid);
		// if (blockade == null) {
		// continue;
		// }
		// if (RoadUtils.isOnSegment(blockade, (fromEdge.getStartX() + fromEdge.getEndX()) / 2, (fromEdge.getStartY() +
		// fromEdge.getEndY()) / 2,
		// (toEdge.getStartX() + toEdge.getEndX()) / 2, (toEdge.getStartY() + toEdge.getEndY()) / 2)) {
		// if (isInClearRange(blockade)) {
		// retblocks.add(blockade);
		// }
		// }
		// }
		// }

		return retblocks;
	}

	public Set<Blockade> getStuckAgentBlockade() {
		Set<Blockade> ret = new HashSet<Blockade>();
		List<Human> list = chs.getAgentsInSight();
		for (StandardEntity se : this.chs.getEntitiesInSight(StandardEntityURN.BLOCKADE)) {
			Blockade b = (Blockade) se;
			if (isInClearRange(b)) {
				for (Human h : list) {
					int x = h.getX();
					int y = h.getY();
					if (GeoUtils.inPolygon(b.getApexes(), x, y)) {
						ret.add(b);
					}
				}
			}
		}
		return ret;
	}

	public Set<Blockade> getCloseAgentBlockade(int threshold) {
		Set<Blockade> ret = new HashSet<Blockade>();
		List<Human> list = chs.getAgentsInSight();
		for (StandardEntity se : this.chs.getEntitiesInSight(StandardEntityURN.BLOCKADE)) {
			Blockade b = (Blockade) se;
			if (isInClearRange(b)) {
				for (Human h : list) {
					int x = h.getX();
					int y = h.getY();
					if (getMinDistanceToBlockade(x, y, b) <= threshold) {
						ret.add(b);
					}
				}
			}
		}
		return ret;
	}

	/**
	 * CleanLevel.All: 路上全部清除 <br>
	 * CleanLevel.Almost: 只留下PeddlingBlock, Path周围的也会清除, 无论是否看到Civilian, 比如听到了CivHelp <br>
	 * CleanLevel.Normal: Fastest + c2c_blockade, Path周围只有看到Civ小路会清除 <br>
	 * CleanLevel.Fastest: 只求通过 <br>
	 * TODO 优先清理困住Agent的Blockade, 也许有问题, 如果一旦 该Agent移动了一些, 又发出Stuck命令, Stuck命令的发送条件? 会对整个系统有影响 <br>
	 * 
	 * @return target blockade, maybe null.
	 */
	public EntityID cleverPathBlockadeSelect(List<EntityID> moveRoute, CleanLevel level) {
		if (moveRoute == null || moveRoute.size() == 0) {
			log.fatal("Illegal Argument, Null Pointer");
			return null;
		}
		// log.info("Clever Path: " + moveRoute + " level:" + level);
		Set<Blockade> sab = getStuckAgentBlockade();
		if (!sab.isEmpty())
			return RandomSelector.randomSelect(sab, 1, this.rand).iterator().next().getID();

		Set<Blockade> blks = new HashSet<Blockade>();
		Set<EntityID> sightRoads = RoadUtils.getRoadIDsInSight(context);
		
		Set<Blockade> closeAgentBlockades = getCloseAgentBlockade(CLOST_BLOCK_DISTANCE);
		
		switch (level) {
		case Fastest:

			EntityID fbid = RoadUtils.getBlockadeOnPath(context, CleanType.Partial, moveRoute);
			if (fbid != null)
				blks.add((Blockade) world.getEntity(fbid));

			blks.addAll(getOnPathNormalBlockades(moveRoute, sightRoads));

			// return RoadUtils.getBlockadeOnPath(context, CleanType.Partial, moveRoute);
			if (blks.isEmpty())
				return null;

			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		case Normal:
			if (!closeAgentBlockades.isEmpty()) {
				// log.info("Clean Close agent blockade");
				return RandomSelector.randomSelect(closeAgentBlockades, 1, this.rand).iterator().next().getID();
			}
			
			EntityID nbid = RoadUtils.getBlockadeOnPath(context, CleanType.Partial, moveRoute);
			
			if (nbid != null) {
				Blockade tempb = (Blockade) world.getEntity(nbid);
				if (!isInClearRange(tempb)) {
					log.error("Not in clear range!!");
				}
				// log.info("Add blk:" + tempb);
				blks.add((Blockade) world.getEntity(nbid));
			}

			blks.addAll(getOnPathNormalBlockades(moveRoute, sightRoads));
			blks.addAll(getAvailableRoadBlockadeToTruthCivBuilding(sightRoads));

			if (blks.isEmpty())
				return null;

			Blockade retblk = RandomSelector.randomSelect(blks, 1, this.rand).iterator().next();
			// log.info("Select:" + retblk);
			return retblk.getID();

		case Almost:
			if (!closeAgentBlockades.isEmpty())
				return RandomSelector.randomSelect(closeAgentBlockades, 1, this.rand).iterator().next().getID();
			
			// TODO
			blks.addAll(getOnMyPathInsightInRangeBlockadeSubset(moveRoute));
			blks.addAll(getInsightInRangeBlockades());
			removePadddling(blks);
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		case All:
			if (!closeAgentBlockades.isEmpty())
				return RandomSelector.randomSelect(closeAgentBlockades, 1, this.rand).iterator().next().getID();
			
			// blks.addAll(getOnMyPathInsightInRangeBlockadeSubset(moveRoute));
			blks.addAll(getInsightInRangeBlockades());
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();
		default:
			return null;
		}
	}

	/**
	 * CleanLevel.All: 周围全部清除 <br>
	 * CleanLevel.Almost: 只留下PeddlingBlock, Area周围的也会清除, 无论是否看到Civilian, 比如听到了CivHelp <br>
	 * CleanLevel.Normal: 只留下PeddlingBlock, Area周围只有看到Civ小路会清除 <br>
	 * CleanLevel.Fastest: 只留下PeddlingBlock, 只清理当前Area <br>
	 * TODO 优先清理困住Agent的Blockade, 也许有问题, 如果一旦 该Agent移动了一些, 又发出Stuck命令, Stuck命令的发送条件?, 会对整个系统有影响 <br>
	 * 
	 * @return target blockade, maybe null.
	 */
	public EntityID cleverAroundBlockadeSelect(CleanLevel level) {
		// log.info("Clever Around level:" + level);
		
		Set<Blockade> sab = getStuckAgentBlockade();
		if (!sab.isEmpty())
			return RandomSelector.randomSelect(sab, 1, this.rand).iterator().next().getID();

		Set<Blockade> blks = getInsightInRangeBlockades();
		if (blks.isEmpty())
			return null;
		
		Set<Blockade> closeAgentBlockades = getCloseAgentBlockade(CLOST_BLOCK_DISTANCE * 2);
		if (!closeAgentBlockades.isEmpty())
			return RandomSelector.randomSelect(closeAgentBlockades, 1, this.rand).iterator().next().getID();
		
		switch (level) {
		case Fastest:
			removePadddling(blks);
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		case Normal:
			removePadddling(blks);
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		case Almost:
			removePadddling(blks);
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		case All:
			if (blks.isEmpty())
				return null;
			return RandomSelector.randomSelect(blks, 1, this.rand).iterator().next().getID();

		default:
			return null;
		}
	}

	public boolean hasSmartAroundBlockade(CleanLevel level) {
		EntityID id = cleverAroundBlockadeSelect(level);
		return (id != null);
	}

	// ???
	// private boolean isSecondJudgeToRoadNeedClean(Area from, Road to) {
	// if (to.isBlockadesDefined() && to.getBlockades().size() == 0)
	// return false;
	//
	// int margin = 500 * 2;
	// Edge e = from.getEdgeTo(to.getID());
	// if (e == null) {
	// log.error("not connected");
	// return true;
	// }
	// int x = (e.getStartX() + e.getEndX()) / 2;
	// int y = (e.getStartY() + e.getEndY()) / 2;
	// for (EntityID id : to.getBlockades()) {
	// Blockade b = (Blockade) world.getEntity(id);
	// List<Segment> ss = Converter.generateSegments(b.getApexes());
	// for (Segment s : ss) {
	// if (GeoUtils.getDistanceToLine(s, x, y) < margin) {
	// return true;
	// }
	// }
	// }
	// return false;
	// }

	@Deprecated
	public boolean isNormalClearAllAround(Set<Blockade> rr) {
		for (Blockade b : rr) {
			Road r = (Road) world.getEntity(b.getPosition());
			if (!isBlockPeddling(b, r)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return null, if no normal-clear-blk
	 */
	@Deprecated
	public Blockade getNormalTargetCleanBlockade(Set<Blockade> rr) {
		Set<Blockade> tmp = new HashSet<Blockade>();
		for (Blockade b : rr) {
			Road r = (Road) world.getEntity(b.getPosition());
			if (!isBlockPeddling(b, r)) {
				tmp.add(b);
			}
		}
		if (tmp.isEmpty())
			return null;
		return RandomSelector.randomSelect(tmp, 1, rand).iterator().next();
	}

}
