package ant.agent.job;

import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.Human;
import ant.AgentContext;
import ant.MessageProducer;
import ant.agent.behavior.AntBehavior;
import ant.agent.command.AntCommand;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Fun, Ivan
 * 
 * @param <E>
 */
public abstract class AbstractJob<E extends Human> implements MessageProducer {
	
	protected JobState state;
	protected AntBehavior<? extends Human> behavior;
	protected final AgentContext context;
	protected final E agent;
	
	private Set<IMessage> voiceMessages = new HashSet<IMessage>();
	private Set<IMessage> radioMessages = new HashSet<IMessage>();
	
	public AbstractJob(AgentContext context) {
		this.state = JobState.Inactive;
		this.context = context;
		this.behavior = context.getBehavior();
		this.agent = (E) context.getHuman();
	}
	
	/**
	 * {ShouldExecuted==false} callback
	 */
	public void overleap() {}

	public abstract void overtaken();

	public abstract boolean shouldExecute();

	public abstract AntCommand execute();

	public final JobState getState() {
		return this.state;
	}
	
	public abstract String getJobName();
	
	protected final void addVoiceMessage(IMessage msg) {
		voiceMessages.add(msg);
	}
	
	protected final void addRadioMessage(IMessage msg) {
		radioMessages.add(msg);
	}
	
	@Override
	public final Set<IMessage> pollVoiceMessages() {
		HashSet<IMessage> temp = new HashSet<IMessage>(voiceMessages);;
		voiceMessages.clear();
		return temp;
	}
	
	@Override
	public final Set<IMessage> pollRadioMessages() {
		HashSet<IMessage> temp = new HashSet<IMessage>(radioMessages);;
		radioMessages.clear();
		return temp;
	}
	
	
}
