package ant.agent.job.at;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.command.AntMoveXY;
import ant.agent.job.AntAmbulanceTeamJob;
import ant.agent.job.JobState;
import ant.component.at.Exploration;
import ant.component.at.ExplorationRecordComponent;

public class ATExplorationJob extends AntAmbulanceTeamJob{

	private static Log log = LogFactory.getLog(ATExplorationJob.class);
	
	public static final int MIN_RETRAVEL_INTERVAL = 20;
	private AgentMover amover;
	private Exploration<Building> exploration;
	private List<EntityID> path;
	private ExplorationRecordComponent erc;
	
	
	public ATExplorationJob(AgentContext context) {
		super(context);
		erc = (ExplorationRecordComponent) context.getAntComponentKit().getAntComponent(ExplorationRecordComponent.class);
		amover = new AgentMover(context);
	}
		
	private JobState getExceptedDo(){
		JobState ret = JobState.Inactive;
		if(state == JobState.Inactive || state == JobState.Exploring){
			ret = JobState.Exploring;
		}else if(state == JobState.StuckInBuilding){
			ret = JobState.ExploringRandom;
		}else if(state == JobState.ExploringRandom){
			ret = JobState.Exploring;
		}

		return ret;
	}

	private boolean isExplorationCompeted() {
		// FIXME
		this.exploration = erc.getExplorationBuilding();
		return exploration.isExplorationCompeted();
	}

	@Override
	public void overtaken() {
		state = JobState.Inactive;
		path = null;
		
	}

	@Override
	public boolean shouldExecute() {
		if(context.getTimestamp() <= ConfigKey.getIgnoreTime(context.getConfig())){
			return false;
		}
		
		state = getExceptedDo();
		
		if ((state == JobState.Inactive || state == JobState.Exploring)
				&& !isExplorationCompeted()) {
			// Stuck In Building
//			if(amover.isStuckInBuilding(time)){
//				log.info(">>>>>>>>>>>>>>>>>>>>>>>> StuckInBuilding Agent:"+agent.getID());
//	    		state = State.StuckInBuilding;
//	    		return true;
//			}
			exploration = erc.getExplorationBuilding();
			List<Building> buildings = exploration.getUnexploredEntities();

//			log.info("UnexploredEntity:"+buildings);
			
			List<Building> safeBuildings = DecisionManager
					.ignoreBurningBuildings(buildings);
			
//			Map<EntityID,BuildingInfo> infos = agent.getSubworld().getSubmodelHistoryInfo().getBuildingInfos();
//			for(Building building: safeBuildings){
//				if(infos.get(building.getID()) == null){
//					continue;
//				}
//				if(infos.get(building.getID()).getFieryness()==8){
//					travelRecorder.getExplorationBuilding().addToExplorationHistory(building);
//				}
//			}

//			log.info("safeBuilding:"+safeBuildings);
			
			Set<StandardEntity> unreachableLocation = new HashSet<StandardEntity>(
					erc.getUnreachableLocation());
			
//			log.info("unreachableLocation :"+unreachableLocation);

			safeBuildings.removeAll(unreachableLocation);

			if (safeBuildings.isEmpty()) {
//				log.info("Go to Unreachable building");
				
				safeBuildings = DecisionManager
						.ignoreBurningBuildings(buildings);
				unreachableLocation.removeAll(erc
						.getUnreachableLocationOlderThan(context.getTimestamp()
								- MIN_RETRAVEL_INTERVAL));
				safeBuildings.removeAll(unreachableLocation);
			}

			if (safeBuildings.isEmpty()) {
//				log.debug("No safe building is found to explore.");
				return false;
			}

//			log.info("safeBuildings:"+safeBuildings);
			for (int i = 0; i < safeBuildings.size(); i++) {
				
				if (amover.canMove(safeBuildings.get(i).getID())) {
					this.path = amover.getPath();
					
//					log.info("Agent:" + agent.getID() + " @" + agent.me().getPosition()
//							+ " exploration" + safeBuildings.get(i).getID()+ " path:" + path);
					return true;
				}

				erc.addUnreachableLocation(safeBuildings.get(i),
						context.getTimestamp());
				
				// request help
				// call for pf's help
//				EntityID nextArea = amover.getNextAreaStatic(safeBuildings.get(i).getID());
//				if(nextArea != null){
//					if(amover.frontBlocked(nextArea)){
//						Area here = (Area) context.getWorld().getEntity(agent.getPosition());
//						Edge e = here.getEdgeTo(nextArea);
//						if(e != null){
//							Area next = (Area) context.getWorld().getEntity(nextArea);
//							log.info("sendMsg:cleanHigh "+next.getID()+" Agent:"+agent.getID());
//							//agent.pushMessage(new BitCleanEntityHigh(next, (e.getStartX()+e.getEndX())/2, (e.getStartY()+e.getEndY())/2));
//						}else{
//							log.fatal("Area:"+here.getID()+" has no neighbour:"+nextArea+". Agent:"+agent.getID());
//						}
//					}
//				}

//				log.info(agent.getID() + " can't move to "
//						+ safeBuildings.get(i).getID());
			}
//			log.info(agent.getID() + " can't move");
			return false;
		}else if(state == JobState.ExploringRandom){
			path = amover.randomWalk();
			return true;
		}
//		log.info("Exploration is already completed.");
		return false;
	}

	@Override
	public AntCommand execute() {
		if(state == JobState.Inactive || state== JobState.Exploring){
			if (path == null) {
				log.fatal("path == null");
				return null;
			}
//			System.err.println("state:"+state+" ,path:"+path);
			return new AntMove(context.getTimestamp(), path);
		}else if(state == JobState.StuckInBuilding){
			List<EntityID> path = new ArrayList<EntityID>();
    		path.add(agent.getPosition());
    		Area area = behavior.getMyPosition();
    		return new AntMoveXY(context.getTimestamp(), path, area.getX(), area.getY());
		}else if(state == JobState.ExploringRandom){
			return new AntMove(context.getTimestamp(), path);
		}
		return null;
	}

	@Override
	public String getJobName() {
		return "ATExplorationJob";
	}

}
