package ant.agent.job.at;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.map.component.AbstractRouteComponent;
import ant.map.objects.RouteResult;
import ant.utils.blockade.RoadUtils;
import ant.utils.geo.GeoUtils;

/**
 * Temp
 * 
 * @author Fun
 * 
 */
public class AgentMover {
	private static Log log = LogFactory.getLog(AgentMover.class);

	private static final int RANDOM_WALK_LENGTH = 50;
	public static final double BLOCK_MIN_MOVE_DIST = 2000;

	private AgentContext context;
	private Human me;
	private EntityID target;
	private StandardWorldModel world;
	private AbstractRouteComponent rc;
	private int tryLimit = 1;
	private int tryTime = 0;
	private int noPathLimit = 1;
	private int noPathTime = 0;
	private int lastx, lasty;

	private List<EntityID> lastPath;

	private List<EntityID> retPath;

	public AgentMover(AgentContext context) {
		this.context = context;
		this.me = context.getHuman();
		this.rc = context.getAntComponentKit().getAbstractRouteComponent();
		this.world = this.context.getWorld();
	}

	public void initial() {
		this.tryTime = 0;
		this.noPathTime = 0;
	}

	public boolean canMove(EntityID target) {

		if (!target.equals(this.target)) {
			initial();
			this.target = target;
		}

		retPath = null;

		if (me.getPosition().equals(target)) {
			return false;
		} else {
			Area here = (Area) me.getPosition(world);
			int x = me.getX();
			int y = me.getY();

			if (!movedCheck(x, y)) {
				if (tryTime > tryLimit) {
					return false;
				}
			}

			Area targetArea = (Area) world.getEntity(target);
			
//			System.err.println("here:"+here.getID()+" ,"+targetArea.getID());
			RouteResult rr = rc.search(here.getID(), x, y,targetArea.getID());

			if (!rr.hasAcceptableRoute()) {
//				this.noPathTime++;
//				if (this.noPathTime <= this.noPathLimit) {
//					log.info("No Path and retry" + here.getID() + "x:" + x
//							+ " y:" + y);
//					// r = srp.getRoute(this.agent.me().getPosition(), target);
//					// if (r == null) {
//					// log.info("srp return null");
//					// this.retPath = this.lastPath;
//					// } else {
//					// this.retPath = r.getRoutes();
//					// }
//					// return true;
//				} else {
					return false;
//				}
			}
			this.noPathTime = 0;
			List<EntityID> path = rr.getMoveRoute();
//			System.err.println(path);
			this.retPath = path;
			return true;
		}
	}

	public List<EntityID> getPath() {
		return retPath;
	}

	private boolean movedCheck(int x, int y) {
		double movedist = GeoUtils.dist(x, y, lastx, lasty);
		if (movedist < BLOCK_MIN_MOVE_DIST) {
			tryTime++;
			return false;
		} else {
			tryTime = 0;
			return true;
		}
	}

	public void record() {
		this.lastPath = retPath;
		this.lastx = me.getX();
		this.lasty = me.getY();
	}

	// public boolean couldeMoveStatic(EntityID target) {
	// Route r = srp.getRoute(this.agent.me().getPosition(), target);
	//
	// if (r == null) {
	// log.info("srp return null");
	// this.retPath = this.lastPath;
	// } else {
	// if (r.getRoutes().size() > 0) {
	// if (frontBlocked(r.getRoutes().get(0))) {
	// return false;
	// }
	// }
	// this.retPath = r.getRoutes();
	// }
	// return true;
	// }

	public boolean frontBlocked(EntityID nextArea) {
		Area here = (Area) world.getEntity(me.getPosition());
		Area next = (Area) world.getEntity(nextArea);

		if (here.isBlockadesDefined()) {
			List<EntityID> bids1 = here.getBlockades();
			for (EntityID entityID : bids1) {
				if (RoadUtils.isOnSegment((Blockade) world.getEntity(entityID),
						me.getX(), me.getY(), next.getX(), next.getY())) {
					return true;
				}
			}
		}
		if (next.isBlockadesDefined()) {
			List<EntityID> bids2 = next.getBlockades();
			for (EntityID entityID : bids2) {
				if (RoadUtils.isOnSegment((Blockade) world.getEntity(entityID),
						me.getX(), me.getY(), next.getX(), next.getY())) {
					return true;
				}
			}
		}
		return false;
	}

	// public boolean isStuckInBuilding(int time){
	// if(time<5){
	// return false;
	// }
	//
	// StandardEntity here = me.getPosition(world);
	//
	// if(!(here instanceof Building)){
	// return false;
	// }
	//
	// AbstractCommand[] coms = agent.getSendHistory();
	// AbstractCommand ac1 = coms[time - 1];
	// AbstractCommand ac2 = coms[time - 2];
	// AbstractCommand ac3 = coms[time - 3];
	// AbstractCommand ac4 = coms[time - 4];
	//
	// boolean b1 = (ac1 == null) ||
	// ac1.getURN().equals(StandardMessageURN.AK_MOVE.toString());
	// boolean b2 = (ac2 == null) ||
	// ac2.getURN().equals(StandardMessageURN.AK_MOVE.toString());
	// boolean b3 = (ac3 == null) ||
	// ac3.getURN().equals(StandardMessageURN.AK_MOVE.toString());
	// boolean b4 = (ac4 == null) ||
	// ac4.getURN().equals(StandardMessageURN.AK_MOVE.toString());
	//
	// if(b1&b2&b3&b4){
	// EntityID stay = agent.me().getPosition();
	// TravelRecorder travelRecorder =
	// agent.getRecordManager().getTravelRecorder();
	// EntityID p1 = travelRecorder.getPositionHistory(time-1);
	// if(p1 == null || (!p1.equals(stay))){
	// return false;
	// }
	// EntityID p2 = travelRecorder.getPositionHistory(time-2);
	// if(p2 == null || (!p2.equals(stay))){
	// return false;
	// }
	// EntityID p3 = travelRecorder.getPositionHistory(time-3);
	// if(p3 == null || (!p3.equals(stay))){
	// return false;
	// }
	// EntityID p4 = travelRecorder.getPositionHistory(time-4);
	// if(p4 == null || (!p4.equals(stay))){
	// return false;
	// }
	//
	// return true;
	// }
	// return false;
	// }

	public List<EntityID> randomWalk() {
		List<EntityID> result = new ArrayList<EntityID>(RANDOM_WALK_LENGTH);
		Set<EntityID> seen = new HashSet<EntityID>();
		EntityID current = me.getPosition();
		for (int i = 0; i < RANDOM_WALK_LENGTH; ++i) {
			result.add(current);
			seen.add(current);
			
			Area area = (Area) world.getEntity(current);
			List<EntityID> possible = area.getNeighbours();
		
			Collections.shuffle(possible, new Random());
			boolean found = false;
			for (EntityID next : possible) {
				if (seen.contains(next)) {
					continue;
				}
				current = next;
				found = true;
				break;
			}
			if (!found) {
				// We reached a dead-end.
				break;
			}
		}
		return result;
	}

	// public EntityID getNextAreaStatic(EntityID targetID) {
	// Route route = this.srp.getRoute(agent.me().getPosition(), targetID);
	// if (route.getRoutes().size() > 0) {
	// return route.getRoutes().get(0);
	// } else {
	// return null;
	// }
	// }

}
