package ant.agent.job.at;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntLoad;
import ant.agent.command.AntMove;
import ant.agent.command.AntRescue;
import ant.agent.command.AntUnload;
import ant.agent.job.AntAmbulanceTeamJob;
import ant.agent.job.JobState;
import ant.component.at.AmbulanceMemComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.liaison.message.MCivilianRescued;
import ant.liaison.message.MRescueUnreachNear;
import ant.liaison.message.MRescueUnreachable;
import ant.liaison.message.MVoiceRescuingCivilian;
import ant.utils.civilian.AmbulanceCommuless;
import ant.utils.civilian.CivilianInfo;
import ant.utils.civilian.RescueInfoListener;
import ant.utils.geo.GeoUtils;

public class CommuLessSaveJob extends AntAmbulanceTeamJob implements RescueInfoListener {
	private static Log log = LogFactory.getLog(CommuLessSaveJob.class);
	private List<EntityID> path;
	private EntityID currentTarget = null;
	private AmbulanceMemComponent amc;
	private AmbulanceCommuless acl;
	private AgentMover amover;
	private int sendRescued = 0;

	public CommuLessSaveJob(AgentContext context) {
		super(context);
		amc = (AmbulanceMemComponent) context.getAntComponentKit().getAntComponent(AmbulanceMemComponent.class);
		amc.addRescueInfoListener(this);
		amover = new AgentMover(context);
		acl = new AmbulanceCommuless(amc, behavior);
	}

	@Override
	public void overtaken() {
		state = JobState.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		state = getExpectedDo();

		if (state == JobState.UnloadingHuman) {
		} else if (state == JobState.MovingToRefuge) {

			if (sendRescued <= 1) {
				// context.getAntCommunicationKit().sendRadio(new MCivilianRescued(currentTarget));
				addRadioMessage(new MCivilianRescued(currentTarget));
				
				sendRescued = 2;
			}

			if (sendRescued <= 2) {
				// context.getAntCommunicationKit().sendRadio(new MCivilianRescued(currentTarget));
				addRadioMessage(new MCivilianRescued(currentTarget));
				
				sendRescued = 0;
			}

			Refuge refuge = behavior.getNearestRefuge();

			if (refuge == null) {
				log.info("No refuge can move.");
				return false;
			}
			if (amover.canMove(refuge.getID())) {
				this.path = amover.getPath();
				return true;
			} else {
				// FIXME
				return false;
			}
		} else if (state == JobState.LoadingHuman) {

		} else if (state == JobState.RescuingHuman) {

		} else if (state == JobState.MovingToHuman) {
			CivilianInfo ci = amc.getCivilianInfoMap().get(currentTarget);
			if (ci == null) {
				log.error("ci == null");
				return false;
			}
			if (amover.canMove(ci.getPosition())) {
				path = amover.getPath();
				return true;
			} else {
				AgentLocationComponent alc = context.getAntComponentKit().getAgentLocationComponent();
				List<AgentLocationInfo> list = alc.getAgentLocationInfos(context.getAgentID());
				if (list.size() >= 3) {
					AgentLocationInfo al0 = list.get(list.size() - 1);
					AgentLocationInfo al1 = list.get(list.size() - 2);
					AgentLocationInfo al2 = list.get(list.size() - 3);
					if (al0.getLocation().equals(al1.getLocation()) && al0.getLocation().equals(al2.getLocation())) {
						if (amc.getUnreachableCivs().containsKey(ci.getID())) {
							int timebefore = amc.getUnreachableCivs().get(ci.getID());
							if ((context.getTimestamp() - timebefore) < 50) {
								amc.getUnreachableCivs().put(ci.getID(),
										context.getTimestamp() + AmbulanceMemComponent.UNREACHABLE_CIV_IGNORE_INTERVAL * 2);
							}
						} else {
							amc.getUnreachableCivs().put(ci.getID(), context.getTimestamp());
						}
						
						
						Area cp = (Area) context.getWorld().getEntity(ci.getPosition());
						if(cp instanceof Building){
							addRadioMessage(new MRescueUnreachable(ci.getPosition()));
							if(GeoUtils.manhattanDistance(cp, context.getHuman().getX(), context.getHuman().getY())<behavior.AVERAGE_MOVE_DISTANCE*3){
								addRadioMessage(new MRescueUnreachNear(ci.getPosition()));
							}
						}
						return false;
					}
				}

				path = context.getAntComponentKit().getStaticCityGraphComponent()
						.getStaticMoveRoute(context.getHuman().getPosition(), ci.getPosition());
			}
		} else if (state == JobState.Inactive) {
			return false;
		} else {
			log.debug(agent.getID() + " unexpected move request: " + state);
			state = JobState.Inactive;
			return false;
		}
		return true;
	}

	private JobState getExpectedDo() {
		EntityID civilian = behavior.getLoadedCivilian();
		if (civilian != null) {
			currentTarget = civilian;
			if (behavior.isInRefuge()) {
				return JobState.UnloadingHuman;
			} else {
				return JobState.MovingToRefuge;
			}
		}

		currentTarget = acl.getTarget();

		if (currentTarget == null) {
			return JobState.Inactive;
		}

		if (behavior.canLoadCivilian(currentTarget)) {
			if (amc.isTheMinToLoad(currentTarget)) {
				return JobState.LoadingHuman;
			}

			// next target
			currentTarget = acl.getTarget();
		}

		if (behavior.canRescueHuman(currentTarget)) {
			return JobState.RescuingHuman;
		}
		return JobState.MovingToHuman;
	}

	@Override
	public AntCommand execute() {

		if (state == JobState.Inactive) {
		}
		if (state == JobState.MovingToHuman) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.RescuingHuman) {
			amc.setRescuing(currentTarget, false);
			addVoiceMessage(new MVoiceRescuingCivilian(currentTarget));

			return new AntRescue(context.getTimestamp(), currentTarget);
		}
		if (state == JobState.LoadingHuman) {
			amc.setLoading(currentTarget);
			addRadioMessage(new MCivilianRescued(currentTarget));
			addVoiceMessage(new MCivilianRescued(currentTarget));
			return new AntLoad(context.getTimestamp(), currentTarget);
		}
		if (state == JobState.MovingToRefuge) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.UnloadingHuman) {
			return new AntUnload(context.getTimestamp());
		}
		return null;
	}

	@Override
	public String getJobName() {
		return "CommuLessSaveJob";
	}

	@Override
	public void updateRescueInfo() {

	}

}
