package ant.agent.job.at;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntLoad;
import ant.agent.command.AntMove;
import ant.agent.command.AntRescue;
import ant.agent.command.AntUnload;
import ant.agent.job.AntAmbulanceTeamJob;
import ant.agent.job.JobState;
import ant.component.at.AmbulanceMemComponent;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.component.universal.ChangeSetDetailComponent;
import ant.component.universal.HumanInfoComponent;
import ant.liaison.message.MCivilianRescued;
import ant.liaison.message.MRescueUnreachNear;
import ant.liaison.message.MRescueUnreachable;
import ant.liaison.message.MVoiceRescuingCivilian;
import ant.utils.civilian.CivilianInfo;
import ant.utils.civilian.RescueAciton;
import ant.utils.civilian.RescueInfoListener;
import ant.utils.geo.GeoUtils;

public class SaveCivilianJobAdv extends AntAmbulanceTeamJob implements RescueInfoListener {
	private static Log log = LogFactory.getLog(SaveCivilianJobAdv.class);
	
	public static final int KEEP_RESCEUING = 3;
	
	private StandardWorldModel world;
	private ChangeSetDetailComponent csd;
	private HumanInfoComponent hic;
	private AmbulanceMemComponent amc;
	private BuildingConditionComponent bcc;
	private int sendResuced;
	private int stableCounter = KEEP_RESCEUING;
	private List<EntityID> path;
	private AgentMover amover;

	public SaveCivilianJobAdv(AgentContext context) {
		super(context);
		this.world = context.getWorld();
		this.csd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.hic = context.getAntComponentKit().getHumanInfoComponent();
		this.amc = (AmbulanceMemComponent) context.getAntComponentKit().getAntComponent(AmbulanceMemComponent.class);
		this.bcc = context.getAntComponentKit().getBuildingConditionComponent();
		amc.addRescueInfoListener(this);
		amover = new AgentMover(context);
	}

	private JobState getExpectedDo() {
		EntityID human;

		human = behavior.getLoadedCivilian();

		if (human != null) {
			amc.setCurrenTarget(human);

			Human ci = (Human) world.getEntity(human);

			if (ci.isHPDefined()) {
				if (ci.getHP() <= 0) {
					return JobState.UnloadingHuman;
				}
			}

			if (behavior.isInRefuge()) {
				return JobState.UnloadingHuman;
			} else {
				return JobState.MovingToRefuge;
			}
		}

		human = getCivilianToLoad();

		if (human != null) {
			if (behavior.isCivilian(human)) {
				amc.setCurrenTarget(human);
				amc.ignoreCivlian(human);
				return JobState.LoadingHuman;
			} else {
				amc.ignoreCivlian(human);
			}
		}

		human = getCivilianToRescue();

		if (human != null) {
			amc.setCurrenTarget(human);
			return JobState.RescuingHuman;
		}
		return JobState.MovingToHuman;
	}

	private EntityID getCivilianToRescue() {
		List<Human> humansAround = new ArrayList<Human>();
		List<Human> humans = new ArrayList<Human>();

		for (Civilian next : behavior.getNearbyCivilian()) {
			humansAround.add((Human) next);
		}

		for (Human human : humansAround) {
			if (hic.isCivilianIgnored(human.getID())) {
				continue;
			}
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = world.getEntity(positionId);
				if (position instanceof Building) {
					if (positionId.equals(agent.getPosition())) {
						if (!(position instanceof Refuge)) {
							if (human.isBuriednessDefined()) {
								if (human.getBuriedness() > 0) {
									if (human.isHPDefined()) {
										if (human.getHP() > 0) {
											humans.add(human);
											if (behavior.canSee(human.getID())) {
												humans.add(human);
											} else {
												// log.error(human
												// + " is not seen.");
												amc.ignoreCivlian(human.getID());
											}
										} else {
											// log.info(human
											// + "hp is zero defined");
											amc.ignoreCivlian(human.getID());
										}
									} else {
										// log.info(human +
										// "hp is not defined");
										amc.ignoreCivlian(human.getID());
									}
								} else {
									// log.info(human + "buriedness is zero");
								}
							} else {
								// log.info(human +
								// "buriedness is not defined");
							}
						} else {
							amc.ignoreCivlian(human.getID());
							// log.info(human + " is in the refuge");
						}
					} else {
						// log.info(human + " is not in the same building");
					}
				} else {
					// amc.ignoreCivlian(human.getID());
					// log.info(human + " is not in building");
				}
			}
		}
		if (humans.isEmpty()) {
			return null;
		} else {
			Collections.sort(humans, new BuriedHumanComparator());

			for (Human next : humans) {
				CivilianInfo ci = hic.getCivilianInfoMap().get(next.getID());
				if (ci != null) {
					int availableTime = ci.getTimeToDie() - context.getTimestamp() - ci.getTimeCostToRefuge();
					if (availableTime < 5) {
						int atnum = behavior.getNearbyTeammates().size() + 1;
						int buriedness = ci.getBuriedness();
						int timeNeed = buriedness / atnum;
						if (timeNeed < availableTime) {
							stableCounter = KEEP_RESCEUING;
							return next.getID();
						}
					} else if (availableTime < 15) {
						RescueAciton lastaciton = amc.getLastRescueAction();
						if(lastaciton != null && lastaciton.getState().equals(JobState.MovingToHuman) && (behavior.getNearbyTeammates().size() + 1) <=5){
							if(ci.getID().equals(lastaciton.getTarget())){
								stableCounter = 0;
							}
						}
						
						if(stableCounter < KEEP_RESCEUING){	//防止摆动
							stableCounter ++;
							return next.getID();
						}
						
						int atnum = (int) ((behavior.getNearbyTeammates().size() + 1) * 1.5);
						int buriedness = ci.getBuriedness();
						int timeNeed = buriedness / atnum;
						if (timeNeed < availableTime) {
							return next.getID();
						}
					} else {
						stableCounter = KEEP_RESCEUING;
						return next.getID();
					}
				} else {
					log.error("ci == null!!!!");
				}
			}

			return null;
		}
	}

	private EntityID getCivilianToLoad() {
		List<Human> humans;
		List<Human> tmp = new ArrayList<Human>();
		for (Civilian next : behavior.getNearbyCivilian()) {
			tmp.add((Human) next);
		}
		humans = tmp;

		for (Human human : humans) {
			if (!human.isPositionDefined()) {
				continue;
			}

			EntityID positionId;
			StandardEntity position;
			positionId = human.getPosition();
			position = world.getEntity(positionId);

			if (position instanceof Refuge) {
				continue;
			}

			if (!positionId.equals(agent.getPosition())) {
				continue;
			}
			
			if (!human.isBuriednessDefined()) {
				continue;
			}
			
			if (human.getBuriedness() > 0) {
				continue;
			}
			
			if (!human.isHPDefined()) {
				continue;
			}
			
			if (human.getHP() <= 0) {
				continue;
			}
			
			if (!(position instanceof Building)) {
				
				if(!human.isDamageDefined()){
					continue;
				}
				
				if(human.getDamage() <= 0){
					continue;
				}
				
			}

			if (amc.isTheMinToLoad(human.getID())) {
				return human.getID();
			}

		}
		return null;
	}
	
	private Set<Human> getNeedHelpCivilianOnRoadInSight(){
		Set<Human> ret = new HashSet<Human>();
		List<StandardEntity> civsInSight = csd.getEntitiesInSight(StandardEntityURN.CIVILIAN);
		for(StandardEntity se: civsInSight){
			Human civ = (Human) se;
			Area posa = behavior.getArea(civ.getPosition());
			
			if(!(posa instanceof Road)){
				continue;
			}
			
			if(context.getHuman().getPosition().equals(civ.getPosition())){ // Check
				continue;
			}
			
			if (!civ.isHPDefined()) {
				continue;
			}
			
			if (civ.getHP() <= 0) {
				continue;
			}
			
			if(!civ.isDamageDefined()){
				continue;
			}
			
			if(civ.getDamage() <= 0){
				continue;
			}
			
			ret.add(civ);
		}
		
		return ret;
	}

	@Override
	public boolean shouldExecute() {
		JobState expectedDo;
		expectedDo = getExpectedDo();

		if (expectedDo == JobState.MovingToHuman) {
			//-------------- road ------------------------
			Set<Human> civsOnRoad = getNeedHelpCivilianOnRoadInSight();
			if(civsOnRoad.size() > 0){
				for(Human civ: civsOnRoad){
					if(behavior.canBeSavedNaive(civ.getHP(), civ.getDamage(), civ.getBuriedness(), context.getTimestamp())){
						if(amover.canMove(civ.getPosition())){
							amc.setCurrenTarget(civ.getID());
							amc.setMovingToHuman(civ.getID());
							this.path = amover.getPath();
							state = expectedDo;
							return true;
						}
					}
				}
			}
			
			//-------------- building --------------------
			List<Building> buildingsWithCivilian;
			buildingsWithCivilian = getBuildingsWithCivilian();
			if (!buildingsWithCivilian.isEmpty()) {
				if (buildingsWithCivilian.size() > 1) {
					Collections.sort(buildingsWithCivilian, new BuildingDistanceComparator(agent));
				}
				
				for (Building building : buildingsWithCivilian) {
					Set<EntityID> civsInBuilding = hic.getBuildingCivilianMap().get(building.getID());
					for (EntityID cid : civsInBuilding) {
						if (hic.isCivilianIgnored(cid)) {
							continue;
						}

						CivilianInfo ci = hic.getCivilianInfoMap().get(cid);
						if (ci != null) {
							
							if(amc.isUnreachableCiv(cid)){
								continue;
							}
							
							int timeCostMoveToArea = behavior.getTimeCostMoveToAreaNaive(building.getID());
							if (ci.getTimeToDie() > context.getTimestamp()
									+ timeCostMoveToArea + ci.getTimeCostToRefuge() + behavior.AT_MIN_RESCUE_COST) { // 剔除肯定不能被救的对象
								if(!behavior.isATEnough(ci)||buildingsWithCivilian.size()<=3){	//剔除AT人数过多的对象
									if (amover.canMove(building.getID())) {
										amc.setCurrenTarget(cid);
										amc.setMovingToHuman(cid);
										this.path = amover.getPath();
										state = expectedDo;
										return true;
									}else{
										if(context.getTimestamp()>5){
											double dist = GeoUtils.manhattanDistance(building, context.getHuman().getX(), context.getHuman().getY());
											if(dist<=behavior.AVERAGE_MOVE_DISTANCE){
												addRadioMessage(new MRescueUnreachNear(building.getID()));
												addVoiceMessage(new MRescueUnreachNear(building.getID()));
												amc.addUnreacheableCiv(cid, context.getTimestamp());
											}
											if(dist<=behavior.AVERAGE_MOVE_DISTANCE*5){
												addRadioMessage(new MRescueUnreachable(building.getID()));
												amc.addUnreacheableCiv(cid, context.getTimestamp());
											}
											
										}
									}
								}
							}
						}
					}
				}
				log.debug(agent.getID() + " can't move to any building.");
				return false;
			} else {
				log.debug(agent.getID() + " no human to rescue");
				state = JobState.Inactive;
				return false;
			}
		} else if (expectedDo == JobState.RescuingHuman) {
			state = expectedDo;
			return true;
		} else if (expectedDo == JobState.LoadingHuman) {
			state = expectedDo;
			sendResuced = 1;
			return true;
		} else if (expectedDo == JobState.MovingToRefuge) {

			if (sendResuced == 1) {
				EntityID c = behavior.getLoadedCivilian();
				if (c != null) {
					addRadioMessage(new MCivilianRescued(c));
					addVoiceMessage(new MCivilianRescued(c));
				}
				sendResuced = 2;
			}
			if (sendResuced == 2) {
				EntityID c = behavior.getLoadedCivilian();
				if (c != null) {
					addRadioMessage(new MCivilianRescued(c));
				}
				sendResuced = 0;
			}

			Refuge refuge = behavior.getNearestRefuge();

			if (refuge == null) {
				log.info("No refuge can move.");
				return false;
			}
			if (amover.canMove(refuge.getID())) {
				this.path = amover.getPath();
				state = expectedDo;
				return true;
			} else {
				// TODO
				return false;
			}

		} else if (expectedDo == JobState.UnloadingHuman) {
			state = expectedDo;
			return true;
		} else {
			log.debug(agent.getID() + " unexpected move request: " + expectedDo);
			state = JobState.Inactive;
			return false;
		}
	}

	private List<Building> getBuildingsWithCivilian() {
		List<Building> ret = new ArrayList<Building>();
		for (Entry<EntityID, Set<EntityID>> entry : hic.getBuildingCivilianMap().entrySet()) {
			if (!entry.getValue().isEmpty()) {
				BuildingCondition bc = bcc.getBuildingCondition(entry.getKey());
				if(bc.isWorthyOfSearch()){ 
					Building building = (Building) world.getEntity(entry.getKey());
					ret.add(building);
				}
			}
		}
		return ret;
	}

	@Override
	public void updateRescueInfo() {
		// TODO Auto-generated method stub

	}

	@Override
	public AntCommand execute() {
		if (state == JobState.Inactive) {
		}
		if (state == JobState.MovingToHuman) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.RescuingHuman) {
			amc.setRescuing(amc.getCurrenTarget(), false);
			addVoiceMessage(new MVoiceRescuingCivilian(amc.getCurrenTarget()));
			return new AntRescue(context.getTimestamp(), amc.getCurrenTarget());
		}
		if (state == JobState.LoadingHuman) {
			amc.setLoading(amc.getCurrenTarget());
			addRadioMessage(new MCivilianRescued(amc.getCurrenTarget()));
			addVoiceMessage(new MCivilianRescued(amc.getCurrenTarget()));
			return new AntLoad(context.getTimestamp(), amc.getCurrenTarget());
		}
		if (state == JobState.MovingToRefuge) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.UnloadingHuman) {
			amc.ignoreCivlian(amc.getCurrenTarget());
			return new AntUnload(context.getTimestamp());
		}
		return null;
	}

	@Override
	public void overtaken() {
		if (state != JobState.MovingToRefuge) {
			state = JobState.Inactive;
		}
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

}
