package ant.agent.job.at;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.behavior.ATBehavior;
import ant.agent.command.AntCommand;
import ant.agent.command.AntLoad;
import ant.agent.command.AntMove;
import ant.agent.command.AntRescue;
import ant.agent.command.AntUnload;
import ant.agent.job.AntAmbulanceTeamJob;
import ant.agent.job.JobState;
import ant.component.at.AmbulanceMemComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.component.universal.AgentPartitionAssignComponent;
import ant.liaison.message.IMessage;
import ant.liaison.message.MAgentBuried;
import ant.liaison.message.MAgentRescued;
import ant.liaison.message.MCivilianRescued;
import ant.liaison.message.MFindCivilianInBuilding;
import ant.liaison.message.MRescueUnreachNear;
import ant.liaison.message.MRescueUnreachable;
import ant.liaison.message.MVoiceRescuingCivilian;
import ant.map.partition.Partition;
import ant.utils.civilian.RescueInfoListener;
import ant.utils.geo.GeoUtils;

public class SaveHumanJob extends AntAmbulanceTeamJob implements RescueInfoListener {

	public static Log log = LogFactory.getLog(SaveHumanJob.class);

	private StandardWorldModel world;
	private AgentPartitionAssignComponent apac;
	private AmbulanceMemComponent amc;
	private List<SaveHumanTask> saveHumanTasks;
	private Set<EntityID> savedHumans;
	private Map<EntityID, Integer> noSavedHistory;
	private EntityID currentHuman;
	private int sendResuced;
	private List<EntityID> path;
	private boolean forAgent = false;
	private AgentMover amover;

	public SaveHumanJob(AgentContext context, boolean forAgent) {
		super(context);
		this.forAgent = forAgent;
		world = context.getWorld();
		amc = (AmbulanceMemComponent) context.getAntComponentKit().getAntComponent(AmbulanceMemComponent.class);
		amc.addRescueInfoListener(this);
		saveHumanTasks = new ArrayList<SaveHumanTask>();
		savedHumans = new HashSet<EntityID>();
		noSavedHistory = new HashMap<EntityID, Integer>();
		amover = new AgentMover(context);
		apac = (AgentPartitionAssignComponent) context.getAntComponentKit().getAntComponent(
				AgentPartitionAssignComponent.class);
	}

	@Override
	public void overtaken() {
		if (state != JobState.MovingToRefuge) {
			state = JobState.Inactive;
		}
	}

	@Override
	public boolean shouldExecute() {
		JobState expectedDo;
		expectedDo = getExpectedDo();

		if (expectedDo == JobState.MovingToHuman) {
			List<Building> buildingsWithHuman;
			buildingsWithHuman = getBuildingsWithHuman();
			if (!buildingsWithHuman.isEmpty()) {

				if (buildingsWithHuman.size() > 1) {
					Collections.sort(buildingsWithHuman, new BuildingDistanceComparator(agent));
				}
				List<SaveHumanTask> saveTaskToRemove = new ArrayList<SaveHumanTask>();
				for (int i = 0; i < buildingsWithHuman.size(); i++) {
					if (amover.canMove(buildingsWithHuman.get(i).getID())) {
						this.path = amover.getPath();
						// log.info("#Path:" + path);
						state = expectedDo;
						return true;
					}

					requestPathTo(buildingsWithHuman.get(i).getID());

					for (SaveHumanTask task : getTaskByBuilding(buildingsWithHuman.get(i))) {
						saveTaskToRemove.add(task);
					}
				}
				
				for (SaveHumanTask task : saveTaskToRemove) {
					Integer dorpTime = noSavedHistory.get(task.getHuman());
					if (dorpTime != null && context.getTimestamp() - dorpTime >= ATBehavior.AT_TASK_READD_INTERVAL) {
						if (!behavior.canSee(task.getHuman())) {
							AgentLocationComponent alc = context.getAntComponentKit().getAgentLocationComponent();
							List<AgentLocationInfo> list = alc.getAgentLocationInfos(context.getAgentID());
							if (list.size() >= 3) {
								AgentLocationInfo al0 = list.get(list.size() - 1);
								AgentLocationInfo al1 = list.get(list.size() - 2);
								AgentLocationInfo al2 = list.get(list.size() - 3);
								if (al0.getLocation().equals(al1.getLocation())
										&& al0.getLocation().equals(al2.getLocation())) {
									// Stuck
									// TODO may cause bug
								} else {
									state = expectedDo;
									path = context
											.getAntComponentKit()
											.getStaticCityGraphComponent()
											.getStaticMoveRoute(context.getHuman().getPosition(),
													task.getLocationOfHuman().getID());
									return true;
								}
							} else {
								state = expectedDo;
								path = context
										.getAntComponentKit()
										.getStaticCityGraphComponent()
										.getStaticMoveRoute(context.getHuman().getPosition(),
												task.getLocationOfHuman().getID());
								return true;
							}
						}
					}
					// send unreachable
					Area cp = (Area) context.getWorld().getEntity(task.getLocationOfHuman().getID());
					if(cp instanceof Building){
						if(context.getTimestamp()>7){
							addRadioMessage(new MRescueUnreachable(cp.getID()));
							if(GeoUtils.manhattanDistance(cp, context.getHuman().getX(), context.getHuman().getY())<behavior.AVERAGE_MOVE_DISTANCE*3){
									addRadioMessage(new MRescueUnreachNear(cp.getID()));
									addVoiceMessage(new MRescueUnreachNear(cp.getID()));
							}
						}
					}
					
					removeSaveTaskNoSaved(task, context.getTimestamp());
				}
				log.debug(agent.getID() + " can't move to any building.");
				return false;
			} else {
				log.debug(agent.getID() + " no human to rescue");
				state = JobState.Inactive;
				return false;
			}
		} else if (expectedDo == JobState.RescuingHuman) {
			state = expectedDo;
			return true;
		} else if (expectedDo == JobState.LoadingHuman) {
			state = expectedDo;
			sendResuced = 1;
			removeSaveTask(currentHuman);
			return true;
		} else if (expectedDo == JobState.MovingToRefuge) {

			if (sendResuced == 1) {
				EntityID c = behavior.getLoadedCivilian();
				if (c != null) {
					addRadioMessage(new MCivilianRescued(c));
				}
				sendResuced = 2;
			}
			if (sendResuced == 2) {
				EntityID c = behavior.getLoadedCivilian();
				if (c != null) {
					addRadioMessage(new MCivilianRescued(c));
				}
				sendResuced = 0;
			}

			Refuge refuge = behavior.getNearestRefuge();

			if (refuge == null) {
				log.info("No refuge can move.");
				return false;
			}
			if (amover.canMove(refuge.getID())) {
				this.path = amover.getPath();
				state = expectedDo;
				return true;
			} else {
				// TODO
				return false;
			}

		} else if (expectedDo == JobState.UnloadingHuman) {
			state = expectedDo;
			return true;
		} else {
			log.debug(agent.getID() + " unexpected move request: " + expectedDo);
			state = JobState.Inactive;
			return false;
		}
	}

	@Override
	public AntCommand execute() {
		if (state == JobState.Inactive) {
		}
		if (state == JobState.MovingToHuman) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.RescuingHuman) {
			amc.setRescuing(currentHuman, forAgent);
			if(!forAgent){
				addVoiceMessage(new MVoiceRescuingCivilian(currentHuman));
			}
			return new AntRescue(context.getTimestamp(), currentHuman);
		}
		if (state == JobState.LoadingHuman) {
			amc.setLoading(currentHuman);
			addRadioMessage(new MCivilianRescued(currentHuman));
			addVoiceMessage(new MCivilianRescued(currentHuman));
			return new AntLoad(context.getTimestamp(), currentHuman);
		}
		if (state == JobState.MovingToRefuge) {
			if (path != null) {
				return new AntMove(context.getTimestamp(), path);
			} else {
				log.fatal("path == null");
			}
		}
		if (state == JobState.UnloadingHuman) {
			savedHumans.add(currentHuman);
			return new AntUnload(context.getTimestamp());
		}
		return null;
	}

	public void updateRescueInfo() {
		// update messages
		for (IMessage msg : context.getAntCommunicationKit().getRadioReceiveBox()) {
			if (msg instanceof MAgentBuried && forAgent) {
				MAgentBuried ab = (MAgentBuried) msg;
				EntityID agentid = ab.getSender();
				EntityID location = context.getAntComponentKit().getAgentLocationComponent().getAgentLastLocation(agentid);
				int hp = ab.getHp();
				int dmg = ab.getDamage();
				int buriedness = ab.getBuriedness();
				Area here = (Area) this.world.getEntity(agent.getPosition());
				Area cp = (Area) this.world.getEntity(location);

				Partition partition = apac.getPartition(context.getAgentID());

				if (partition.getAreaIDSet().contains(location) || GeoUtils.manhattanDistance(here, cp) < 80000) {
					// on fire?
					if (!hasTaskForHuman(agentid)) {
						StandardEntity se = world.getEntity(location);
						SaveHumanTask st = new SaveHumanTask(agentid, se, behavior.getDeathTimeNaive(hp, dmg,
								context.getTimestamp()));
						this.addSaveTask(st, context.getTimestamp());
					} else {
						updateDeathtimeOfTask(agentid, behavior.getDeathTimeNaive(hp, dmg, context.getTimestamp()));
					}
				}

			} else if (msg instanceof MFindCivilianInBuilding && (!forAgent)) {
				MFindCivilianInBuilding fcm = (MFindCivilianInBuilding) msg;
				EntityID cid = fcm.getId();
				EntityID location = fcm.getPosition();
				int hp = fcm.getHp();
				int dmg = fcm.getDamage();
				int buriedness = fcm.getBuriedness();

				Area here = (Area) this.world.getEntity(agent.getPosition());
				Area cp = (Area) this.world.getEntity(location);

				Partition partition = apac.getPartition(context.getAgentID());
				if (partition.getAreaIDSet().contains(location) || saveHumanTasks.size() == 0
						|| GeoUtils.manhattanDistance(here, cp) < 320000) {

					if (!hasTaskForHuman(cid)) {
						StandardEntity se = world.getEntity(location);
						SaveHumanTask st = new SaveHumanTask(cid, se, behavior.getDeathTimeNaive(hp, dmg,
								context.getTimestamp()));
						this.addSaveTask(st, context.getTimestamp());
					} else {// update
						updateDeathtimeOfTask(cid, behavior.getDeathTimeNaive(hp, dmg, context.getTimestamp()));
					}
				}

			} else if (msg instanceof MCivilianRescued && (!forAgent)) {
				MCivilianRescued crd = (MCivilianRescued) msg;
				EntityID cid = crd.getCivilian();
				
//				System.err.println(context.getTimestamp()+" "+context.getAgentID()+"sj recive CivilianRescued Radio:"+ cid);
				
				this.removeSaveTask(cid);
			} else if (msg instanceof MAgentRescued && forAgent) {
				MAgentRescued ard = (MAgentRescued) msg;
				EntityID aid = ard.getAgent();
				this.removeSaveTask(aid);
			} else if (msg instanceof MRescueUnreachNear) {
				MRescueUnreachNear ru = (MRescueUnreachNear) msg;
				EntityID loaction = ru.getLoaction();
				List<SaveHumanTask> toRemove = new ArrayList<SaveHumanTask>();
				for (SaveHumanTask task : saveHumanTasks) {
					if (task.location.getID().equals(loaction)) {
						toRemove.add(task);
					}
				}
				for (SaveHumanTask task : toRemove) {
//					log.info("remove Unreachable Near:"+task.getHuman());
					removeSaveTaskNoSaved(task, context.getTimestamp() - 1);
				}
			}
		}

		// from observing
		updateHumanAroundInfo();
		removeInvalidTasks();
	}

	private void requestPathTo(EntityID destination) {
		// request help
		// call for pf's help
		// TODO
		// EntityID nextArea = amover.getNextAreaStatic(destination);
		// if (nextArea != null) {
		// if (amover.frontBlocked(nextArea)) {
		// Area here = (Area) agent.getWorld().getEntity(
		// agent.me().getPosition());
		// Edge e = here.getEdgeTo(nextArea);
		// if (e != null) {
		// Area next = (Area) agent.getWorld().getEntity(nextArea);
		// log.info("sendMsg:cleanHigh " + next.getID() + " Agent:"
		// + agent.getID());
		// agent.pushMessage(new BitCleanEntityHigh(next, (int) e
		// .getMidx(), (int) e.getMidy()));
		// } else {
		// log.fatal("Area:" + here.getID() + " has no neighbour:"
		// + nextArea + ". Agent:" + agent.getID());
		// }
		// }
		// }
	}

	private List<EntityID> getHumenInBuilding(Building building) {
		List<EntityID> ret = new ArrayList<EntityID>();
		for (SaveHumanTask task : saveHumanTasks) {
			if (building.getID().equals(task.getLocationOfHuman().getID())) {
				ret.add(task.getHuman());
			}
		}
		return ret;
	}

	private List<SaveHumanTask> getTaskByBuilding(Building building) {
		List<SaveHumanTask> ret = new ArrayList<SaveHumanTask>();
		for (SaveHumanTask task : saveHumanTasks) {
			if (building.getID().equals(task.getLocationOfHuman().getID())) {
				ret.add(task);
			}
		}
		return ret;
	}

	private List<Building> getBuildingsWithHuman() {
		List<Building> buildings;

		buildings = new ArrayList<Building>();
		Set<SaveHumanTask> saveTaskToRemove = new HashSet<SaveHumanTask>();
		for (SaveHumanTask task : saveHumanTasks) {
			StandardEntity position;
			if (!savedHumans.contains(task.getHuman())) {
				position = task.getLocationOfHuman();
				if (position != null && position instanceof Building) {
					Building building;

					building = (Building) position;
					if (!building.isOnFire() && building.getFierynessEnum() != Fieryness.BURNT_OUT) {
						buildings.add(building);
					} else {
						// log.info(agent.getID() +
						// " building is on fire, remove"
						// + task);
						// TODO
						saveTaskToRemove.add(task);
					}
				} else {
					log.info(agent.getID() + " position is nol or not a building" + task);
				}
			} else {
				log.info(agent.getID() + " already saved" + task);
			}
		}

		// remove
		for (SaveHumanTask task : saveTaskToRemove) {
			removeSaveTask(task);
		}
		return buildings;
	}

	private void removeInvalidTasks() {
		Area area;
		List<SaveHumanTask> temp;

		area = behavior.getMyPosition();
		temp = new ArrayList<SaveHumanTask>(saveHumanTasks);
		if (area instanceof Building) {
			for (SaveHumanTask task : temp) {
				if (area.equals(task.getLocationOfHuman())) {
					if (!behavior.canSee(task.getHuman())) {
						log.info("Invaliad task is removed.");
						if (hasTaskForHuman(task.getHuman())) {
							if (forAgent) {
								// context.getAntCommunicationKit().sendRadio(new MAgentRescued(task.getHuman()));
								addRadioMessage(new MAgentRescued(task.getHuman()));
							}
							removeSaveTask(task);
						}
					}
				}
			}
		}

		if (forAgent) {
			for (SaveHumanTask task : temp) {
				if (behavior.isAgentRescued(task.getHuman())) {
					removeSaveTask(task.getHuman());
				}
			}
		}
	}

	private JobState getExpectedDo() {
		EntityID human;

		if (!forAgent) {
			// FIXME
			human = behavior.getLoadedCivilian();
			if (human != null) {
				currentHuman = human;
				
				Human ci = (Human)world.getEntity(human);
				
				if(ci.isHPDefined()){
					if(ci.getHP()<=0){
						return JobState.UnloadingHuman;
					}
				}
				
				if (behavior.isInRefuge()) {
					return JobState.UnloadingHuman;
				} else {
					return JobState.MovingToRefuge;
				}
			}
		}
		human = getHumanToLoad();

		if (human != null) {
			if (behavior.isCivilian(human)) {
				currentHuman = human;
				return JobState.LoadingHuman;
			} else {
				removeSaveTask(human);
			}
		}

		human = getHumanToRescue();

		if (human != null) {
			currentHuman = human;
			return JobState.RescuingHuman;
		}
		return JobState.MovingToHuman;
	}

	private EntityID getHumanToRescue() {
		List<Human> humansAround;
		List<Human> humans;

		humans = new ArrayList<Human>();

		if (forAgent) {
			humansAround = behavior.getNearbyAgents();
		} else {
			List<Human> tmp = new ArrayList<Human>();
			for (Civilian next : behavior.getNearbyCivilian()) {
				tmp.add((Human) next);
			}
			humansAround = tmp;
		}
		for (Human human : humansAround) {
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = world.getEntity(positionId);
				if (position instanceof Building) {
					if (positionId.equals(agent.getPosition())) {
						if (!(position instanceof Refuge)) {
							if (human.isBuriednessDefined()) {
								if (human.getBuriedness() > 0) {
									if (human.isHPDefined()) {
										if (human.getHP() > 0) {
											humans.add(human);
											if (behavior.canSee(human.getID())) {
												humans.add(human);
											} else {
												// log.error(human
												// + " is not seen.");
												checkTaskList(human);
											}
										} else {
											// log.info(human
											// + "hp is zero defined");
											checkTaskList(human);
										}
									} else {
										// log.info(human +
										// "hp is not defined");
										checkTaskList(human);
									}
								} else {
									// log.info(human + "buriedness is zero");
								}
							} else {
								// log.info(human +
								// "buriedness is not defined");
							}
						} else {
							checkTaskList(human);
							// log.info(human + " is in the refuge");
						}
					} else {
						// log.info(human + " is not in the same building");
					}
				} else {
					checkTaskList(human);
					// log.info(human + " is not in building");
				}
			}
		}
		if (humans.isEmpty()) {
			return null;
		} else {
			BuriedHumanComparator comparator;
			comparator = new BuriedHumanComparator();

			Collections.sort(humans, comparator);
			return humans.get(0).getID();
		}
	}

	private void updateHumanAroundInfo() {
		List<Human> humansAround;

		if (forAgent) {
			humansAround = behavior.getNearbyAgents();
		} else {
			List<Human> tmp = new ArrayList<Human>();
			for (Civilian next : behavior.getNearbyCivilian()) {
				tmp.add((Human) next);
			}
			humansAround = tmp;
		}
		for (Human human : humansAround) {
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = world.getEntity(positionId);
				if (!(position instanceof Building)) {
					// log.info(human + " is not in building");
					checkTaskList(human);
					continue;
				}

				if (position instanceof Refuge) {
					// log.info(human + " is in the refuge");
					checkTaskList(human);
					continue;
				}
				if (!human.isBuriednessDefined()) {
					// log.info(human +
					// "buriedness is not defined");
					continue;
				}
				if (human.getBuriedness() <= 0) {
					// log.info(human + "buriedness is zero");
					checkTaskList(human);
					continue;
				}
				if (!human.isHPDefined()) {
					// log.info(human +
					// "hp is not defined");
					continue;
				}
				if (human.getHP() <= 0) {
					// log.info(human
					// + "hp is zero defined");
					checkTaskList(human);
					continue;
				}
				if (!behavior.canSee(human.getID())) {
					// log.error(human
					// + " is not seen.");
					checkTaskList(human);
					continue;
				}
				int hp = human.getHP();
				int dmg = human.getDamage();
				int buriedness = human.getBuriedness();

				if (hasTaskForHuman(human.getID())) {
					updateDeathtimeOfTask(human.getID(), behavior.getDeathTimeNaive(hp, dmg, context.getTimestamp()));
				} else {
					SaveHumanTask se = new SaveHumanTask(human.getID(), position, behavior.getDeathTimeNaive(hp, dmg,
							context.getTimestamp()));
					addSaveTask(se, context.getTimestamp());
				}
			}
		}

	}

	private void checkTaskList(Human human) {
		if (hasTaskForHuman(human.getID())) {
			if (!forAgent) {
				addRadioMessage(new MCivilianRescued(human.getID()));
				addVoiceMessage(new MCivilianRescued(human.getID()));
			} else {
				addRadioMessage(new MAgentRescued(human.getID()));
				addVoiceMessage(new MAgentRescued(human.getID()));
			}
			removeSaveTask(human.getID());
		}
	}

	private EntityID getHumanToLoad() {
		List<Human> humans;
		if (forAgent) {
			return null;
		} else {
			List<Human> tmp = new ArrayList<Human>();
			for (Civilian next : behavior.getNearbyCivilian()) {
				tmp.add((Human) next);
			}
			humans = tmp;
		}

		for (Human human : humans) {
			if (!human.isPositionDefined()) {
				continue;
			}

			EntityID positionId;
			StandardEntity position;
			positionId = human.getPosition();
			position = world.getEntity(positionId);

			if (position instanceof Refuge) {
				continue;
			}

			if (!positionId.equals(agent.getPosition())) {
				continue;
			}
			if (!(position instanceof Building)) {
				continue;
			}
			if (!human.isBuriednessDefined()) {
				continue;
			}
			if (human.getBuriedness() > 0) {
				continue;
			}
			if (!human.isHPDefined()) {
				continue;
			}
			if (human.getHP() <= 0) {
				continue;
			}

			if (amc.isTheMinToLoad(human.getID())) {
				return human.getID();
			}

		}
		return null;
	}

	private boolean hasTaskForHuman(EntityID human) {
		for (SaveHumanTask task : saveHumanTasks) {
			if (task.getHuman().equals(human)) {
				return true;
			}
		}
		return false;
	}

	private void addSaveTask(SaveHumanTask saveHumanTask, int time) {
		EntityID human;
		StandardEntity location;
		AgentLocationComponent alc = context.getAntComponentKit().getAgentLocationComponent();
		Map<EntityID, EntityID> atol = null;
		human = saveHumanTask.getHuman();
		location = saveHumanTask.getLocationOfHuman();
		if (!hasTaskForHuman(human) && !savedHumans.contains(human) && !(location instanceof Refuge)
				&& (location instanceof Building)) {
			Integer dorpTime = noSavedHistory.get(human);
			if (dorpTime != null && time - dorpTime < ATBehavior.AT_TASK_READD_INTERVAL) {
				if (atol == null) {
					atol = alc.getAgentsLastLocation();
				}
				for (Entry<EntityID, EntityID> next : atol.entrySet()) {
					if (next.getValue().equals(location.getID())) {
						if (!alc.getAgentInitialLocation(next.getKey()).equals(next.getValue())) {
							saveHumanTasks.add(saveHumanTask); // TODO May cause
																// bug
							return;
						}
					}
				}
				return;
			}
			saveHumanTasks.add(saveHumanTask);
		}
	}

	private void removeSaveTask(EntityID humanId) {
		SaveHumanTask saveHumanTask = null;
		// if (humanId.getValue() == 53867) {
		// saveHumanTask.getDeathTime();
		// }

		for (SaveHumanTask next : saveHumanTasks) {
			if (next.getHuman().getValue() == humanId.getValue()) {
				saveHumanTask = next;
				break;
			}
		}
		if (saveHumanTask != null) {
			removeSaveTask(saveHumanTask);
		}
	}

	private void removeSaveTask(SaveHumanTask task) {
		if (task != null) {
			// log.info(">>>>>>>>>>>>>>>>>>>>>> remove saved task:"
			// + task.getHuman());
			saveHumanTasks.remove(task);
			savedHumans.add(task.getHuman());
		}
	}

	private void removeSaveTaskNoSaved(EntityID humanId, int time) {
		SaveHumanTask saveHumanTask = null;

		for (SaveHumanTask next : saveHumanTasks) {
			if (next.getHuman().getValue() == humanId.getValue()) {
				saveHumanTask = next;
				break;
			}
		}
		if (saveHumanTask != null) {
			removeSaveTaskNoSaved(saveHumanTask, time);
		}
	}

	private void removeSaveTaskNoSaved(SaveHumanTask task, int time) {
		if (task != null) {
			// log.info(">>>>>>>>>>>>>>>>>>>>>> remove no saved task:"
			// + task.getHuman());
			saveHumanTasks.remove(task);
			noSavedHistory.put(task.getHuman(), time);
		}
	}

	private void updateDeathtimeOfTask(EntityID humanId, int deathTime) {
		for (SaveHumanTask next : saveHumanTasks) {
			if (next.getHuman().getValue() == humanId.getValue()) {
				next.setDeathtime(deathTime);
				break;
			}
		}

	}

	@Override
	public String getJobName() {
		return "SaveHumanJob";
	}
}
