package ant.agent.job.fb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.command.AntRest;
import ant.agent.job.AbstractJob;
import ant.agent.job.JobState;
import ant.component.universal.AgentPartitionAssignComponent;
import ant.map.component.AbstractRouteComponent;
import ant.map.objects.RouteResult;
import ant.map.partition.Partition;
import ant.utils.comparator.DistanceComparator;

/**
 * 
 * @author Oliver
 * 
 * @param <E>
 *            Type of agent
 */
public class GotoPartitionJob<E extends Human> extends AbstractJob<E> {
	private static Log log = LogFactory.getLog(GotoPartitionJob.class);

	private static final int MAX_LEAVE_PARTITION_TIME = 5;
	private static final int MAX_CANDITATE_SIZE = 10;
	private AgentPartitionAssignComponent apa;
	private AbstractRouteComponent arc;
	private Partition partition;
	private int leavePartitionTimeCount;

	private RouteResult routeToPartition;

	public GotoPartitionJob(AgentContext context) {
		super(context);
		this.apa = (AgentPartitionAssignComponent) context.getAntComponentKit().getAntComponent(
				AgentPartitionAssignComponent.class);
		this.arc = context.getAntComponentKit().getAbstractRouteComponent();
		this.partition = apa.getPartition(context.getAgentID());
		this.leavePartitionTimeCount = 0;
	}

	@Override
	public void overtaken() {
		state = JobState.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		if (partition.getAreaIDSet().contains(agent.getPosition())) {
			leavePartitionTimeCount = 0;
			return false;
		} else {
			leavePartitionTimeCount++;
			if (leavePartitionTimeCount <= MAX_LEAVE_PARTITION_TIME) {
				return false;
			} else {
				List<Area> list = new ArrayList<Area>(partition.getAreaSet());
				Collections.sort(list, new DistanceComparator(agent, context.getWorld()));

				int tryTime = 0;
				for (Area a : list) {
					if (a instanceof Road) {
						this.routeToPartition = arc.search(agent.getPosition(), agent.getX(),
								agent.getY(), a.getID());
						if (routeToPartition.hasAcceptableRoute()) {
							return true;
						} else {
							tryTime++;
							if (tryTime > MAX_CANDITATE_SIZE) {
								return false;
							}
						}
					}
				}
				return false;
			}
		}
	}

	@Override
	public AntCommand execute() {
		if (!routeToPartition.hasAcceptableRoute()) {
			log.error("No route to partition, but in going to partition state!");
			return new AntRest(context.getTimestamp());
		} else {
			return new AntMove(context.getTimestamp(), routeToPartition.getMoveRoute());
		}
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

}
