package ant.agent.job.fb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.job.JobState;
import ant.component.fb.ExtinguishRecordComponent;
import ant.component.fb.FireBuilding;
import ant.component.fb.FireConvexHullComponent;
import ant.component.fb.FireSimulatorComponent;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;

/**
 * 
 * @author Oliver
 * 
 */
public class PreExtinguishEntityJob extends AbstractExtinguishJob {
	private static Log log = LogFactory.getLog(PreExtinguishEntityJob.class);
	private static final float SHOULD_PRE_EXTINGUISH_D_TEMPERATURE = 15;

	private BuildingConditionComponent bcc;
	private FireSimulatorComponent fsc;
	private ExtinguishRecordComponent erc;
	private FireConvexHullComponent fch;
	private FBComparatorForPreExtinguish fbc;

	public PreExtinguishEntityJob(AgentContext context) {
		super(context);
		this.bcc = (BuildingConditionComponent) context.getAntComponentKit().getAntComponent(
				BuildingConditionComponent.class);
		this.fsc = (FireSimulatorComponent) context.getAntComponentKit().getAntComponent(
				FireSimulatorComponent.class);
		this.erc = (ExtinguishRecordComponent) context.getAntComponentKit().getAntComponent(
				ExtinguishRecordComponent.class);
		this.fbc = new FBComparatorForPreExtinguish(me, erc, fsc);
	}

	@Override
	public void overtaken() {
		this.state = JobState.Inactive;
	}

	@Override
	public void overleap() {
		super.overleap();
		overtaken();
	}

	@Override
	public boolean shouldExecute() {
		// FIXME 附近没其他FB的时候 预浇水更合适
		ArrayList<FireBuilding> list = new ArrayList<FireBuilding>();
		for (BuildingCondition bc : bcc.getCurrentUpdatedConditions()) {
			Building b = bc.getBuilding();
			if (!bc.isOnFire() && isInExtinguishRange(b)) {
				FireBuilding fb = fsc.getFireBuilding(b.getID());
				if (fb.getIgnitionPoint() - bc.getTemperature() < SHOULD_PRE_EXTINGUISH_D_TEMPERATURE) {
					list.add(fb);
				}
			}
		}
		Collections.sort(list, fbc);
		if (list.isEmpty()) {
			return false;
		}
		return false;
	}

	@Override
	public AntCommand execute() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

	private class FBComparatorForPreExtinguish implements Comparator<FireBuilding> {
		private Human agent;
		private ExtinguishRecordComponent erc;
		private FireSimulatorComponent fsc;

		public FBComparatorForPreExtinguish(Human agent, ExtinguishRecordComponent erc,
				FireSimulatorComponent fsc) {
			this.agent = agent;
			this.erc = erc;
			this.fsc = fsc;
		}

		@Override
		public int compare(FireBuilding o1, FireBuilding o2) {
			double dCost = cost(o1) - cost(o2);
			if (dCost > 0) {
				return 1;
			} else if (dCost < 0) {
				return -1;
			} else {
				return 0;
			}
		}

		private double cost(FireBuilding fb) {
			// TODO Auto-generated method stub
			return 0;
		}
	}
}