package ant.agent.job.fb;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.command.AntRest;
import ant.agent.job.AbstractJob;
import ant.agent.job.JobState;
import ant.map.component.AbstractRouteComponent;
import ant.map.objects.RouteResult;

/**
 * 
 * @author Oliver
 * 
 * @param <E>
 */
public class SearchJob<E extends Human> extends AbstractJob<E> {
	private static Log log = LogFactory.getLog(SearchJob.class);

	private ArrayList<Road> roads;
	private Road targetRoad;
	private AbstractRouteComponent arc;
	private RouteResult route;

	public SearchJob(AgentContext context) {
		super(context);
		this.arc = context.getAntComponentKit().getAbstractRouteComponent();

		this.roads = new ArrayList<Road>();
		for (StandardEntity entity : context.getWorld().getEntitiesOfType(StandardEntityURN.ROAD)) {
			Road r = (Road) entity;
			roads.add(r);
		}
	}

	@Override
	public void overtaken() {
		state = JobState.Inactive;
		targetRoad = null;
	}

	@Override
	public boolean shouldExecute() {
		getTargetRoad();
		return true;
	}

	@Override
	public AntCommand execute() {
		if (route.hasAcceptableRoute()) {
			state = JobState.Exploring;
//			log.info("Search:" + targetRoad);
			return new AntMove(context.getTimestamp(), route.getMoveRoute());
		} else {
			state = JobState.Waiting;
			return new AntRest(context.getTimestamp());
		}
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

	private void getTargetRoad() {
		Area here = (Area) context.getHuman().getPosition(context.getWorld());
		if (targetRoad != null && !hasArrivedTargetRoad(here)) {
			this.route = arc.search(here.getID(), here.getX(), here.getY(), targetRoad.getID());
			if (route.hasAcceptableRoute()) {
				return;
			}
		}
		// for (int i = 0; i < roads.size(); i++) {
		targetRoad = roads.get(0);
		roads.remove(0);
		roads.add(targetRoad);
		this.route = arc.search(here.getID(), here.getX(), here.getY(), targetRoad.getID());
		// if (route.hasAcceptableRoute()) {
		// break;
		// } else {
		// targetRoad = null;
		// continue;
		// }
		// }
		// if (targetRoad == null) {
		// // has to find a road, even it is unreachable
		// // TODO
		// log.error("No target Road found on search!");
		// }
	}

	private boolean hasArrivedTargetRoad(Area here) {
		if (targetRoad == null)
			return false;
		if (here.equals(targetRoad)) {
			return true;
		}
		return false;
	}
}
