package ant.agent.job.fb;

import java.util.TreeSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.config.Config;
import rescuecore2.standard.entities.Building;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.agent.command.AntCommand;
import ant.agent.command.AntExtinguish;
import ant.agent.command.AntMoveXY;
import ant.agent.command.AntRest;
import ant.agent.job.AntFireBrigadeJob;
import ant.agent.job.JobState;
import ant.component.fb.FireBuilding;
import ant.component.fb.FireBuildingComparator;
import ant.component.fb.FireSimulatorComponent;
import ant.map.component.AbstractRouteComponent;
import ant.map.objects.RouteResult;

/**
 * 
 * @author Oliver
 * 
 */
public class SimpleExtinguishEntityJob extends AntFireBrigadeJob {
	private static Log log = LogFactory.getLog(SimpleExtinguishEntityJob.class);

	private final static int EXTINGUISH_RANGE = 100000;
	private final static int RECENT_LEVEL = 5;

	private int maxPower;
	private AbstractRouteComponent arc;
	private FireSimulatorComponent fsc;

	private EntityID target;
	private Building targetBuilding;
	private FireBuilding targetFireBuilding;
	private ExtinguishPositionSearcher eps;

	public SimpleExtinguishEntityJob(AgentContext context) {
		super(context);
		Config config = context.getConfig();
		this.maxPower = ConfigKey.getFireMaxPower(config);
		this.arc = context.getAntComponentKit().getAbstractRouteComponent();
		this.fsc = (FireSimulatorComponent) context.getAntComponentKit().getAntComponent(
				FireSimulatorComponent.class);

	}

	@Override
	public void overtaken() {
		this.state = JobState.Inactive;

		// reset values about target
		target = null;
		targetBuilding = null;
	}

	@Override
	public boolean shouldExecute() {
		// find a target
		if (!searchNextTarget()) {
			return false;
		} else {
			return true;
		}
	}

	@Override
	public AntCommand execute() {
		if (eps.haveArrivedPosition()) {
			this.state = JobState.Extinguishing;
			return new AntExtinguish(context.getTimestamp(), target, maxPower);
		} else {
			RouteResult route = arc.search(agent.getPosition(), agent.getX(), agent.getY(),
					eps.getPositionID());
			if (!route.hasAcceptableRoute()) {
				log.error("There is no valid route to target:" + target);
				// TODO
				this.state = JobState.Waiting;
				return new AntRest(context.getTimestamp());
			} else {
				this.state = JobState.MovingToFire;
				// log.info("moving to " + eps.getPosition() + "(" +
				// eps.getExtinguishCoordinate().x()
				// + "," + eps.getExtinguishCoordinate().y() + ")" +
				// " agent at ("
				// + agent.getX() + "," + agent.getY() + ")" +
				// " to extinguish! route:"
				// + route.getMoveRoute() + " (" + route.getFrom() + "," +
				// route.getTo() + ")"
				// + " target:" + target);
				return new AntMoveXY(context.getTimestamp(), route.getMoveRoute(), eps
						.getExtinguishCoordinate().x(), eps.getExtinguishCoordinate().y());
			}
		}
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

	/**
	 * take burningBuildingsInRange and recently found burningBuildings into
	 * consideration
	 * 
	 * @return
	 */
	private boolean searchNextTarget() {
		TreeSet<FireBuilding> temp = new TreeSet<FireBuilding>(new FireBuildingComparator(
				context));
		temp.addAll(fsc.getBurningBuildingsInRange(agent.getPosition(), EXTINGUISH_RANGE,
				RECENT_LEVEL));
		temp.addAll(fsc.getRecentlyBurningBuildings(RECENT_LEVEL));
		// ArrayList<FireBuilding> searchBuildings = new
		// ArrayList<FireBuilding>(temp);
		// Collections.sort(searchBuildings, new
		// FireBuildingComparatorByCost(context));

		for (FireBuilding fireBuilding : temp) {
			targetFireBuilding = fireBuilding;
			targetBuilding = targetFireBuilding.getBuilding();
			target = targetBuilding.getID();
			eps = new ExtinguishPositionSearcher(context, targetFireBuilding);
			if (eps.isPositionValid()) {
				return true;
			} else {
				continue;
			}
		}
		targetFireBuilding = null;
		targetBuilding = null;
		target = null;
		return false;
	}
}
