package ant.agent.job.fb;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMoveXY;
import ant.agent.command.AntRest;
import ant.agent.job.AbstractJob;
import ant.agent.job.JobState;
import ant.component.fb.FireTraceComponent;
import ant.component.fb.FireTraceComponent.TraceInfo;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.map.component.AbstractRouteComponent;
import ant.map.objects.RouteResult;
import ant.utils.Pair;
import ant.utils.geo.Point;

/**
 * 
 * @author Oliver
 * 
 * @param <E>
 */
public class TraceFireJob<E extends Human> extends AbstractJob<E> {
	private static final int TRACE_RANGE = 150000;
	private static final int TRACE_BURNING_BUILDING_MAX = 0;

	private static Log log = LogFactory.getLog(TraceFireJob.class);

	private AbstractRouteComponent arc;
	private FireTraceComponent ftc;
	private RouteResult route;

	private StandardWorldModel world;

	private BuildingConditionComponent bcc;
	private Pair<Point, Road> target;

	public TraceFireJob(AgentContext context) {
		super(context);
		this.world = context.getWorld();
		this.arc = context.getAntComponentKit().getAbstractRouteComponent();
		this.bcc = (BuildingConditionComponent) context.getAntComponentKit().getAntComponent(
				BuildingConditionComponent.class);
		this.ftc = (FireTraceComponent) context.getAntComponentKit().getAntComponent(
				FireTraceComponent.class);
	}

	@Override
	public void overtaken() {
		this.state = JobState.Inactive;
	}

	@Override
	public void overleap() {
		super.overleap();
		this.state = JobState.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		List<TraceInfo> list = ftc.getCurrentTraceList();
		if (list.isEmpty()) {
			return false;
		}

		Human me = context.getHuman();
		int nearBurningBuildingCount = 0;
		for (StandardEntity entity : world.getObjectsInRange(me.getX(), me.getY(), TRACE_RANGE)) {
			if (entity instanceof Building) {
				BuildingCondition bc = bcc.getBuildingCondition(entity.getID());
				if (bc.isOnFire()) {
					nearBurningBuildingCount++;
				}
			}
		}
		if (nearBurningBuildingCount > TRACE_BURNING_BUILDING_MAX) {
			return false;
		}

		for (TraceInfo ti : list) {
			// FIXME 不直接进入房子来探索将更好
			Building b = ti.getBuildingCondition().getBuilding();
			this.target = behavior.getPositionInRoadToMove(b);
			if (target != null) {
				this.route = arc.search(me.getPosition(), me.getX(), me.getY(), target.second()
						.getID());
				if (this.route.hasAcceptableRoute()) {
					this.state = JobState.ExploringFire;
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public AntCommand execute() {
		if (route.hasAcceptableRoute()) {
			return new AntMoveXY(context.getTimestamp(), route.getMoveRoute(), target.first()
					.getX(), target.first().getY());
		} else {
			return new AntRest(context.getTimestamp());
		}
	}

	@Override
	public String getJobName() {
		return getClass().getName();
	}

}
