package ant.agent.job.pf;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveJigsawSystemComponent;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotJamStuckInRoad;
import ant.component.pf.spot.SpotLevel;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.Pair;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(AbstractOjExecutor.class);

	protected AgentContext context;
	protected Human human;
	protected PFBehavior behavior;
	protected StandardWorldModel world;
	protected StaticCityGraphComponent city;
	protected ObjectiveJigsawSystemComponent ojsc;
	protected ObjectiveJigsawMetaComponent ojmetac;

	protected AbstractOjExecutor(AgentContext context, PFBehavior behavior) {
		this.context = context;
		this.human = context.getHuman();
		this.behavior = behavior;
		this.world = context.getWorld();
		this.city = context.getAntComponentKit().getStaticCityGraphComponent();
		this.ojsc = (ObjectiveJigsawSystemComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawSystemComponent.class);
		this.ojmetac = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);

	}

	public abstract boolean isDone();

	/**
	 * @return null, if over
	 */
	public abstract AntCommand execute();

	public abstract ObjectiveJigsaw getObjectiveJigsaw();

	protected AntCommand executeFastestNaiveCleanTo(EntityID where) {
		return OjExecutorUtils.executeFastestNaiveCleanTo(human, ojsc, behavior, world, context
				.getTimestamp(), where);
	}

	protected AntCommand executeFastestNaiveCleanTo(EntityID where, int x, int y) {
		return OjExecutorUtils.executeFastestNaiveCleanTo(human, ojsc, behavior, world, context
				.getTimestamp(), where, x, y);
	}

	protected AntCommand executeSpot(AbstractSpot nextSpot) {
		if (nextSpot == null) {
			log.error("Null SPOT!");
			return null;
		}

		if (nextSpot.isDone()) {
			log.warn("Already done SPOT: " + nextSpot);
			return null;
		}

		if (nextSpot.getWhere() == null) {
			log.error(nextSpot + " 's Where is null!");
			return null;
		}

		switch (nextSpot.getType()) {
		case AgentBuried:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());

		case AgentStuckingInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());

		case AgentStuckingInRoad:

			SpotJamStuckInRoad sjsr = (SpotJamStuckInRoad) nextSpot;
			Pair<Integer, Integer> pp = sjsr.getStuckXY();
			if (pp != null)
				return executeFastestNaiveCleanTo(nextSpot.getWhere(), pp.first(), pp.second());
			else
				return executeFastestNaiveCleanTo(nextSpot.getWhere());

		case BuildingExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case BuildingObserving:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case Refuge:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case CivInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyRoadArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyJigsawArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentSpawn:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case SuperiorExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case FindCivBuildingEnter:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		}

		return null;
	}

	// protected ObjectiveJigsaw getUnderfootingObjectiveJigsaw() {
	// // underfooting
	// ObjectiveJigsaw footing =
	// ojmetac.getExMappingObjectiveJigsaw(context.getHuman()
	// .getPosition());
	//
	// return footing;
	// }

	protected boolean isInObjectiveJigsaw(ObjectiveJigsaw target) {
		ObjectiveJigsaw footing = ojmetac.getExMappingObjectiveJigsaw(context.getHuman()
				.getPosition());

		if (footing == null) {
			log.error("Not found oj for " + context.getHuman().getPosition());
			return false;
		}

		if (footing.getObjectiveID() == target.getObjectiveID()) {
			return true;
		} else {
			return false;
		}
	}

	protected Pair<AbstractSpot, Integer> getUnderfootingHighSpot() {
		ObjectiveJigsaw footing = ojmetac.getExMappingObjectiveJigsaw(context.getHuman()
				.getPosition());

		if (footing == null)
			return null;
		if (!footing.isHighlySpoted())
			return null;

		AbstractSpot spot = OjExecutorUtils.getNearestSpot(human, footing
				.getLevelSpots(SpotLevel.High));
		
		if (spot == null)
			return null;
		
		return new Pair<AbstractSpot, Integer>(spot, footing.getObjectiveID());
	}
	
	protected Pair<AbstractSpot, Integer> getUnderfootingHMSpot() {
		ObjectiveJigsaw footing = ojmetac.getExMappingObjectiveJigsaw(context.getHuman()
				.getPosition());

		if (footing == null)
			return null;
		if (!footing.isHighlySpoted())
			return null;

		AbstractSpot spot = OjExecutorUtils.getNearestSpot(human, footing.getHMSpots());
		
		if (spot == null)
			return null;
		
		return new Pair<AbstractSpot, Integer>(spot, footing.getObjectiveID());
	}
	
	protected Pair<AbstractSpot, Integer> getUnderfootingSpot() {
		ObjectiveJigsaw footing = ojmetac.getExMappingObjectiveJigsaw(context.getHuman()
				.getPosition());

		if (footing == null)
			return null;

		if (!footing.isSpoted())
			return null;
		
		AbstractSpot spot = OjExecutorUtils.getNearestSpot(human, footing.getSpots());
		
		if (spot == null)
			return null;
		
		return new Pair<AbstractSpot, Integer>(spot, footing.getObjectiveID());
	}
}