package ant.agent.job.pf;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawAvenue;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotJamStuckInRoad;
import ant.map.component.StaticCityGraphComponent;
import ant.map.jigsaw.JigsawCross;
import ant.utils.Pair;

/**
 * 
 * @author Ivan
 * 
 */
public class ObjectiveJigsawAvenueExecutor extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawAvenueExecutor.class);
	
	private static final double M2C_CLEAR_RATE = 0.75f;

	private ObjectiveJigsawAvenue target;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;
	private StaticCityGraphComponent scg;

	private boolean cleanAvenue = false;
	private JigsawCross cleanAvenueDest = null;
	private boolean isHighly;
	private boolean checkBest = true;

	public ObjectiveJigsawAvenueExecutor(AgentContext context, PFBehavior behavior, ObjectiveJigsawAvenue target, boolean checkBest) {
		super(context, behavior);
		this.target = target;

		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(ObjectiveJigsawMetaComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		
		this.isHighly = target.isHighlySpoted();
		this.checkBest = checkBest;
	}
	
	public ObjectiveJigsawAvenueExecutor(AgentContext context, PFBehavior behavior, ObjectiveJigsawAvenue target) {
		super(context, behavior);
		this.target = target;

		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(ObjectiveJigsawMetaComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		
		this.isHighly = target.isHighlySpoted();
		this.checkBest = true;
	}
	

	private boolean isHighlyDone() {
		// Clean Avenue Stage
		if (cleanAvenue) {
			EntityID where = context.getHuman().getPosition();
			return (cleanAvenueDest.contains(where));
		}
		
		// Objective Stage
		boolean isSpoted = this.target.isSpoted();

		if (isSpoted) {
			if (checkBest) {
				if (!getObjectiveJigsaw().getDiscuss().isBestApplicant()) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		} else {
			
			ObjectiveJigsaw temp = ojmc.getExMappingObjectiveJigsaw(context.getHuman().getPosition());
			if (temp == null) {
				return true;
			}
			
			if (temp.getObjectiveID() != target.getObjectiveID()) {
				// I'm not in JIGSAW, but target is done
				return true;
			}
			
			if (target.getInfoStat().getClearRate() >= M2C_CLEAR_RATE) {
				// Oh, too cleanly
				return true;
			}
			
			// Into Stage 2
			cleanAvenue = true;
		
			EntityID where = context.getHuman().getPosition();
			
			int mostnc = 0;
			for (JigsawCross jc : this.target.getJigsawAvenue().getCrosses()) {
				int nc = 0;
				List<EntityID> route = scg.getStaticMoveRoute(where, jc.getCenter().getID());
				if (route != null) {
					for (EntityID id : route) {
						if (target.getJigsawAvenue().contains(id)) {
							if (!target.getInfoStat().isRoadClean(id)) {
								nc++;
							}
						}
					}
				}
				
				if (nc >= mostnc) {
					mostnc = nc;
					cleanAvenueDest = jc;
				}
			}
			
			if (cleanAvenueDest != null) 
			{
				cleanAvenue = true;
				return false;
			} else {
				return true;
			}
		}
	}

	private AntCommand executeHighly() {
		if (cleanAvenue) {
			return OjExecutorUtils.executeNormalCleanTo(human, city, behavior, world, context.getTimestamp(), cleanAvenueDest.getCenter().getID());
		}

		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}
		
		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}
		
		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}
		
		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
		}
		
		// nextSpot unfinished
		switch (nextSpot.getType()) {
		case AgentBuried:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInRoad:
			
			SpotJamStuckInRoad sjsr = (SpotJamStuckInRoad) nextSpot;
			Pair<Integer, Integer> pp = sjsr.getStuckXY();
			if (pp != null)
				return executeFastestNaiveCleanTo(nextSpot.getWhere(), pp.first(), pp.second());
			else
				return executeFastestNaiveCleanTo(nextSpot.getWhere());
			
		case BuildingExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case BuildingObserving:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case Refuge:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case CivInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyRoadArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyJigsawArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentSpawn:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case SuperiorExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case FindCivBuildingEnter:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		}
		log.error("Executor return null:" + nextSpot);
		return null;
	}
	
	private boolean isLowDone() {
		if (checkBest) {
			if (!getObjectiveJigsaw().getDiscuss().isBestApplicant()) {
				return true;
			}
		}
		
		return !this.target.isSpoted();
	}
	
	private AntCommand executeLow() {
		if (isDone())
			return null;
		
		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}
		
		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			if (nextSpot == null)
				return null;
		}
		
		// nextSpot unfinished
		switch (nextSpot.getType()) {
		case AgentBuried:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInRoad:
			
			SpotJamStuckInRoad sjsr = (SpotJamStuckInRoad) nextSpot;
			Pair<Integer, Integer> pp = sjsr.getStuckXY();
			if (pp != null)
				return executeFastestNaiveCleanTo(nextSpot.getWhere(), pp.first(), pp.second());
			else
				return executeFastestNaiveCleanTo(nextSpot.getWhere());
			
		case BuildingExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case BuildingObserving:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case Refuge:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case CivInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyRoadArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyJigsawArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentSpawn:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case SuperiorExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case FindCivBuildingEnter:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		}
		
		return null;
	}
	
	@Override
	public AntCommand execute() {
		if (isHighly)
			return executeHighly();
		else
			return executeLow();
	}

	@Override
	public boolean isDone() {
		if (isHighly)
			return isHighlyDone();
		else
			return isLowDone();
		
	}
	
	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public String toString() {
		return "ObjectiveJigsawAvenueExecutorAdvanced [target=" + target + ", cleanAvenue=" + cleanAvenue + ", isHighly=" + isHighly + "]";
	}

	
}
