package ant.agent.job.pf;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import ant.AgentContext;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawCross;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotJamStuckInRoad;
import ant.utils.Pair;

/**
 * 
 * @author Ivan
 * 
 */
public class ObjectiveJigsawCrossExecutor extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawCrossExecutor.class);
	
	private ObjectiveJigsawCross target;
	private AbstractSpot nextSpot;
	private boolean checkBest = true;

	public ObjectiveJigsawCrossExecutor(AgentContext context, PFBehavior behavior, ObjectiveJigsawCross target) {
		super(context, behavior);
		this.target = target;
		this.checkBest = true;
	}
	
	public ObjectiveJigsawCrossExecutor(AgentContext context, PFBehavior behavior, ObjectiveJigsawCross target, boolean checkBest) {
		super(context, behavior);
		this.target = target;
		this.checkBest = checkBest;
	}
	

	@Override
	public boolean isDone() {
		if (checkBest) {
			if (!getObjectiveJigsaw().getDiscuss().isBestApplicant()) {
				return true;
			}
		}
		
		return !this.target.isSpoted();
	}

	@Override
	public AntCommand execute() {
		if (isDone())
			return null;
		
		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}
		
		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			if (nextSpot == null)
				return null;
		}
		
		// nextSpot unfinished
		switch (nextSpot.getType()) {
		case AgentBuried:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentStuckingInRoad:
			
			SpotJamStuckInRoad sjsr = (SpotJamStuckInRoad) nextSpot;
			Pair<Integer, Integer> pp = sjsr.getStuckXY();
			if (pp != null)
				return executeFastestNaiveCleanTo(nextSpot.getWhere(), pp.first(), pp.second());
			else
				return executeFastestNaiveCleanTo(nextSpot.getWhere());
			
		case BuildingExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case BuildingObserving:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case Refuge:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case CivInBuilding:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyRoadArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case AgentSpawn:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case HighlyJigsawArrive:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case SuperiorExploration:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		case FindCivBuildingEnter:
			return executeFastestNaiveCleanTo(nextSpot.getWhere());
		}
		
		log.error("Executor return null:" + nextSpot);
		return null;
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public String toString() {
		return "ObjectiveJigsawCrossExecutor [target=" + target + "]";
	}

}
