package ant.agent.job.pf;

import java.util.List;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.agent.behavior.PFBehavior;
import ant.agent.behavior.PFBehavior.CleanLevel;
import ant.agent.command.AntClear;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMoveXY;
import ant.component.pf.ObjectiveJigsawSystemComponent;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotLevel;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 */
public class OjExecutorUtils {

	/**
	 * @return null, if all done
	 */
	public static AbstractSpot getNearestSpot(Human human, Set<AbstractSpot> spots) {
		AbstractSpot ret = null;
		ret = getNearestLevelSpot(human, spots, SpotLevel.High);
		if (ret != null)
			return ret;

		ret = getNearestLevelSpot(human, spots, SpotLevel.Mid);
		if (ret != null)
			return ret;

		ret = getNearestLevelSpot(human, spots, SpotLevel.Low);
		return ret;

	}

	private static AbstractSpot getNearestLevelSpot(Human human, Set<AbstractSpot> spots, SpotLevel level) {

		int x = human.getX();
		int y = human.getY();

		double mindist = Double.MAX_VALUE;
		AbstractSpot pick = null;

		for (AbstractSpot spot : spots) {
			if (spot.isDone())
				continue;
			if (spot.getType().getLevel() != level)
				continue;

			double temp = GeoUtils.manhattanDistance(spot.getX(), spot.getY(), x, y);
			if (temp < mindist) {
				mindist = temp;
				pick = spot;
			}
		}

		return pick;
	}

	public static AntCommand executeFastestNaiveCleanTo(Human human, ObjectiveJigsawSystemComponent ojsc, PFBehavior behavior, StandardWorldModel world,
			int time, EntityID where) {
		// List<EntityID> path = ojsc.getStaticMoveRoute(human.getPosition(), where);
		List<EntityID> path = ojsc.getFastestMoveRoute(human.getPosition(), where);
		
		EntityID blk = null;
		if (!human.getPosition().equals(where)) {
			blk = behavior.cleverPathBlockadeSelect(path, CleanLevel.Fastest);
		} else {
			blk = behavior.cleverAroundBlockadeSelect(CleanLevel.Almost);
		}
		if (blk == null) {
			Area tmp = (Area) world.getEntity(where);
			return new AntMoveXY(time, path, tmp.getX(), tmp.getY());
		} else {
			return new AntClear(time, blk);
		}
	}

	public static AntCommand executeFastestNaiveCleanTo(Human human, ObjectiveJigsawSystemComponent ojsc, PFBehavior behavior, StandardWorldModel world,
			int time, EntityID where, int x, int y) {
		// List<EntityID> path = ojsc.getStaticMoveRoute(human.getPosition(), where);
		List<EntityID> path = ojsc.getFastestMoveRoute(human.getPosition(), where);
		
		EntityID blk = null;
		if (!human.getPosition().equals(where)) {
			blk = behavior.cleverPathBlockadeSelect(path, CleanLevel.Fastest);
		} else {
			blk = behavior.cleverAroundBlockadeSelect(CleanLevel.Almost);
		}
		if (blk == null) {
			return new AntMoveXY(time, path, x, y);
		} else {
			return new AntClear(time, blk);
		}
	}

	public static AntCommand executeNormalCleanTo(Human human, StaticCityGraphComponent city, PFBehavior behavior, StandardWorldModel world,
			int time, EntityID where) {
		List<EntityID> path = city.getStaticMoveRoute(human.getPosition(), where);
		EntityID blk = null;
		if (!human.getPosition().equals(where)) {
			blk = behavior.cleverPathBlockadeSelect(path, CleanLevel.Normal);
		} else {
			blk = behavior.cleverAroundBlockadeSelect(CleanLevel.Almost);
		}
		if (blk == null) {
			Area tmp = (Area) world.getEntity(where);
			return new AntMoveXY(time, path, tmp.getX(), tmp.getY());
		} else {
			return new AntClear(time, blk);
		}
	}
}
