package ant.agent.job.pf;

import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.behavior.PFBehavior.CleanLevel;
import ant.agent.command.AntClear;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMoveXY;
import ant.agent.command.AntRest;
import ant.agent.job.AntPoliceForceJob;
import ant.agent.job.JobState;
import ant.map.component.StaticCityGraphComponent;

/**
 * 
 * @author Ivan
 *
 */
public class PFSaveMySelfJob extends AntPoliceForceJob {
	
	private EntityID ref = null;
	private boolean danger = false;
	private StaticCityGraphComponent cityComp;
	
	public PFSaveMySelfJob(AgentContext context) {
		super(context);
		this.cityComp = context.getAntComponentKit().getStaticCityGraphComponent();
	}
	
	@Override
	public void overtaken() {
		ref = null;
		danger = false;
		state = JobState.Inactive;
	}

	@Override
	public void overleap() {
		super.overleap();
		ref = null;
	}
	
	@Override
	public boolean shouldExecute() {
		if (ref == null) {
			danger = shouldGoToRefuge();
			if (danger == true) {
				Refuge r = behavior.getNearestRefuge();
				if (r == null) return false;
				ref = r.getID();
			}
			return danger;
		} else {
			if (context.getHuman().getPosition().equals(ref)) {
				return false;
			} else {
				return true;
			}
		}
	}

	@Override
	public AntCommand execute() {
		if (!agent.getPosition().equals(ref)) {
			List<EntityID> path = cityComp.getStaticMoveRoute(agent.getPosition(), ref);
			EntityID id = behavior.cleverPathBlockadeSelect(path, CleanLevel.Fastest);
			if (id == null) {
				Area tmp = (Area) context.getWorld().getEntity(ref);
				return new AntMoveXY(this.context.getTimestamp(), path, tmp.getX(), tmp.getY());
			} else {
				return new AntClear(this.context.getTimestamp(), id);
			}
		} else {
			return new AntRest(this.context.getTimestamp());
		}
	}

	@Override
	public String getJobName() {
		return PFSaveMySelfJob.class.getName();
	}
	
	private boolean shouldGoToRefuge() {
		Human me = context.getHuman();
		if (me.getHP() < 3000 && me.getDamage() > 0) {
			return true;
		} else if (me.getHP() < me.getDamage() * 15) {
			return true;
		} else if (me.getDamage() > 50) {
			return true;
		}
		return false;
	}
	
}
