package ant.agent.job.pf.adv;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.agent.job.pf.AbstractOjExecutor;
import ant.agent.job.pf.OjExecutorUtils;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawAvenue;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveMiscComponent;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotLevel;
import ant.map.component.StaticCityGraphComponent;
import ant.map.jigsaw.JigsawCross;
import ant.utils.Pair;

/**
 * 
 * @author Ivan
 * 
 */
public class ObjectiveJigsawAvenueExecutorAdv extends AbstractOjExecutor {

	private AbstractOjExecutor _executor;
	private ObjectiveMiscComponent misc;
	private ObjectiveJigsawAvenue target;

	public ObjectiveJigsawAvenueExecutorAdv(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawAvenue target, ExecutorLevel level) {

		super(context, behavior);

		this.misc = (ObjectiveMiscComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveMiscComponent.class);
		this.target = target;

		if (level == ExecutorLevel.Highly || level == ExecutorLevel.HighlyNoCheck) {
			if (level == ExecutorLevel.Highly)
				_executor = new ObjectiveJigsawAvenueExecutorAdvHighly(context, behavior, target,
						true);
			else
				_executor = new ObjectiveJigsawAvenueExecutorAdvHighly(context, behavior, target,
						false);
		} else {
			if (level == ExecutorLevel.HighlyAndMid) {
				_executor = new ObjectiveJigsawAvenueExecutorAdvNormal(context, behavior, target);
			} else {
				_executor = new ObjectiveJigsawAvenueExecutorAdvAll(context, behavior, target);
			}
		}
	}

	@Override
	public AntCommand execute() {
		misc.setDoing(target);
		return _executor.execute();
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return _executor.getObjectiveJigsaw();
	}

	@Override
	public boolean isDone() {
		boolean b = _executor.isDone();
		if (b)
			misc.setDoing(null);
		return b;
	}

	@Override
	public String toString() {
		return _executor.toString();
	}
}

/******************************** Highly Executor( Fastest Action ) *********************************/
class ObjectiveJigsawAvenueExecutorAdvHighly extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawAvenueExecutorAdvHighly.class);
	private static final double M2C_CLEAR_RATE = 0.75f;

	private ObjectiveJigsawAvenue target;
	private ObjectiveJigsawMetaComponent ojmc;
	private boolean checkBest;
	private boolean cleanAvenueStage = false;
	private JigsawCross cleanAvenueDest = null;
	private AbstractSpot nextSpot;
	private int nextSpotOid;
	private StaticCityGraphComponent scg;

	protected ObjectiveJigsawAvenueExecutorAdvHighly(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawAvenue target, boolean checkBest) {
		super(context, behavior);

		this.target = target;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		this.checkBest = checkBest;
	}

	@Override
	public AntCommand execute() {
		if (cleanAvenueStage) {
			return OjExecutorUtils.executeNormalCleanTo(human, city, behavior, world, context
					.getTimestamp(), cleanAvenueDest.getCenter().getID());
		}

		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			// this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getLevelSpots(SpotLevel.High));
			
//			if (!isInObjectiveJigsaw(target)) {
//				this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getLevelSpots(SpotLevel.High));
//			} else {
//				this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getHMSpots());
//			}
			
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingHighSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		// Clean Avenue Stage
		if (cleanAvenueStage) {
			EntityID where = context.getHuman().getPosition();
			return (cleanAvenueDest.contains(where));
		}

		// Objective Stage
		boolean isSpoted = this.target.isSpoted();
		boolean isHighlySpoted = this.target.isHighlySpoted();

		if (isHighlySpoted) {
			if (checkBest) {
				if (!getObjectiveJigsaw().getDiscuss().isBestApplicant()) {
					return true;
				} else {
					return false;
				}
			}
			return false;
		} else {
			// First Into CleanAvenueStage
			ObjectiveJigsaw temp = ojmc.getExMappingObjectiveJigsaw(context.getHuman()
					.getPosition());

			if (temp == null) {
				return true;
			}

			if (temp.getObjectiveID() != target.getObjectiveID()) {
				// I'm not in JIGSAW, but target is done
				return true;
			}

			if (target.getInfoStat().getClearRate() >= M2C_CLEAR_RATE) {
				// Oh, too cleanly
				return true;
			}

			// Into Stage 2
			cleanAvenueStage = true;

			EntityID where = context.getHuman().getPosition();

			int mostnc = 0;
			for (JigsawCross jc : this.target.getJigsawAvenue().getCrosses()) {
				int nc = 0;
				List<EntityID> route = scg.getStaticMoveRoute(where, jc.getCenter().getID());
				if (route != null) {
					for (EntityID id : route) {
						if (target.getJigsawAvenue().contains(id)) {
							if (!target.getInfoStat().isRoadClean(id)) {
								nc++;
							}
						}
					}
				}

				if (nc >= mostnc) {
					mostnc = nc;
					cleanAvenueDest = jc;
				}
			}

			if (cleanAvenueDest != null) {
				cleanAvenueStage = true;
				return false;
			} else {
				return true;
			}
		}
	}

	@Override
	public String toString() {
		return "ObjectiveJigsawAvenueExecutorAdvHighly [target=" + target + "]";
	}

}

/******************************** Normal Executor *********************************/
class ObjectiveJigsawAvenueExecutorAdvNormal extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawAvenueExecutorAdvNormal.class);

	private ObjectiveJigsawAvenue target;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;
	private int nextSpotOid;

	protected ObjectiveJigsawAvenueExecutorAdvNormal(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawAvenue target) {
		super(context, behavior);
		this.target = target;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
	}

	@Override
	public AntCommand execute() {
		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getHMSpots());
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingHMSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		return !this.target.isHMSpoted();
	}

	@Override
	public String toString() {
		return "ObjectiveJigsawAvenueExecutorAdvNormal [target=" + target + "]";
	}

}

/******************************** ALL Executor *********************************/
class ObjectiveJigsawAvenueExecutorAdvAll extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawAvenueExecutorAdvNormal.class);

	private ObjectiveJigsawAvenue target;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;
	private int nextSpotOid;

	protected ObjectiveJigsawAvenueExecutorAdvAll(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawAvenue target) {
		super(context, behavior);
		this.target = target;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
	}

	@Override
	public AntCommand execute() {
		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		return !this.target.isSpoted();
	}

	@Override
	public String toString() {
		return "ObjectiveJigsawAvenueExecutorAdvAll [target=" + target + "]";
	}

}