package ant.agent.job.pf.adv;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import ant.AgentContext;
import ant.agent.behavior.PFBehavior;
import ant.agent.command.AntCommand;
import ant.agent.job.pf.AbstractOjExecutor;
import ant.agent.job.pf.OjExecutorUtils;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawCross;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveMiscComponent;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotLevel;
import ant.utils.Pair;

/**
 * 
 * @author Ivan
 * 
 */
public class ObjectiveJigsawCrossExecutorAdv extends AbstractOjExecutor {

	private AbstractOjExecutor _executor;
	private ObjectiveMiscComponent misc;
	private ObjectiveJigsawCross target;

	public ObjectiveJigsawCrossExecutorAdv(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawCross target, ExecutorLevel level) {
		super(context, behavior);

		this.misc = (ObjectiveMiscComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveMiscComponent.class);
		this.target = target;

		if (level == ExecutorLevel.HighlyNoCheck || level == ExecutorLevel.Highly) {
			if (level == ExecutorLevel.Highly) {
				_executor = new ObjectiveJigsawCrossExecutorAdvHighly(context, behavior, target,
						true);
			} else {
				_executor = new ObjectiveJigsawCrossExecutorAdvHighly(context, behavior, target,
						false);
			}
		} else {
			if (level == ExecutorLevel.HighlyAndMid) {
				_executor = new ObjectiveJigsawCrossExecutorAdvNormal(context, behavior, target);
			} else {
				_executor = new ObjectiveJigsawCrossExecutorAdvAll(context, behavior, target);
			}
		}
	}

	@Override
	public AntCommand execute() {
		misc.setDoing(target);
		return _executor.execute();
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return _executor.getObjectiveJigsaw();
	}

	@Override
	public boolean isDone() {
		boolean b = _executor.isDone();
		if (b)
			misc.setDoing(null);
		return b;
	}

	@Override
	public String toString() {
		return _executor.toString();
	}
}

/*********************** Highly Executor *************************/
class ObjectiveJigsawCrossExecutorAdvHighly extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawCrossExecutorAdvHighly.class);

	private ObjectiveJigsawCross target;
	private boolean checkBest;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;

	private int nextSpotOid;

	protected ObjectiveJigsawCrossExecutorAdvHighly(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawCross target, boolean checkBest) {
		super(context, behavior);
		this.target = target;
		this.checkBest = checkBest;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
	}

	@Override
	public AntCommand execute() {
		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getLevelSpots(SpotLevel.High));
//			if (!isInObjectiveJigsaw(target)) {
//				this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getLevelSpots(SpotLevel.High));
//			} else {
//				this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getHMSpots());
//			}
			
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingHighSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		// boolean isSpoted = this.target.isSpoted();
		boolean isHighlySpoted = this.target.isHighlySpoted();

		if (isHighlySpoted) {
			if (checkBest) {
				if (!getObjectiveJigsaw().getDiscuss().isBestApplicant()) {
					return true;
				} else {
					return false;
				}
			}
			return false;
		} else {
			return true;
		}
	}

	@Override
	public String toString() {
		return super.toString() + ":" + this.target.toString();
	}
}

/*********************** Normal Executor *************************/
class ObjectiveJigsawCrossExecutorAdvNormal extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawCrossExecutorAdvHighly.class);

	private ObjectiveJigsawCross target;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;

	private int nextSpotOid;

	protected ObjectiveJigsawCrossExecutorAdvNormal(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawCross target) {
		super(context, behavior);
		this.target = target;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
	}

	@Override
	public AntCommand execute() {
		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getHMSpots());
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingHMSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		return !this.target.isHMSpoted();
	}

	@Override
	public String toString() {
		return super.toString() + ":" + this.target.toString();
	}
}


/*********************** Normal Executor *************************/
class ObjectiveJigsawCrossExecutorAdvAll extends AbstractOjExecutor {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawCrossExecutorAdvHighly.class);

	private ObjectiveJigsawCross target;
	private AbstractSpot nextSpot;
	private ObjectiveJigsawMetaComponent ojmc;

	private int nextSpotOid;

	protected ObjectiveJigsawCrossExecutorAdvAll(AgentContext context, PFBehavior behavior,
			ObjectiveJigsawCross target) {
		super(context, behavior);
		this.target = target;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
	}

	@Override
	public AntCommand execute() {
		if (isDone()) {
			log.error("Executor is done!");
			return null;
		}

		if (nextSpot != null && nextSpot.isDone()) {
			nextSpot = null;
		}

		if (nextSpot == null) {
			this.nextSpot = OjExecutorUtils.getNearestSpot(human, target.getSpots());
			this.nextSpotOid = target.getObjectiveID();
			if (nextSpot == null) {
				log.error("Not found next Spot");
				return null;
			}
		}

		if (nextSpotOid == target.getObjectiveID()) {
			if (!isInObjectiveJigsaw(target)) {
				Pair<AbstractSpot, Integer> item = getUnderfootingSpot();
				if (item != null) {
					nextSpot = item.first();
					nextSpotOid = item.second();
				}
			}
		}

		if (nextSpot.getWhere() == null) {
			log.error("@" + context.getTimestamp() + " " + nextSpot + " 's Where is null!");
			return null;
		}

		return executeSpot(nextSpot);
	}

	@Override
	public ObjectiveJigsaw getObjectiveJigsaw() {
		return target;
	}

	@Override
	public boolean isDone() {
		return !this.target.isSpoted();
	}

	@Override
	public String toString() {
		return super.toString() + ":" + this.target.toString();
	}
}