package ant.agent.job.pf.adv;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.agent.behavior.PFBehavior.CleanLevel;
import ant.agent.command.AntClear;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.job.AntPoliceForceJob;
import ant.agent.job.pf.AbstractOjExecutor;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawAvenue;
import ant.component.pf.ObjectiveJigsawCross;
import ant.component.pf.ObjectiveJigsawDecisionComponent;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveRaceComponent;
import ant.component.pf.Race;
import ant.component.pf.adv.ObjectiveAdvProbeComponent;
import ant.component.pf.adv.ObjectiveAdvProbeSpriteComponent;
import ant.component.pf.adv.ObjectiveAdvProbeWorkerComponent;
import ant.component.universal.ChangeSetDetailComponent;
import ant.liaison.channel.assign.ChannelAssignResult.BandwidthLevel;
import ant.map.component.StaticCityGraphComponent;
import ant.map.partition.Partition;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ObjectiveRaceComponent}
 */
public class PFPartitionalProbeJobAdv extends AntPoliceForceJob {
	private static final Log log = LogFactory.getLog(PFPartitionalProbeJobAdv.class);

	private ObjectiveRaceComponent opc;
	private Race race;
	private PFWorkerPartitionalProbeJobAdv workerjob;
	private PFSpritePartitionalProbeJobAdv spritejob;

	public PFPartitionalProbeJobAdv(AgentContext context) {
		super(context);
		this.opc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.race = opc.getMyRace();
		if (race == Race.Worker) {
			this.workerjob = new PFWorkerPartitionalProbeJobAdv(context);
		} else {
			this.spritejob = new PFSpritePartitionalProbeJobAdv(context);
		}
	}

	@Override
	public void overtaken() {
		if (race == Race.Worker)
			workerjob.overtaken();
		else
			spritejob.overtaken();
	}

	@Override
	public void overleap() {
		super.overleap();
		if (race == Race.Worker)
			workerjob.overleap();
		else
			spritejob.overleap();
	}

	@Override
	public boolean shouldExecute() {
		if (race == Race.Worker) {

			long _time_st = System.currentTimeMillis();
			boolean ret = workerjob.shouldExecute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalProbeJob<Worker>.shouldExecute cost: "
						+ (_time_ed - _time_st) + " milliseconds");
			}
			return ret;
		} else {

			long _time_st = System.currentTimeMillis();
			boolean ret = spritejob.shouldExecute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalProbeJob<Sprite>.shouldExecute cost: "
						+ (_time_ed - _time_st) + " milliseconds");
			}
			return ret;
		}
	}

	@Override
	public AntCommand execute() {
		if (race == Race.Worker) {

			long _time_st = System.currentTimeMillis();
			AntCommand ret = workerjob.execute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalProbeJob<Worker>.execute cost: " + (_time_ed - _time_st)
						+ " milliseconds");
			}
			if (ret == null)
				log.error("WorkerJob return null");
			return ret;
		} else {

			long _time_st = System.currentTimeMillis();
			AntCommand ret = spritejob.execute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalProbeJob<Sprite>.execute cost: " + (_time_ed - _time_st)
						+ " milliseconds");
			}
			if (ret == null)
				log.error("SpriteJob return null");
			return ret;
		}
	}

	@Override
	public String getJobName() {
		return PFPartitionalProbeJobAdv.class.getName();
	}

}

/**************************************************************************************************************************/
/*********************************************** ABSTRACT PARTITIONAL PROBE ***********************************************/
/**************************************************************************************************************************/

abstract class PFAbstractPartitionalProbeJobAdv extends AntPoliceForceJob {

	protected ObjectiveJigsawMetaComponent ojmc;
	protected ObjectiveJigsawDecisionComponent odc;
	protected ObjectiveRaceComponent opc;
	protected StaticCityGraphComponent city;
	protected ChangeSetDetailComponent csd;
	protected Human human;
	// protected ObjectiveProbeComponent probc;
	protected ObjectiveAdvProbeComponent oapc;

	protected PFAbstractPartitionalProbeJobAdv(AgentContext context) {
		super(context);
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.odc = (ObjectiveJigsawDecisionComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawDecisionComponent.class);
		this.opc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.city = context.getAntComponentKit().getStaticCityGraphComponent();
		this.csd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.oapc = (ObjectiveAdvProbeComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveAdvProbeComponent.class);
		this.human = context.getHuman();
		
	}

	protected boolean canAddDeepExplore(ObjectiveJigsaw oj) {
		if (context.getTimestamp() <= ConfigKey.getIgnoreTime(context.getConfig()) + 2)
			return false;
		
		BandwidthLevel bwl = context.getAntCommunicationKit().getBandwidthLevel();
		if (bwl == BandwidthLevel.Poor || bwl == BandwidthLevel.None) {
			if (context.getTimestamp() > 75) 
				return false;
			else
				return true;
		}
		
		return false;
	}
	
	protected boolean canAddVoiceCivExplore(ObjectiveJigsaw oj) {
		return true;
	}
}

/**************************************************************************************************************************/
/*********************************************** WORKER PARTITIONAL PROBE ***********************************************/
/**************************************************************************************************************************/

class PFWorkerPartitionalProbeJobAdv extends PFAbstractPartitionalProbeJobAdv {
	private static final Log log = LogFactory.getLog(PFWorkerPartitionalProbeJobAdv.class);

	private enum WppStage {
		Inactive, MovingToPart, PartWorking
	}

	private Partition mypart;
	private ObjectiveAdvProbeWorkerComponent oapwc;
	private WppStage stage;

	private EntityID moveToPartTarget;
	private ObjectiveJigsaw partWorkingTarget;
	private AbstractOjExecutor executor = null;

	public PFWorkerPartitionalProbeJobAdv(AgentContext context) {
		super(context);
		this.mypart = opc.getMyPartition();
		this.oapwc = oapc.getObjectiveAdvProbeWorkerComponent();
		this.stage = WppStage.Inactive;
	}

	@Override
	public void overtaken() {
		stage = WppStage.Inactive;
		executor = null;
	}

	@Override
	public void overleap() {
		super.overleap();
		stage = WppStage.Inactive;
		executor = null;
	}

	@Override
	public boolean shouldExecute() {
		if (executor != null) {
			if (executor.isDone()) {
				executor = null;
			} else {
				return true;
			}
		}

		if (executor == null) {
			// here-executor
			EntityID loc = human.getPosition();
			ObjectiveJigsaw oj = ojmc.getExMappingObjectiveJigsaw(loc);
			
			boolean ade = oj.belongPartId(mypart.getID()) && canAddDeepExplore(oj) && oj.getInfoStat().shouldDeepExplore();
			boolean avce = oj.belongPartId(mypart.getID()) && canAddVoiceCivExplore(oj) && oj.getInfoStat().shouldVoiceCivExplore();
			
			if (oj.isSpoted() || ade || avce) {
				// if (oj.isSpoted() || (oj.belongPartId(mypart.getID()) &&
				// oj.getInfoStat().shouldDeepExplore())) {
				// if (oj.belongPartId(mypart.getID()) &&
				// oj.getInfoStat().shouldDeepExplore()) {
				// oj.getInfoStat().addDeepExploreSpots();
				// }
				if (ade) {
					oj.getInfoStat().addDeepExploreSpots();
				}
				
				if (avce) {
					oj.getInfoStat().addVoiceCivExplore();
				}
				
				if (oj instanceof ObjectiveJigsawCross) {
					executor = new ObjectiveJigsawCrossExecutorAdv(context, behavior,
							(ObjectiveJigsawCross) oj, ExecutorLevel.All);
					if (executor.isDone()) {
						log.error("Add doneExecutor!");
					}
					return true;
				} else if (oj instanceof ObjectiveJigsawAvenue) {
					executor = new ObjectiveJigsawAvenueExecutorAdv(context, behavior,
							(ObjectiveJigsawAvenue) oj, ExecutorLevel.All);
					if (executor.isDone()) {
						log.error("Add doneExecutor!");
					}
					return true;
				}
			}
		}

		// Executor == null
		boolean isInMyPart = oapwc.isInMyPartition();
		if (!isInMyPart) {
			// Move to myPartition

			// 如果上个状态是Inactive, 那么movetopart, return true
			if (stage == WppStage.Inactive) {
				stage = WppStage.MovingToPart;
				moveToPartTarget = oapwc.getNearestRoadToPartition();
				return true;
			}

			if (stage == WppStage.MovingToPart) {
				if (moveToPartTarget == null) {
					moveToPartTarget = oapwc.getNearestRoadToPartition();
				}
				return true;
			}

			if (stage == WppStage.PartWorking) {
				// 继续做完你的partworking吧
				if (partWorkingTarget == null) {
					return tryGetTask();
				} else {
					return true;
				}
			}

			return true;

		} else {
			// In myPartition
			stage = WppStage.PartWorking;
			
			if (partWorkingTarget != null) {
				if (agent.getPosition().equals(partWorkingTarget.getJigsawObject().getCenter().getID())) {
					// 如果所在Jigsaw Totally clean
					ObjectiveJigsaw newone = oapwc.getTask();
					if (newone == null) {
						return false;
					} else {
						if (newone.getObjectiveID() == partWorkingTarget.getObjectiveID()) {
							log.info("Pick same Jigsaw, we need a deep explore");
						} else {
							partWorkingTarget = newone;
						}
						return true;
					}
				} else {
					return true;
				}
			} else {
				return tryGetTask();
			}
		}
	}

	private boolean tryGetTask() {
		partWorkingTarget = oapwc.getTask();
		if (partWorkingTarget == null)
			return false;
		else
			return true;
	}

	@Override
	public AntCommand execute() {
		int time = context.getTimestamp();

		if (executor != null) {
			AntCommand ret = executor.execute();
			if (ret == null) {
				log.error("Executor :" + executor + " ret null");
			}
			return ret;
		}

		if (stage == WppStage.MovingToPart) {
			List<EntityID> path = city.getStaticMoveRoute(agent.getPosition(), moveToPartTarget);
			EntityID id = behavior.cleverPathBlockadeSelect(path, CleanLevel.Normal);
			if (id == null) {
				return new AntMove(time, path);
			} else {
				return new AntClear(time, id);
			}
		} else {
			// PartWork
			Area center = partWorkingTarget.getJigsawObject().getCenter();
			if (agent.getPosition().equals(center.getID())) {
				EntityID id = behavior.cleverAroundBlockadeSelect(CleanLevel.Almost);
				if (id != null) {
					return new AntClear(time, id);
				} else {
					log.error("Bad Rest");
					// FIXME !!
					return new AntMove(time, behavior.getRandomWalk());
				}
			} else {
				List<EntityID> path = oapwc.getMidMoveRoute(agent.getPosition(), center.getID());
				EntityID id = behavior.cleverPathBlockadeSelect(path, CleanLevel.Normal);

				if (id != null) {
					return new AntClear(time, id);
				} else {
					return new AntMove(time, path);
				}
			}
		}
	}

	@Override
	public String getJobName() {
		return PFWorkerPartitionalProbeJobAdv.class.getName();
	}

}

/**************************************************************************************************************************/
/*********************************************** SPRITE PARTITIONAL PROBE ***********************************************/
/**************************************************************************************************************************/

class PFSpritePartitionalProbeJobAdv extends PFAbstractPartitionalProbeJobAdv {
	private static final Log log = LogFactory.getLog(PFAbstractPartitionalProbeJobAdv.class);

	private AbstractOjExecutor executor = null;
	private ObjectiveJigsaw target = null;

	private ObjectiveAdvProbeSpriteComponent oapsc;

	public PFSpritePartitionalProbeJobAdv(AgentContext context) {
		super(context);
		this.oapsc = oapc.getObjectiveAdvProbeSpriteComponent();
	}

	@Override
	public void overtaken() {
		executor = null;
		target = null;
	}

	@Override
	public void overleap() {
		super.overleap();
		executor = null;
		target = null;
	}

	@Override
	public boolean shouldExecute() {
		if (executor != null) {
			if (executor.isDone()) {
				executor = null;
			} else {
				return true;
			}
		}

		EntityID loc = human.getPosition();
		ObjectiveJigsaw oj = ojmc.getExMappingObjectiveJigsaw(loc);
		if (executor == null) {
			
			boolean ade = canAddDeepExplore(oj) && oj.getInfoStat().shouldDeepExplore();;
			boolean avce = canAddVoiceCivExplore(oj) && oj.getInfoStat().shouldVoiceCivExplore();
			
			if (oj.isSpoted() || ade || avce) {
				
				if (ade) {
					oj.getInfoStat().addDeepExploreSpots();
				}
				
				if (avce) {
					oj.getInfoStat().addVoiceCivExplore();
				}

				if (oj instanceof ObjectiveJigsawCross) {
					executor = new ObjectiveJigsawCrossExecutorAdv(context, behavior,
							(ObjectiveJigsawCross) oj, ExecutorLevel.All);
					if (executor.isDone()) {
						log.error("Add doneExecutor!");
					}
					return true;
				} else if (oj instanceof ObjectiveJigsawAvenue) {
					executor = new ObjectiveJigsawAvenueExecutorAdv(context, behavior,
							(ObjectiveJigsawAvenue) oj, ExecutorLevel.All);
					if (executor.isDone()) {
						log.error("Add doneExecutor!");
					}
					return true;
				}
			}
		}

		if (target != null) {
			if (agent.getPosition().equals(target.getJigsawObject().getCenter().getID())) {
				target = null;
			} else {
				return true;
			}
		}

		if (target == null) {
			return tryGetTask();
		} else {
			return false;
		}
	}

	private boolean tryGetTask() {
		this.target = oapsc.getTask();
		if (target == null)
			return false;
		else
			return true;
	}

	@Override
	public AntCommand execute() {
		if (executor != null) {
			AntCommand ret = executor.execute();
			if (ret == null) {
				log.error("Executor ret null:" + executor);
			}
			return ret;
		}

		int time = context.getTimestamp();
		EntityID tw = target.getJigsawObject().getCenter().getID();
		if (agent.getPosition().equals(tw)) {
			EntityID id = behavior.cleverAroundBlockadeSelect(CleanLevel.Almost);
			if (id != null) {
				return new AntClear(time, id);
			} else {
				log.error("Bad Rest");
				// FIXME !!
				return new AntMove(time, behavior.getRandomWalk());
			}
		} else {
			List<EntityID> path = oapsc.getMidMoveRoute(agent.getPosition(), tw);
			EntityID id = behavior.cleverPathBlockadeSelect(path, CleanLevel.Normal);
			if (id != null) {
				return new AntClear(time, id);
			} else {
				return new AntMove(time, path);
			}
		}
	}

	@Override
	public String getJobName() {
		return PFAbstractPartitionalProbeJobAdv.class.getName();
	}

}
