package ant.agent.job.pf.adv;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntRest;
import ant.agent.job.AntPoliceForceJob;
import ant.agent.job.pf.AbstractOjExecutor;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawAvenue;
import ant.component.pf.ObjectiveJigsawCross;
import ant.component.pf.ObjectiveJigsawDecisionComponent;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveRaceComponent;
import ant.component.pf.Race;
import ant.component.pf.adv.ObjectiveSpriteWantedComponent;
import ant.component.pf.adv.ObjectiveWantedComponent;
import ant.component.pf.adv.ObjectiveWorkerWantedComponent;
import ant.map.partition.Partition;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ObjectiveRaceComponent}
 */
public class PFPartitionalWantedJobAdv extends AntPoliceForceJob {
	private static final Log log = LogFactory.getLog(PFPartitionalWantedJobAdv.class);

	private ObjectiveRaceComponent opc;
	private Race race;
	private PFWorkerPartitionalWantedJobAdv workerjob;
	private PFSpritePartitionalWantedJobAdv spritejob;

	public PFPartitionalWantedJobAdv(AgentContext context) {
		super(context);
		this.opc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.race = opc.getMyRace();
		if (race == Race.Worker) {
			this.workerjob = new PFWorkerPartitionalWantedJobAdv(context);
		} else {
			this.spritejob = new PFSpritePartitionalWantedJobAdv(context);
		}
	}

	@Override
	public void overtaken() {
		if (race == Race.Worker)
			workerjob.overtaken();
		else
			spritejob.overtaken();
	}

	@Override
	public void overleap() {
		super.overleap();
		if (race == Race.Worker)
			workerjob.overleap();
		else
			spritejob.overleap();
	}

	@Override
	public boolean shouldExecute() {
		if (race == Race.Worker) {
			long _time_st = System.currentTimeMillis();
			boolean ret = workerjob.shouldExecute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalWantedJob<Worker>.shouldExecute cost: "
						+ (_time_ed - _time_st) + " milliseconds");
			}
			return ret;
		} else {
			long _time_st = System.currentTimeMillis();
			boolean ret = spritejob.shouldExecute();
			long _time_ed = System.currentTimeMillis();
			if (_time_ed - _time_st > 75) {
				log.info("PFPartitionalWantedJob<Sprite>.shouldExecute cost: "
						+ (_time_ed - _time_st) + " milliseconds");
			}
			return ret;
		}
	}

	@Override
	public AntCommand execute() {
		if (race == Race.Worker) {
			return workerjob.execute();
		} else {
			return spritejob.execute();
		}
	}

	@Override
	public String getJobName() {
		return PFPartitionalWantedJobAdv.class.getName();
	}

}

abstract class PFAbstractPartitionalWantedJobAdv extends AntPoliceForceJob {

	protected ObjectiveJigsawMetaComponent omc;
	protected ObjectiveJigsawDecisionComponent odc;
	protected ObjectiveRaceComponent opc;
	protected ObjectiveWantedComponent owc;
	protected Human human;

	public PFAbstractPartitionalWantedJobAdv(AgentContext context) {
		super(context);
		this.omc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.odc = (ObjectiveJigsawDecisionComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawDecisionComponent.class);
		this.opc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.owc = (ObjectiveWantedComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveWantedComponent.class);
		this.human = context.getHuman();
	}

}

class PFWorkerPartitionalWantedJobAdv extends PFAbstractPartitionalWantedJobAdv {
	private static final Log log = LogFactory.getLog(PFWorkerPartitionalWantedJobAdv.class);

	private AbstractOjExecutor executor = null;
	private Partition mypart;
	private ObjectiveWorkerWantedComponent owwc;

	public PFWorkerPartitionalWantedJobAdv(AgentContext context) {
		super(context);
		this.mypart = opc.getMyPartition();
		this.owwc = owc.getObjectiveWorkerWantedComponent();
	}

	@Override
	public void overtaken() {
	}

	@Override
	public void overleap() {
		super.overleap();
		executor = null;
	}

	@Override
	public boolean shouldExecute() {
		if (executor != null) {
			if (executor.isDone()) {
				executor = null;
			}
		}

		if (executor == null) {
			ObjectiveJigsaw oj = owwc.getTask();
			if (oj != null) {
				if (oj.belongPartId(mypart.getID())) {
					if (oj instanceof ObjectiveJigsawCross) {
						executor = new ObjectiveJigsawCrossExecutorAdv(context, behavior,
								(ObjectiveJigsawCross) oj, ExecutorLevel.HighlyAndMid);
					} else if (oj instanceof ObjectiveJigsawAvenue) {
						executor = new ObjectiveJigsawAvenueExecutorAdv(context, behavior,
								(ObjectiveJigsawAvenue) oj, ExecutorLevel.HighlyAndMid);
					}
				}
			}

			if (executor == null) {
				return false;
			}
		}

		// GO~
		return true;
	}

	@Override
	public AntCommand execute() {
		if (executor != null) {
			// log.info("Execute:" + executor);

			AntCommand ret = executor.execute();
			if (ret != null) {
				// addRadioMessage(new
				// MJigsawDoing(executor.getObjectiveJigsaw().getObjectiveID()));
				return ret;
			} else {
				log.error("Bad state");
				return new AntRest(context.getTimestamp());
			}
		} else {
			log.error("Bad state");
			return new AntRest(context.getTimestamp());
		}
	}

	@Override
	public String getJobName() {
		return PFWorkerPartitionalWantedJobAdv.class.getName();
	}

}

class PFSpritePartitionalWantedJobAdv extends PFAbstractPartitionalWantedJobAdv {
	private static final Log log = LogFactory.getLog(PFSpritePartitionalWantedJobAdv.class);

	private AbstractOjExecutor executor = null;
	private ObjectiveSpriteWantedComponent oswc;

	public PFSpritePartitionalWantedJobAdv(AgentContext context) {
		super(context);
		this.oswc = owc.getObjectiveSpriteWantedComponent();
	}

	@Override
	public void overtaken() {
	}

	@Override
	public void overleap() {
		super.overleap();
		executor = null;
	}

	@Override
	public boolean shouldExecute() {
		if (executor != null) {
			if (executor.isDone()) {
				executor = null;
			}
		}

		if (executor == null) {
			ObjectiveJigsaw oj = oswc.getTask();
			if (oj != null) {
				if (oj instanceof ObjectiveJigsawCross) {
					executor = new ObjectiveJigsawCrossExecutorAdv(context, behavior,
							(ObjectiveJigsawCross) oj, ExecutorLevel.HighlyAndMid);
				} else if (oj instanceof ObjectiveJigsawAvenue) {
					executor = new ObjectiveJigsawAvenueExecutorAdv(context, behavior,
							(ObjectiveJigsawAvenue) oj, ExecutorLevel.HighlyAndMid);
				}
			}

			if (executor == null) {
				return false;
			}
		}

		// GO~
		return true;
	}

	@Override
	public AntCommand execute() {
		if (executor != null) {
			AntCommand ret = executor.execute();
			if (ret != null) {
				// addRadioMessage(new
				// MJigsawDoing(executor.getObjectiveJigsaw().getObjectiveID()));
				return ret;
			} else {
				log.error("Bad state");
				return new AntRest(context.getTimestamp());
			}
		} else {
			log.error("Bad state");
			return new AntRest(context.getTimestamp());
		}
	}

	@Override
	public String getJobName() {
		return PFSpritePartitionalWantedJobAdv.class.getName();
	}

}
