package ant.agent.job.universal;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.command.AntMoveXY;
import ant.agent.job.JobState;
import ant.liaison.message.MStuckInRoad;
import ant.liaison.message.MVoiceAgentStuckInRoad;
import ant.utils.Pair;
import ant.utils.geo.Converter;
import ant.utils.geo.GeoUtils;
import ant.utils.geo.Point;
import ant.utils.geo.Segment;

/**
 * 
 * @author Fun
 * 
 */
public class AdvanceStuckInRoadJob<E extends Human> extends AbstractStuckJob<E> {
	private static Log log = LogFactory.getLog(AdvanceStuckInRoadJob.class);

	private static final int CHECK_STEP_1 = 2;
	private static final int CHECK_STEP_2 = 5;
	private Road stuckRoad;
	private Pair<Integer, Integer> target = null;
	private EntityID nextArea;
	private int tryTime = 0;

	private int target1try = 0;
	private int target1failed = 0;
	private int target2try = 0;
	private int target2failed = 0;

	public AdvanceStuckInRoadJob(AgentContext context) {
		super(context);

	}

	@Override
	public boolean shouldExecute() {
		// no need to check stuck at start
		if (context.getTimestamp() < ignoreTimestamp + CHECK_STEP_1)
			return false;

		StandardEntity here = context.getHuman().getPosition(world);

		if (here instanceof Road && isStuck(CHECK_STEP_1)) {
			if (tryTime < 2) {
				this.state = JobState.MovingOutOfStuck;
				stuckRoad = (Road) here;
				List<EntityID> leastMovePath = getLeastMovePath();
				nextArea = null;
				target = null;

				if (leastMovePath != null && leastMovePath.size() > 0) {
					for (EntityID next : leastMovePath) {
						if (next.equals(stuckRoad.getID())) {
							continue;
						}

						if (stuckRoad.getNeighbours().contains(next)) {
							nextArea = next;
						}
					}
				}

				if (nextArea != null) {
					if (tryTime < 1) {
						target = getTargetXY(context, nextArea);
					} else {
						target = getTargetXY2(context, nextArea);
					}
					if (target != null) {
						tryTime++;

						if (tryTime == 1) {
							target1try++;
						} else if (tryTime == 2) {
							target2try++;
							target1failed++;
						}
						if (context.getTimestamp() % 50 == 0) {
							log.info("target1try:" + target1try + " failed:" + target1failed
									+ "# target2try:" + target2try + " failed:" + target2failed);
						}
						return true;
					}
				}
			} else {
				tryTime++;
			}

		}

		if (tryTime == 3) {
			target2failed++;
		}

		if (here instanceof Road && isStuck(CHECK_STEP_2)) {
			this.state = JobState.MovingOutOfStuck;
			stuckRoad = (Road) here;
			sendStuckMessage();
		}

		if (tryTime == 1) {
			target1try++;
		} else if (tryTime == 2) {
			target2try++;
			target1failed++;
		}

		if (context.getTimestamp() % 50 == 0) {
			log.info("target1try:" + target1try + " failed:" + target1failed + "# target2try:"
					+ target2try + " failed:" + target2failed);
		}

		return false;
	}

	private List<EntityID> getLeastMovePath() {
		int time = context.getTimestamp();
		for (int i = 1; i <= CHECK_STEP_2; i++) {
			AntCommand ac = this.context.getCommands().get(time - i);
			if (ac instanceof AntMove) {
				return ((AntMove) ac).getPath();
			}
		}
		return null;
	}

	@Override
	public AntCommand execute() {
		List<EntityID> path = new ArrayList<EntityID>();
		path.add(nextArea);

		// if(tryTime == 2){
		// SenceMaker.printArea(stuckRoad, world);
		// Area next = (Area) world.getEntity(nextArea);
		// SenceMaker.printArea(next, world);
		// System.out.println("StuckInRoad: try move to ("+target.first()+", "+target.second()+")");
		// }

		return new AntMoveXY(context.getTimestamp(), path, target.first(), target.second());
	}

	@Override
	public void overtaken() {
		super.overtaken();
		this.stuckRoad = null;
		this.tryTime = 0;
	}

	@Override
	public void overleap() {
		super.overleap();
		this.stuckRoad = null;
	}

	private void sendStuckMessage() {
		if (stuckRoad != null) {
			addRadioMessage(new MStuckInRoad(stuckRoad, context.getHuman().getX(), context
					.getHuman().getY()));
			addVoiceMessage(new MVoiceAgentStuckInRoad(context.getAgentID(), stuckRoad, context
					.getHuman().getX(), context.getHuman().getY()));
		} else {
			log.error("No stuckBuilding, why send stuck message?");
		}
	}

	
	private Segment getDivSegment(Road stuckRoad, EntityID neighbour ,int ax, int ay){
		Edge edge = stuckRoad.getEdgeTo(neighbour);
		int lenX = edge.getEndX()-edge.getStartX();
		int lenY = edge.getEndY()-edge.getStartY();
		Segment seg1 = new Segment(ax-10*lenX,ay-10*lenY,ax,ay);
		Segment seg2 = new Segment(ax,ay,ax+10*lenX,ay+10*lenY);
		Point mid = new Point(ax,ay);
		
		List<Segment> segs = Converter.generateSegments(stuckRoad.getApexList());
		
		for(Segment next: segs){
			Point intp = GeoUtils.getSegmentIntersectionPoint(seg1, next);
			if(intp != null){
				seg1 = new Segment(intp,mid);
			}
			intp = GeoUtils.getSegmentIntersectionPoint(seg2, next);
			if(intp != null){
				seg2 = new Segment(mid, intp);
			}
		}
		
		Segment tmp = new Segment(seg1.getOrigin(), seg2.getEnd());
		Line2D line = new Line2D(tmp.getX1(),tmp.getY1(),tmp.getX2(),tmp.getY2());
		Point2D p1 = GeoUtils.getClosestPointOnSegment(line, edge.getStart());
		Point2D p2 = GeoUtils.getClosestPointOnSegment(line, edge.getEnd());
		if(p1 != null){
			if(GeoUtils.isOnSegment((int)p1.getX(), (int)p1.getY(), seg1.getX1(), seg1.getY1(), seg1.getX2(), seg1.getY2())){
				seg1 = new Segment(new Point((int)p1.getX(),(int)p1.getY()), mid);
			}else if(GeoUtils.isOnSegment((int)p1.getX(), (int)p1.getY(), seg2.getX1(), seg2.getY1(), seg2.getX2(), seg2.getY2())){
				seg2 = new Segment(mid,new Point((int)p1.getX(),(int)p1.getY()));
			}
		}
		if(p2 != null){
			if(GeoUtils.isOnSegment((int)p2.getX(), (int)p2.getY(), seg1.getX1(), seg1.getY1(), seg1.getX2(), seg1.getY2())){
				seg1 = new Segment(new Point((int)p2.getX(),(int)p2.getY()),mid);
			}else if(GeoUtils.isOnSegment((int)p2.getX(), (int)p2.getY(), seg2.getX1(), seg2.getY1(), seg2.getX2(), seg2.getY2())){
				seg2 = new Segment(mid,new Point((int)p2.getX(),(int)p2.getY()));
			}
		}
		
		return new Segment(seg1.getOrigin(),seg2.getEnd());
	}
	
	private Set<Blockade> getFacedBlocakesOnStuckRoad(Segment divSeg, Road stuckRoad,EntityID neighbour, StandardWorldModel world){
		Edge edge = stuckRoad.getEdgeTo(neighbour);
		Point flag = new Point((edge.getStartX()+edge.getEndX())/2,(edge.getStartY()+edge.getEndY())/2);
		
		Set<Blockade> ret = new HashSet<Blockade>();
		
		if(!stuckRoad.isBlockadesDefined()){
			return ret;
		}
		for(EntityID bid: stuckRoad.getBlockades()){
			Blockade blockade = (Blockade) world.getEntity(bid);
			if(GeoUtils.onSameSide(divSeg.getOrigin(), divSeg.getEnd(), flag, new Point(blockade.getX(),blockade.getY()))){
				ret.add(blockade);
			}
		}
		return ret;
	}

}
