package ant.agent.job.universal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.job.AbstractJob;
import ant.agent.job.JobState;
import ant.map.objects.RouteResult;

/**
 * 
 * @author Fun
 * 
 */
public class RandomWalkJob<E extends Human> extends AbstractJob<E> {

	private static Log log = LogFactory.getLog(RandomWalkJob.class);

	private static final int RANDOM_WALK_LENGTH = 8;
	private List<EntityID> path;
	private int trytime = 0;
	private int seed;
	
	public RandomWalkJob(AgentContext context) {
		super(context);
		seed = context.getAgentID().getValue() + 1;
	}

	@Override
	public void overtaken() {
		state = JobState.Inactive;
		trytime = 0;
		path = null;
	}

	@Override
	public boolean shouldExecute() {
		if(state == JobState.ExploringRandom){
			if(path != null&&behavior.getMyPosition().getID() != path.get(path.size()-1)&&trytime <2){
					state = JobState.ExploringRandom;
			}else{
				state = JobState.Inactive;
			}
		}
		if (state == JobState.Inactive) {
			List<EntityID> dest = randomWalk();
			RouteResult result = null;
			if (dest.size() > 0) {
				result = context
						.getAntComponentKit()
						.getAbstractRouteComponent()
						.search(context.getHuman().getPosition(),
								context.getHuman().getX(),
								context.getHuman().getY(),
								dest.get(dest.size() - 1));
			}
			if (result.hasAcceptableRoute()) {
				path = result.getMoveRoute();
			} else {
				path = dest;
			}
			state = JobState.ExploringRandom;
			trytime=0;
		}
		return true;
	}

	private List<EntityID> randomWalk() {
		List<EntityID> path = new ArrayList<EntityID>();
		Set<EntityID> seen = new HashSet<EntityID>();
		Area current = behavior.getMyPosition();
		for (int i = 0; i < RANDOM_WALK_LENGTH; i++) {
			path.add(current.getID());
			seen.add(current.getID());
			List<EntityID> neighbours = current.getNeighbours();
			Random random = new Random(seed);
			seed += 1;
			Collections.shuffle(neighbours, random);
			boolean found = false;
			for (EntityID next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				StandardEntity ar = context.getWorld().getEntity(next);
				if (ar instanceof Building) {
					continue;
				}
				current = (Area) ar;
				found = true;
				break;
			}
			if (!found) {
				break;
			}

		}
//		log.info("random path:"+path);
		return path;
	}

	@Override
	public AntCommand execute() {
		if (state == JobState.ExploringRandom) {
			if (path != null) {
				trytime++;
				return new AntMove(context.getTimestamp(), path);
			}
		}
		return null;
	}

	@Override
	public String getJobName() {
		return RandomWalkJob.class.getName();
	}

}
