package ant.agent.job.universal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import ant.AgentContext;
import ant.agent.command.AntCommand;
import ant.agent.command.AntMove;
import ant.agent.job.JobState;
import ant.liaison.message.MStuckInRoad;
import ant.liaison.message.MVoiceAgentStuckInRoad;

/**
 * 
 * @author Oliver
 * 
 */
public class SimpleStuckInRoadJob<E extends Human> extends AbstractStuckJob<E> {
	private static final Log log = LogFactory.getLog(SimpleStuckInRoadJob.class);

	private static final int CHECK_STEP = 5;
	private Road stuckRoad;

	public SimpleStuckInRoadJob(AgentContext context) {
		super(context);
	}

	@Override
	public void overtaken() {
		super.overtaken();
		this.stuckRoad = null;
	}

	@Override
	public void overleap() {
		super.overleap();
		this.stuckRoad = null;
	}

	@Override
	public boolean shouldExecute() {
		// no need to check stuck at start
		if (context.getTimestamp() < ignoreTimestamp + CHECK_STEP)
			return false;

		StandardEntity here = context.getHuman().getPosition(world);
		if (here instanceof Road && isStuck(CHECK_STEP)) {
			this.state = JobState.MovingOutOfStuck;
			stuckRoad = (Road) here;
			sendStuckMessage();
		}

		return false;
	}

	private void sendStuckMessage() {
		// FIXME sending policy needed
		if (stuckRoad != null) {
			// cmu.sendRadio(new MStuckInRoad(stuckRoad, context.getHuman().getX(), context.getHuman().getY()));
			addRadioMessage(new MStuckInRoad(stuckRoad, context.getHuman().getX(), context.getHuman().getY()));
			addVoiceMessage(new MVoiceAgentStuckInRoad(context.getAgentID(), stuckRoad, context.getHuman().getX(), context.getHuman().getY()));
		} else {
			log.error("No stuckBuilding, why send stuck message?");
		}
	}

	@Override
	public AntCommand execute() {
		log.error("Why exectue?");
		return new AntMove(this.context.getTimestamp(), behavior.getRandomWalk());
	}

	@Override
	public String getJobName() {
		return SimpleStuckInRoadJob.class.getName();
	}
}
