package ant.component.at;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.HumanInfoComponent;
import ant.component.universal.JamComponent;
import ant.liaison.message.MRescueNextOrder;
import ant.liaison.message.MRescueOrder;
import ant.map.component.AbstractRouteComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.civilian.AmbulanceKernelSimple;
import ant.utils.civilian.CivilianInfo;
import ant.utils.comparator.EntityIDComparator;
/**
 * 
 * @author Fun
 * @depend {@link AgentLocationComponent} {@link AntMapBasisComponent}
 *         {@link HumanInfoComponent} {@link AgentLocationComponent}
 *         {@link AbstractRouteComponent} {@link StaticCityGraphComponent}
 *         {@link AmbulanceMemComponent} {@link AntMapBasisComponent}
 */
public class AmbulanceKernelComponent extends AbstractMessageProducerComponent{
	private static Log log = LogFactory.getLog(AmbulanceKernelComponent.class);
	
	private AgentContext context;
	private HumanInfoComponent hic;
	private AmbulanceMemComponent amc;
	private AntMapBasisComponent mbc;
	private AmbulanceKernelSimple kernel;
	private List<EntityID> ats;
	
	private List<CivilianInfo> rescueableCollection;

	private Map<EntityID, EntityID> firstAssign;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		hic = context.getAntComponentKit().getHumanInfoComponent();
		amc = (AmbulanceMemComponent) context.getAntComponentKit().getAntComponent(AmbulanceMemComponent.class);
		mbc = (AntMapBasisComponent) context.getAntComponentKit().getAntMapBasisComponent();
		ats = new ArrayList<EntityID>();
		for(Human at: mbc.getAtSet()){
			ats.add(at.getID());
		}
		Collections.sort(ats,new EntityIDComparator());
	}
		
	public boolean isLeader() {
		if (context.getAgentID().getValue() == ats.get(3).getValue()) {
			return true;
		}
		return false;
	}
	
	public void perform(){
		if (kernel == null && isLeader()) {
			kernel = new AmbulanceKernelSimple(amc, this);
		}
		if (isLeader()) {
			kernel.perform();
		}
	}
	
	public Map<EntityID, EntityID> getFirstAssign(){
		return this.firstAssign;
	}
	
	public List<CivilianInfo> getRescueableCollection(){
		return this.rescueableCollection;
	}
	
	/**
	 * for ambulance kernel
	 */
	public void sendOrderForKernel(Map<EntityID, EntityID> firstAssign,
			Map<EntityID, EntityID> secondAssign, List<CivilianInfo> rescueableCollection) {
		this.firstAssign = firstAssign;
		this.rescueableCollection = rescueableCollection;
		
		for (Entry<EntityID, EntityID> next : firstAssign.entrySet()) {
			CivilianInfo ci = hic.getCivilianInfoMap().get(next.getValue());
			MRescueOrder msg = new MRescueOrder(next.getKey(), next.getValue(), ci.getPosition());
			addRadioMessage(msg);
		}
		for (Entry<EntityID, EntityID> next : secondAssign.entrySet()) {
			CivilianInfo ci = hic.getCivilianInfoMap().get(next.getValue());
			addRadioMessage(new MRescueNextOrder(next.getKey(), next.getValue(), ci.getPosition()));
		}
	}
	
	@Override
	public void update() {
	}

}
