package ant.component.at;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.agent.job.JobState;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.HumanInfoComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.Jam;
import ant.liaison.message.IMessage;
import ant.liaison.message.MRescueAgentStart;
import ant.liaison.message.MRescueCivilianStart;
import ant.liaison.message.MRescueUnreachNear;
import ant.liaison.message.MVoiceRescuingCivilian;
import ant.map.component.AbstractRouteComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.civilian.BuildingInfo;
import ant.utils.civilian.CivilianInfo;
import ant.utils.civilian.RescueAciton;
import ant.utils.civilian.RescueInfoListener;
import ant.utils.comparator.EntityIDComparator;
import ant.utils.geo.GeoUtils;

/**
 * This component handles and stores data from message for ambulance team or
 * center
 * 
 * @author Fun
 * @depend {@link AgentLocationComponent} {@link AntMapBasisComponent}
 *         {@link HumanInfoComponent} {@link AgentLocationComponent}
 *         {@link AbstractRouteComponent} {@link StaticCityGraphComponent}
 *         {@link JamComponent}
 */
// TODO 移除BuildingInfo
public class AmbulanceMemComponent extends AbstractMessageProducerComponent {

	private static Log log = LogFactory.getLog(AmbulanceMemComponent.class);

	public static final int UNREACHABLE_CIV_IGNORE_INTERVAL = 20;

	private AgentContext context;
	private StandardWorldModel world;
	private AgentLocationComponent alc;
	private HumanInfoComponent hic;
	private JamComponent jc;

	private List<RescueInfoListener> rescueInofListeners;

	/**
	 * 定义world model 的边界量
	 */
	private int minX, maxX, width, minY, maxY, height;
	/**
	 * 此时周期
	 */
	private int currentTime = -1;
	/**
	 * 获得所有的ambulanceTeam的队列
	 */
//	private List<EntityID> ambulanceTeams;
	private int freeATnum;
	
	private Map<Building,Set<EntityID>> buildingATmap;

	/**
	 * 维护一个map来和Building队列，里面的value是CivBuilding
	 */
	private Map<EntityID, BuildingInfo> buildingInfoMap;

	/**
	 * 暂时无法到达的Civilian <id,timestamp>
	 */
	private Map<EntityID, Integer> unreachableCivs;

	private Map<EntityID, EntityID> minCooperator;
	
	private EntityID currenTarget;

	private RescueAciton actionHistory[];
	
	public void setCurrenTarget(EntityID target){
		currenTarget = target;
	}
	
	public EntityID getCurrenTarget(){
		return currenTarget;
	}
	
	public int getCurrentTime() {
		return currentTime;
	}
	/**
	 * 
	 * @return 返回上个周期的记录，如果没有则返回null
	 */
	public RescueAciton getLastRescueAction(){
		return actionHistory[currentTime-1];
	}

	/**
	 * Set history, send RescueStart message
	 * 
	 * @param target
	 */
	public void setRescuing(EntityID target, boolean forAgent) { // TODO
		actionHistory[currentTime] = new RescueAciton(JobState.RescuingHuman, target);
		if (getLastRescueTarget() == null) { // FIXME
			if (forAgent) {
				addRadioMessage(new MRescueAgentStart(target));
			} else {
				addRadioMessage(new MRescueCivilianStart(target));
			}
		} else if (!getLastRescueTarget().equals(target)) {
			if (forAgent) {
				addRadioMessage(new MRescueAgentStart(target));
			} else {
				addRadioMessage(new MRescueCivilianStart(target));
			}
		}
	}

	public EntityID getLastRescueTarget() {
		if (currentTime >= 0) {
			if (actionHistory[currentTime - 1] != null) {
				RescueAciton action = actionHistory[currentTime - 1];
				if(action.getState().equals(JobState.RescuingHuman)){
					return action.getTarget();
				}
			}
		}
		return null;
	}

	/**
	 * 
	 * @param target
	 */
	public void setLoading(EntityID target) {
		if(target == null){
			log.error("loading target is null");
		}
		actionHistory[currentTime] = new RescueAciton(JobState.LoadingHuman, target);
	}
	
	public void setMovingToHuman(EntityID targetHuman){
		if(targetHuman == null){
			log.error("loading target is null");
		}
		actionHistory[currentTime] = new RescueAciton(JobState.MovingToHuman, targetHuman);
	}

//	public List<EntityID> getAmbulanceTeams() {
//		return ambulanceTeams;
//	}

	public List<EntityID> getAvailableATs(){
		return jc.getActiveAgentsOfType(StandardEntityURN.AMBULANCE_TEAM);
	}
	
	public EntityID getAmbulanceTeamPosition(EntityID id) {
		return alc.getAgentLastLocation(id);
	}

	public List<CivilianInfo> getUnsavedCivilians() {
		return hic.getUnsavedCivilians();
	}

	public Map<EntityID, CivilianInfo> getCivilianInfoMap() {
		return hic.getCivilianInfoMap();
	}

	public Set<EntityID> getRefugesID() {
		return hic.getRefugesID();
	}
	
	public Set<EntityID> getATSetInBuilding(Building building){
		return buildingATmap.get(building);
	}

	public Map<EntityID, BuildingInfo> getBuildingInfoMap() {
		return buildingInfoMap;
	}
	
	public boolean isCivilianIgnored(EntityID cid){
		return hic.isCivilianIgnored(cid);
	}
	
	public void ignoreCivlian(EntityID cid){
		this.hic.ignoreCivilian(cid);
	}

	public Map<EntityID, Integer> getUnreachableCivs() {
		return unreachableCivs;
	}

	public boolean isTheMinToLoad(EntityID target) {
		EntityID miniCoper = minCooperator.get(target); // can cause bug for
														// order
		if (miniCoper == null) {
			return true;
		}

		if (this.context.getAgentID().getValue() <= miniCoper.getValue()) {
			minCooperator.remove(target);
			return true;
		}

		if (!this.context.getBehavior().canSee(miniCoper)) {
			log.error("no one load " + target);
			minCooperator.remove(target);
			return true;
		}
		return false;
	}

	public boolean isUnreachableCiv(EntityID cid) {
		if (unreachableCivs.containsKey(cid)) {
			int time = unreachableCivs.get(cid);
			if ((currentTime - time) < UNREACHABLE_CIV_IGNORE_INTERVAL) {
				return true;
			}
		}
		return false;
	}

	public void addUnreacheableCiv(EntityID cid, int timestamp){
		if(!hic.isCivilianIgnored(cid)){
			if(!unreachableCivs.containsKey(cid)){
				unreachableCivs.put(cid, timestamp);
			}else{
				int time = unreachableCivs.get(cid);
				if(time<timestamp){
					unreachableCivs.put(cid, timestamp);
				}
			}
		}
	}
	
	public int getFreeATnum() {
		return this.freeATnum;
	}

	public void addRescueInfoListener(RescueInfoListener listener) {
		this.rescueInofListeners.add(listener);
	}

	public void removeRescueInofListener(RescueInfoListener listener) {
		this.rescueInofListeners.remove(listener);
	}

	/**
	 * distance between AmbulanceTeams, Civilians, Building(Refuge) or Road
	 * 
	 * @param id1
	 * @param urn1
	 * @param id2
	 * @param urn2
	 * @return Integer.MAX_VALUE if unknown
	 */
	public long distance(EntityID id1, StandardEntityURN urn1, EntityID id2, StandardEntityURN urn2) {
		int[] xy1 = getEntityXY(id1, urn1);
		int[] xy2 = getEntityXY(id2, urn2);
		if (xy1[0] == Integer.MAX_VALUE || xy2[0] == Integer.MAX_VALUE) {
			return Integer.MAX_VALUE;
		} else {
			return (long) GeoUtils.dist(xy1[0], xy1[1], xy2[0], xy2[1]);
		}
	}

	/**
	 * manhattan distance between AmbulanceTeams, Civilians, Building(Refuge) or
	 * Road
	 * 
	 * @param id1
	 * @param urn1
	 * @param id2
	 * @param urn2
	 * @return Integer.MAX_VALUE if unknown
	 */
	public long manhattanDistance(EntityID id1, StandardEntityURN urn1, EntityID id2,
			StandardEntityURN urn2) {
		int[] xy1 = getEntityXY(id1, urn1);
		int[] xy2 = getEntityXY(id2, urn2);
		if (xy1[0] == Integer.MAX_VALUE || xy2[0] == Integer.MAX_VALUE) {
			return Integer.MAX_VALUE;
		} else {
			return (long) GeoUtils.manhattanDistance(xy1[0], xy1[1], xy2[0], xy2[1]);
		}
	}

	private int[] getEntityXY(EntityID id, StandardEntityURN urn) {
		int[] ret = new int[2];
		ret[0] = ret[1] = Integer.MAX_VALUE;
		if (urn.equals(StandardEntityURN.AMBULANCE_TEAM)) {
			Area area = (Area) world.getEntity(alc.getAgentLastLocation(id));
			if (area == null) {
				log.error("area == null");
				return ret;
			}
			ret[0] = area.getX();
			ret[1] = area.getY();
		} else if (urn.equals(StandardEntityURN.CIVILIAN)) {
			CivilianInfo ci = getCivilianInfoMap().get(id);
			StandardEntity entity = world.getEntity(ci.getPosition());
			if (entity instanceof Area) {
				Area area = (Area) entity;
				ret[0] = area.getX();
				ret[1] = area.getY();
			} else {
				log.error("Civilian location Error!");
			}
		} else if (urn.equals(StandardEntityURN.BUILDING) || urn.equals(StandardEntityURN.ROAD)
				|| urn.equals(StandardEntityURN.REFUGE)) {
			Area area = (Area) world.getEntity(alc.getAgentLastLocation(id));
			if (area == null) {
				log.error("area == null");
				return ret;
			}
			ret[0] = area.getX();
			ret[1] = area.getY();
		} else {
			log.error("Wrong URN!");
		}
		return ret;
	}

	/**
	 * Get the width of the world
	 * 
	 * @return The width of the world
	 */
	public int getWidth() {
		if (width <= 0)
			calculateDimensions();
		return width;
	}

	/**
	 * Get the height of the world
	 * 
	 * @return The height of the world
	 */
	public int getHeight() {
		if (height <= 0)
			calculateDimensions();

		return height;
	}

	@Override
	public String getComponentName() {
		return AmbulanceMemComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.hic = context.getAntComponentKit().getHumanInfoComponent();
		this.jc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);

		buildingATmap = new HashMap<Building,Set<EntityID>>();

		buildingInfoMap = new HashMap<EntityID, BuildingInfo>();

		unreachableCivs = new HashMap<EntityID, Integer>();
		this.minCooperator = new HashMap<EntityID, EntityID>();
		this.actionHistory = new RescueAciton[1024];
		rescueInofListeners = new ArrayList<RescueInfoListener>();

		if (context.getHuman().getID().getValue() < 0) {
			// in case viewer launch component
			return;
		}

		initialMem();
	}

	private void initialMem() {

		for (StandardEntity next : world) {
			if (next instanceof Building) {
				BuildingInfo bi = new BuildingInfo(next.getID());
				buildingInfoMap.put(next.getID(), bi);
			} 
		}
	}

	public int getTimeToNearestRefugeOfMap(EntityID civLocateArea) {
		return hic.getTimeToNearestRefugeOfMap(civLocateArea);
	}

	// 通过worldBounds这个变量来获取四个边界值。以此来得到地图的边界
	private void calculateDimensions() {
		Rectangle2D worldBounds = this.world.getBounds();
		minY = (int) worldBounds.getMinY();
		minX = (int) worldBounds.getMinX();
		maxX = (int) worldBounds.getMaxX();
		maxY = (int) worldBounds.getMaxY();

		width = maxX - minX;
		height = maxY - minY;
	}

	// ------------------------------- update -------------------------------
	@Override
	public void update() {
		this.currentTime = context.getTimestamp();

		if (context.getHuman().getID().getValue() < 0) {
			// in case viewer launch component
			return;
		}

		Map<EntityID, EntityID> atol = null;

		long _time_st = System.currentTimeMillis();
		updateFromMessages();
		updateFromObserving();
		long _time_ed = System.currentTimeMillis();
		if ((_time_ed - _time_st) > 100) {
			log.error("@" + context.getTimestamp() + " " + getComponentName()
					+ " update Msg&Observing " + " update cost: " + (_time_ed - _time_st)
					+ " milliseconds");
		}

		_time_st = System.currentTimeMillis();
		// 更新不可到达的Civilian信息
		
		Iterator<Entry<EntityID, Integer>> it = unreachableCivs.entrySet().iterator();
		Human at = context.getHuman(); 
		int ct = 0;
		while (it.hasNext()) {
			Entry<EntityID, Integer> entry = it.next();

			CivilianInfo ci = hic.getCivilianInfoMap().get(entry.getKey());

			if (ci == null) {
				continue;
			}
			EntityID civilianPositionID = ci.getPosition();
			if (civilianPositionID == null) {
				continue;
			}

			if (hic.isCivilianIgnored(ci.getID())) {
				it.remove();
				continue;
			}

			ct++;

			StandardEntity se = world.getEntity(ci.getPosition());
			
			if(se == null && !(se instanceof Area)){
				log.error("civilian position is not area" + ci);
				continue;
			}
			
			Area cip = (Area) se;
			
			/**
			if (arc.search(cip.getID(),cip.getX(),cip.getY(),
					at.getPosition()).hasAcceptableRoute()) {

				it.remove();
				continue;
			}
			*/
			
			if (atol == null) {
				atol = alc.getAgentsLastLocation();
			}

			for (Entry<EntityID, EntityID> next : atol.entrySet()) {
				if (next.getValue().equals(ci.getPosition())) {
					if (!alc.getAgentInitialLocation(next.getKey()).equals(
							next.getValue())) {
						it.remove();
						break;
					}
				}
			}

		}

		_time_ed = System.currentTimeMillis();
		if ((_time_ed - _time_st) > 100) {

			log.error("ct:" + ct);
			log.error("@" + context.getTimestamp() + " " + getComponentName()
					+ " upadte unreachable civilians " + " update cost: " + (_time_ed - _time_st)
					+ " milliseconds");
		}

		_time_st = System.currentTimeMillis();
		// update civilian's at set
		for (CivilianInfo next : hic.getCivilianInfoMap().values()) {
			if (hic.isCivilianIgnored(next.getID())) {
				continue;
			}
			Iterator<EntityID> atit = next.getResuceATSet().iterator();
			while (atit.hasNext()) {
				EntityID atid = atit.next();
				EntityID lastposition = alc.getAgentLastLocation(atid);
				if (!lastposition.equals(next.getPosition())) {
					atit.remove();
				}
			}
		}
		_time_ed = System.currentTimeMillis();
		if ((_time_ed - _time_st) > 100) {
			log.error("@" + context.getTimestamp() + " " + getComponentName()
					+ " update civilian's at set " + " update cost: " + (_time_ed - _time_st)
					+ " milliseconds");
		}

		_time_st = System.currentTimeMillis();
		// update available at number
		int busyat = 0;
		for (CivilianInfo next : hic.getCivilianInfoMap().values()) {
			if (hic.isCivilianIgnored(next.getID())) {
				continue;
			}
			busyat += next.getResuceATSet().size();
		}
		freeATnum = getAvailableATs().size() - busyat;
		freeATnum = freeATnum < 0 ? 0 : freeATnum;
		_time_ed = System.currentTimeMillis();
		if ((_time_ed - _time_st) > 100) {
			log.error("@" + context.getTimestamp() + " " + getComponentName()
					+ " update available at number " + " update cost: " + (_time_ed - _time_st)
					+ " milliseconds");
		}

		// update building at map
		atol = alc.getAgentsLastLocation();
		this.buildingATmap.clear();
		for(Entry<EntityID,EntityID> entry: atol.entrySet()){
			StandardEntity agent = world.getEntity(entry.getKey());
			if(!agent.getStandardURN().equals(StandardEntityURN.AMBULANCE_TEAM)){
				continue;
			}
			StandardEntity se = world.getEntity(entry.getValue());
			if(se instanceof Building){
				if(!buildingATmap.containsKey(se.getID())){
					Set<EntityID> set = new HashSet<EntityID>();
					set.add(entry.getKey());
					buildingATmap.put((Building) se, set);
				}else{
					buildingATmap.get((Building) se).add(entry.getKey());
				}
			}
		}
		
		_time_st = System.currentTimeMillis();
		// update job info
		for (RescueInfoListener listener : this.rescueInofListeners) {
			listener.updateRescueInfo();
		}
		_time_ed = System.currentTimeMillis();
		if ((_time_ed - _time_st) > 100) {
			log.error("@" + context.getTimestamp() + " " + getComponentName() + " update job info "
					+ " update cost: " + (_time_ed - _time_st) + " milliseconds");
		}
	}

	/**
	 * 更新视野中的信息
	 */
	private void updateFromObserving() {
	}

	/**
	 * 更新MRescueUnreachable，MRescueCivilianStart
	 */
	private void updateFromMessages() {
		updateFromRadio();
		updateFromVoice();
	}

	private void updateFromRadio() {
		for (IMessage msg : context.getAntCommunicationKit().getRadioReceiveBox()) {
			// Start Rescue
			if (msg instanceof MRescueCivilianStart) {
				MRescueCivilianStart sr = (MRescueCivilianStart) msg;
				EntityID cid = sr.getCivilian();
				EntityID atid = sr.getSender();
				CivilianInfo ci = hic.getCivilianInfoMap().get(cid);
				if (ci != null) {
					if (!ci.getResuceATSet().contains(atid)) {
						ci.getResuceATSet().add(atid);
					}
				}
				hic.ignoreCivilian(cid);
				if (hic.isCivilianIgnored(cid)) {
					hic.setNotIgnoreCivilian(cid);
				}
			}
			
			// Civilian Unreachable			
			if(msg instanceof MRescueUnreachNear){
				MRescueUnreachNear run = (MRescueUnreachNear) msg;
				EntityID location = run.getLoaction();
				Set<EntityID> civSet = hic.getBuildingCivilianMap().get(location);
				if(civSet != null && civSet.size() > 0){
					for(EntityID cid: civSet){
						if(hic.isCivilianIgnored(cid)){
							continue;
						}
						unreachableCivs.put(cid, run.getTimestamp());
					}
				}				
			}
			// Building info?
		}
	}

	/**
	 * 
	 */
	private void updateFromVoice() {
		for (IMessage msg : context.getAntCommunicationKit().getVoiceReceiveBox()) {
			// MVoiceRescuingCivilian
			if (msg instanceof MVoiceRescuingCivilian) {
				MVoiceRescuingCivilian rcm = (MVoiceRescuingCivilian) msg;
				EntityID cid = rcm.getCivilian();
				EntityID atId = rcm.getSender();
				EntityID minCoper = minCooperator.get(cid);
				if (minCoper == null) {
					minCooperator.put(cid, atId);
				} else if (atId.getValue() < minCoper.getValue()) {
					minCooperator.put(cid, atId);
				}
			}
		}
	}

}
