package ant.component.at;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.component.universal.AgentPartitionAssignComponent;
import ant.map.partition.Partition;

/**
 * 
 * @author Fun
 * @depend  {@link AgentPartitionAssignComponent}
 */
public class ExplorationRecordComponent implements AntComponent{
	private static Log log = LogFactory.getLog(ExplorationRecordComponent.class);
	
	private AgentContext context;
	private StandardWorldModel world;
	private Exploration<Building> exploration;
	private Map<StandardEntity, Integer> unreachableLocation;
	
	
	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		initExploration();
	}

	private void initExploration() {
		exploration = new Exploration<Building>(world);
		unreachableLocation = new HashMap<StandardEntity, Integer>();
		List<Building> buildings = new ArrayList<Building>();

		AgentPartitionAssignComponent apac = (AgentPartitionAssignComponent) context.getAntComponentKit().getAntComponent( AgentPartitionAssignComponent.class);
		Partition partition = apac.getPartition(context.getAgentID());
		
		for(Area next: partition.getAreaSet()){
			if(next instanceof Building){
				buildings.add((Building) next);
			}
		}
		
		exploration.addEntitiesToExploreAndSort(buildings, context.getHuman()
				.getPosition(world));
	}
	
	@Override
	public void update() {
		if (exploration.isExplorationCompeted()) {
			initExploration();
		}
		
		Area location = context.getBehavior().getMyPosition();
		if (location instanceof Building) {
			Building building = (Building) location;
			// log.info("Agent:"+agent.getID()+" ##Building:"+location.getID()+" is removed.");
			this.exploration.addToExplorationHistory(building);
			this.unreachableLocation.remove(location);
			// log.info("Agent:"+agent.getID()+" UnReachAfterRM:"+this.unreachableLocation);
		}

//		Map<EntityID, AgentInfo> agentinfos = agent.getSubworld()
//				.getSubmodelHistoryInfo().getAgentInfos();
//		for (AgentInfo info : agentinfos.values()) {
//			PositionRecord position = info.getPositionRecord();
//			StandardEntity entity = world.getEntity(position.getPosition());
//			if (entity instanceof Building) {
//				Building building = (Building) entity;
//				this.exploration.addToExplorationHistory(building);
//			}
//		}
		
	}
	
	public void addUnreachableLocation(StandardEntity location, int time) {
		unreachableLocation.put(location, time);
	}

	public void removeFromUnreachableLocation(StandardEntity location) {
		unreachableLocation.remove(location);
	}

	public void removeFromUnreachableLocation(EntityID id) {
		Set<StandardEntity> locations = unreachableLocation.keySet();
		for (StandardEntity location : locations) {
			if (location.getID().equals(id)) {
				unreachableLocation.remove(location);
				return;
			}
		}
	}

	public Set<StandardEntity> getUnreachableLocation() {
		return unreachableLocation.keySet();
	}

	/**
	 * 
	 * @return return null if unreachableLocation's size is 0
	 */
	public StandardEntity getOldestUnreachableLoation() {
		StandardEntity ret = null;
		int minTimeStep = Integer.MAX_VALUE;
		for (StandardEntity entity : unreachableLocation.keySet()) {
			int timestep = unreachableLocation.get(entity);
			if (minTimeStep > timestep) {
				minTimeStep = timestep;
				ret = entity;
			}
		}
		return ret;
	}

	public Set<StandardEntity> getUnreachableLocationOlderThan(int timestep) {

		Set<StandardEntity> ret = new HashSet<StandardEntity>();
		if (timestep < 0 || timestep > 300) {
			return ret;
		}
		for (StandardEntity entity : unreachableLocation.keySet()) {
			int temp = unreachableLocation.get(entity);
			if (temp < timestep) {
				ret.add(entity);
			}
		}
		return ret;
	}

	public Exploration<Building> getExplorationBuilding() {
		return this.exploration;
	}
	
	@Override
	public String getComponentName() {
		return ExplorationRecordComponent.class.getName();
	}


}
