package ant.component.fb;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;

/**
 * 
 * @author Oliver
 * @depend {@link FireSimulatorComponent}
 */
public abstract class AbstractFireSpliteComponent implements AntComponent {
	private static Log log = LogFactory.getLog(AbstractFireSpliteComponent.class);

	protected static final float DEFALUT_MIN_CONNECT_VALUE_FOR_SPLIT = 0f;
	/**
	 * set for speedup random access
	 */
	protected List<Set<FireBuilding>> splitSetResult;
	/**
	 * list for traversal and convex hull
	 */
	protected List<List<FireBuilding>> splitListResult;
	protected AgentContext context;
	protected StandardWorldModel world;
	protected FireSimulatorComponent fsc;

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.splitSetResult = new ArrayList<Set<FireBuilding>>();
		this.splitListResult = new ArrayList<List<FireBuilding>>();
		this.fsc = (FireSimulatorComponent) context.getAntComponentKit().getAntComponent(
				FireSimulatorComponent.class);
		initial();
	}

	protected abstract void initial();

	protected void split() {
		long sTime = System.currentTimeMillis();
		splitSetResult.clear();
		splitListResult.clear();
		Set<FireBuilding> buildingsForSplit = getBuildingsForSplit();
		while (!buildingsForSplit.isEmpty()) {
			HashSet<FireBuilding> splitSet = new HashSet<FireBuilding>();
			ArrayList<FireBuilding> splitList = new ArrayList<FireBuilding>();
			splitSetResult.add(splitSet);
			splitListResult.add(splitList);
			split(splitSet, splitList, buildingsForSplit);
		}
		long dTime = System.currentTimeMillis() - sTime;
		if (dTime > 50) {
			log.error("split cost:" + dTime);
		}
	}

	private Set<FireBuilding> getBuildingsForSplit() {
		HashSet<FireBuilding> buildings = new HashSet<FireBuilding>();
		for (FireBuilding fireBuilding : fsc.getAllFireBuildings()) {
			if (isFireBuildingFitSpliteCondition(fireBuilding))
				buildings.add(fireBuilding);
		}
		return buildings;
	}

	private void split(Set<FireBuilding> splitSet, List<FireBuilding> splitList,
			Set<FireBuilding> buildingsForSplit) {
		if (buildingsForSplit.isEmpty()) {
			return;
		}
		Iterator<FireBuilding> it = buildingsForSplit.iterator();
		// get first building for initial
		FireBuilding firstBuilding = it.next();
		// remove the building from the buildingSplit set
		it.remove();

		// add first building to the set
		splitSet.add(firstBuilding);
		// add first building to the list for traversal
		splitList.add(firstBuilding);

		int indexForList = 0;
		while (indexForList < splitList.size()) {
			// get building from the list for traversal
			FireBuilding fireBuilding = splitList.get(indexForList);
			indexForList++;
			for (Entry<FireBuilding, Float> entry : fireBuilding.getNearFireBuildings().entrySet()) {
				FireBuilding nearFireBuilding = entry.getKey();
				if (buildingsForSplit.contains(nearFireBuilding)
						&& entry.getValue() > getMinConnectValueForSplit()
						&& isFireBuildingFitSpliteCondition(nearFireBuilding)) {
					// only if the neighbor building has't been added to set
					if (splitSet.add(nearFireBuilding)) {
						// synchronous the set and the list for traversal
						splitList.add(nearFireBuilding);
						// remove the building form the burning list
						buildingsForSplit.remove(nearFireBuilding);
					}
				}
			}
		}

	}

	public String printResult() {
		String info = "FireSplit Result, split into " + splitSetResult.size() + " part";
		int setIndex = 0;
		for (Set<FireBuilding> set : splitSetResult) {
			info += "\npart " + setIndex++ + ":";
			for (FireBuilding fireBuilding : set) {
				info += "\t" + fireBuilding.getBuilding().getID();
			}
		}
		return info;
	}

	protected abstract boolean isFireBuildingFitSpliteCondition(FireBuilding fireBuilding);

	protected float getMinConnectValueForSplit() {
		return DEFALUT_MIN_CONNECT_VALUE_FOR_SPLIT;
	}

	public List<Set<FireBuilding>> getSplitSetResult() {
		return splitSetResult;
	}

	public List<List<FireBuilding>> getSplitListResult() {
		return splitListResult;
	}

	public int getSplitNum(EntityID buildingID) {
		FireBuilding fireBuilding = fsc.getFireBuilding(buildingID);
		if (fireBuilding != null) {
			return getSplitNum(fireBuilding);
		} else {
			return -1;
		}
	}

	public int getSplitNum(FireBuilding fireBuilding) {
		for (int i = 0; i < splitSetResult.size(); i++) {
			Set<FireBuilding> set = splitSetResult.get(i);
			if (set.contains(fireBuilding)) {
				return i;
			}
		}
		return -1;
	}

}
