package ant.component.fb;

import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.JamState;
import ant.liaison.message.IMessage;
import ant.liaison.message.MOutOfWater;
import ant.map.component.AntMapBasisComponent;

/**
 * 
 * @author Oliver
 * @depend {@link AntMapBasisComponent} {@link AgentLocationComponent}
 *         {@link JamComponent}
 */
public class ActiveFireBrigadeComponent implements AntComponent {

	private AgentContext context;
	private AgentLocationComponent alc;
	private HashSet<EntityID> outOfWaterAgents;
	private Set<EntityID> fbs;
	private Set<EntityID> refuges;
	private HashSet<EntityID> inRefugeAgents;
	private JamComponent jc;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		AntMapBasisComponent amb = context.getAntComponentKit().getAntMapBasisComponent();
		this.alc = (AgentLocationComponent) context.getAntComponentKit().getAntComponent(
				AgentLocationComponent.class);
		this.jc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);

		this.fbs = new HashSet<EntityID>();
		for (FireBrigade fb : amb.getFbSet()) {
			fbs.add(fb.getID());
		}

		this.refuges = new HashSet<EntityID>();
		for (Refuge r : amb.getRefugeSet()) {
			refuges.add(r.getID());
		}

		this.outOfWaterAgents = new HashSet<EntityID>();
		this.inRefugeAgents = new HashSet<EntityID>();
	}

	@Override
	public void update() {
		for (IMessage msg : context.getAntCommunicationKit().getRadioReceiveBox()) {
			if (msg instanceof MOutOfWater) {
				outOfWaterAgents.add(msg.getSender());
			}
		}
		for (IMessage msg : context.getAntCommunicationKit().getVoiceReceiveBox()) {
			if (msg instanceof MOutOfWater) {
				outOfWaterAgents.add(msg.getSender());
			}
		}

		for (AgentLocationInfo info : alc.getUpdatedAgentCurrentLocationInfoSet()) {
			EntityID agentID = info.getAgentID();
			if (!fbs.contains(agentID)) {
				continue;
			}

			if (refuges.contains(info.getLocation())) {
				outOfWaterAgents.remove(agentID);
				inRefugeAgents.add(agentID);
			} else {
				inRefugeAgents.remove(agentID);
			}

		}

		// FIXME what if I'm out of water
	}

	public Set<EntityID> getOutOfWaterAgents() {
		return outOfWaterAgents;
	}

	public boolean isAgentOutOfWater(EntityID agentID) {
		return outOfWaterAgents.contains(agentID);
	}

	public boolean isAgentInRefuge(EntityID agentID) {
		return inRefugeAgents.contains(agentID);
	}

	public boolean isAgentActive(EntityID agentID) {
		JamState state = jc.getJam(agentID).getState();
		return !isAgentOutOfWater(agentID) && !isAgentInRefuge(agentID) && state != JamState.Buried
				&& state != JamState.Unknown;
	}
}
