package ant.component.fb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractAntCommunicationKit;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.agent.command.AntCommand;
import ant.agent.command.AntExtinguish;
import ant.liaison.message.IMessage;
import ant.liaison.message.MVoiceExtinguishBuilding;

/**
 * 
 * @author Oliver
 * 
 */
public class ExtinguishRecordComponent implements AntComponent {
	private static Log log = LogFactory.getLog(ExtinguishRecordComponent.class);

	private AgentContext context;
	private int maxExtinguishPower;
	private AbstractAntCommunicationKit cmu;

	private Map<EntityID, BuildingExtinguishRecord> records;
	private Map<EntityID, WaterQuantity> buildingWaterQuanatityMap;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.maxExtinguishPower = ConfigKey.getFireMaxPower(context.getConfig());
		this.cmu = context.getAntCommunicationKit();
		this.records = new HashMap<EntityID, BuildingExtinguishRecord>();
		this.buildingWaterQuanatityMap = new HashMap<EntityID, WaterQuantity>();
	}

	@Override
	public void update() {
		// self record
		AntCommand cmd = context.getCommands().get(context.getTimestamp() - 1);
		if (cmd instanceof AntExtinguish) {
			AntExtinguish ae = (AntExtinguish) cmd;
			EntityID buildingID = ae.getTarget();
			BuildingExtinguishRecord ber = records.get(buildingID);
			if (ber == null) {
				ber = new BuildingExtinguishRecord();
				records.put(buildingID, ber);
			}
			if (ber.update(ae, context.getAgentID())) {
				WaterQuantity wq = buildingWaterQuanatityMap.get(buildingID);
				if (wq == null) {
					wq = new WaterQuantity(maxExtinguishPower);
				} else {
					wq.setQuantity(wq.getQuantity() + maxExtinguishPower);
				}
				buildingWaterQuanatityMap.put(buildingID, wq);
			}
		}

		// update msg
		for (IMessage msg : cmu.getVoiceReceiveBox()) {
			if (msg instanceof MVoiceExtinguishBuilding) {
				MVoiceExtinguishBuilding eb = (MVoiceExtinguishBuilding) msg;
				EntityID buildingID = eb.getBuilding();
				BuildingExtinguishRecord ber = records.get(buildingID);
				if (ber == null) {
					ber = new BuildingExtinguishRecord();
					records.put(buildingID, ber);
				}
				if (ber.update(eb, maxExtinguishPower)) {
					WaterQuantity wq = buildingWaterQuanatityMap.get(buildingID);
					if (wq == null) {
						wq = new WaterQuantity(maxExtinguishPower);
					} else {
						wq.setQuantity(wq.getQuantity() + maxExtinguishPower);
					}
					buildingWaterQuanatityMap.put(buildingID, wq);
				}
			}
		}
	}

	public BuildingExtinguishRecord getExtinguishRecord(EntityID id) {
		return records.get(id);
	}

	public Map<EntityID, WaterQuantity> getBuildingWaterQuanatityMap() {
		return buildingWaterQuanatityMap;
	}

	public static class WaterQuantity {
		public int quantity;

		public WaterQuantity(int quantity) {
			this.quantity = quantity;
		}

		public int getQuantity() {
			return quantity;
		}

		public void setQuantity(int quantity) {
			this.quantity = quantity;
		}
	}

	public static class BuildingExtinguishRecord {
		private Map<EntityID, List<AntExtinguish>> extinguishCommandMap;

		public BuildingExtinguishRecord() {
			this.extinguishCommandMap = new HashMap<EntityID, List<AntExtinguish>>();
		}

		boolean update(AntExtinguish cmd, EntityID agentID) {
			List<AntExtinguish> list = extinguishCommandMap.get(agentID);
			if (list == null) {
				list = new ArrayList<AntExtinguish>();
				list.add(cmd);
				extinguishCommandMap.put(agentID, list);
				return true;
			} else {
				return updateCommand(list, cmd);
			}
		}

		boolean update(MVoiceExtinguishBuilding msg, int maxExtinguishPower) {
			AntExtinguish cmd = new AntExtinguish(msg.getTimestamp(), msg.getBuilding(),
					maxExtinguishPower);
			return update(cmd, msg.getSender());
		}

		private boolean updateCommand(List<AntExtinguish> cmdList, AntExtinguish newCmd) {
			AntExtinguish lastCmd = cmdList.get(cmdList.size() - 1);
			if (lastCmd == null || newCmd.getTime() <= lastCmd.getTime()) {
				return false;
			}
			cmdList.add(newCmd);
			return true;
		}

		public int getExtinguishCountSinceLastUpdate(int buildingUpdateTime) {
			int extinguishCountSinceLastUpdate = 0;
			for (List<AntExtinguish> list : extinguishCommandMap.values()) {
				for (int i = list.size() - 1; i >= 0; i--) {
					AntExtinguish cmd = list.get(i);
					if (cmd.getTime() >= buildingUpdateTime) {
						extinguishCountSinceLastUpdate++;
					} else {
						break;
					}
				}
			}
			return extinguishCountSinceLastUpdate;
		}

		public int getExtinguishCommandTotal() {
			int rv = 0;
			for (List<AntExtinguish> list : extinguishCommandMap.values()) {
				rv += list.size();
			}
			return rv;
		}
	}

}
