package ant.component.fb;

import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.agent.job.fb.BuidlingQualification;
import ant.component.fb.ExtinguishRecordComponent.BuildingExtinguishRecord;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.component.universal.ChangeSetDetailComponent;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Oliver
 * 
 */
public class FireBuildingComparator implements Comparator<FireBuilding> {
	private static Log log = LogFactory.getLog(FireBuildingComparator.class);

	private static final int EXTINGUISH_TIME_COEFFICENT = 5;
	private static final int MIN_EXTINGUISH_TIME_FOR_BONUS = 2;
	private static final int EXTINGUISH_TIME_BONUS = MIN_EXTINGUISH_TIME_FOR_BONUS + 10;
	private static final double MAX_DISTANCE_RANK_RATE_FOR_BONUS = 0.3;
	private static final int BONUS_FOR_DISTANCE_RANK = 100;
	private static final double MIN_DISTANCE_RANK_RATE_FOR_PUNISHMENT = 0.7;
	private static final int PUNISHMENT_FOR_DISTANCE_RANK = 500;
	private static final int MIN_ACTIVE_FIREBRIGADE_NUM_FOR_BONUS_AND_PUNISHMENT = 5;
	private static final double AGENT_SPEED = 40000;
	private static final double SHOULD_MOVE_COST = 30;
	private static final double PER_MOVE_COST = 15;
	private static final double NOT_IN_CHANGESET_COST = PER_MOVE_COST * 3;
	private static final double NOT_IN_SMALL_HULL_COST = PER_MOVE_COST * 2.5;

	private AgentContext context;
	private ExtinguishRecordComponent erc;
	private Map<EntityID, StandardEntity> buildingsInSight;
	private int fireMaxDistance;

	public FireBuildingComparator(AgentContext context) {
		this.context = context;
		ChangeSetDetailComponent csd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.fireMaxDistance = ConfigKey.getFireMaxDistance(context.getConfig());
		this.buildingsInSight = csd.getEntitiesInSightMap(StandardEntityURN.BUILDING);
		this.erc = (ExtinguishRecordComponent) context.getAntComponentKit().getAntComponent(
				ExtinguishRecordComponent.class);
	}

	@Override
	public int compare(FireBuilding arg0, FireBuilding arg1) {
		return cost(arg0) - cost(arg1);
	}

	public int cost(FireBuilding fireBuilding) {
		FireBrigade me = (FireBrigade) context.getHuman();
		Building building = fireBuilding.getBuilding();
		BuildingCondition condition = fireBuilding.getCondition();
		BuidlingQualification qualification = fireBuilding.getQualification();
		BuildingExtinguishRecord extinguishRecord = erc.getExtinguishRecord(building.getID());

		double cost = 0;
		// qualification
		if (!qualification.isInSmallHull()) {
			cost += NOT_IN_SMALL_HULL_COST;
			double distanceToNearestEdge = qualification.getDistanceToNearestEdge();
			double radius = qualification.getHullRadius();
			// TODO
		}

		// distance
		double distance = GeoUtils.dist(me.getX(), me.getY(), building.getX(), building.getY());
		if (distance > fireMaxDistance) {
			double timeToMove = (distance - fireMaxDistance) / AGENT_SPEED;
			double distanceCost = 0;
			if (timeToMove <= 0.5) {
				distanceCost = timeToMove * PER_MOVE_COST * 0.8;
			} else if (timeToMove <= 2) {
				distanceCost = timeToMove * PER_MOVE_COST * 1.2;
			} else if (timeToMove <= 4) {
				distanceCost = timeToMove * PER_MOVE_COST * 1.6;
			} else {
				distanceCost = timeToMove * PER_MOVE_COST * 2.2;
			}
			cost += distanceCost + SHOULD_MOVE_COST;
		}

		// fieryness
		cost += 40 * (condition.getFieryness() - 1) + context.getTimestamp()
				- condition.getTimestamp();

		// estiFieryness
		cost += 30 * (fireBuilding.getCurrentEstiFieryness() - 1);

		// totalHits/totalRays
		cost += 10 * fireBuilding.getHitRate();

		// neighbors
		for (Iterator<Entry<FireBuilding, Float>> iter = fireBuilding.getNearFireBuildings()
				.entrySet().iterator(); iter.hasNext();) {
			Entry<FireBuilding, Float> entry = iter.next();
			FireBuilding fb = entry.getKey();
			if (condition.isOnFire())
				cost += 10 * (fb.getCurrentEstiFieryness() - 1) * entry.getValue();
		}

		// estiIgnition time
		if (fireBuilding.getIgnitionTime() != -1) {
			cost += 10 * (context.getTimestamp() - fireBuilding.getIgnitionTime());
		}

		// estiFuel
		cost += Math.min(70, (1 - fireBuilding.getFuelRate())
				* (fireBuilding.getInitFuel() / 8000 + 20));
		// Temperature
		double dTemperature = fireBuilding.getCurrentEstiTemperature()
				- fireBuilding.getIgnitionPoint();
		if (dTemperature > 80) {
			cost += PER_MOVE_COST * 0.5;
		} else if (dTemperature > 200) {
			cost += PER_MOVE_COST * 1;
		} else if (dTemperature > 500) {
			cost += PER_MOVE_COST * 1.5;
		}

		// distance of rank, TODO test for it
		// ArrayList<EntityID> activeFireBrigades = me.getActiveFireBrigades();
		// if (activeFireBrigades.size() >=
		// MIN_ACTIVE_FIREBRIGADE_NUM_FOR_BONUS_AND_PUNISHMENT) {
		// Collections.sort(activeFireBrigades, new DistanceComparator(me,
		// building.getID()));
		// int distanceRank = activeFireBrigades.indexOf(me);
		// if (distanceRank == -1) {
		// log.info(me + "is not in activeFireBrigades");
		// return (int) (cost < 0 ? 0 : cost);
		// }
		// double distanceRankRate = distanceRank / activeFireBrigades.size();
		// if (distanceRankRate <= MAX_DISTANCE_RANK_RATE_FOR_BONUS) {
		// cost -= BONUS_FOR_DISTANCE_RANK * (1 - distanceRankRate);
		// } else if (distanceRankRate >= MIN_DISTANCE_RANK_RATE_FOR_PUNISHMENT)
		// {
		// cost += PUNISHMENT_FOR_DISTANCE_RANK * distanceRankRate;
		// }
		// }
		//
		// log.info("Cost for " + me.getID() + " to extinguish " +
		// fireBuilding.getBuilding().getID()
		// + " is " + cost + " activeFb");

		// better to extinguish the buildings in the changeset
		if (buildingsInSight.get(building.getID()) == null)
			cost += NOT_IN_CHANGESET_COST;

		// extinguishTime TODO
		// if (extinguishRecord != null) {
		// int extinguishTime = extinguishRecord.getExtinguishCount();
		// if (extinguishTime > MIN_EXTINGUISH_TIME_FOR_BONUS) {
		// double bonus = (EXTINGUISH_TIME_BONUS - extinguishTime)
		// * EXTINGUISH_TIME_COEFFICENT;
		// if (bonus > 0)
		// cost -= bonus;
		// }
		// }

		return (int) (cost < 0 ? 0 : cost);
	}
}
