package ant.component.fb;

import java.util.Comparator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.component.universal.ChangeSetDetailComponent;
import ant.component.universal.UpdateSource;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Oliver
 * 
 */
public class FireBuildingComparatorWithFSC implements Comparator<FireBuilding> {
	private static Log log = LogFactory.getLog(FireBuildingComparatorWithFSC.class);

	private static final double INITIAL_COST = 200;

	private static final double AGENT_SPEED = 40000;
	private static final double BASE_PER_MOVE_COST = 10;
	private static final double SHOULD_MOVE_COST = BASE_PER_MOVE_COST * 2.2;
	private static final double MAX_DISTANCE_COST = BASE_PER_MOVE_COST * 10;
	private static final double NOT_IN_CHANGESET_COST = BASE_PER_MOVE_COST * 1.2;

	private static final int RECENT_UPDATE_TIME_MAX = 4;

	private static final double MAX_FUEL_VALUE_FOR_IGNITION_BUILDING = 4000000;
	private static final double IGNITION_BUILDING_MAX_COST = 25;
	private static final double IGNITION_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_IGNITION_BUILDING
			/ IGNITION_BUILDING_MAX_COST;
	private static final double IGNITION_BUILDING_MIN_COST = 6;

	private static final double MAX_FUEL_VALUE_FOR_ESTI_IGNITION_BUILDING = 4000000;
	private static final double ESTI_IGNITION_BUILDING_MAX_COST = 25;
	private static final double ESTI_IGNITION_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_ESTI_IGNITION_BUILDING
			/ ESTI_IGNITION_BUILDING_MAX_COST;
	private static final double ESTI_IGNITION_BUILDING_MIN_AWARD = 6;

	private static final double MAX_FUEL_VALUE_FOR_HEATING_BUILDING = 4000000;
	private static final double RECENT_UPDATED_HEATING_BUILDING_MAX_COST = 78;
	private static final double RECENT_UPDATED_HEATING_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_HEATING_BUILDING
			/ RECENT_UPDATED_HEATING_BUILDING_MAX_COST;
	private static final double DEFAULT_HEATING_BUILDING_MAX_COST = 65;
	private static final double DEFAULT_HEATING_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_HEATING_BUILDING
			/ DEFAULT_HEATING_BUILDING_MAX_COST;
	private static final double HEATING_BUILDING_MIN_COST = 20;

	private static final double MAX_FUEL_VALUE_FOR_BURNING_BUILDING = 4000000;
	private static final double BURNING_BUILDING_MAX_COST = 80;
	private static final double BURNING_BUILDING_MIN_COST = 36;
	private static final double BURNING_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_BURNING_BUILDING
			* 2 / (BURNING_BUILDING_MAX_COST + BURNING_BUILDING_MIN_COST);

	private static final double MAX_FUEL_VALUE_FOR_INFERNO_BUILDING = 4000000;
	private static final double INFERNO_BUILDING_MAX_COST = 60;
	private static final double INFERNO_BUILDING_MIN_COST = 25;
	private static final double INFERNO_BUILDING_FUEL_COEFFICIENT = MAX_FUEL_VALUE_FOR_INFERNO_BUILDING
			* 2 / (INFERNO_BUILDING_MAX_COST + INFERNO_BUILDING_MIN_COST);

	private static final double MAX_FUEL_LEFT_VALUE_FOR_AWARD = 4000000;
	private static final double FUEL_LEFT_MAX_AWARD = 40;
	private static final double FUEL_LEFT_COEFFICIENT = FUEL_LEFT_MAX_AWARD
			/ MAX_FUEL_LEFT_VALUE_FOR_AWARD;

	private static final float MAX_TEMPERATURE = 400;
	private static final float TEMPERATURE_COEFFICIENT = MAX_TEMPERATURE / 32;
	private static final int lOW_TEMPERATURE = 180;
	private static final double LOW_TEMPERATURE_AWARD = 15;

	private static final double IN_SIGHT_HEATING_BUILDING_AWARD = 25;
	private static final double JUST_NOTICED_HEATING_BUILDING_AWARD = 18;

	private static final double CONFIRM_COST = 22;
	private static final int HIT_RATE_COEFFICIENT = 18;

	private AgentContext context;
	private FireSimulatorComponent fsc;
	private FireConfirmComponent fcc;
	private Map<EntityID, StandardEntity> buildingsInSight;
	private int fireMaxDistance;

	private double perMoveCost;
	private double notInChangeSetCost;

	private double shouldMoveCost;

	private double maxDistanceCost;

	public FireBuildingComparatorWithFSC(AgentContext context) {
		this.context = context;
		ChangeSetDetailComponent csd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.fireMaxDistance = ConfigKey.getFireMaxDistance(context.getConfig());
		this.buildingsInSight = csd.getEntitiesInSightMap(StandardEntityURN.BUILDING);
		this.fsc = (FireSimulatorComponent) context.getAntComponentKit().getAntComponent(
				FireSimulatorComponent.class);
		this.fcc = (FireConfirmComponent) context.getAntComponentKit().getAntComponent(
				FireConfirmComponent.class);
		double worldFireBuildingSituation = fsc.getWorldFireBuildingSituation();
		double coefficient = worldFireBuildingSituation > 1 ? worldFireBuildingSituation * 0.5 + 0.5
				: worldFireBuildingSituation;
		this.perMoveCost = BASE_PER_MOVE_COST * coefficient;
		this.shouldMoveCost = SHOULD_MOVE_COST * coefficient;
		this.maxDistanceCost = MAX_DISTANCE_COST * coefficient;
		this.notInChangeSetCost = NOT_IN_CHANGESET_COST * coefficient;
	}

	@Override
	public int compare(FireBuilding arg0, FireBuilding arg1) {
		return cost(arg0) - cost(arg1);
	}

	public int cost(FireBuilding fireBuilding) {
		double cost = INITIAL_COST;

		FireBrigade me = (FireBrigade) context.getHuman();
		Building building = fireBuilding.getBuilding();
		BuildingCondition condition = fireBuilding.getCondition();

		// distance and should move
		double distance = GeoUtils.dist(me.getX(), me.getY(), building.getX(), building.getY());
		if (distance > fireMaxDistance) {
			double timeToMove = (distance - fireMaxDistance) / AGENT_SPEED;
			double distanceCost = 0;
			if (timeToMove <= 0.5) {
				distanceCost = timeToMove * perMoveCost * 0.4;
			} else if (timeToMove <= 2) {
				distanceCost = timeToMove * perMoveCost * 0.6;
			} else if (timeToMove <= 4) {
				distanceCost = timeToMove * perMoveCost * 0.8;
			} else {
				distanceCost = timeToMove * perMoveCost * 1.0;
			}
			if (distanceCost > MAX_DISTANCE_COST) {
				distanceCost = MAX_DISTANCE_COST;
			}
			cost += distanceCost + shouldMoveCost;
		}

		// currentEstiFieryness
		Fieryness currentEstiFieryness = fireBuilding.getCurrentEstiFierynessEnum();
		float currentEstiFuel = fireBuilding.getCurrentEstiFuel();
		float initFuel = fireBuilding.getInitFuel();
		// ignition
		if (currentEstiFieryness == Fieryness.HEATING
				&& context.getTimestamp() - fireBuilding.getIgnitionTime() <= RECENT_UPDATE_TIME_MAX) {
			double award = Math.min(IGNITION_BUILDING_MAX_COST, currentEstiFuel
					/ IGNITION_BUILDING_FUEL_COEFFICIENT);
			cost -= Math.max(award, IGNITION_BUILDING_MIN_COST);
		} else if (fireBuilding.getEstiIgnitionTime() != -1
				&& context.getTimestamp() - fireBuilding.getEstiIgnitionTime() <= RECENT_UPDATE_TIME_MAX) {
			double award = Math.min(ESTI_IGNITION_BUILDING_MAX_COST, currentEstiFuel
					/ ESTI_IGNITION_BUILDING_FUEL_COEFFICIENT);
			cost -= Math.max(award, ESTI_IGNITION_BUILDING_MIN_AWARD);
		}

		// fuel
		boolean isRecentlyUpdated = (context.getTimestamp() - condition.getTemperature() <= RECENT_UPDATE_TIME_MAX);
		if (currentEstiFieryness == Fieryness.HEATING) {
			double award = 0;
			if (isRecentlyUpdated) {
				award = Math.min(RECENT_UPDATED_HEATING_BUILDING_MAX_COST, currentEstiFuel
						/ RECENT_UPDATED_HEATING_BUILDING_FUEL_COEFFICIENT);
			} else {
				award = Math.min(DEFAULT_HEATING_BUILDING_MAX_COST, currentEstiFuel
						/ DEFAULT_HEATING_BUILDING_FUEL_COEFFICIENT);
			}
			cost -= Math.max(award, HEATING_BUILDING_MIN_COST);
		} else if (currentEstiFieryness == Fieryness.BURNING) {
			double dCost = initFuel / BURNING_BUILDING_FUEL_COEFFICIENT;
			if (dCost > BURNING_BUILDING_MAX_COST) {
				dCost = BURNING_BUILDING_MAX_COST;
			} else if (dCost < BURNING_BUILDING_MIN_COST) {
				dCost = BURNING_BUILDING_MIN_COST;
			}
			cost += dCost;
		} else if (currentEstiFieryness == Fieryness.INFERNO) {
			double dCost = initFuel / INFERNO_BUILDING_FUEL_COEFFICIENT;
			if (dCost > INFERNO_BUILDING_MAX_COST) {
				dCost = INFERNO_BUILDING_MAX_COST;
			} else if (dCost < INFERNO_BUILDING_MIN_COST) {
				dCost = INFERNO_BUILDING_MIN_COST;
			}
			cost += dCost;
		}

		// totalHits/totalRays
		cost += HIT_RATE_COEFFICIENT * fireBuilding.getHitRate();

		// Temperature
		if (currentEstiFieryness == Fieryness.HEATING) {
			double dTemperature = fireBuilding.getCurrentEstiTemperature()
					- fireBuilding.getIgnitionPoint();
			cost += Math.min(MAX_TEMPERATURE, dTemperature) / TEMPERATURE_COEFFICIENT;
			if (dTemperature < lOW_TEMPERATURE) {
				cost -= LOW_TEMPERATURE_AWARD;
			}
		}

		float fuelLeft = initFuel - currentEstiFuel;
		cost -= Math.min(FUEL_LEFT_MAX_AWARD, fuelLeft * FUEL_LEFT_COEFFICIENT);

		// better to extinguish the buildings in the changeset
		if (buildingsInSight.get(building.getID()) == null) {
			cost += notInChangeSetCost;
		}

		// just updated
		if (condition.getTimestamp() == context.getTimestamp()
				&& condition.getUpdateSource() == UpdateSource.ChangeSet
				&& condition.getFierynessEnum() == Fieryness.HEATING) {
			cost -= IN_SIGHT_HEATING_BUILDING_AWARD;
		} else if (context.getTimestamp() - condition.getTimestamp() <= 2
				&& condition.getFierynessEnum() == Fieryness.HEATING) {
			cost -= JUST_NOTICED_HEATING_BUILDING_AWARD;
		}

		// need confirm?
		if (fcc.shouldConfirmBuildingCondition(fireBuilding)) {
			cost += CONFIRM_COST;
		}

		return (int) (cost < 0 ? 0 : cost);
	}
}
