package ant.component.fb;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.component.universal.BuildingConditionComponent;

/**
 * 
 * @author Sarah {@link BuildingConditionComponent}<br>
 */
public class RealTimeFireTraceBuildingManagerComponent implements AntComponent {

	private static Log log = LogFactory.getLog(FireTraceComponent.class);

	private StandardWorldModel world;
	private BuildingConditionComponent bcc;

	private Map<EntityID, FireTraceBuilding> tracedBuildings;
	private Map<EntityID, FireTraceBuilding> halfTracedBuildings;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.world = world;
		this.bcc = (BuildingConditionComponent) context.getAntComponentKit().getAntComponent(
				BuildingConditionComponent.class);
		this.tracedBuildings = new HashMap<EntityID, FireTraceBuilding>();
		this.halfTracedBuildings = new HashMap<EntityID, FireTraceBuilding>();

	}

	@Override
	public void update() {
	}

	private FireTraceBuilding makeHalfFireTraceBuilding(Building b) {
		FireTraceBuilding ftb = new FireTraceBuilding(this, world, b, bcc.getBuildingCondition(b
				.getID()));
		halfTracedBuildings.put(b.getID(), ftb);
		return ftb;
	}

	public FireTraceBuilding getHalfTracedBuilding(EntityID id) {
		FireTraceBuilding rv = halfTracedBuildings.get(id);
		if (rv == null) {
			Building b = (Building) world.getEntity(id);
			rv = makeHalfFireTraceBuilding(b);
		}
		return rv;
	}

	private FireTraceBuilding makeFireTraceBuilding(Building b) {
		FireTraceBuilding rv = null;
		if (halfTracedBuildings.containsKey(b.getID())) {
			rv = finishHalfFireTraceBuilding(b);
		} else {
			EntityID id = b.getID();
			rv = new FireTraceBuilding(this, world, b, bcc.getBuildingCondition(id));
			rv.calConnectValue();
			tracedBuildings.put(id, rv);
			for (FireBuilding fb : getTracedBuilding(id).getNearFireBuildings().keySet()) {
				EntityID fbid = fb.getBuilding().getID();
				if (!tracedBuildings.containsKey(fbid) && !halfTracedBuildings.containsKey(fbid)) {
					halfTracedBuildings.put(fbid, (FireTraceBuilding) fb);
				}
			}
		}
		return rv;
	}

	private FireTraceBuilding finishHalfFireTraceBuilding(Building b) {
		EntityID id = b.getID();
		FireTraceBuilding ftb = getHalfTracedBuilding(id);
		ftb.calConnectValue();
		tracedBuildings.put(id, ftb);
		for (FireBuilding fb : ftb.getNearFireBuildings().keySet()) {
			EntityID fbid = fb.getBuilding().getID();
			if (!halfTracedBuildings.containsKey(fbid)) {
				halfTracedBuildings.put(fbid, (FireTraceBuilding) fb);
			}
		}
		return ftb;
	}

	public FireTraceBuilding getTracedBuilding(EntityID id) {
		Building b = (Building) world.getEntity(id);
		FireTraceBuilding rv = tracedBuildings.get(id);
		if (rv == null) {
			rv = makeFireTraceBuilding(b);
		}
		return rv;

	}
}
