package ant.component.pf;

import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotFindCivBuildingEnter;
import ant.component.pf.spot.SpotLevel;
import ant.component.pf.spot.SpotType;
import ant.component.universal.BeliefCivVoiceComponent;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.component.universal.BuildingWithNoCivilianComponent;
import ant.component.universal.LocationMapComponent;
import ant.map.jigsaw.JigsawAttach;

/**
 * 
 * @author Ivan
 * 
 */
public class JigsawInfoStat {
	private static final int LONGTIME = 20;
	private static final double NEVER_UPDATED_FACTOR = 3;
	private static final int MAXLONGTIME = 100;
	private static final double DEEP_EXPLORE_THRESHOLD = 0.35f;
	private static final double CIV_BASE = 2;
	private static final double CIV_FACTOR = 8;
	private static final double VOICE_CIV_PROBE_THRESHOLD = 0.15f;
	private static final boolean USE_CIV_VOICE_BOOST = true;

	private AgentContext context;
	private ObjectiveJigsaw target;
	private BuildingConditionComponent bcc;
	private BuildingWithNoCivilianComponent bwncc;
	private LocationMapComponent lmc;
	private BeliefCivVoiceComponent bcvc;

	private HashSet<EntityID> bset;
	private Set<EntityID> clearSet;
	private int roadCount;
	private int neverUpdatedCount;
	private int longtimeNoUpdatedCount;

	private double score;
	private double totscore;
	private boolean fireclean;
	private boolean deepExplore;

	public JigsawInfoStat(AgentContext context, ObjectiveJigsaw target) {
		this.context = context;
		this.bcc = context.getAntComponentKit().getBuildingConditionComponent();
		this.bwncc = context.getAntComponentKit().getBuildingWithNoCivilianComponent();
		this.lmc = context.getAntComponentKit().getLocationMapComponent();
		this.bcvc = (BeliefCivVoiceComponent) context.getAntComponentKit().getAntComponent(
				BeliefCivVoiceComponent.class);

		this.target = target;
		this.clearSet = new HashSet<EntityID>();
		this.bset = new HashSet<EntityID>();
		for (JigsawAttach ja : target.getJigsawObject().getAttaches()) {
			for (Area a : ja.getObjects()) {
				if (a instanceof Building) {
					bset.add(a.getID());
				}
			}
		}

		for (Area a : target.getJigsawObject().getObjects()) {
			if (a instanceof Road) {
				roadCount++;
			}
		}

		this.neverUpdatedCount = 0;
		this.longtimeNoUpdatedCount = 0;
	}

	public void triggerUpdate() {
		this.neverUpdatedCount = 0;
		this.longtimeNoUpdatedCount = 0;
		this.score = 0;
		int now = context.getTimestamp();

		/****************** UPDATE SCORE ****************/
		for (EntityID bid : bset) {
			BuildingCondition bc = bcc.getBuildingCondition(bid);

			if (now - bc.getTimestamp() < LONGTIME) {
				continue;
			}

			double tsc = 0;
			if (now - bc.getTimestamp() >= MAXLONGTIME)
				tsc = MAXLONGTIME;
			else
				tsc = now - bc.getTimestamp();

			if (bc.hasBeenUpdated()) {
				if (!bc.isEverOnFire()) {
					this.longtimeNoUpdatedCount++;
					score += tsc / LONGTIME;
				}
			} else {
				this.longtimeNoUpdatedCount++;
				this.neverUpdatedCount++;
				score += NEVER_UPDATED_FACTOR * tsc / LONGTIME;
			}
		}

		this.totscore = score;
		double cnt = 0;
		for (AbstractSpot as : target.getLevelSpots(SpotLevel.Low)) {
			if (as.getType() == SpotType.CivInBuilding) {
				cnt++;
			}
		}

		if (USE_CIV_VOICE_BOOST) {
			for (EntityID bid : bset) {
				Building b = (Building) context.getWorld().getEntity(bid);
				double score = bcvc.getProbScoreInBuilding(b);
				cnt += score;
			}
		}

		if (cnt != 0)
			this.totscore += Math.pow(CIV_BASE, cnt) * CIV_FACTOR;

		double norbcnt = 0;
		boolean burn = false;
		for (EntityID bid : bset) {
			BuildingCondition bc = bcc.getBuildingCondition(bid);
			if (bc.hasBeenUpdated()) {
				if (now - bc.getTimestamp() < 15) {
					if (bc.isEverOnFire()) {
						burn = true;
					}
				}
				// if (bc.getFierynessEnum() == Fieryness.HEATING) {
				// burn = true;
				// } else if (bc.getFierynessEnum() == Fieryness.BURNING) {
				// burn = true;
				// } else if (bc.isEverOnFire() && now - bc.getTimestamp() > 15)
				// {
				// burn = true;
				// }
				if (bc.getFierynessEnum() == Fieryness.UNBURNT) {
					norbcnt++;
				}
			} else {
				norbcnt++;
			}
		}

		this.fireclean = false;
		if (burn) {
			if (bset.size() > 5) {
				if (norbcnt / bset.size() < 0.5f) {
					this.fireclean = true;
				}
			}
		}

		this.deepExplore = false;
		cnt = 0;
		for (EntityID bid : bset) {
			BuildingCondition bc = bcc.getBuildingCondition(bid);
			if (bc.isEverOnFire())
				continue;
			if (lmc.getAreaNode(bid).getVisitCount() != 0)
				continue;

			if (!bc.hasBeenUpdated()) {
				cnt++;
			} else {
				if (!bwncc.isBuildingWithNoCivilian(bid))
					cnt++;
			}
		}

		if (1.0 * cnt / bset.size() > DEEP_EXPLORE_THRESHOLD) {
			deepExplore = true;
		}
	}

	public int getLongtimeNoUpdatedCount() {
		return longtimeNoUpdatedCount;
	}

	public int getNeverUpdatedCount() {
		return neverUpdatedCount;
	}

	public int getBuildingCount() {
		return bset.size();
	}

	public final boolean isRoadClean(EntityID id) {
		return clearSet.contains(id);
	}

	public final void setRoadClean(EntityID id) {
		clearSet.add(id);
	}

	public final double getClearRate() {
		if (roadCount == 0) return 1;
		else
			return 1.0f * (clearSet.size()) / (roadCount);
	}

	public int getRoadCount() {
		return roadCount;
	}

	public double getUpdateScore() {
		return this.score;
	}

	public double getTotalScore() {
		return this.totscore;
	}

	public boolean isFireClean() {
		return this.fireclean;
	}

	public boolean shouldDeepExplore() {
		return this.deepExplore;
	}

	public void addDeepExploreSpots() {
		if (shouldDeepExplore()) {
			for (EntityID bid : bset) {
				boolean add = false;
				Building b = (Building) context.getWorld().getEntity(bid);
				BuildingCondition bc = bcc.getBuildingCondition(bid);

				if (bc.isEverOnFire())
					continue;

				if (lmc.getAreaNode(bid).getVisitCount() != 0)
					continue;

				if (!bc.hasBeenUpdated()) {
					add = true;
				} else {
					if (!bwncc.isBuildingWithNoCivilian(bid)) {
						add = true;
					}
				}

				if (add) {
					this.target.addSpot(new SpotFindCivBuildingEnter(context, b));
				}
			}

		}
	}

	public boolean shouldVoiceCivExplore() {
		if (bcvc == null)
			return false;

		for (EntityID bid : bset) {
			Building b = (Building) context.getWorld().getEntity(bid);
			if (b != null) {
				if (bcvc.getProbScoreInBuilding(b) > VOICE_CIV_PROBE_THRESHOLD) {
					SpotFindCivBuildingEnter sfcbe = new SpotFindCivBuildingEnter(context, b);
					sfcbe.triggerUpdate();
					if (!sfcbe.isDone()) {
						return true;
					}
				}
			}
		}

		return false;
	}

	public void addVoiceCivExplore() {
		if (bcvc == null)
			return;

		for (EntityID bid : bset) {
			Building b = (Building) context.getWorld().getEntity(bid);
			if (b != null) {
				if (bcvc.getProbScoreInBuilding(b) >= VOICE_CIV_PROBE_THRESHOLD) {
					// TODO more judge, may be add a already fin spot
					this.target.addSpot(new SpotFindCivBuildingEnter(context, b));
				}
			}
		}
	}
}
