package ant.component.pf;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import ant.AgentContext;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.SpotLevel;
import ant.map.jigsaw.JigsawObject;
import ant.map.jigsaw.JigsawType;
import ant.utils.IPositioning;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class ObjectiveJigsaw implements IPositioning {
	private static final Log log = LogFactory.getLog(ObjectiveJigsaw.class);

	protected HashSet<AbstractSpot> spots;
	private JigsawObject jo;
	private AgentContext context;
	private Discuss discuss;
	private JigsawInfoStat jigsawStat;
	private int objectiveID;

	protected Set<Integer> partset;
	protected Area centerArea = null;

	public ObjectiveJigsaw(AgentContext context, JigsawObject jo, int objectiveID) {
		if (jo == null || jo.getType() != getJigsawType()) {
			throw new IllegalArgumentException();
		}
		this.context = context;
		this.spots = new HashSet<AbstractSpot>();
		this.partset = new HashSet<Integer>();
		this.jo = jo;
		this.discuss = new Discuss();
		this.objectiveID = objectiveID;

		this.jigsawStat = new JigsawInfoStat(context, this);
	}

	public final JigsawObject getJigsawObject() {
		return this.jo;
	}

	public abstract JigsawType getJigsawType();

	public final boolean addSpot(AbstractSpot spot) {
		if (isInvaild(spot)) {
			log.error("Bad Spot for Jigsaw:" + this.toString() + " SPOT:" + spot);
			return false;
		} else {
			spot.triggerUpdate();
			if (!spot.isDone()) {
				spots.add(spot);
			} else {
				// log.warn("Add a already fin spot:" + spot);
			}
			return true;
		}
		// addSpotInternal(spot);
	}

	// protected abstract void addSpotInternal(AbstractSpot spot);

	/** if spot not belong to the jigsaw **/
	protected boolean isInvaild(AbstractSpot spot) {
		// EntityID where = spot.getWhere();
		// if (jo.getType() == JigsawType.Attach) {
		// JigsawAttach ja = (JigsawAttach) jo;
		// if (!ja.contains(where)) {
		// return true;
		// }
		// } else if (jo.getType() == JigsawType.Avenue) {
		// JigsawAvenue ja = (JigsawAvenue) jo;
		// if (!ja.contains(where)) {
		// return true;
		// }
		// } else if (jo.getType() == JigsawType.Cross) {
		// JigsawCross jc = (JigsawCross) jo;
		// if (!jc.contains(where)) {
		// return true;
		// }
		// } else {
		// return true;
		// }
		//		
		return false;
	}

	public final boolean isSpoted() {
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			else
				return true;
		}

		return false;
	}
	
	public final boolean isMidSpoted() {
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			else {
				if (as.getType().getLevel() == SpotLevel.Mid) {
					return true;
				}
			}
		}

		return false;
	}

	public final boolean isHMSpoted() {
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			else {
				if (as.getType().getLevel() == SpotLevel.Mid || as.getType().getLevel() == SpotLevel.High) {
					return true;
				}
			}
		}

		return false;
	}

	
	public final boolean isHighlySpoted() {
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			else {
				if (as.getType().getLevel() == SpotLevel.High) {
					return true;
				}
			}
		}

		return false;
	}

	public final void triggerUpdate() {
		Iterator<AbstractSpot> iter = this.spots.iterator();
		while (iter.hasNext()) {
			AbstractSpot p = iter.next();
			p.triggerUpdate();
			if (p.isDone()) {
				iter.remove();
			}
		}

		if (!isHighlySpoted()) {
			// if (this.spots.isEmpty()) {
			this.discuss.clear();
		}

		jigsawStat.triggerUpdate();
	}

	public final Set<AbstractSpot> getSpots() {
		Set<AbstractSpot> ret = new HashSet<AbstractSpot>();
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;

			ret.add(as);
		}
		return ret;
	}
	
	public final Set<AbstractSpot> getHMSpots() {
		Set<AbstractSpot> ret = new HashSet<AbstractSpot>();
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;

			if (as.getType().getLevel() == SpotLevel.High || as.getType().getLevel() == SpotLevel.Mid)
				ret.add(as);
		}
		return ret;
	}
	
	public final Set<AbstractSpot> getLevelSpots(SpotLevel level) {
		Set<AbstractSpot> ret = new HashSet<AbstractSpot>();
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			
			if (as.getType().getLevel() == level)
				ret.add(as);
		}
		return ret;
	}
	
	public final int getLevelSpotCount(SpotLevel level) {
		int ret = 0;
		for (AbstractSpot as : this.spots) {
			if (as.isDone())
				continue;
			if (as.getType().getLevel() == level)
				ret++;
		}
		return ret;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((jo == null) ? 0 : jo.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ObjectiveJigsaw other = (ObjectiveJigsaw) obj;
		if (jo == null) {
			if (other.jo != null)
				return false;
		} else if (!jo.equals(other.jo))
			return false;
		return true;
	}

	@Override
	public final int getX() {
		if (centerArea == null)
			calcCenterArea();

		return centerArea.getX();
	}

	@Override
	public final int getY() {
		if (centerArea == null)
			calcCenterArea();

		return centerArea.getY();
	}

	private void calcCenterArea() {
		int x = this.jo.getX();
		int y = this.jo.getY();
		Area tar = null;
		double mindist = Double.MAX_VALUE;
		for (Area a : jo.getObjects()) {
			double temp = GeoUtils.manhattanDistance(a, x, y);
			if (temp < mindist) {
				mindist = temp;
				tar = a;
			}
		}
		centerArea = tar;
	}

	@Override
	public String toString() {
		return "Objective-Jigsaw<" + this.jo.getType() + " " + this.jo.getJigsawID() + "> " + "[" + jo.toString() + "]";
	}

	public final Discuss getDiscuss() {
		return discuss;
	}

	public int getObjectiveID() {
		return this.objectiveID;
	}

	public final void addPartId(int id) {
		this.partset.add(id);
	}

	public final boolean belongPartId(int id) {
		return this.partset.contains(id);
	}

	public final JigsawInfoStat getInfoStat() {
		return this.jigsawStat;
	}
}
