package ant.component.pf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.JamState;
import ant.liaison.message.IMessage;
import ant.liaison.message.MJigsawApplication;
import ant.liaison.message.MJigsawDoing;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.LazyMap;
import ant.utils.Pair;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ObjectiveJigsawMetaComponent} {@link StaticCityGraphComponent} {@link JamComponent}
 *         {@link AntMapBasisComponent} {@link AgentLocationComponent}
 */
public class ObjectiveJigsawDecisionComponent extends AbstractMessageProducerComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawDecisionComponent.class);

	private static final int REC_MESSAGE_INGORE = 2;

	/** active count < ~, **/
	private static final int MINIMUM_NONDELAY_COUNT = 5;

	/** already applicants & better than me **/
	private static final int PENDING_COUNT = 2;
	private static final int PENDING_DELAY = 50;
	private static final int NON_PENDING_DELAY = 3;

	private StandardWorldModel world;
	private AgentContext context;

	private ObjectiveJigsawMetaComponent ojmc;
	private StaticCityGraphComponent scg;
	private JamComponent jamc;
	private AgentLocationComponent alc;
	private Set<Human> pfset;

	private Map<EntityID, Integer> executorMap;
	private Map<EntityID, Set<Integer>> applyMap;

	@Override
	public String getComponentName() {
		return ObjectiveJigsawDecisionComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(ObjectiveJigsawMetaComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		this.pfset = context.getAntComponentKit().getAntMapBasisComponent().getPfSet();
		this.jamc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);
		this.alc = context.getAntComponentKit().getAgentLocationComponent();

		this.executorMap = new HashMap<EntityID, Integer>();
		this.applyMap = new LazyMap<EntityID, Set<Integer>>() {
			@Override
			public Set<Integer> createValue() {
				return new HashSet<Integer>();
			}
		};
	}

	@Override
	public void update() {
		if (context.getTimestamp() >= ConfigKey.getIgnoreTime(context.getConfig()) + REC_MESSAGE_INGORE) {
			updateMessages();
			updateBestApp();
			for (ObjectiveJigsaw oj : ojmc.getSpotedJigsaws()) {
				if (!oj.isHighlySpoted()) {
					oj.getDiscuss().clear();
				}
			}
		}
	}

	private void updateBestApp() {
		for (ObjectiveJigsaw oj : ojmc.getSpotedJigsaws()) {
			if (oj.isHighlySpoted()) {
				// I'm Applicant
				if (!oj.getDiscuss().getApplicants().containsKey(context.getAgentID())) {
					continue;
				}

				// is Best ?
				boolean better = true;

				double myest = estimateApplication(context.getAgentID(), oj);
				for (Entry<EntityID, Integer> entry : oj.getDiscuss().getApplicants().entrySet()) {
					if (entry.getKey().equals(context.getAgentID()))
						continue;

					double otherest = estimateApplication(entry.getKey(), oj);
					if (myest > otherest) {
						better = false;
					}
				}

				// Better than others
				if (better) {
					// Oh, good~
					oj.getDiscuss().setBestApplicant(true);
				} else {
					// No...
					oj.getDiscuss().setBestApplicant(false);
				}

			}
		}
	}

	public void discuss(ObjectiveJigsaw oj) {
		if (!oj.isHighlySpoted()) {
			log.error("Discuss a non-highly-spoted item");
			return;
		}

		if (context.getHuman().isBuriednessDefined()) {
			if (context.getHuman().getBuriedness() > 0) {
				log.error("Bad PFState for Discuss");
				return;
			}
		}

		Discuss discuss = oj.getDiscuss();

		if (discuss.isBestApplicant()) {
			log.error("You are already best applicant!");
			return;
		}

		if (discuss.getApplicants().containsKey(context.getAgentID())) {
			// log.info("You are already in applicant-list");
			return;
		}

		// out of discussion
		if (discuss.getDiscussDelayedTimestamp() == -1) {
			int delay = getDiscussDelay(oj);
			discuss.setDiscussDelayedTimestamp(context.getTimestamp() + delay);
		}

		// timeout
		if (discuss.getDiscussDelayedTimestamp() <= context.getTimestamp()) {
			// if i'm(APP) better than existing other applicants?

			double myest = estimateApplication(context.getAgentID(), oj);
			boolean better = true;
			int poorc = 0;
			for (Entry<EntityID, Integer> entry : discuss.getApplicants().entrySet()) {
				if (entry.getKey().equals(context.getAgentID()))
					continue;

				double otherest = estimateApplication(entry.getKey(), oj);
				if (myest > otherest) {
					better = false;
					poorc++;
				}

				if (poorc >= PENDING_COUNT)
					break;
			}

			// Better than others
			if (better) {
				// My Application, I Want to do this :)
				this.addRadioMessage(new MJigsawApplication(oj.getObjectiveID()));
				discuss.getApplicants().put(context.getAgentID(), context.getTimestamp());

				applyMap.get(context.getAgentID()).add(oj.getObjectiveID());

			} else {
				if (poorc >= PENDING_COUNT) {
					discuss.setDiscussDelayedTimestamp(context.getTimestamp() + PENDING_DELAY);
				} else {
					discuss.setDiscussDelayedTimestamp(context.getTimestamp() + NON_PENDING_DELAY);
				}
			}
		}
	}

	private double estimateApplication(EntityID agentID, ObjectiveJigsaw oj) {
		Area center = oj.getJigsawObject().getCenter();
		if (jamc.getJam(agentID).getState() != JamState.Buried) {
			EntityID aloc = this.alc.getAgentLastLocation(agentID);
			return scg.getStaticDistanceToTreeRoot(aloc, center.getID());
		} else {
			return Double.MAX_VALUE;
		}
	}

	private void updateMessages() {
		for (IMessage im : context.getAntCommunicationKit().getRadioReceiveBox()) {
			if (im instanceof MJigsawApplication) {
				MJigsawApplication mja = (MJigsawApplication) im;
				int ojid = mja.getObjectiveJigsawID();
				ObjectiveJigsaw item = ojmc.getObjectiveJigsawByObjectiveID(ojid);
				if (item != null) {
					Discuss d = item.getDiscuss();
					d.putApplicant(mja.getSender(), mja.getTimestamp());
					applyMap.get(mja.getSender()).add(mja.getObjectiveJigsawID());
				} else {
					log.error("Not found objectiveJigsaw for " + ojid);
				}
			} else if (im instanceof MJigsawDoing) {
				MJigsawDoing mjo = (MJigsawDoing) im;
				int ojid = mjo.getObjectiveJigsawID();
				ObjectiveJigsaw item = ojmc.getObjectiveJigsawByObjectiveID(ojid);
				if (item != null) {
					executorMap.put(mjo.getSender(), ojid);
					for (Integer i : applyMap.get(mjo.getSender())) {
						if (i != ojid) {
							// 删除其他申请信息
							ObjectiveJigsaw other = ojmc.getObjectiveJigsawByObjectiveID(i);
							if (other != null) {
								if (other.getDiscuss().getApplicants().containsKey(mjo.getSender())) {
									other.getDiscuss().getApplicants().remove(mjo.getSender());
								} else {
									log.error("Bad state for discuss");
								}
							} else {
								log.error("Not found jisaw : " + i);
							}
						}
					}
					applyMap.clear();
					applyMap.get(mjo.getSender()).add(ojid);
					Discuss d = item.getDiscuss();
					d.putApplicant(mjo.getSender(), mjo.getTimestamp());

				} else {
					log.error("Not found objectiveJigsaw for " + ojid);
				}
			}
		}
	}

	private int getDiscussDelay(ObjectiveJigsaw oj) {
		// Executor, Applicants, Others

		// TODO
		// 1. 如果有执行者, 如果我比执行者远, delay 25c
		// 2. 如果有执行者, 如果我比执行者近, delay 0c

		// 3. 如果没有执行者, 如果我比所有申请人远, delay 25c
		// 4. 如果没有执行者, 如果我处于所有人前30%, delay 0c
		// 5. 如果没有执行者, 如果我处于所有人中30%, delay 2c
		// 5. 如果没有执行者, 如果我处于所有人后30%, delay 4c
		List<Pair<EntityID, Double>> dismap = new ArrayList<Pair<EntityID, Double>>();
		Area center = oj.getJigsawObject().getCenter();

		for (Human h : this.pfset) {
			if (jamc.getJam(h.getID()).getState() != JamState.Buried) {
				Area temp = (Area) world.getEntity(alc.getAgentLastLocation(h.getID()));
				double h2oj = GeoUtils.manhattanDistance(center, temp);
				dismap.add(new Pair<EntityID, Double>(h.getID(), h2oj));
			}
		}

		Collections.sort(dismap, new Comparator<Pair<EntityID, Double>>() {
			@Override
			public int compare(Pair<EntityID, Double> o1, Pair<EntityID, Double> o2) {
				if (o1.second() < o2.second())
					return -1;
				else if (o1.second() > o2.second())
					return 1;
				else {
					if (o1.first().getValue() < o2.first().getValue())
						return -1;
					else if (o1.first().getValue() > o2.first().getValue())
						return 1;
					else
						return 0;
				}
			}
		});

		
		if (dismap.size() <= MINIMUM_NONDELAY_COUNT)
			return 0;

		int delay0Threshold = dismap.size() / 3;
		int delay1Threshold = delay0Threshold + dismap.size() / 3;

		for (int i = 0; i < dismap.size(); i++) {
			Pair<EntityID, Double> pair = dismap.get(i);
			if (pair.first().equals(context.getAgentID())) {
				if (i <= delay0Threshold)
					return 0;
				if (i <= delay1Threshold)
					return 2;
			}
		}

		// Otherwise
		return 4;
	}

	

	//
	// private double estimateMyApplication(ObjectiveJigsaw oj, ObjectiveJigsaw nowtar) {
	// Area center = oj.getJigsawObject().getCenter();
	// EntityID aloc = context.getHuman().getPosition();
	//
	// if (nowtar == null) {
	// return this.scg.getStaticDistanceToTreeRoot(aloc, center.getID());
	// } else {
	// Area tarcen = nowtar.getJigsawObject().getCenter();
	//
	// double d1 = this.scg.getStaticDistanceToTreeRoot(aloc, tarcen.getID());
	// double d2 = this.scg.getStaticDistanceToTreeRoot(tarcen.getID(), center.getID());
	// return d1 + d2;
	// }
	// }
	//
	// private double estimateOtherApplication(EntityID agentID, ObjectiveJigsaw oj) {
	// Area center = oj.getJigsawObject().getCenter();
	// if (jamc.getJam(agentID).getState() != JamState.Buried) {
	// EntityID aloc = this.alc.getAgentLastLocation(agentID);
	// return scg.getStaticDistanceToTreeRoot(aloc, center.getID());
	// } else {
	// return Double.MAX_VALUE;
	// }
	// }
}
