package ant.component.pf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.map.jigsaw.JigsawAttach;
import ant.map.jigsaw.JigsawAvenue;
import ant.map.jigsaw.JigsawCross;
import ant.map.jigsaw.JigsawMainComponent;
import ant.map.jigsaw.JigsawObject;
import ant.map.jigsaw.JigsawType;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link JigsawMainComponent} {@link StaticCityGraphComponent}
 * 
 */
public class ObjectiveJigsawMetaComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawMetaComponent.class);

	private StandardWorldModel world;
	private AgentContext context;

	private JigsawMainComponent jimain;
	private StaticCityGraphComponent scg;

	private int objectiveCount = 0;
	private ArrayList<ObjectiveJigsaw> objectiveJigsawList;
	private Map<EntityID, ObjectiveJigsaw> objectiveReversedTable;
	private Map<Integer, Integer> jigsawIdToObjectiveIdMap;


	@Override
	public String getComponentName() {
		return ObjectiveJigsawMetaComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;

		this.jimain = (JigsawMainComponent) context.getAntComponentKit().getAntComponent(JigsawMainComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		
		this.objectiveJigsawList = new ArrayList<ObjectiveJigsaw>();
		this.objectiveReversedTable = new HashMap<EntityID, ObjectiveJigsaw>();
		this.jigsawIdToObjectiveIdMap = new HashMap<Integer, Integer>();

		ArrayList<JigsawObject> temp = jimain.getJigsawObjects();
		for (int i = 0; i < temp.size(); i++) {
			JigsawObject jo = temp.get(i);
			if (jo.getType() == JigsawType.Avenue || jo.getType() == JigsawType.Cross) {
				if (jo.getType() == JigsawType.Avenue) {
					ObjectiveJigsawAvenue oja = new ObjectiveJigsawAvenue(context, (JigsawAvenue) jo, objectiveCount);
					objectiveJigsawList.add(oja);
					jigsawIdToObjectiveIdMap.put(i, objectiveCount);
					for (Area a : jo.getObjects()) {
						objectiveReversedTable.put(a.getID(), oja);
					}
					objectiveCount++;
				} else if (jo.getType() == JigsawType.Cross) {
					ObjectiveJigsawCross ojc = new ObjectiveJigsawCross(context, (JigsawCross) jo, objectiveCount);
					objectiveJigsawList.add(ojc);
					jigsawIdToObjectiveIdMap.put(i, objectiveCount);
					for (Area a : jo.getObjects()) {
						objectiveReversedTable.put(a.getID(), ojc);
					}
					objectiveCount++;
				}
			}
		}

	}

	@Override
	public void update() {
	}

	public Set<ObjectiveJigsaw> getSpotedJigsaws() {
		Set<ObjectiveJigsaw> ret = new HashSet<ObjectiveJigsaw>();
		for (ObjectiveJigsaw aoj : this.objectiveJigsawList) {
			if (aoj.isSpoted()) {
				ret.add(aoj);
			}
		}
		return ret;
	}

	public Set<ObjectiveJigsaw> getHighlySpotedJigsaws() {
		Set<ObjectiveJigsaw> ret = new HashSet<ObjectiveJigsaw>();
		for (ObjectiveJigsaw aoj : this.objectiveJigsawList) {
			if (aoj.isHighlySpoted()) {
				ret.add(aoj);
			}
		}
		return ret;
	}

	public Set<ObjectiveJigsaw> getMidSpotedJigsaws() {
		Set<ObjectiveJigsaw> ret = new HashSet<ObjectiveJigsaw>();
		for (ObjectiveJigsaw aoj : this.objectiveJigsawList) {
			if (aoj.isMidSpoted()) {
				ret.add(aoj);
			}
		}
		return ret;
	}
	/**
	 * @return null, if not found
	 */
	public ObjectiveJigsaw getAbstractObjectiveJigsaw(EntityID whereId) {
		return this.objectiveReversedTable.get(whereId);
	}

	public ObjectiveJigsaw getObjectiveJigsawByObjectiveID(int objectiveID) {
		return this.objectiveJigsawList.get(objectiveID);
	}

	/**
	 * @param whereId
	 *            , 支持WhereID=BuildingID
	 */
	public ObjectiveJigsaw getExMappingObjectiveJigsaw(EntityID whereId) {
		ObjectiveJigsaw ret = this.objectiveReversedTable.get(whereId);
		if (ret == null) {
			// Attach
			JigsawObject jo = jimain.getJigsaw(whereId);
			if (jo.getType() == JigsawType.Attach) {
				JigsawAttach ja = (JigsawAttach) jo;
				JigsawObject fixed = ja.getFixedEnclosed();
				if (fixed != null) {
					if (fixed.getType() != JigsawType.Attach) {
						return getMappingObjectiveJigsaw(fixed);
					}
				}
				log.error("bad jigsaw for " + whereId + ", " + fixed);
				return null;
			} else {
				log.error("bad jigsaw for " + whereId + "," + jo.getJigsawID());
				return null;
			}
		} else {
			return ret;
		}
	}

	public ObjectiveJigsaw getMappingObjectiveJigsaw(JigsawObject jigsawObject) {
		if (jigsawObject == null) {
			log.error("Null input");
			return null;
		} else {
			if (jigsawObject.getType() == JigsawType.Attach) {
				log.error("Not found in mapping");
				return null;
			} else {
				Integer id = this.jigsawIdToObjectiveIdMap.get(jigsawObject.getJigsawID());
				if (id == null) {
					log.error("Not found in mapping for " + jigsawObject);
					return null;
				} else {
					try {
						return this.objectiveJigsawList.get(id);
					} catch (Exception e) {
						log.error(e);
						return null;
					}
				}
			}
		}
	}

	public int getObjectiveJigsawCount() {
		return objectiveJigsawList.size();
	}

	public ArrayList<ObjectiveJigsaw> getObjectiveJigsaws() {
		return this.objectiveJigsawList;
	}
	
	public List<ObjectiveJigsaw> getSortedHighlySpotedCandicateObjectiveJigsaws() {
		if (context.getHuman().isBuriednessDefined()) {
			if (context.getHuman().getBuriedness() > 0) {
				log.error("Bad PFState for App");
				return Collections.emptyList();
			}
		}

		List<ObjectiveJigsaw> list = new ArrayList<ObjectiveJigsaw>();
		for (ObjectiveJigsaw oj : this.getSpotedJigsaws()) {
			if (oj.isHighlySpoted()) {
				list.add(oj);
			}
		}

		EntityID where = context.getHuman().getPosition();
		Collections.sort(list, new ObjectiveJigsawComparator(scg, where));

		return list;
	}

	private static class ObjectiveJigsawComparator implements Comparator<ObjectiveJigsaw> {

		private EntityID where;
		private StaticCityGraphComponent scg;

		public ObjectiveJigsawComparator(StaticCityGraphComponent scg, EntityID where) {
			this.scg = scg;
			this.where = where;
		}

		@Override
		public int compare(ObjectiveJigsaw o1, ObjectiveJigsaw o2) {
			double d1 = scg.getStaticDistanceToTreeRoot(o1.getJigsawObject().getCenter().getID(), where);
			double d2 = scg.getStaticDistanceToTreeRoot(o2.getJigsawObject().getCenter().getID(), where);

			if (d1 < d2) {
				return -1;
			} else if (d1 > d2) {
				return 1;
			} else {
				if (o1.getObjectiveID() < o2.getObjectiveID()) {
					return -1;
				} else if (o1.getObjectiveID() > o2.getObjectiveID()) {
					return 1;
				} else {
					return 0;
				}
			}
		}

	}
	
}

// public ObjectiveJigsaw getAbstractObjectiveJigsaw(EntityID whereId, JigsawType type) {
// try {
// ObjectiveJigsaw tmp = objectiveJigsawList.get(jimain.getJigsaw(whereId).getId());
// if (tmp.getJigsawType() == type) {
// return tmp;
// } else {
// log.error("Bad Jigsaw Type");
// return null;
// }
// } catch (Exception e) {
// log.error(e);
// return null;
// }
// }

// public ObjectiveJigsaw getObjectiveJigsaw(int jid) {
// return this.objectiveJigsawList.get(jid);
// }

// private void updateDiscussion() {
// // if now > deadtime removed discuss
//
// // Update Acceptions Messages
// for (IMessage im : context.getAntCommunicationKit().getRadioReceiveBox()) {
// if (im instanceof MJigsawApplication) {
// MJigsawApplication mja = (MJigsawApplication) im;
// ObjectiveJigsaw jo = getObjectiveJigsaw(mja.getJigsawID());
// if (jo != null) {
// jo.getDiscuss().getApplicants().put(mja.getSender(), mja.getTimestamp());
// }
// }
// }
//
// if (context.getHuman().isBuriednessDefined()) {
// if (context.getHuman().getBuriedness() > 0) {
// return;
// }
// }
//
// for (ObjectiveJigsaw oj : getAllSpotedJigsaw()) {
// Discuss dis = oj.getDiscuss();
// if (oj.isSpoted()) {
// // not yet discussion
// if (dis.getDiscussDelayedTimestamp() == -1) {
// int delay = getDelay(oj);
// dis.setDiscussDelayedTimestamp(context.getTimestamp() + delay);
// }
//
// if (dis.getDiscussDelayedTimestamp() <= context.getTimestamp()) {
// // if i'm better than other applicants?
// double myest = applyEstimate(context.getAgentID(), oj);
// boolean better = true;
// for (Entry<EntityID, Integer> entry : dis.getApplicants().entrySet()) {
// double otherest = applyEstimate(entry.getKey(), oj);
// if (myest > otherest) {
// better = false;
// break;
// }
// }
// if (better) {
// addRadioMessage(new MJigsawApplication(oj.getJigsawObject().getId()));
// dis.getApplicants().put(context.getAgentID(), context.getTimestamp());
// }
// }
// }
// }
// }

// private AbstractSSSPTree<LabeledArea> getAbstractSSSPTree(EntityID center) {
// if (!center.equals(this.scgorigin)) {
// this.scgorigin = center;
// this.scgtree = this.scg.getAbstractSSSPTree(center);
// }
// return this.scgtree;
// }
//
// private int getDelay(ObjectiveJigsaw oj) {
// Area center = oj.getJigsawObject().getCenter();
// AbstractSSSPTree<LabeledArea> tree = getAbstractSSSPTree(center.getID());
//
// List<Pair<EntityID, Double>> dismap = new ArrayList<Pair<EntityID, Double>>();
// for (Human h : this.pfset) {
// double h2oj = applyEstimate(h.getID(), oj);
// dismap.add(new Pair<EntityID, Double>(h.getID(), h2oj));
// }
//
// Collections.sort(dismap, new Comparator<Pair<EntityID, Double>>() {
// @Override
// public int compare(Pair<EntityID, Double> o1, Pair<EntityID, Double> o2) {
// if (o1.second() < o2.second())
// return -1;
// else if (o1.second() > o2.second())
// return 1;
// else
// return 0;
// }
// });
//
// // IF size <= 5, no delay
// if (dismap.size() <= 5)
// return 0;
// int delay0Threshold = dismap.size() / 3;
// int delay1Threshold = delay0Threshold + dismap.size() / 3;
//
// for (int i = 0; i < dismap.size(); i++) {
// Pair<EntityID, Double> pair = dismap.get(i);
// if (pair.first().equals(context.getAgentID())) {
// if (i <= delay0Threshold)
// return 0;
// if (i <= delay1Threshold)
// return 1;
// }
// }
//
// return 2;
// }
//
// private double applyEstimate(EntityID sender, ObjectiveJigsaw jo) {
// Area center = jo.getJigsawObject().getCenter();
// AbstractSSSPTree<LabeledArea> tree = getAbstractSSSPTree(center.getID());
//
// if (jamc.getJam(sender).getState() != JamState.Buried) {
// EntityID aloc = this.alc.getAgentLastLocation(sender);
// LabeledArea la = this.scg.getLabeledAreaMap().get(aloc);
// // dismap.add(new Pair<EntityID, Double>(h.getID(), h2oj));
// return tree.getDistance(la);
//
// } else {
// return Double.MAX_VALUE;
// }
//
// }

// /** return null, if no applicant **/
// public EntityID getBestApplicant(ObjectiveJigsaw oj) {
// double min = Double.MAX_VALUE;
// EntityID who = null;
// for (Entry<EntityID, Integer> item : oj.getDiscuss().getApplicants().entrySet()) {
// double d = applyEstimate(item.getKey(), oj);
// if (d < min) {
// who = item.getKey();
// min = d;
// }
// }
// return who;
// }

// /**
// * @return null, if no acc or no spoted
// */
// public ObjectiveJigsaw pickAppropriateObjectiveJigsaw() {
// for (ObjectiveJigsaw oj : objectiveJigsawList) {
// if (oj.isSpoted()) {
// if (oj.isAccMyself()) {
// return oj;
// }
// }
// }
// return null;
// }

// // Just distance to center ?
// public double getDistanceToObjectiveJigsaw(ObjectiveJigsaw aoj, Area from) {
// if (aoj == null)
// throw new IllegalArgumentException();
//
// return GeoUtils.manhattanDistance(aoj, from);
// }