package ant.component.pf;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.pf.spot.AbstractSpot;
import ant.component.pf.spot.AbstractSpotJam;
import ant.component.pf.spot.SpotAgentSpawn;
import ant.component.pf.spot.SpotBuildingExploration;
import ant.component.pf.spot.SpotBuildingObserving;
import ant.component.pf.spot.SpotCivInBuilding;
import ant.component.pf.spot.SpotHighlyJigsawArrive;
import ant.component.pf.spot.SpotJamAgentBuried;
import ant.component.pf.spot.SpotJamStuckInBuilding;
import ant.component.pf.spot.SpotJamStuckInRoad;
import ant.component.pf.spot.SpotLevel;
import ant.component.pf.spot.SpotRefuge;
import ant.component.pf.spot.SpotSuperiorExploration;
import ant.component.universal.BeliefClearComponent;
import ant.component.universal.BeliefClearListener;
import ant.component.universal.BeliefTrailComponent;
import ant.component.universal.BeliefTrailComponent.BeliefTrail;
import ant.component.universal.HumanInfoComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.Jam;
import ant.component.universal.JamComponent.JamState;
import ant.liaison.channel.assign.ChannelAssignResult.BandwidthLevel;
import ant.liaison.message.IMessage;
import ant.liaison.message.MRescueUnreachable;
import ant.liaison.message.MTargeBuildingUnreachable;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.map.jigsaw.JigsawType;
import ant.map.objects.LabeledArea;
import ant.map.objects.LabeledGraph;
import ant.map.objects.Route;
import ant.map.objects.StandardGraphEdge;
import ant.map.search.StandardLabeledSPFASearch;
import ant.utils.civilian.CivilianInfo;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ObjectiveJigsawMetaComponent} {@link AntMapBasisComponent}
 *         {@link JamComponent} {@link ObjectiveMiscComponent}
 *         {@link BeliefClearComponent} {@link BeliefTrailComponent}
 *         {@link StaticCityGraphComponent}
 */
public class ObjectiveJigsawSystemComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveJigsawSystemComponent.class);
	private static final int TRACE_LEN = 15;
	private static final double TARGET_CLEAR_PENALTY = 0.40;
	private static final int MIDUP_SPOT_COUNT = 3;
	private static final int SPAWN_LIMITS = 4;
	private static final int SUPERIOR_CHECKTIME = 25;
	private static final int SUPERIOR_STARTTIME = 50;
	
	private StandardWorldModel world;
	private AgentContext context;

	private BeliefClearComponent bcc;
	private BeliefTrailComponent btc;
	private ObjectiveJigsawMetaComponent ojmc;
	private ObjectiveMiscComponent omc;
	private JamComponent jamc;
	private HumanInfoComponent hic;

	private Map<EntityID, AbstractSpotJam> jamSpotMap;
	private Set<Human> agentset;
	private Set<Human> pfset;
	private StaticCityGraphComponent scg;
	private LabeledGraph<LabeledArea, StandardGraphEdge<LabeledArea>> graph;
	private StandardLabeledSPFASearch algorithm;
	private HashSet<EntityID> unclearRoadSet;

	@Override
	public String getComponentName() {
		return ObjectiveJigsawSystemComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;

		this.btc = (BeliefTrailComponent) context.getAntComponentKit().getAntComponent(
				BeliefTrailComponent.class);
		this.bcc = context.getAntComponentKit().getBeliefClearComponent();
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.omc = (ObjectiveMiscComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveMiscComponent.class);
		this.jamc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);
		this.hic = context.getAntComponentKit().getHumanInfoComponent();
		this.scg = this.context.getAntComponentKit().getStaticCityGraphComponent();

		this.agentset = context.getAntComponentKit().getAntMapBasisComponent().getAgentSet();
		this.pfset = context.getAntComponentKit().getAntMapBasisComponent().getPfSet();
		this.jamSpotMap = new HashMap<EntityID, AbstractSpotJam>();

		if (bcc != null) {
			bcc.addBeleifClearListener(new BeliefClearListener() {
				@Override
				public void beliefClear(Road r) {
					processRoad(r);
				}
			});
		}

		this.graph = this.scg.createGraph();
		this.algorithm = new StandardLabeledSPFASearch(this.graph);
		this.bcc.addBeleifClearListener(new BeliefClearListener() {
			@Override
			public void beliefClear(Road r) {
				graphvisit(r);
			}
		});

		this.unclearRoadSet = new HashSet<EntityID>();
		for (Road r : context.getAntComponentKit().getAntMapBasisComponent().getRoadSet()) {
			unclearRoadSet.add(r.getID());
		}

		updateRefugeSpot();
		
	}

	@Override
	public void update() {
		for (ObjectiveJigsaw aoj : ojmc.getObjectiveJigsaws()) {
			aoj.triggerUpdate();
		}

		// Remove all completed spot from map
		Iterator<Entry<EntityID, AbstractSpotJam>> iter = jamSpotMap.entrySet().iterator();
		while (iter.hasNext()) {
			if (iter.next().getValue().isDone()) {
				iter.remove();
			}
		}

		try {
			if (context.getTimestamp() == ConfigKey.getIgnoreTime(context.getConfig()) + 2) {
				updateSpawnSpot();
			}
			updateSuperior();
			updateJamSpot();
			updateObservingAndExplorationSpot();
			updateAvenueClean();
			updateMidUp();
		} catch (Exception e) {
			log.error(e);
		}
	}

	private void updateMidUp() {
		for (ObjectiveJigsaw oj : ojmc.getObjectiveJigsaws()) {
			if (!oj.isHighlySpoted()) {
				if (oj.getLevelSpotCount(SpotLevel.Mid) >= MIDUP_SPOT_COUNT) {
					oj.addSpot(new SpotHighlyJigsawArrive(context, oj));
				}
			}
		}
	}

	private void updateObservingAndExplorationSpot() {
		for (IMessage im : context.getAntCommunicationKit().getRadioReceiveBox()) {
			AbstractSpot ajs = null;
			if (im instanceof MTargeBuildingUnreachable) {
				Building b = (Building) world.getEntity(((MTargeBuildingUnreachable) im)
						.getTarget());
				if (!omc.getActivePoliceForceVisitedBuildingSet().contains(b.getID()))
					ajs = new SpotBuildingObserving(context, b, im.getTimestamp());

			} else if (im instanceof MRescueUnreachable) {
				Building b = (Building) world.getEntity(((MRescueUnreachable) im).getLoaction());
				if (!omc.getActivePoliceForceVisitedBuildingSet().contains(b.getID())) {
					ajs = new SpotBuildingExploration(context, b);
				}
			}

			if (ajs != null) {
				ObjectiveJigsaw aoj = ojmc.getExMappingObjectiveJigsaw(ajs.getWhere());

				if (aoj != null) {
					aoj.addSpot(ajs);
				} else {
					log.error("Bad objective Jigsaw:" + aoj + " Spot:" + ajs);
				}
			}
		}

		for (CivilianInfo item : hic.getCivIncNow()) {
			StandardEntity se = world.getEntity(item.getPosition());
			if (se instanceof Building) {
				Building b = (Building) se;
				AbstractSpot ajs = null;
				if (!omc.getActivePoliceForceVisitedBuildingSet().contains(b.getID())) {
					ajs = new SpotCivInBuilding(context, b, item.getID());
				}
				if (ajs != null) {
					ObjectiveJigsaw aoj = ojmc.getExMappingObjectiveJigsaw(ajs.getWhere());

					if (aoj != null) {
						aoj.addSpot(ajs);
					} else {
						log.error("Bad objective Jigsaw:" + aoj + " Spot:" + ajs);
					}
				}
			}
		}

	}

	private void updateJamSpot() {
		// Add new JamSpot
		for (Human h : agentset) {
			if (jamSpotMap.containsKey(h.getID())) {
				// TODO Check inconsistent
				AbstractSpotJam asj = jamSpotMap.get(h.getID());
				Jam jam = jamc.getJam(h.getID());
				if (asj.isDone()) {
					log.error("Done ASJ:" + asj + " but in jsm");
				} else {
					if (asj.getWhere() == null) {
						log.error("Null where : " + asj);
					}
				}
			} else {
				Jam jam = jamc.getJam(h.getID());

				AbstractSpotJam asj = null;
				if (jam.getState() == JamState.Buried || jam.getState() == JamState.StuckInBuilding
						|| jam.getState() == JamState.StuckInRoad) {
					try {
						if (jam.getState() == JamState.Buried) {
							// Never add visited building
							if (!omc.getActivePoliceForceVisitedBuildingSet().contains(
									jam.getBuriedBuilding())) {
								asj = new SpotJamAgentBuried(context, h.getID());
							}
						} else if (jam.getState() == JamState.StuckInBuilding) {
							asj = new SpotJamStuckInBuilding(context, h.getID());
						} else if (jam.getState() == JamState.StuckInRoad) {
							asj = new SpotJamStuckInRoad(context, h.getID());
						}
					} catch (Exception e) {
						e.printStackTrace();
						log.error(e);
					}
				}

				if (asj != null) {
					// ObjectiveJigsaw aoj =
					// getAbstractObjectiveJigsaw(ajs.getWhere());
					ObjectiveJigsaw aoj = ojmc.getExMappingObjectiveJigsaw(asj.getWhere());

					if (aoj != null) {
						aoj.addSpot(asj);
						jamSpotMap.put(h.getID(), asj);
					} else {
						log.error("Bad objective Jigsaw:" + aoj + " Spot:" + asj);
					}
				}
			}
		}
	}

	private void updateRefugeSpot() {
		for (Refuge ref : context.getAntComponentKit().getAntMapBasisComponent().getRefugeSet()) {
			ObjectiveJigsaw item = ojmc.getExMappingObjectiveJigsaw(ref.getID());
			if (item != null) {
				item.addSpot(new SpotRefuge(context, ref));
				// log.info("Add Refuge Spot to:" + item);
			}
		}
	}
	
	private void updateSuperior() {
		BandwidthLevel bwl = context.getAntCommunicationKit().getBandwidthLevel();
		if (!(bwl == BandwidthLevel.None || bwl == BandwidthLevel.Poor
				|| bwl == BandwidthLevel.Unknown)) {
			
			if (context.getTimestamp() >= SUPERIOR_STARTTIME && context.getTimestamp() % SUPERIOR_CHECKTIME == 0) {
				double maxscore = 0;
				ObjectiveJigsaw pick = null;
				for (ObjectiveJigsaw oj : ojmc.getObjectiveJigsaws()) {
					if (oj.getInfoStat().getUpdateScore() > maxscore) {
						maxscore = oj.getInfoStat().getUpdateScore();
						pick = oj;
					}
				}
				
				if (pick != null) {
					// log.info("Add SuperExploration at: " + pick);
					// System.err.println("Add SuperExploration at: " + pick + "!!!!");
					pick.addSpot(new SpotSuperiorExploration(context, pick));
				}
			}
		}
	}
	
	private void updateSpawnSpot() {
		BandwidthLevel bwl = context.getAntCommunicationKit().getBandwidthLevel();
		if (bwl == BandwidthLevel.None || bwl == BandwidthLevel.Poor
				|| bwl == BandwidthLevel.Unknown) {
			for (Human h : this.agentset) {
				if (!h.isPositionDefined())
					continue;
				EntityID where = h.getPosition();
				if (h instanceof PoliceForce) {
					if ((world.getEntity(where) instanceof Road)) {
						continue;
					}
				}

				if (isNaiveNearestN(where, SPAWN_LIMITS)) {
					ObjectiveJigsaw item = ojmc.getExMappingObjectiveJigsaw(where);
					if (item != null) {
						item.addSpot(new SpotAgentSpawn(context, h.getID(), where));
					}
				}
			}
		}
	}

	private boolean isNaiveNearestN(EntityID where, int n) {
		int cnt = 0;
		Area areaw = (Area) world.getEntity(where);
		double mydist = GeoUtils.manhattanDistance(areaw, (Area) context.getHuman().getPosition(
				world));
		for (Human h : pfset) {
			double otherdist = GeoUtils.manhattanDistance(areaw, (Area) h.getPosition(world));
			if (otherdist < mydist) {
				cnt++;
			}
		}

		if (cnt < n)
			return true;
		else
			return false;

	}

	private void updateAvenueClean() {
		try {
			if (btc == null)
				return;

			for (Human h : agentset) {
				List<BeliefTrail> list = this.btc.getTrails(h.getID());
				if (!list.isEmpty()) {
					// Last one
					BeliefTrail trail = list.get(list.size() - 1);
					if (trail.isInvalid()) {
						if (list.size() >= 2) {
							trail = list.get(list.size() - 2);
						}
					}
					if (!trail.isInvalid()) {
						List<EntityID> traillist = trail.getTrail();
						int to = traillist.size(); // Not included
						int from = to - TRACE_LEN;
						if (from < 0)
							from = 0;
						List<EntityID> tmp = traillist.subList(from, to);
						processSubTrail(tmp);
						// process(h.getID(), trail.getTrail());
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e);
		}
	}

	// 结合Clear命令, 可以使start, end两点进一步更新
	private void processSubTrail(List<EntityID> list) {
		int size = list.size();
		if (list.size() < 2)
			return;
		for (int i = 1; i < size - 1; i++) {
			EntityID loc0 = list.get(i - 1);
			EntityID loc1 = list.get(i);
			EntityID loc2 = list.get(i + 1);
			if (loc0.equals(loc1) || loc1.equals(loc2) || loc0.equals(loc2)) {
				if (loc0.equals(loc1) || loc1.equals(loc2)) {
					log.error("Dereplication Error");
				}
			} else {
				Area a0 = (Area) world.getEntity(loc0);
				Area a1 = (Area) world.getEntity(loc1);
				Area a2 = (Area) world.getEntity(loc2);

				boolean isb0 = (a0 instanceof Building);
				boolean isb1 = (a1 instanceof Building);
				boolean isb2 = (a2 instanceof Building);

				if (isb1)
					return;

				try {
					Road target = (Road) a1;
					processRoad(target);
				} catch (Exception e) {
					log.error(e);
				}
			}
		}
	}

	private void processRoad(Road target) {
		graphvisit(target);
		ObjectiveJigsaw oj = ojmc.getAbstractObjectiveJigsaw(target.getID());
		if (oj != null) {
			if (oj.getJigsawType() == JigsawType.Avenue) {
				((ObjectiveJigsawAvenue) oj).getInfoStat().setRoadClean(target.getID());
			}
		}
	}

	private void graphvisit(Road se) {
		if (unclearRoadSet.contains(se.getID())) {
			unclearRoadSet.remove(se.getID());
			LabeledArea to = this.scg.getLabeledAreaMap().get(se.getID());
			for (StandardGraphEdge<LabeledArea> edge : this.graph.edgesOf(to)) {
				edge.setWeight(edge.getWeight() * TARGET_CLEAR_PENALTY);
			}
		}
	}
	
	public boolean isUnclear(EntityID id) {
		return unclearRoadSet.contains(id);
	}

	public List<EntityID> getFastestMoveRoute(EntityID from, EntityID to) {
		LabeledArea labelFrom = this.scg.getLabeledAreaMap().get(from);
		LabeledArea labelTo = this.scg.getLabeledAreaMap().get(to);
		Route<LabeledArea> data = this.algorithm.spSearch(labelFrom, labelTo);
		if (data == null) {
			return null;
		} else {
			List<EntityID> r = new ArrayList<EntityID>();
			if (data.getRoute().size() > 1) {
				for (int i = 1; i < data.getRoute().size(); i++) {
					r.add(data.getRoute().get(i).getID());
				}
			} else {
				for (int i = 0; i < data.getRoute().size(); i++) {
					r.add(data.getRoute().get(i).getID());
				}
			}
			return r;
		}
	}
}
