package ant.component.pf;

import java.util.HashSet;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.agent.command.AntClear;
import ant.agent.command.AntCommand;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.JamState;
import ant.map.component.AntMapBasisComponent;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link JamComponent} {@link AntMapBasisComponent} {@link AgentLocationComponent}
 * 
 */
public class ObjectiveMiscComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveMiscComponent.class);
	private static final double TARGET_CLEAR_PENALTY = 0.5;

	private AgentContext context;
	private StandardWorldModel world;
	
	private AgentLocationComponent alc;
	private JamComponent jc;
	private AntMapBasisComponent ambc;
	// private BuildingConditionComponent bcc;
	// private AntMapBasisComponent amb;
	// private PartitionAreaComponent pc;

	private Set<EntityID> activeVisitedBuildingSet;
	private Set<Human> pfset;
	private int clearCommandCount = 0;
	private int bnsc = 0;
	private ObjectiveJigsaw doing;

	
	@Override
	public String getComponentName() {
		return ObjectiveMiscComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.jc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);
		this.ambc = context.getAntComponentKit().getAntMapBasisComponent();
		// this.bcc = context.getAntComponentKit().getBuildingConditionComponent();
		// this.amb = context.getAntComponentKit().getAntMapBasisComponent();
		// this.pc = context.getAntComponentKit().getPartitionAreaComponent();
		this.pfset = ambc.getPfSet();
		this.activeVisitedBuildingSet = new HashSet<EntityID>();
		
	}

	@Override
	public void update() {
		// Update Already visited building
		updateVisitedBuildings();
		updateCommandStat();
	}
	
//	private void updateSuperiorExploration() {
//		if (context.getTimestamp() < ConfigKey.getIgnoreTime(context.getConfig()))
//			return;
//
//		int last = bnsc;
//		if (context.getTimestamp() % 1 == 0) {
//			bnsc = 0;
//			for (Building b : ambc.getBuildingSet()) {
//				if (!bcc.getBuildingCondition(b.getID()).hasBeenUpdated()) {
//					bnsc++;
//				}
//			}
//		}
//
//		// log.info("NSBC:" + bnsc + " Total:" + ambc.getBuildingCount() + "dsc:" + (last - bnsc) + " % " + (1.0 * bnsc)
//		// / ambc.getBuildingCount());
//	}

	private void updateCommandStat() {
		for (Entry<Integer, AntCommand> entry : context.getCommands().entrySet()) {
			AntCommand cc;
			cc = entry.getValue();
			if (cc instanceof AntClear)
				clearCommandCount++;
		}
	}

	private void updateVisitedBuildings() {
		for (Human h : pfset) {
			if (jc.getJam(h.getID()).getState() == JamState.Active) {
				EntityID loc = alc.getAgentLastLocation(h.getID());
				if (world.getEntity(loc) instanceof Building) {
					activeVisitedBuildingSet.add(loc);
				}
			}
		}
	}

	public Set<EntityID> getActivePoliceForceVisitedBuildingSet() {
		return this.activeVisitedBuildingSet;
	}
	
	public void setDoing(ObjectiveJigsaw oj) {
		this.doing = oj;
	}

	public ObjectiveJigsaw getDoing() {
		return doing;
	}
}
