package ant.component.pf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.jigsaw.JigsawMarkerComponent;
import ant.map.jigsaw.JigsawMarkerComponent.RoadMark;
import ant.map.partition.Partition;
import ant.map.partition.PartitionAreaComponent;
import ant.utils.CounterSet;
import ant.utils.DualisticFunction;
import ant.utils.IdempotentMatch;
import ant.utils.RLMatch;
import ant.utils.comparator.EntityIDComparator;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * 
 * @depend {@link AntMapBasisComponent} {@link PartitionAreaComponent}
 *         {@link ObjectiveJigsawMetaComponent}
 */
public class ObjectiveRaceComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveRaceComponent.class);

	private StandardWorldModel world;
	private AgentContext context;
	private PartitionAreaComponent pc;
	private AntMapBasisComponent amb;
	private ObjectiveJigsawMetaComponent ojmc;
	private JigsawMarkerComponent ojmk;

	private int workerCount;
	private Race myrace;
	private Partition mypart;

	private HashMap<EntityID, Race> racemap;
	private HashMap<EntityID, Integer> workerPartitionAssignment;
	private HashSet<EntityID> spriteSet;
	
	@Override
	public String getComponentName() {
		return ObjectiveRaceComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.pc = context.getAntComponentKit().getPartitionAreaComponent();
		this.amb = context.getAntComponentKit().getAntMapBasisComponent();
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.ojmk = (JigsawMarkerComponent) context.getAntComponentKit().getAntComponent(
				JigsawMarkerComponent.class);
		initRace();
		initJigsawPart();
	}

	@Override
	public void update() {
	}

	private void initJigsawPart() {
		for (ObjectiveJigsaw oj : ojmc.getObjectiveJigsaws()) {
			for (Area a : oj.getJigsawObject().getObjects()) {
				int pi = pc.findPartitionIndex(a.getID());
				oj.addPartId(pi);
			}
		}
	}

	private void initRace() {
		this.spriteSet = new HashSet<EntityID>();

		List<EntityID> pflist = new ArrayList<EntityID>();
		List<EntityID> buildingpflist = new ArrayList<EntityID>();
		List<EntityID> roadtogpflist = new ArrayList<EntityID>();
		List<EntityID> smallroadonlypflist = new ArrayList<EntityID>();
		List<EntityID> bigroadonlypflist = new ArrayList<EntityID>();

		CounterSet<EntityID> loccount = new CounterSet<EntityID>();
		for (Human atid : amb.getAgentSet()) {
			loccount.inc(atid.getPosition());
		}

		for (Human pf : amb.getPfSet()) {
			Area a = (Area) pf.getPosition(world);
			if (a instanceof Building) {
				buildingpflist.add(pf.getID());
			} else {
				if (loccount.count(a.getID()) > 1) {
					roadtogpflist.add(pf.getID());
				} else {
					RoadMark mark = ojmk.getMark(a.getID());
					if (mark == RoadMark.Entrance)
						smallroadonlypflist.add(pf.getID());
					else
						bigroadonlypflist.add(pf.getID());
				}
			}
		}
		Collections.sort(buildingpflist, new EntityIDComparator());
		Collections.sort(roadtogpflist, new EntityIDComparator());
		Collections.sort(smallroadonlypflist, new EntityIDComparator());
		Collections.sort(bigroadonlypflist, new EntityIDComparator());

		for (EntityID id : bigroadonlypflist)
			pflist.add(id);

		for (EntityID id : smallroadonlypflist)
			pflist.add(id);

		for (EntityID id : roadtogpflist)
			pflist.add(id);

		for (EntityID id : buildingpflist)
			pflist.add(id);

		this.workerCount = pc.getPartitionCount();
		log.info("WorkerCount: " + workerCount + " InOnlyBigRoad: " + bigroadonlypflist.size()
				+ " InOnlySmallRoad: " + smallroadonlypflist.size() + " InBuilding:"
				+ buildingpflist.size() + " InRoad: " + roadtogpflist.size());

		this.racemap = new HashMap<EntityID, Race>();
		this.workerPartitionAssignment = new HashMap<EntityID, Integer>();
		ArrayList<EntityID> wlist = new ArrayList<EntityID>();
		ArrayList<Partition> plist = new ArrayList<Partition>();

		if (bigroadonlypflist.size() < workerCount) {
			log.info("IM Match!!");
			for (int i = 0; i < pflist.size(); i++) {
				if (i < workerCount) {
					racemap.put(pflist.get(i), Race.Worker);
					wlist.add(pflist.get(i));
					plist.add(pc.getPartitions().get(i));
				} else {
					racemap.put(pflist.get(i), Race.Sprite);
					spriteSet.add(pflist.get(i));
				}
			}

			// log.info("AccWorkerCount: " + workerPartitionAssignment.size());

			

			IdempotentMatch<EntityID, Partition> im = new IdempotentMatch<EntityID, Partition>(
					wlist, plist, new DualisticFunction<EntityID, Partition>() {
						@Override
						public double invoke(EntityID t1, Partition t2) {
							Area a = (Area) ((Human) world.getEntity(t1)).getPosition(world);
							Area c = t2.getCenter();
							if (c != null)
								return GeoUtils.manhattanDistance(a, c);
							else
								return GeoUtils.manhattanDistance(a, 0, 0);
						}
					});

			im.minMatch();
			for (EntityID h : wlist) {
				workerPartitionAssignment.put(h, im.getAmatch(h).getID());
			}
		} else {
			log.info("RL Match!!");
			// BRPF >= WC
			for (int i = 0; i < bigroadonlypflist.size(); i++) {
				wlist.add(pflist.get(i));
			}
			for (int i = 0; i < workerCount; i++) {
				plist.add(pc.getPartitions().get(i));
			}
			RLMatch<EntityID, Partition> rlm = new RLMatch<EntityID, Partition>(wlist, plist,
					new DualisticFunction<EntityID, Partition>() {
						@Override
						public double invoke(EntityID t1, Partition t2) {
							Area a = (Area) ((Human) world.getEntity(t1)).getPosition(world);
							Area c = t2.getCenter();
							if (c != null)
								return GeoUtils.manhattanDistance(a, c);
							else
								return GeoUtils.manhattanDistance(a, 0, 0);
						}
					});
			rlm.minMatch();
			for (int i = 0; i < pflist.size(); i++) {
				racemap.put(pflist.get(i), Race.Sprite);
			}
			for (EntityID h : wlist) {
				if (rlm.getAmatch(h) != null) {
					racemap.put(h, Race.Worker);
					workerPartitionAssignment.put(h, rlm.getAmatch(h).getID());
				}
			}
		}

		myrace = racemap.get(context.getAgentID());
		if (myrace == Race.Worker) {
			this.mypart = pc.getPartitions().get(
					workerPartitionAssignment.get(context.getAgentID()));
		}
	}

	public Race getMyRace() {
		return myrace;
	}

	public Partition getMyPartition() {
		return mypart;
	}

	public HashSet<EntityID> getSpriteSet() {
		return spriteSet;
	}
}
