package ant.component.pf.adv;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveRaceComponent;
import ant.component.universal.BeliefClearComponent;
import ant.component.universal.BeliefClearListener;
import ant.map.component.StaticCityGraphComponent;
import ant.map.objects.LabeledArea;
import ant.map.objects.LabeledGraph;
import ant.map.objects.Route;
import ant.map.objects.StandardGraphEdge;
import ant.map.search.StandardLabeledSPFASearch;
import ant.utils.RandomSelector;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 *
 */
public class ObjectiveAdvProbeSpriteComponent implements AntComponent {

	// private static final int IGNORE_TIMESTAMP = 3;
	private static final int TARGET_CLEAR_PENALTY = 4;
	private static final int MAXSCORE_RANGE = 200000;
	private static final int MAXSCORE_TOPN = 3;

	private AgentContext context;
	private StandardWorldModel world;

	private ObjectiveRaceComponent opc;
	private StaticCityGraphComponent scg;
	private BeliefClearComponent bcc;
	private ObjectiveJigsawMetaComponent ojmc;

	private LabeledGraph<LabeledArea, StandardGraphEdge<LabeledArea>> graph;
	private StandardLabeledSPFASearch algorithm;
	private Set<EntityID> unclearRoadSet;
	private Random rand;

	@Override
	public String getComponentName() {
		return ObjectiveAdvProbeSpriteComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.opc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);

		this.bcc = context.getAntComponentKit().getBeliefClearComponent();
		this.scg = this.context.getAntComponentKit().getStaticCityGraphComponent();
		this.graph = this.scg.createGraph();
		this.algorithm = new StandardLabeledSPFASearch(this.graph);
		this.bcc.addBeleifClearListener(new BeliefClearListener() {
			@Override
			public void beliefClear(Road r) {
				graphvisit(r);
			}
		});
		unclearRoadSet = new HashSet<EntityID>();
		for (Road r : context.getAntComponentKit().getAntMapBasisComponent().getRoadSet()) {
			unclearRoadSet.add(r.getID());
		}
		this.rand = new Random(context.getAgentID().getValue() + 1);
	}

	private void graphvisit(Road se) {
		if (unclearRoadSet.contains(se.getID())) {
			unclearRoadSet.remove(se.getID());
			LabeledArea to = this.scg.getLabeledAreaMap().get(se.getID());
			for (StandardGraphEdge<LabeledArea> edge : this.graph.edgesOf(to)) {
				edge.setWeight(edge.getWeight() * TARGET_CLEAR_PENALTY);
			}
		}
	}

	@Override
	public void update() {
	}

	public List<EntityID> getMidMoveRoute(EntityID from, EntityID to) {
		LabeledArea labelFrom = this.scg.getLabeledAreaMap().get(from);
		LabeledArea labelTo = this.scg.getLabeledAreaMap().get(to);
		Route<LabeledArea> data = this.algorithm.spSearch(labelFrom, labelTo);
		if (data == null) {
			return null;
		} else {
			List<EntityID> r = new ArrayList<EntityID>();
			if (data.getRoute().size() > 1) {
				for (int i = 1; i < data.getRoute().size(); i++) {
					r.add(data.getRoute().get(i).getID());
				}
			} else {
				for (int i = 0; i < data.getRoute().size(); i++) {
					r.add(data.getRoute().get(i).getID());
				}
			}
			return r;
		}
	}

	public ObjectiveJigsaw getTask() {
		// int x = context.getHuman().getX();
		// int y = context.getHuman().getY();
		//		
		// ObjectiveJigsaw pick = null;
		// double mindist = Double.MAX_VALUE;
		Set<ObjectiveJigsaw> cands = new HashSet<ObjectiveJigsaw>();
		for (ObjectiveJigsaw oj : ojmc.getObjectiveJigsaws()) {
			if (oj.isSpoted() || oj.getInfoStat().shouldDeepExplore())
				cands.add(oj);
		}

		cands.addAll(getInRangeTopNScoreObjectiveJigsaw());
		
		if (cands.isEmpty())
			return null;

		//		
		// double temp = GeoUtils.manhattanDistance(x, y, oj.getX(), oj.getY());
		// if (temp < mindist) {
		// mindist = temp;
		// pick = oj;
		// }
		// }
		//		
		// if (pick != null) {
		// return pick;
		// }
		//		
		HashSet<ObjectiveJigsaw> alist = new HashSet<ObjectiveJigsaw>(ojmc.getObjectiveJigsaws());
		return multiscaleSelect(alist);
	}

	private Collection<ObjectiveJigsaw> getInRangeTopNScoreObjectiveJigsaw() {
		List<ObjectiveJigsaw> list = new ArrayList<ObjectiveJigsaw>();

		int hx = context.getHuman().getX();
		int hy = context.getHuman().getY();
		for (ObjectiveJigsaw oj : ojmc.getObjectiveJigsaws()) {
			double dist = GeoUtils.manhattanDistance(hx, hy, oj.getX(), oj.getY());
			if (dist <= MAXSCORE_RANGE) {
				list.add(oj);
			}
		}

		Collections.sort(list, new Comparator<ObjectiveJigsaw>() {
			@Override
			public int compare(ObjectiveJigsaw a0, ObjectiveJigsaw a1) {
				double d = a0.getInfoStat().getTotalScore() - a1.getInfoStat().getTotalScore();
				if (d < 0)
					return -1;
				else if (d > 0)
					return 1;
				else
					return 0;
			}
		});
		
		List<ObjectiveJigsaw> ret = new ArrayList<ObjectiveJigsaw>();
		
		for (int i = list.size() - 1; i >= 0; i--) {
			if (ret.size() < MAXSCORE_TOPN) {
				ret.add(list.get(i));
			} else {
				break;
			}
		}
		
		return ret;
	}

	/**
	 * near -> far
	 */
	private ObjectiveJigsaw multiscaleSelect(Set<ObjectiveJigsaw> cands) {
		if (cands.isEmpty())
			return null;
		int x = this.context.getHuman().getX();
		int y = this.context.getHuman().getY();
		int range = 200000;
		Set<ObjectiveJigsaw> candidate = new HashSet<ObjectiveJigsaw>();
		while (true) {
			candidate.clear();
			range *= 2;
			for (ObjectiveJigsaw a : cands) {
				if (GeoUtils.manhattanDistance(x, y, a.getX(), a.getY()) < range) {
					candidate.add(a);
				}
			}
			if (!candidate.isEmpty()) {
				ObjectiveJigsaw pick = RandomSelector.randomSelect(candidate, 1, rand).iterator()
						.next();
				return pick;
			}
		}
	}

}
