package ant.component.pf.adv;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.agent.behavior.AntBehavior;
import ant.component.pf.ObjectiveJigsaw;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.component.pf.ObjectiveJigsawSystemComponent;
import ant.component.pf.ObjectiveRaceComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.JamState;
import ant.liaison.channel.assign.ChannelAssignResult.BandwidthLevel;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.RandomSelector;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link AgentLocationComponent} {@link ObjectiveJigsawSystemComponent}
 *         {@link ObjectiveRaceComponent}
 */
public class ObjectiveSpriteWantedComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(ObjectiveSpriteWantedComponent.class);
	private static final int QUERY_CLOSE_IGNORE_COUNT = 3;
	private static final double LOW_BANDWIDTH_OVERTIME = 30;

	private AgentContext context;
	private StandardWorldModel world;
	private AgentLocationComponent alc;
	private ObjectiveJigsawMetaComponent ojmc;
	private ObjectiveRaceComponent orc;
	private StaticCityGraphComponent scg;
	private JamComponent jc;
	private boolean nearestPolicy = true;
	private Random rand;
	private HashSet<EntityID> sprites;

	private Set<Human> pfset;

	@Override
	public String getComponentName() {
		return ObjectiveSpriteWantedComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveJigsawMetaComponent.class);
		this.jc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);
		this.orc = (ObjectiveRaceComponent) context.getAntComponentKit().getAntComponent(
				ObjectiveRaceComponent.class);
		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		this.sprites = orc.getSpriteSet();
		this.pfset = context.getAntComponentKit().getAntMapBasisComponent().getPfSet();
		this.rand = new Random(context.getAgentID().getValue() + 1);
	}

	@Override
	public void update() {
		if (context.getTimestamp() == ConfigKey.getIgnoreTime(context.getConfig()) + 2) {
			updatePolicy();
		}
	}

	private void updatePolicy() {
		BandwidthLevel bwl = context.getAntCommunicationKit().getBandwidthLevel();
		if (bwl == BandwidthLevel.None || bwl == BandwidthLevel.Poor
				|| bwl == BandwidthLevel.Unknown) {
			nearestPolicy = true;
		} else {
			nearestPolicy = false;
		}
	}

	public ObjectiveJigsaw getTask() {
		if (nearestPolicy == true) {
			return getTaskNearest();
		} else {
			return getTaskSelect();
		}
	}

	private ObjectiveJigsaw getTaskSelect() {
		Set<ObjectiveJigsaw> queries = ojmc.getMidSpotedJigsaws();

		if (queries.isEmpty())
			return null;

		Set<ObjectiveJigsaw> cands = new HashSet<ObjectiveJigsaw>();
		for (ObjectiveJigsaw oj : queries) {
			if (countCloseThanMe(oj.getJigsawObject().getCenter().getID()) <= QUERY_CLOSE_IGNORE_COUNT) {
				cands.add(oj);
			}
		}

		if (cands.isEmpty())
			return null;

		return closestQueriesSelectInner(cands);
	}

	private ObjectiveJigsaw closestQueriesSelectInner(Set<ObjectiveJigsaw> cands) {
		ObjectiveJigsaw pick = null;
		double mindist = Double.MAX_VALUE;
		for (ObjectiveJigsaw oj : cands) {
			double tempdist = scg.getStaticDistanceToTreeRoot(oj.getJigsawObject().getCenter()
					.getID(), context.getHuman().getPosition());
			if (tempdist < mindist) {
				mindist = tempdist;
				pick = oj;
			}
		}

		return pick;
	}

	private int countCloseThanMe(EntityID tar) {
		// Close Than other sprites
		Area targetArea = (Area) world.getEntity(tar);
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();
		int c = 0;
		double mydist = (int) (GeoUtils.manhattanDistance(targetArea, x, y) / 1000);
		for (Human other : pfset) {
			if (jc.getJam(other.getID()).getState() == JamState.Buried)
				continue;
			
			Area oa = (Area) world.getEntity(alc.getAgentLastLocation(other.getID()));
			double otherdist = (int) (GeoUtils.manhattanDistance(targetArea, oa) / 1000);
			if (otherdist < mydist) {
				c++;
			} else if (otherdist == mydist) {
				if (other.getID().getValue() < context.getHuman().getID().getValue()) {
					c++;
				}
			}
		}
		return c;
	}

	private ObjectiveJigsaw getTaskNearest() {
		Set<ObjectiveJigsaw> cands = new HashSet<ObjectiveJigsaw>();
		
		int hx = context.getHuman().getX();
		int hy = context.getHuman().getY();
		for (ObjectiveJigsaw oj : ojmc.getMidSpotedJigsaws()) {
			double dist = GeoUtils.manhattanDistance(oj.getX(), oj.getY(), hx, hy) * 2;
			if (dist / AntBehavior.AVERAGE_MOVE_DISTANCE < LOW_BANDWIDTH_OVERTIME) {
				cands.add(oj);
			}
		}
		
		if (cands.isEmpty())
			return null;

		return multiscaleSelect(cands);
	}

	/**
	 * near -> far
	 */
	private ObjectiveJigsaw multiscaleSelect(Set<ObjectiveJigsaw> cands) {
		if (cands.isEmpty())
			return null;
		int x = this.context.getHuman().getX();
		int y = this.context.getHuman().getY();
		int range = 200000;
		Set<ObjectiveJigsaw> candidate = new HashSet<ObjectiveJigsaw>();
		while (true) {
			candidate.clear();
			range *= 2;
			for (ObjectiveJigsaw a : cands) {
				if (GeoUtils.manhattanDistance(x, y, a.getX(), a.getY()) < range) {
					candidate.add(a);
				}
			}
			if (!candidate.isEmpty()) {
				ObjectiveJigsaw pick = RandomSelector.randomSelect(candidate, 1, rand).iterator()
						.next();
				return pick;
			}
		}
	}
}
