package ant.component.pf.spot;

import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.pf.ObjectiveMiscComponent;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.JamComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.jigsaw.JigsawMainComponent;
import ant.utils.IPositioning;


/**
 * 
 * @author Ivan
 * 
 * @depend {@link AntMapBasisComponent} {@link JigsawMainComponent} {@link AgentLocationComponent} {@link JamComponent} {@link ObjectiveMiscComponent}
 */
public abstract class AbstractSpot implements IPositioning {
	protected AgentContext context;
	protected boolean isDone = false;
	protected int timestamp;
	
	protected AgentLocationComponent alc;
	protected JigsawMainComponent jimain;
	protected Set<Human> agentset;
	protected Set<Human> pfset;
	protected JamComponent jc;
	protected StandardWorldModel world;
	
	public AbstractSpot(AgentContext context) {
		this.context = context;
		this.world = context.getWorld();
		this.isDone = false;
		this.timestamp = context.getTimestamp();
		this.jimain = (JigsawMainComponent) context.getAntComponentKit().getAntComponent(JigsawMainComponent.class);
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.jc = (JamComponent) context.getAntComponentKit().getAntComponent(JamComponent.class);
		this.pfset = context.getAntComponentKit().getAntMapBasisComponent().getPfSet();
		this.agentset = context.getAntComponentKit().getAntMapBasisComponent().getAgentSet();
	}
	
	public abstract SpotType getType();

	public boolean isDone() {
		return isDone;
	}
		
	public void done() {
		this.isDone = true;
	}
	
//	public void undone() {
//		this.isDone = false;
//	}
	
	public final int getTimestamp() {
		return timestamp;
	}
	
	public abstract void triggerUpdate();
	
	/**
	 * @return null, if done
	 */
	public abstract EntityID getWhere();
	
	@Override
	public String toString() {
		return "[SPOT " + getType().toString() + " where:" + getWhere() + " isDone:" + isDone + "]";
	}
	
	@Override
	public int getX() {
		Area a = (Area) world.getEntity(getWhere());
		return a.getX();
	}
	
	@Override
	public int getY() {
		Area a = (Area) world.getEntity(getWhere());
		return a.getY();
	}
	
	
}
