package ant.component.pf.spot;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.universal.JamComponent.Jam;
import ant.component.universal.JamComponent.JamState;
import ant.exception.StatusInconsistentException;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class AbstractSpotJam extends AbstractSpot {
	private static final Log log = LogFactory.getLog(AbstractSpotJam.class);

	protected Jam jam;
	protected EntityID stuckAgent;

	public AbstractSpotJam(AgentContext context, EntityID stuckAgent) throws StatusInconsistentException {
		super(context);
		this.stuckAgent = stuckAgent;
		this.jam = this.jc.getJam(stuckAgent);
		if (!checkJam())
			throw new StatusInconsistentException("Discordance jam state");
	}

	private boolean checkJam() {
		try {
			if (this.getType() == SpotType.AgentBuried) {
				if (jam.getState() == JamState.Buried) {
					StandardEntity se = world.getEntity(jam.getBuriedBuilding());
					if (se instanceof Building)
						return true;
				}
			} else if (this.getType() == SpotType.AgentStuckingInBuilding) {
				if (jam.getState() == JamState.StuckInBuilding) {
					StandardEntity se = world.getEntity(jam.getStuckBuilding());
					if (se instanceof Building)
						return true;
				}
			} else if (this.getType() == SpotType.AgentStuckingInRoad) {
				if (jam.getState() == JamState.StuckInRoad) {
					if (jam.getStuckRoodCoord().getRoad() != null) {
						return true;
					}
				}
			}
		} catch (Exception e) {
			log.error(e);
		}
		return false;
	}

	public final Jam getJam() {
		return this.jam;
	}

	public final EntityID getStuckAgent() {
		return this.stuckAgent;
	}

	@Override
	public final EntityID getWhere() {
		try {
			if (jam.getState() == JamState.Buried) {
				StandardEntity se = world.getEntity(jam.getBuriedBuilding());
				if (se instanceof Building)
					return se.getID();
			} else if (jam.getState() == JamState.StuckInBuilding) {
				StandardEntity se = world.getEntity(jam.getStuckBuilding());
				if (se instanceof Building)
					return se.getID();
			}
			if (jam.getState() == JamState.StuckInRoad) {
				return jam.getStuckRoodCoord().getRoad().getID();
			}
		} catch (Exception e) {
			log.error(e);
		}
		return null;
	}
	
	@Override
	public String toString() {
		return super.toString() + " for " + stuckAgent;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((stuckAgent == null) ? 0 : stuckAgent.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AbstractSpotJam other = (AbstractSpotJam) obj;
		if (stuckAgent == null) {
			if (other.stuckAgent != null)
				return false;
		} else if (!stuckAgent.equals(other.stuckAgent))
			return false;
		return true;
	}
	
}
