package ant.component.pf.spot;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.component.universal.BuildingConditionComponent;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.component.universal.BuildingWithNoCivilianComponent;
import ant.component.universal.ChangeSetDetailComponent;
import ant.component.universal.LocationMapComponent;

/**
 * 
 * @author Ivan
 * 
 *         Single Agent Spot
 */
public class SpotFindCivBuildingEnter extends AbstractSpot {
	private static final Log log = LogFactory.getLog(SpotFindCivBuildingEnter.class);
	
	private Building tar;
	private BuildingWithNoCivilianComponent bnc;
	private LocationMapComponent lmc;
	private ChangeSetDetailComponent csd;
	private BuildingConditionComponent bcc;

	public SpotFindCivBuildingEnter(AgentContext context, Building b) {
		super(context);
		this.tar = b;
		this.bnc = context.getAntComponentKit().getBuildingWithNoCivilianComponent();
		this.lmc = context.getAntComponentKit().getLocationMapComponent();
		// this.csd =
		// context.getAntComponentKit().getChangeSetDetailComponent();
		this.bcc = context.getAntComponentKit().getBuildingConditionComponent();
	}

	@Override
	public SpotType getType() {
		return SpotType.FindCivBuildingEnter;
	}

	public Building getTargetBuilding() {
		return tar;
	}

	@Override
	public void triggerUpdate() {
		if (this.isDone)
			return;

		for (Human h : agentset) {
			for (AgentLocationInfo ali : alc.getUpdatedAgentLocationInfos(h.getID())) {
				EntityID loc = ali.getLocation();
				if (tar.getID().equals(loc)) {
					this.done();
				}
			}
		}
		
		if (bnc.isBuildingWithNoCivilian(tar.getID())) {
			this.done();
		}

		if (lmc.getAreaNode(tar.getID()).getVisitCount() != 0) {
			// log.error("ALC is not same as LMC");
			this.done();
		}

		BuildingCondition tarbc = bcc.getBuildingCondition(tar.getID());
		if (tarbc.isEverOnFire())
			this.done();
	}

	@Override
	public EntityID getWhere() {
		return tar.getID();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((tar == null) ? 0 : tar.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SpotFindCivBuildingEnter other = (SpotFindCivBuildingEnter) obj;
		if (tar == null) {
			if (other.tar != null)
				return false;
		} else if (!tar.equals(other.tar))
			return false;
		return true;
	}

}
