package ant.component.pf.spot;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.component.universal.AgentLocationComponent;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.component.universal.JamComponent;
import ant.component.universal.JamComponent.JamState;
import ant.map.component.AntMapBasisComponent;
import ant.map.jigsaw.JigsawAttach;
import ant.map.jigsaw.JigsawMainComponent;

/**
 * 
 * @author Ivan
 *
 *
 * @depend {@link AntMapBasisComponent} {@link JigsawMainComponent} {@link AgentLocationComponent} {@link JamComponent}
 */
public class SpotRefuge extends AbstractSpot {
	private static final Log log = LogFactory.getLog(SpotRefuge.class);
	private Refuge refuge;
	private JigsawAttach refJigsaw;

	public SpotRefuge(AgentContext context, Refuge refuge) {
		super(context);
		this.refuge = refuge;
		this.refJigsaw = (JigsawAttach) jimain.getJigsaw(refuge.getID());
	}

	@Override
	public SpotType getType() {
		return SpotType.Refuge;
	}

	
	/** 检查是否有PF在RefJigsaw(Only Building) 里面 **/
	@Override
	public void triggerUpdate() {
		if (this.isDone) return;
		// Case 1 发现其他ActivePF到了 Refuge的JigsawAttach(Without Road)里面
		// Case 2 我自己到了Refuge的JigsawAttach里面
		// log.info("Trigger Update: @" + context.getTimestamp());
		
		for (Human h : pfset) {
			if (this.jc.getJam(h.getID()).getState() != JamState.Buried) {
				for (AgentLocationInfo ali : alc.getUpdatedAgentLocationInfos(h.getID())) {
					EntityID loc = ali.getLocation();
					Area a = (Area) context.getWorld().getEntity(loc);
					if (a instanceof Building) {
						if (refJigsaw.contains(loc)) {
							this.done();
//							log.info("Refuge: " + this.refuge.getID() + " is done by " + h.getID());
							return;
						}
					}
				}
			}
		}
		
		// Case 3 某人说自己已经清理了该Refuge
		// TODO
	}
	
	public Refuge getTargetRefuge() {
		return this.refuge;
	}

	@Override
	public EntityID getWhere() {
		return this.refuge.getID();
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((refuge == null) ? 0 : refuge.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SpotRefuge other = (SpotRefuge) obj;
		if (refuge == null) {
			if (other.refuge != null)
				return false;
		} else if (!refuge.equals(other.refuge))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return super.toString() + " Refuge:" + refuge.getID();
	}
}
