package ant.component.universal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.partition.Partition;
import ant.map.partition.PartitionAreaComponent;
import ant.utils.Pair;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Oliver
 * @depend {@link PartitionAreaComponent}
 */
public class AgentPartitionAssignComponent implements AntComponent {
	private static Log log = LogFactory.getLog(AgentPartitionAssignComponent.class);

	private PartitionAreaComponent pac;
	private ArrayList<Human> agents;
	private HashMap<EntityID, Partition> partitionMap;

	private double maxDistance;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.pac = context.getAntComponentKit().getPartitionAreaComponent();
		this.partitionMap = new HashMap<EntityID, Partition>();

		this.agents = new ArrayList<Human>();
		StandardEntityURN urn = context.getAgentType();

		for (StandardEntity e : world.getEntitiesOfType(urn)) {
			agents.add((Human) e);
		}

		this.maxDistance = Double.MIN_VALUE;

		assign(agents);
	}

	@Override
	public void update() {
		// TODO What if agent blocked or died
	}

	public Partition getPartition(EntityID agentID) {
		return partitionMap.get(agentID);
	}

	private void assign(ArrayList<Human> agents) {
		HashMap<EntityID, Map<Partition, Double>> distances = calculateDistanceToPartitions(agents,
				pac.getPartitions());
		int agentSize = agents.size();
		for (int i = 0; i < agentSize; i++) {
			Pair<Human, Partition> assign = partitionAssign(agents, distances);
			Human agent = assign.first();
			Partition partition = assign.second();

			partitionMap.put(agent.getID(), partition);
			agents.remove(agent);

			for (Map<Partition, Double> map : distances.values()) {
				Double dis = map.get(partition);
				dis += maxDistance;
				map.put(partition, dis);
			}
		}
	}

	public void print() {
		String info = "partition result " + partitionMap.size() + ":";
		for (Iterator<Entry<EntityID, Partition>> it = partitionMap.entrySet().iterator(); it
				.hasNext();) {
			Entry<EntityID, Partition> entry = it.next();
			info += "(" + entry.getKey() + "," + entry.getValue().getID() + ")";
		}
		log.info(info);
	}

	private HashMap<EntityID, Map<Partition, Double>> calculateDistanceToPartitions(
			ArrayList<Human> agents, ArrayList<Partition> partitions) {
		HashMap<EntityID, Map<Partition, Double>> distanceMap = new HashMap<EntityID, Map<Partition, Double>>();
		for (Human agent : agents) {
			HashMap<Partition, Double> map = new HashMap<Partition, Double>();
			distanceMap.put(agent.getID(), map);
			for (Partition p : partitions) {
				double dis = GeoUtils.dist(agent.getX(), agent.getY(), p.getX(), p.getY());
				map.put(p, dis);
				if (dis > maxDistance) {
					maxDistance = dis;
				}
			}
		}
		return distanceMap;
	}

	private Pair<Human, Partition> partitionAssign(ArrayList<Human> agents,
			HashMap<EntityID, Map<Partition, Double>> distances) {
		double minDis = Double.MAX_VALUE;
		Partition partition = null;
		Human agent = null;
		for (int i = 0; i < agents.size(); i++) {
			Human a = agents.get(i);
			for (Iterator<Entry<Partition, Double>> it = distances.get(a.getID()).entrySet()
					.iterator(); it.hasNext();) {
				Entry<Partition, Double> entry = it.next();
				if (entry.getValue() < minDis) {
					minDis = entry.getValue();
					partition = entry.getKey();
					agent = a;
				}
			}
		}
		if (agent == null || partition == null) {
			log.error("Partition assign failed!");
			return null;
		} else {
			return new Pair<Human, Partition>(agent, partition);
		}
	}
}
