package ant.component.universal;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.liaison.message.AbstractCivilianMessage;
import ant.liaison.message.AbstractCivilianMessage.MCivilianSayHelp;
import ant.liaison.message.IMessage;
import ant.liaison.message.MVoiceHearCivHelp;
import ant.utils.civilian.CivilianInfo;

/**
 * 
 * @author Ivan
 * @depend {@link HumanInfoComponent} {@link BuildingWithNoCivilianComponent}
 *         {@link AgentLocationComponent}
 */
public class BeliefCivVoiceComponent extends AbstractMessageProducerComponent implements
		AntComponent {
	private static final Log log = LogFactory.getLog(BeliefCivVoiceComponent.class);
	private static final double RELEX_FACTOR_1 = 1.1;
	private static final double RELEX_FACTOR_2 = 1.2;
	private AgentContext context;
	private int voiceRange;
	private StandardWorldModel world;
	private Map<EntityID, Set<Building>> civ2bmap;
	private Set<EntityID> visitedBuilding;

	private HumanInfoComponent hic;
	private BuildingWithNoCivilianComponent bwncc;
	private AgentLocationComponent alc;
	private ChangeSetDetailComponent csd;

	@Override
	public String getComponentName() {
		return BeliefCivVoiceComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		// TODO
		this.voiceRange = 30000;
		this.civ2bmap = new HashMap<EntityID, Set<Building>>();
		this.visitedBuilding = new HashSet<EntityID>();

		this.hic = context.getAntComponentKit().getHumanInfoComponent();
		this.bwncc = context.getAntComponentKit().getBuildingWithNoCivilianComponent();
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.csd = context.getAntComponentKit().getChangeSetDetailComponent();
		
	}

	@Override
	public void update() {
		if (context.getTimestamp() < ConfigKey.getIgnoreTime(context.getConfig()))
			return;

		if (!context.getHuman().isPositionDefined())
			return;
		if (!context.getHuman().isXDefined())
			return;
		if (!context.getHuman().isYDefined())
			return;

		EntityID here = context.getHuman().getPosition();
		for (AbstractCivilianMessage msg : context.getAntCommunicationKit()
				.getMessagesFromCivilian()) {
			if (msg instanceof MCivilianSayHelp) {
				// log.info("Recieve a CivHelp!");
				MCivilianSayHelp mcsh = (MCivilianSayHelp) msg;
				if (mcsh.getCivilianID() != null) {
					pushCivMsg(mcsh.getCivilianID(), context.getHuman().getX(), context.getHuman()
							.getY(), voiceRange);
					addVoiceMessage(new MVoiceHearCivHelp(mcsh.getCivilianID()));
				}
			}
		}
		for (IMessage im : context.getAntCommunicationKit().getVoiceReceiveBox()) {
			if (im instanceof MVoiceHearCivHelp) {
				MVoiceHearCivHelp mvhch = (MVoiceHearCivHelp) im;
				EntityID sender = mvhch.getSender();
				if (sender != null && mvhch.getCivID() != null) {
					if (sender.equals(context.getAgentID())) {
						continue;
					}
					AgentLocationInfo ali = alc.getAgentLastLocationInfo(sender);
					if (ali.isCoordinateDefined()) {
						pushCivMsg(mvhch.getCivID(), ali.getX(), ali.getY(),
								(int) (voiceRange * RELEX_FACTOR_1));
					} else {
						EntityID loc = ali.getLocation();
						if (loc != null && world.getEntity(loc) instanceof Area) {
							Area loca = (Area) world.getEntity(loc);
							pushCivMsg(mvhch.getCivID(), loca.getX(), loca.getY(),
									(int) (voiceRange * RELEX_FACTOR_2));
						}
					}
				}
			}
		}

		for (CivilianInfo ci : hic.getCivIncNow()) {
			// StandardEntity se = world.getEntity(ci.getPosition());
			// if (se instanceof Building) {
			// if (civ2bmap.containsKey(ci.getID())) {
			// civ2bmap.get(ci.getID()).clear();
			// civ2bmap.get(ci.getID()).add((Building) se);
			// } else {
			// Set<Building> waits = new HashSet<Building>();
			// waits.add((Building) se);
			// civ2bmap.put(ci.getID(), waits);
			// }
			// } else {
			// // Ignore
			// }
			if (civ2bmap.containsKey(ci.getID())) {
				civ2bmap.remove(ci.getID());
			}
		}

		for (CivilianInfo ci : hic.getCivDelNow()) {
			if (civ2bmap.containsKey(ci.getID())) {
				civ2bmap.remove(ci.getID());
			}
		}

		
		
		StandardEntity se = context.getHuman().getPosition(context.getWorld());
		if (se instanceof Building) {
			visitedBuilding.add(se.getID());
		}
		
		for (Entry<EntityID, Set<Building>> entry : civ2bmap.entrySet()) {
			Set<Building> bs = entry.getValue();
			Iterator<Building> iter = bs.iterator();
			while (iter.hasNext()) {
				Building b = iter.next();
				if (bwncc.isBuildingWithNoCivilian(b.getID())) {
					iter.remove();
				} else if (visitedBuilding.contains(b.getID())) {
					iter.remove();
				} else if (b instanceof Refuge) {
					iter.remove();
				}
			}
		}
	}

	private void pushCivMsg(EntityID civilianID, int x, int y, int range) {
		if (hic.getCivilianInfoMap().containsKey(civilianID))
			return;

		Set<Building> waits = new HashSet<Building>();
		for (StandardEntity se : world.getObjectsInRange(x, y, range)) {
			if (se instanceof Building) {
				if (!bwncc.isBuildingWithNoCivilian(se.getID())) {
					// Never Seen the building
					if (!visitedBuilding.contains(se.getID()))
						waits.add((Building) se);
				}
			}
		}

		if (civ2bmap.containsKey(civilianID)) {
			Set<Building> have = civ2bmap.get(civilianID);
			Iterator<Building> iter = have.iterator();
			while (iter.hasNext()) {
				Building b = iter.next();
				if (!waits.contains(b)) {
					iter.remove();
				}
			}
		} else {
			civ2bmap.put(civilianID, waits);
		}
	}

	public int getScoreInBuilding(Building b) {
		int c = 0;
		for (Entry<EntityID, Set<Building>> entry : civ2bmap.entrySet()) {
			if (entry.getValue().contains(b))
				c++;
		}

		return c;
	}

	public double getProbScoreInBuilding(Building b) {
		double c = 0;
		for (Entry<EntityID, Set<Building>> entry : civ2bmap.entrySet()) {
			if (entry.getValue().contains(b)) {
				if (entry.getValue().size() != 0) {
					c += 1.0f / entry.getValue().size();
				} else {
					log.error("Civilian:" + entry.getValue() + " cannot found");
				}
			}
		}
		return c;
	}
	
}
