package ant.component.universal;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.agent.behavior.PFBehavior;
import ant.liaison.message.IMessage;
import ant.liaison.message.MAgentRoadCorLocation;
import ant.liaison.message.MAgentRoadLocation;
import ant.map.component.AntMapBasisComponent;

/**
 * TODO 前50个周期不认为PF-Clear了?
 * TODO viewer
 * 
 * @author Ivan
 * @depend {@link AgentLocationComponent} {@link ChangeSetDetailComponent} {@link AntMapBasisComponent}
 */
public class BeliefClearComponent implements AntComponent {
	private static final int PF_CLEAR_SAMPLING_TIME = 50;
	private Set<Road> clearRoadSet;
	private Set<Road> unclearRoadSet;
	private Set<Human> pfSet;
	private AgentLocationComponent alc;
	private StandardWorldModel world;
	private ChangeSetDetailComponent cgd;
	private AgentContext context;
	private List<BeliefClearListener> listeners;

	@Override
	public String getComponentName() {
		return BeliefClearComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.pfSet = new HashSet<Human>();
		this.listeners = new ArrayList<BeliefClearListener>();

		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.cgd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.unclearRoadSet = new HashSet<Road>();
		this.unclearRoadSet.addAll(context.getAntComponentKit().getAntMapBasisComponent().getRoadSet());
		
		for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getAgentSet()) {
			if (h instanceof PoliceForce) {
				pfSet.add(h);
			}
		}
		this.clearRoadSet = new HashSet<Road>();
	}

	@Override
	public void update() {
		if (context.getTimestamp() >= PF_CLEAR_SAMPLING_TIME) {
			for (Human h : pfSet) {
				EntityID id = alc.getAgentLastLocation(h.getID());
				if (world.getEntity(id) instanceof Road) {
					Road r = (Road) world.getEntity(id);
					updateBeliefClearRoad(r);
				}
			}
		}
		for (StandardEntity se : cgd.getEntitiesInSight(StandardEntityURN.ROAD)) {
			Road r = (Road) se;
			if (PFBehavior.isRoadClean(world, r)) {
				updateBeliefClearRoad(r);
			}
		}
		for (IMessage im : context.getAntCommunicationKit().getRadioReceiveBox()) {
			if (im instanceof MAgentRoadCorLocation) {
				MAgentRoadCorLocation msg = (MAgentRoadCorLocation) im;
				if (msg.isBlockadeSeenOnTheRoad() == false) {
					updateBeliefClearRoad(msg.getRoadCoord().getRoad());
				}
			} else if (im instanceof MAgentRoadLocation) {
				MAgentRoadLocation msg = (MAgentRoadLocation) im;
				if (msg.isBlockadeSeenOnTheRoad() == false) {
					Road r = (Road) world.getEntity(msg.getLocation());
					updateBeliefClearRoad(r);
				}
			}
		}
		// TODO voice
		// TODO else message
	}

	private void updateBeliefClearRoad(Road r) {
		
		unclearRoadSet.remove(r.getID());
		clearRoadSet.add(r);
		for (BeliefClearListener l : listeners) {
			l.beliefClear(r);
		}
	}

	public boolean isBeliefUnclear(EntityID roadID) {
		return unclearRoadSet.contains(roadID);
	}

	public void addBeleifClearListener(BeliefClearListener listener) {
		this.listeners.add(listener);
	}
	
	public void removeBeleifClearListener(BeliefClearListener listener) {
		this.listeners.remove(listener);
	}
	
	public Set<Road> getClearRoadSet() {
		return clearRoadSet;
	}
	
	public Set<Road> getUnclearRoadSet() {
		return unclearRoadSet;
	}
}
