package ant.component.universal;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.component.AntMapBasisComponent;
import ant.utils.CounterSet;
import ant.utils.DimBool;
import ant.utils.LazyMap;

/**
 * 在无通信的情况下会起到很大作用.
 * 
 * @author Ivan
 * 
 * @depend {@link AgentLocationComponent} {@link AntMapBasisComponent}
 */

@Deprecated
public class BeliefInactiveAgentComponent implements AntComponent {
	private static final int TRIGGER_TIME = 7;
	private static final Log log = LogFactory.getLog(BeliefInactiveAgentComponent.class);
	
	private AgentContext context;
	private AgentLocationComponent alc;
	private AntMapBasisComponent amb;

	/**
	 * agentID - areaID
	 */
	private Map<EntityID, EntityID> initPosition;
	private Map<EntityID, Set<EntityID>> initInactivePositions;
	private Set<EntityID> candiate;

	@Override
	public String getComponentName() {
		return BeliefInactiveAgentComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.amb = context.getAntComponentKit().getAntMapBasisComponent();

		this.candiate = new HashSet<EntityID>();
		this.initPosition = new HashMap<EntityID, EntityID>();
		this.initInactivePositions = new LazyMap<EntityID, Set<EntityID>>() {
			@Override
			public Set<EntityID> createValue() {
				return new HashSet<EntityID>();
			}
		};

		CounterSet<EntityID> loccount = new CounterSet<EntityID>();
		for (Human atid : amb.getAgentSet()) {
			loccount.inc(atid.getPosition());
		}

		for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getAgentSet()) {
			candiate.add(h.getID());
			initPosition.put(h.getID(), h.getPosition());
			initInactivePositions.get(h.getID()).add(h.getPosition());
			if (loccount.count(h.getPosition()) > 1) {
				initInactivePositions.get(h.getID()).addAll(((Area) h.getPosition(world)).getNeighbours());
			}
		}
	}

	@Override
	public void update() {
		Iterator<EntityID> iter = candiate.iterator();
		while (iter.hasNext()) {
			EntityID h = iter.next();
			if (initPosition.containsKey(h)) {
				EntityID where = alc.getAgentLastLocation(h);
				if (!initInactivePositions.get(h).contains(where)) {
					initPosition.remove(h);
					iter.remove();
				}
			}
		}
	}

	/**
	 * if return null, not trigger <Br>
	 * <AgentID, LocID>
	 */
	public Map<EntityID, EntityID> getInactiveMap() {
		if (context.getTimestamp() >= TRIGGER_TIME) {
			return this.initPosition;
		} else {
			return null;
		}
	}

	public Map<EntityID, Set<EntityID>> getInactivePlacesMap() {
		if (context.getTimestamp() >= TRIGGER_TIME) {
			return this.initInactivePositions;
		} else {
			return null;
		}
	}

	public DimBool isInactive(EntityID id) {
		if (id.equals(context.getAgentID())) {
			if (context.getAgentType() == StandardEntityURN.POLICE_FORCE) {
				if (context.getHuman().isBuriednessDefined()) {
					if (context.getHuman().getBuriedness() == 0) {
						return DimBool.FALSE;
					}
				}
			}
		}
		Map<EntityID, EntityID> tmp = getInactiveMap();
		if (tmp == null)
			return DimBool.UNKNOWN;
		else {
			if (tmp.containsKey(id))
				return DimBool.TRUE;
			else
				return DimBool.FALSE;
		}
	}

}
