package ant.component.universal;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.StaticCityGraphComponent;
import ant.map.jigsaw.JigsawMarkerComponent;
import ant.map.objects.AreaGraph;
import ant.map.objects.Route;
import ant.map.search.RangeStepBFSSearch;
import ant.utils.LazyMap;
import ant.utils.ListDereplication;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link AgentLocationComponent} {@link AntMapBasisComponent} {@link StaticCityGraphComponent} {@link JigsawMarkerComponent}
 */
public class BeliefTrailComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(BeliefTrailComponent.class);
	private static final int OBSERVED_COUNT = 60;
	private static final int TIME_STEP = 4;
	// private static final int RANGE = 75000;
	private static final int RANGE_STEP = 10;

	private AgentContext context;
	private StandardWorldModel world;
	private AgentLocationComponent locomp;
	private JigsawMarkerComponent jmc;
	
	private RangeStepBFSSearch alg;
	private Map<EntityID, List<BeliefTrail>> allTrails;
	private Set<Human> agentset;

	// private Map<EntityID, BeliefTrail> transientTrails;
	// private Map<EntityID, Integer> lasttime;

	// TODO remove
	private int ctrail = 0;
	private int ctrail2 = 0;
	private int ctrail3 = 0;
	private int ctrail4 = 0;

	@Override
	public String getComponentName() {
		return BeliefTrailComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.locomp = context.getAntComponentKit().getAgentLocationComponent();
		this.jmc = (JigsawMarkerComponent) context.getAntComponentKit().getAntComponent(JigsawMarkerComponent.class);
		
		// this.transientTrails = new HashMap<EntityID, BeliefTrail>();
		AreaGraph graph = context.getAntComponentKit().getStaticCityGraphComponent().getStaticAreaGraph();
		// this.alg = new RangeSPFASearch(graph, world, RANGE);
		this.alg = new RangeStepBFSSearch(jmc, graph, world, RANGE_STEP);
		
		processAgentSet();

		this.allTrails = new LazyMap<EntityID, List<BeliefTrail>>() {
			@Override
			public List<BeliefTrail> createValue() {
				return new ArrayList<BeliefTrail>();
			}
		};
	}

	private void processAgentSet() {
		// int cc = OBSERVED_COUNT;
		this.agentset = new HashSet<Human>();
		// for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getFbSet()) {
		// agentset.add(h);
		// cc--;
		// if (cc == 0)
		// return;
		// }
		// for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getPfSet()) {
		// agentset.add(h);
		// cc--;
		// if (cc == 0)
		// return;
		// }
		// for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getAtSet()) {
		// agentset.add(h);
		// cc--;
		// if (cc == 0)
		// return;
		// }

		// TODO 细化
		for (Human h : context.getAntComponentKit().getAntMapBasisComponent().getAgentSet()) {
			agentset.add(h);
		}
	}

	@Override
	public void update() {
		// transientTrails.clear();
		// Set<EntityID> waits = new HashSet<EntityID>();
		// for (IMessage msg : context.getAntCommunicationKit().getReceiveBox())
		// {
		// if (msg instanceof AbstractAgentLocationMessage) {
		// waits.add(msg.getSender());
		// }
		// }
		// for (EntityID wait : waits) {
		// processWait(wait);
		// }
		for (Human human : agentset) {
			process(human.getID());
		}
		// log.info("Here:" + context.getHuman().getPosition());
		// log.info("MLoc:" + locomp.getAgentLastLocation(context.getHuman().getID()));
		// log.info(context.getHuman().getID() + "," +
		// allTrails.get(context.getHuman().getID()).get(allTrails.get(context.getHuman().getID()).size() - 1));

		// if (context.getTimestamp() % 10 == 0) {
		// log.info("@" + context.getTimestamp() + " TrailsStat count:" + ctrail + " hit[2]:" + ctrail2 + " hit[3]:"
		// + ctrail3);
		// }
	}

	private void process(EntityID wait) {

		AgentLocationInfo now = locomp.getAgentLastLocationInfo(wait);
		if (allTrails.get(wait).size() == 0) {
			BeliefTrail bt = makeIllTrail(now);
			allTrails.get(wait).add(bt);
		} else {
			BeliefTrail trail = allTrails.get(wait).get(allTrails.get(wait).size() - 1);
			if (trail.isInvalid()) {
				Area start = (Area) world.getEntity(trail.getStart());
				Area end = (Area) world.getEntity(now.getLocation());
				Route<Area> route = makeRoute(start, end);
				if (route != null) {
					trail.append(now.getLocation(), toEntityIDList(route.getRoute()), now.getTimestamp());
				} else {
					// Oh ill and ill
					resetIllTrail(trail, now);
				}
			} else {
				Area start = (Area) world.getEntity(trail.getEnd());
				Area end = (Area) world.getEntity(now.getLocation());

				
				this.ctrail++;
				
				// Hit[1]
				if (start.getID().equals(end.getID())) {
					// this.ctrail1++;
					return;
				}

				// Hit[2]
				if (start.getEdgeTo(end.getID()) != null) {
					trail.append1(now.getLocation(), end.getID(), now.getTimestamp());
					this.ctrail2++;
					return;
				}

				// Hit[3]
				// for (EntityID nei : start.getNeighbours()) {
				// if (end.getEdgeTo(nei) != null) {
				// this.ctrail3++;
				// }
				// }

				Route<Area> route = makeRoute(start, end);
				if (route != null) {
					// trail.append(now.getLocation(), toEntityIDList(route.getRoute()), now.getTimestamp());
					trail.append(now.getLocation(), route, now.getTimestamp());

				} else {
					// Oh disconnected
					BeliefTrail bt = makeIllTrail(now);
					allTrails.get(wait).add(bt);
				}
			}
		}
	}

	private Route<Area> makeRoute(Area st, Area ed) {

		if (st.getID().equals(ed.getID()) || st.getEdgeTo(ed.getID()) != null) {
			Route<Area> r = new Route<Area>();
			List<Area> tmp = new ArrayList<Area>();
			tmp.add(st);
			if (!st.getID().equals(ed.getID()))
				tmp.add(ed);
			r.setRoute(tmp);

			// Route<Area> algroute = alg.spSearch(st, ed);
			// if (!algroute.getRoute().equals(tmp)) {
			// log.error(algroute.getRoute());
			// log.error(tmp);
			// log.error("---------------------");
			// }

			return r;
		} else {
			return alg.spSearch(st, ed);
		}
	}

	private BeliefTrail makeIllTrail(AgentLocationInfo single) {
		List<EntityID> tmp = new ArrayList<EntityID>();
		tmp.add(single.getLocation());
		// tmp.add(single.getLocation());
		return new BeliefTrail(single.getLocation(), null, tmp, single.getTimestamp(), -1);
	}

	private void resetIllTrail(BeliefTrail trail, AgentLocationInfo single) {
		List<EntityID> tmp = new ArrayList<EntityID>();
		tmp.add(single.getLocation());
		// tmp.add(single.getLocation());
		trail.setEnd(null);
		trail.setEndTime(-1);
		trail.setStart(single.getLocation());
		trail.setStartTime(single.getTimestamp());
		trail.setTrail(tmp);
	}

	public List<BeliefTrail> getTrails(EntityID who) {
		return allTrails.get(who);
	}

	public Set<Human> getObstervedAgentSet() {
		return this.agentset;
	}

	private List<EntityID> toEntityIDList(List<Area> input) {
		List<EntityID> tmp = new ArrayList<EntityID>();
		for (Area a : input) {
			tmp.add(a.getID());
		}
		return tmp;
	}

	public static class BeliefTrail {
		private EntityID start;
		private EntityID end;
		private List<EntityID> trail;
		private int startTime;
		private int endTime;

		public BeliefTrail(EntityID start, EntityID end, List<EntityID> trail, int startTime, int endTime) {
			this.start = start;
			this.end = end;
			this.trail = trail;
			this.startTime = startTime;
			this.endTime = endTime;
		}

		public void append(EntityID newEnd, List<EntityID> newTrail, int newEndTime) {
			this.end = newEnd;
			this.endTime = newEndTime;

			trail.addAll(newTrail);

			trail = ListDereplication.dereplication(trail);
		}

		public void append(EntityID newEnd, Route<Area> ra, int newEndTime) {
			this.end = newEnd;
			this.endTime = newEndTime;

			for (Area a : ra.getRoute()) {
				trail.add(a.getID());
			}

			trail = ListDereplication.dereplication(trail);
		}

		public void append1(EntityID newEnd, EntityID newTrailOne, int newEndTime) {
			this.end = newEnd;
			this.endTime = newEndTime;

			trail.add(newTrailOne);
		}

		public void append2(EntityID newEnd, EntityID newTrailOne, EntityID newTrailTwo, int newEndTime) {
			this.end = newEnd;
			this.endTime = newEndTime;

			trail.add(newTrailOne);

			if (!newTrailOne.equals(newTrailTwo))
				trail.add(newTrailTwo);
		}

		public EntityID getStart() {
			return start;
		}

		public void setStart(EntityID start) {
			this.start = start;
		}

		public EntityID getEnd() {
			return end;
		}

		public void setEnd(EntityID end) {
			this.end = end;
		}

		public List<EntityID> getTrail() {
			return trail;
		}

		public void setTrail(List<EntityID> trail) {
			this.trail = trail;
		}

		public int getStartTime() {
			return startTime;
		}

		public void setStartTime(int startTime) {
			this.startTime = startTime;
		}

		public int getEndTime() {
			return endTime;
		}

		public void setEndTime(int endTime) {
			this.endTime = endTime;
		}

		public boolean isInvalid() {
			return (start == null || end == null);
		}

		@Override
		public String toString() {
			if (isInvalid())
				return "Trail-Ill [" + start + "]";
			else
				return "Trail [" + trail + "]";
		}

	}

}
