package ant.component.universal;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractAntCommunicationKit;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.agent.behavior.AntBehavior;
import ant.component.universal.LineOfSightPerceptionComponent.LineInfo;
import ant.liaison.message.IBuildingCivilianMessage;
import ant.liaison.message.IMessage;
import ant.liaison.message.MBuildingCivilian;
import ant.utils.civilian.CivilianInfo;

/**
 * 
 * @author Oliver
 * @depend {@link ChangeSetDetailComponent}
 *         {@link LineOfSightPerceptionComponent} {@link HumanInfoComponent}
 */
public class BuildingWithNoCivilianComponent extends AbstractMessageProducerComponent {
	private static Log log = LogFactory.getLog(BuildingWithNoCivilianComponent.class);

	private AntBehavior<? extends Human> behavior;
	private AgentContext context;
	private ChangeSetDetailComponent csd;
	private LineOfSightPerceptionComponent los;
	private HumanInfoComponent hic;
	private Map<EntityID, Set<EntityID>> buildingCivilianMap;
	private AbstractAntCommunicationKit cmu;
	private HashSet<EntityID> buildingWithNoCivilianSet;

	private int checkValidCount;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.behavior = context.getBehavior();
		this.csd = context.getAntComponentKit().getChangeSetDetailComponent();
		this.los = (LineOfSightPerceptionComponent) context.getAntComponentKit().getAntComponent(
				LineOfSightPerceptionComponent.class);
		this.hic = (HumanInfoComponent) context.getAntComponentKit().getAntComponent(
				HumanInfoComponent.class);
		this.buildingCivilianMap = hic.getBuildingCivilianMap();
		this.cmu = context.getAntCommunicationKit();

		this.buildingWithNoCivilianSet = new HashSet<EntityID>();

		this.checkValidCount = 0;
	}

	@Override
	public void update() {
		updateFromMessage(cmu.getRadioReceiveBox());
		updateFromMessage(cmu.getVoiceReceiveBox());
		updateFromOtherComponents();

		checkAroundBuildings();
		// if (context.getTimestamp() % 51 == 0) {
		// log.info("checkValidCount:" + checkValidCount);
		// }
	}

	private void updateFromOtherComponents() {
		for (CivilianInfo info : hic.getCivIncNow()) {
			buildingWithNoCivilianSet.remove(info.getPosition());
		}
		for (CivilianInfo info : hic.getCivDelNow()) {
			EntityID location = info.getPosition();
			if (!isBuildingWithCivilian(location)) {
				buildingWithNoCivilianSet.add(location);
			}
		}
	}

	private void updateFromMessage(Set<IMessage> messages) {
		for (IMessage msg : messages) {
			if (msg instanceof IBuildingCivilianMessage) {
				IBuildingCivilianMessage bcm = (IBuildingCivilianMessage) msg;
				if (bcm.isCivilianInBuilding()) {
					buildingWithNoCivilianSet.remove(bcm.getLocation());
				} else {
					buildingWithNoCivilianSet.add(bcm.getLocation());
				}
			}
			// TODO： else if(other case)
		}
	}

	private void checkAroundBuildings() {
		StandardEntity here = context.getHuman().getPosition(context.getWorld());
		if (!(here instanceof Area)) {
			return;
		}

		Set<LineInfo> lines = new HashSet<LineInfo>();
		for (StandardEntity entity : csd.getEntitiesInSight(StandardEntityURN.BUILDING)) {
			lines.addAll(los.getBuidlingLines((Building) entity));
		}
		for (StandardEntity entity : csd.getEntitiesInSight(StandardEntityURN.BUILDING)) {
			Building b = (Building) entity;
			if (!buildingWithNoCivilianSet.contains(b.getID())
					&& !isBuildingWithCivilian(b.getID())
					&& los.canSeeBuildingXY(context.getHuman().getX(), context.getHuman().getY(),
							b.getX(), b.getY(), lines)) {
				// can see center of building, but found no civilian in
				// building
				buildingWithNoCivilianSet.add(b.getID());
				checkValidCount++;

				MBuildingCivilian msg = new MBuildingCivilian(b.getID(), false);
				addRadioMessage(msg);
				addVoiceMessage(msg);
			}
		}
	}

	private boolean isBuildingWithCivilian(EntityID builduingID) {
		Set<EntityID> set = buildingCivilianMap.get(builduingID);
		if (set == null || set.isEmpty()) {
			return false;
		}
		return true;
	}

	public boolean isBuildingWithNoCivilian(EntityID buildingID) {
		return buildingWithNoCivilianSet.contains(buildingID);
	}
}
