package ant.component.universal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import rescuecore2.worldmodel.Property;
import ant.AgentContext;
import ant.AntComponent;

/**
 * 
 * @author Oliver
 * @depend
 */
public class ChangeSetDetailComponent implements AntComponent {
	private static Log log = LogFactory.getLog(ChangeSetDetailComponent.class);
	private AgentContext context;
	private StandardWorldModel world;

	private Map<StandardEntityURN, Map<EntityID, EntityHistoryInfo>> previousEntitiesInSight;
	private Map<StandardEntityURN, Map<EntityID, StandardEntity>> entitiesInSight;
	private Map<StandardEntityURN, Map<EntityID, StandardEntity>> entitiesChanged;

	public ChangeSetDetailComponent() {
		previousEntitiesInSight = buildPreviousEntitiesInSightMap();
		entitiesInSight = buildEntityMap();
		entitiesChanged = buildEntityMap();
	}

	private Map<StandardEntityURN, Map<EntityID, EntityHistoryInfo>> buildPreviousEntitiesInSightMap() {
		Map<StandardEntityURN, Map<EntityID, EntityHistoryInfo>> map = new HashMap<StandardEntityURN, Map<EntityID, EntityHistoryInfo>>();
		map.put(StandardEntityURN.ROAD, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.BLOCKADE, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.BUILDING, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.REFUGE, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.FIRE_STATION, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.AMBULANCE_CENTRE, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.POLICE_OFFICE, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.CIVILIAN, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.FIRE_BRIGADE, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.AMBULANCE_TEAM, new HashMap<EntityID, EntityHistoryInfo>());
		map.put(StandardEntityURN.POLICE_FORCE, new HashMap<EntityID, EntityHistoryInfo>());
		return map;
	}

	private Map<StandardEntityURN, Map<EntityID, StandardEntity>> buildEntityMap() {
		Map<StandardEntityURN, Map<EntityID, StandardEntity>> map = new HashMap<StandardEntityURN, Map<EntityID, StandardEntity>>();
		map.put(StandardEntityURN.ROAD, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.BLOCKADE, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.BUILDING, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.REFUGE, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.FIRE_STATION, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.AMBULANCE_CENTRE, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.POLICE_OFFICE, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.CIVILIAN, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.FIRE_BRIGADE, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.AMBULANCE_TEAM, new HashMap<EntityID, StandardEntity>());
		map.put(StandardEntityURN.POLICE_FORCE, new HashMap<EntityID, StandardEntity>());
		return map;
	}

	private void clearEntityMap(Map<StandardEntityURN, Map<EntityID, StandardEntity>> entityMap) {
		for (Map<EntityID, StandardEntity> m : entityMap.values()) {
			m.clear();
		}
	}

	public String printEnitityMap(Map<StandardEntityURN, Map<EntityID, StandardEntity>> entityMap) {
		String info = "";
		for (Iterator<Entry<StandardEntityURN, Map<EntityID, StandardEntity>>> it = entityMap
				.entrySet().iterator(); it.hasNext();) {
			Entry<StandardEntityURN, Map<EntityID, StandardEntity>> entry = it.next();
			info += "\n" + entry.getKey();
			for (Iterator<Entry<EntityID, StandardEntity>> _it = entry.getValue().entrySet()
					.iterator(); _it.hasNext();) {
				Entry<EntityID, StandardEntity> _entry = _it.next();
				info += "\n\t" + _entry.getKey() + ":" + _entry.getValue();
			}
		}
		return info;
	}

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
	}

	@Override
	public void update() {
		ChangeSet changeSet = context.getChangeSet();
		updateEntitesInSight(changeSet);
		updateEntitiesChanged();
		// debug();
		updatePreviousEntitesInSight();

		// TODO not used yet
		Set<EntityID> deletedEntities = changeSet.getDeletedEntities();
		if (deletedEntities.size() != 0) {
			// log.warn("deletedEntities in changeSet is not empty!! " +
			// deletedEntities);
			// blockade deleted
			// attention never use changeSet.getChangedProperties() to search
			// entity that is not existed
		}
	}

	public void debug() {
		String info = "";
		// info +=
		// "\n******************************entitiesInSight******************************"
		// + printEnitityMap(entitiesInSight);
		// info +=
		// "\n******************************previousEntitiesInSight******************************"
		// + printEnitityMap(previousEntitiesInSight);
		info += "\n******************************entitiesChanged******************************"
				+ printEnitityMap(entitiesChanged);
		log.info(info);
	}

	private void updatePreviousEntitesInSight() {
		for (Iterator<Entry<StandardEntityURN, Map<EntityID, StandardEntity>>> it = entitiesInSight
				.entrySet().iterator(); it.hasNext();) {
			Entry<StandardEntityURN, Map<EntityID, StandardEntity>> entry = it.next();
			Map<EntityID, StandardEntity> map = entry.getValue();
			Map<EntityID, EntityHistoryInfo> preMap = previousEntitiesInSight.get(entry.getKey());
			for (Iterator<Entry<EntityID, StandardEntity>> _it = map.entrySet().iterator(); _it
					.hasNext();) {
				Entry<EntityID, StandardEntity> _entry = _it.next();
				EntityID id = _entry.getKey();
				StandardEntity entity = _entry.getValue();
				EntityHistoryInfo preValue = preMap.get(id);
				if (preValue == null) {
					preMap.put(id, getEntityHistoryInfo(entity));
				} else {
					preValue.update(context.getTimestamp(), entity);
				}
			}
		}
	}

	private EntityHistoryInfo getEntityHistoryInfo(StandardEntity entity) {
		int updateTime = context.getTimestamp();
		switch (entity.getStandardURN()) {
		case ROAD:
			return new RoadHistoryInfo(updateTime, (Road) entity);
		case BLOCKADE:
			return new BlockadeHistoryInfo(updateTime, (Blockade) entity);
		case BUILDING:
		case REFUGE:
		case AMBULANCE_CENTRE:
		case FIRE_STATION:
		case POLICE_OFFICE:
			return new BuildingHistoryInfo(updateTime, (Building) entity);
		case CIVILIAN:
		case AMBULANCE_TEAM:
		case FIRE_BRIGADE:
		case POLICE_FORCE:
			return new HumanHistoryInfo(updateTime, (Human) entity);
		default:
			return null;
		}
	}

	public void updateEntitesInSight(ChangeSet changeSet) {
		clearEntityMap(entitiesInSight);

		for (EntityID id : changeSet.getChangedEntities()) {
			StandardEntityURN urn = StandardEntityURN.fromString(changeSet.getEntityURN(id));
			Map<EntityID, StandardEntity> map = entitiesInSight.get(urn);
			if (map != null) {
				map.put(id, world.getEntity(id));
			} else {
				log.error("UnKnown URN:" + urn);
			}
		}
	}

	private void updateEntitiesChanged() {
		clearEntityMap(entitiesChanged);

		for (Iterator<Entry<StandardEntityURN, Map<EntityID, StandardEntity>>> it = entitiesInSight
				.entrySet().iterator(); it.hasNext();) {
			Entry<StandardEntityURN, Map<EntityID, StandardEntity>> entry = it.next();
			StandardEntityURN urn = entry.getKey();
			Map<EntityID, EntityHistoryInfo> preMap = previousEntitiesInSight.get(urn);
			Map<EntityID, StandardEntity> changedMap = entitiesChanged.get(urn);
			for (Iterator<Entry<EntityID, StandardEntity>> _it = entry.getValue().entrySet()
					.iterator(); _it.hasNext();) {
				Entry<EntityID, StandardEntity> _entry = _it.next();
				EntityID id = _entry.getKey();
				StandardEntity value = _entry.getValue();
				EntityHistoryInfo preValue = preMap.get(id);
				if (preValue == null || preValue.hasBeenUpdated(value)) {
					changedMap.put(id, value);
				}
			}
		}

		// 额外处理blockade变化，认为blockade变化，即为其上的road发生了变化
		Map<EntityID, StandardEntity> roadsChangedMap = entitiesChanged.get(StandardEntityURN.ROAD);
		for (StandardEntity entity : entitiesChanged.get(StandardEntityURN.BLOCKADE).values()) {
			EntityID r = ((Blockade) entity).getPosition();
			if (!roadsChangedMap.containsKey(r)) {
				roadsChangedMap.put(r, world.getEntity(r));
			}
		}
	}

	public Map<EntityID, StandardEntity> getEntitiesInSightMap(StandardEntityURN urn) {
		return entitiesInSight.get(urn);
	}

	public List<StandardEntity> getEntitiesInSight(StandardEntityURN urn) {
		return new ArrayList<StandardEntity>(entitiesInSight.get(urn).values());
	}

	public List<StandardEntity> getEntitiesInSight(StandardEntityURN... urns) {
		List<StandardEntity> rv = new ArrayList<StandardEntity>();
		for (StandardEntityURN urn : urns) {
			Map<EntityID, StandardEntity> map = entitiesInSight.get(urn);
			if (map != null) {
				rv.addAll(map.values());
			} else {
				log.error("UnKnown URN:" + urn);
			}
		}
		return rv;
	}

	public List<Human> getAgentsInSight() {
		return getAgentsInSight(StandardEntityURN.FIRE_BRIGADE, StandardEntityURN.POLICE_FORCE,
				StandardEntityURN.AMBULANCE_TEAM);
	}

	public List<Human> getAgentsInSight(StandardEntityURN... urns) {
		List<Human> rv = new ArrayList<Human>();
		for (StandardEntityURN urn : urns) {
			if (urn == StandardEntityURN.FIRE_BRIGADE || urn == StandardEntityURN.POLICE_FORCE
					|| urn == StandardEntityURN.AMBULANCE_TEAM) {
				Map<EntityID, StandardEntity> map = entitiesInSight.get(urn);
				for (StandardEntity entity : map.values()) {
					Human h = (Human) entity;
					rv.add(h);
				}
			} else {
				throw new IllegalArgumentException("urn:" + urn + " illegal!");
			}
		}
		return rv;
	}

	public Map<EntityID, StandardEntity> getChangedEntities(StandardEntityURN urn) {
		return entitiesChanged.get(urn);
	}

	public void printChangeSet() {
		ChangeSet changeSet = context.getChangeSet();
		String info = "changeSet @ " + context.getTimestamp() + " in position "
				+ context.getHuman().getPosition() + "[" + context.getHuman().getX() + ","
				+ context.getHuman().getY() + "]";
		Set<EntityID> deletedEntities = changeSet.getDeletedEntities();
		info += "\ndeletedEntities:" + deletedEntities;
		for (EntityID id : changeSet.getChangedEntities()) {
			Set<Property> changedProperties = changeSet.getChangedProperties(id);
			String urn = changeSet.getEntityURN(id);

			info += "\nid:" + id + " " + urn + "\n\tchangedProperties:";
			for (Property p : changedProperties) {
				info += "\n\t\t" + p;
			}
		}
		log.info(info);
	}

	private static abstract class EntityHistoryInfo {
		protected static final int INVALID_INT_VALUE = -1;
		private int updateTime;

		public EntityHistoryInfo(int updateTime) {
			this.updateTime = updateTime;
		}

		public int getUpdateTime() {
			return updateTime;
		}

		public void setUpdateTime(int updateTime) {
			this.updateTime = updateTime;
		}

		public abstract boolean hasBeenUpdated(StandardEntity entity);

		public abstract void update(int time, StandardEntity entity);
	}

	private static class BlockadeHistoryInfo extends EntityHistoryInfo {
		private int[] apexes;
		private int repairCost;

		public BlockadeHistoryInfo(int updateTime, Blockade b) {
			super(updateTime);
			setBlockade(b);
		}

		private void setBlockade(Blockade b) {
			apexes = b.isApexesDefined() ? b.getApexes() : null;
			repairCost = b.isRepairCostDefined() ? b.getRepairCost() : INVALID_INT_VALUE;
		}

		@Override
		public boolean hasBeenUpdated(StandardEntity entity) {
			Blockade b = (Blockade) entity;

			if (b.isApexesDefined()) {
				int[] _apexes = b.getApexes();
				if (apexes == null || apexes.length != _apexes.length) {
					return true;
				}

				for (int i = 0; i < apexes.length; i++) {
					if (apexes[i] != _apexes[i]) {
						return true;
					}
				}
			} else if (apexes != null) {
				return true;
			}

			if ((b.isRepairCostDefined() && b.getRepairCost() != repairCost)
					|| (!b.isRepairCostDefined() && repairCost != INVALID_INT_VALUE)) {
				return true;
			}

			return false;
		}

		@Override
		public void update(int time, StandardEntity entity) {
			setUpdateTime(time);
			setBlockade((Blockade) entity);
		}

	}

	private static class BuildingHistoryInfo extends EntityHistoryInfo {
		private int ignition;
		private int fieryness;
		private int temperature;

		public BuildingHistoryInfo(int updateTime, Building b) {
			super(updateTime);
			setBuilding(b);
		}

		private void setBuilding(Building b) {
			ignition = b.isIgnitionDefined() ? (b.getIgnition() ? 1 : 0) : INVALID_INT_VALUE;
			fieryness = b.isFierynessDefined() ? b.getFieryness() : INVALID_INT_VALUE;
			temperature = b.isTemperatureDefined() ? b.getTemperature() : INVALID_INT_VALUE;
		}

		@Override
		public boolean hasBeenUpdated(StandardEntity entity) {
			Building b = (Building) entity;
			if ((b.isIgnitionDefined() && (b.getIgnition() ? 1 : 0) != ignition)
					|| (!b.isIgnitionDefined() && ignition != INVALID_INT_VALUE)) {
				return true;
			}
			if ((b.isFierynessDefined() && b.getFieryness() != fieryness)
					|| (!b.isFierynessDefined() && fieryness != INVALID_INT_VALUE)) {
				return true;
			}
			if ((b.isTemperatureDefined() && b.getTemperature() != temperature)
					|| (!b.isTemperatureDefined() && temperature != INVALID_INT_VALUE)) {
				return true;
			}
			return false;
		}

		@Override
		public void update(int time, StandardEntity entity) {
			setUpdateTime(time);
			setBuilding((Building) entity);
		}

	}

	private static class HumanHistoryInfo extends EntityHistoryInfo {
		private int x;
		private int y;
		private EntityID position;
		private int travelDistance;
		private int direction;
		private int stamina;
		private int hp;
		private int damage;
		private int buriedness;

		public HumanHistoryInfo(int updateTime, Human h) {
			super(updateTime);
			setHuman(h);
		}

		private void setHuman(Human h) {
			x = h.isXDefined() ? h.getX() : INVALID_INT_VALUE;
			y = h.isYDefined() ? h.getY() : INVALID_INT_VALUE;
			position = h.isPositionDefined() ? h.getPosition() : new EntityID(INVALID_INT_VALUE);
			travelDistance = h.isTravelDistanceDefined() ? h.getTravelDistance()
					: INVALID_INT_VALUE;
			direction = h.isDirectionDefined() ? h.getDirection() : INVALID_INT_VALUE;
			stamina = h.isStaminaDefined() ? h.getStamina() : INVALID_INT_VALUE;
			hp = h.isHPDefined() ? h.getHP() : INVALID_INT_VALUE;
			damage = h.isDamageDefined() ? h.getDamage() : INVALID_INT_VALUE;
			buriedness = h.isBuriednessDefined() ? h.getBuriedness() : INVALID_INT_VALUE;
		}

		@Override
		public boolean hasBeenUpdated(StandardEntity entity) {
			Human h = (Human) entity;
			if ((h.isXDefined() && h.getX() != x) || (!h.isXDefined() && x != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isYDefined() && h.getY() != y) || (!h.isYDefined() && y != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isPositionDefined() && !h.getPosition().equals(position))
					|| (!h.isPositionDefined() && position.getValue() != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isTravelDistanceDefined() && h.getTravelDistance() != travelDistance)
					|| (!h.isTravelDistanceDefined() && travelDistance != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isDirectionDefined() && h.getDirection() != direction)
					|| (!h.isDirectionDefined() && direction != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isStaminaDefined() && h.getStamina() != stamina)
					|| (!h.isStaminaDefined() && stamina != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isHPDefined() && h.getHP() != hp)
					|| (!h.isHPDefined() && hp != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isDamageDefined() && h.getDamage() != damage)
					|| (!h.isDamageDefined() && damage != INVALID_INT_VALUE)) {
				return true;
			}
			if ((h.isBuriednessDefined() && h.getBuriedness() != buriedness)
					|| (!h.isBuriednessDefined() && buriedness != INVALID_INT_VALUE)) {
				return true;
			}

			return false;
		}

		@Override
		public void update(int time, StandardEntity entity) {
			setUpdateTime(time);
			setHuman((Human) entity);
		}
	}

	private static class RoadHistoryInfo extends EntityHistoryInfo {
		private List<EntityID> blockades;

		public RoadHistoryInfo(int updateTime, Road r) {
			super(updateTime);
			setRoad(r);
		}

		public void setRoad(Road r) {
			if (r.isBlockadesDefined()) {
				blockades = new ArrayList<EntityID>(r.getBlockades());
			} else {
				blockades = null;
			}
		}

		@Override
		public boolean hasBeenUpdated(StandardEntity entity) {
			Road r = (Road) entity;
			if ((r.isBlockadesDefined() && !r.getBlockades().equals(blockades))
					|| (!r.isBlockadesDefined() && blockades != null)) {
				return true;
			}
			return false;
		}

		public void update(int time, StandardEntity entity) {
			super.setUpdateTime(time);
			setRoad((Road) entity);
		}

	}
}
