package ant.component.universal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.agent.behavior.AntBehavior;
import ant.component.universal.BuildingConditionComponent.BuildingCondition;
import ant.liaison.message.IMessage;
import ant.liaison.message.IVoiceAgentBuried;
import ant.liaison.message.MAgentBuried;
import ant.liaison.message.MAgentRescued;
import ant.liaison.message.MCivilianRescued;
import ant.liaison.message.MFindCivilianInBuilding;
import ant.liaison.message.MFindCivilianOnRoad;
import ant.liaison.message.MVoiceAgentBuriedSimple;
import ant.map.component.StaticCityGraphComponent;
import ant.utils.Pair;
import ant.utils.civilian.BuriedAgentInfo;
import ant.utils.civilian.CivilianInfo;
import ant.utils.comparator.AgentDeathTimeComparator;
import ant.utils.comparator.DeathTimeComparator;

/**
 * 维护Civilian信息，处理MFindCivilian,MCivilianRescued
 * 
 * @author Fun
 * @depend {@link ChangeSetDetailComponent} {@link StaticCityGraphComponent} {@link BuildingConditionComponent}
 */
public class HumanInfoComponent extends AbstractMessageProducerComponent {
	private static Log log = LogFactory.getLog(HumanInfoComponent.class);

	private AgentContext context;
	private StandardWorldModel world;
	private StaticCityGraphComponent scg;
	private AntBehavior behavior;
	private Set<EntityID> refugesID;
	private Map<EntityID, CivilianInfo> civilianInfoMap;
	private Set<EntityID> ignoredCiv;

	private Map<EntityID, BuriedAgentInfo> needHelpAgentMap;
	private Set<EntityID> ignoredAgents;

	private Set<CivilianInfo> civIncNow; // 当前周期增删Civilian集合
	private Set<CivilianInfo> civDelNow;

	private Map<EntityID, Set<EntityID>> buildingCivilianMap;// Building ID ->
																// Civilian Set

	// transfer msg
	public static final int TRANSFER_RESCUEDCIVS_MSG_NUM = 4;
	public static final int RESCUEDCIVS_MSG_TIME_THRESHOLD = 40;

	public static final int TRANSFER_RESCUEDAGENTS_MSG_NUM = 1;
	public static final int RESCUEDAGENTS_MSG_TIME_THRESHOLD = 50;

	public static final int TRANSFER_FINDCIV_MSG_NUM = 12;
	public static final int TRANSFER_AGENTBURIED_MSG_NUM = 3;

	public static final int MIN_RESCUE_TIME = 4;

	private List<Pair<Integer, EntityID>> rescuedCivilians; // rescued 信息转发

	private List<Pair<Integer, EntityID>> rescuedAgents; // rescued Agent 信息转发

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;

		this.scg = context.getAntComponentKit().getStaticCityGraphComponent();
		this.behavior = context.getBehavior();

		this.refugesID = new HashSet<EntityID>();
		this.civilianInfoMap = new HashMap<EntityID, CivilianInfo>();
		this.ignoredCiv = new HashSet<EntityID>();

		this.needHelpAgentMap = new HashMap<EntityID, BuriedAgentInfo>();
		this.ignoredAgents = new HashSet<EntityID>();

		this.buildingCivilianMap = new HashMap<EntityID, Set<EntityID>>();
		this.rescuedCivilians = new ArrayList<Pair<Integer, EntityID>>();
		this.rescuedAgents = new ArrayList<Pair<Integer, EntityID>>();

		this.civIncNow = new HashSet<CivilianInfo>();
		this.civDelNow = new HashSet<CivilianInfo>();

		for (StandardEntity next : world) {
			if (next instanceof Civilian) {
				Civilian civilian = (Civilian) next;
				CivilianInfo ci = new CivilianInfo(civilian.getID(),
						civilian.getPosition(),
						civilian.isHPDefined() ? civilian.getHP() : 0,
						civilian.isDamageDefined() ? civilian.getDamage() : 0,
						civilian.isBuriednessDefined() ? civilian
								.getBuriedness() : 0, context.getTimestamp());
				ci.setTimeCostToRefuge(getTimeToNearestRefugeOfMap(civilian
						.getPosition()));
				civilianInfoMap.put(civilian.getID(), ci);

				StandardEntity se = world.getEntity(ci.getPosition());
				if (se != null && se instanceof Building) {
					Set<EntityID> cset = buildingCivilianMap.get(se.getID());
					if (cset == null) {
						cset = new HashSet<EntityID>();
						buildingCivilianMap.put(se.getID(), cset);
					}
					cset.add(ci.getID());
				}

			} else if (next instanceof Refuge) {
				refugesID.add(next.getID());
			}
		}

	}

	@Override
	public void update() {
		if (context.getHuman().getID().getValue() < 0) {
			// in case viewer launch component
			return;
		}

		this.civIncNow.clear();
		this.civDelNow.clear();

		// 用location更新Agent信息 TODO

		updateFromObserving();
		updateFromRadio();
		updateFromVoice();
		checkBuildingCondition();
		
		sendHumanInfoMsg();
	}
	
	private void checkBuildingCondition(){
		BuildingConditionComponent bcc= context.getAntComponentKit().getBuildingConditionComponent();
		Set<EntityID> toRemove = new HashSet<EntityID>();
		for(EntityID bid: buildingCivilianMap.keySet()){
			BuildingCondition bc = bcc.getBuildingCondition(bid);
			if(bc.isBurntOut()){
				Set<EntityID> civs = buildingCivilianMap.get(bid);
				toRemove.addAll(civs);
			}
		}
		for(EntityID cid: toRemove){
			ignoreCivilian(cid);
		}
	}

	private void updateFromObserving() {
		// Civilian
		Collection<StandardEntity> civiliansInSight = context
				.getAntComponentKit().getChangeSetDetailComponent()
				.getEntitiesInSightMap(StandardEntityURN.CIVILIAN).values();
		for (StandardEntity se : civiliansInSight) {
			Civilian c = (Civilian) se;
			StandardEntity position = world.getEntity(c.getPosition());

			if (isCivilianIgnored(c.getID())) { // 跳过忽略集合
				continue;
			}

			if (position instanceof AmbulanceTeam) { // loaded
				ignoreCivilian(c.getID());
				MCivilianRescued cr = new MCivilianRescued(c.getID());
				addVoiceMessage(cr); // 只发送Voice
				continue;
			}
			if (!(position instanceof Building)) { // not in building
				continue;
			}
			if (position instanceof Refuge) { // saved
				ignoreCivilian(c.getID());
				// MCivilianRescued cr = new MCivilianRescued(c.getID());
				// addVoiceMessage(cr); //只发送Voice
				continue;
			}

			if (!c.isBuriednessDefined()) {
				// ignoredCiv.add(c.getID());
				continue;
			}

			if (!c.isHPDefined()) {
				ignoreCivilian(c.getID());
				MCivilianRescued cr = new MCivilianRescued(c.getID());
				addVoiceMessage(cr); // 发送Voice，Radio
				addRadioMessage(cr);
				continue;
			}

			if (!c.isDamageDefined()) {
				continue;
			}

			int hp = c.getHP();
			int dmg = c.getDamage();
			int buriedness = c.getBuriedness();

			if (dmg == 0 && buriedness == 0) { // 忽略可以移动的
				continue;
			}

			if (hp <= 0) {
				if (!isCivilianIgnored(c.getID())) { // 如未被忽略
					MCivilianRescued cr = new MCivilianRescued(c.getID());
					addVoiceMessage(cr); // 发送Voice，Radio
					addRadioMessage(cr);
					ignoreCivilian(c.getID());
				}
			}

			updateCivilianInfo(c.getID(), position.getID(), hp, dmg,
					buriedness, context.getTimestamp());
		}

		// 忽略Civilian不再位于原building中
		Area location = behavior.getMyPosition();
		if (location instanceof Building) {
			if (buildingCivilianMap.containsKey(location.getID())) {
				Set<EntityID> civset = buildingCivilianMap
						.get(location.getID());
				if (civset == null) {
					log.error("civset == null!");
				} else {
					Iterator<EntityID> it = civset.iterator();
					while (it.hasNext()) {
						EntityID civ = it.next();
						if (!behavior.canSee(civ)) {
							ignoredCiv.add(civ);
							it.remove();
							addRadioMessage(new MCivilianRescued(civ));
							addVoiceMessage(new MCivilianRescued(civ));
							log.info("civ:" + civ + " is not in building "
									+ location.getID());
						}
					}
				}
			}
		}

		// Agent
		List<Human> agentAround = behavior.getNearbyAgents();

		for (Human human : agentAround) {
			if (isAgentIgnored(human.getID())) {
				continue;
			}

			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = world.getEntity(positionId);
				if (!(position instanceof Building)) {
					ignoreAgent(human.getID());
					continue;
				}

				if (position instanceof Refuge) {
					ignoreAgent(human.getID());
					continue;
				}
				if (!human.isBuriednessDefined()) {
					ignoreAgent(human.getID());
					continue;
				}
				if (human.getBuriedness() <= 0) {
					ignoreAgent(human.getID());
					continue;
				}
				if (!human.isHPDefined()) {
					ignoreAgent(human.getID());
					continue;
				}
				if (human.getHP() <= 0) {
					ignoreAgent(human.getID());
					continue;
				}

				int hp = human.getHP();
				int dmg = human.getDamage();
				int buriedness = human.getBuriedness();

				if (needHelpAgentMap.containsKey(human.getID())) {
					BuriedAgentInfo bai = needHelpAgentMap.get(human.getID());
					bai.update(
							hp,
							dmg,
							buriedness,
							behavior.getDeathTimeNaive(hp, dmg,
									context.getTimestamp()),
							context.getTimestamp());
				} else {
					BuriedAgentInfo bai = new BuriedAgentInfo(human.getID(),
							hp, dmg, buriedness, (Building) position,
							behavior.getDeathTimeNaive(hp, dmg,
									context.getTimestamp()),
							context.getTimestamp());
					needHelpAgentMap.put(human.getID(), bai);
				}
			}
		}

	}

	/**
	 * 处理MFindCivilian,MCivilianRescued
	 */
	private void updateFromRadio() {
		for (IMessage msg : context.getAntCommunicationKit()
				.getRadioReceiveBox()) {
			// Find Civilian for AT
			if (msg instanceof MFindCivilianInBuilding) {
				MFindCivilianInBuilding fcm = (MFindCivilianInBuilding) msg;
				EntityID cid = fcm.getId();
				EntityID location = fcm.getPosition();
				int hp = fcm.getHp();
				int dmg = fcm.getDamage();
				int buriedness = fcm.getBuriedness();

				if (dmg == 0 && buriedness == 0) { // 忽略可以移动的
					continue;
				}

				updateCivilianInfo(cid, location, hp, dmg, buriedness,
						fcm.getTimestamp());
			}

			// Civilian Rescued for AT
			if (msg instanceof MCivilianRescued) {
				MCivilianRescued cr = (MCivilianRescued) msg;
				EntityID cid = cr.getCivilian();
				if (!isCivilianIgnored(cid)) { // 未被忽略
					rescuedCivilians.add(new Pair<Integer, EntityID>(cr
							.getTimestamp(), cid));
					ignoreCivilian(cid);
				}
			}

			// Agent Buried for AT
			if (msg instanceof MAgentBuried) {
				MAgentBuried ab = (MAgentBuried) msg;
				EntityID agentid = ab.getSender();
				EntityID location = context.getAntComponentKit()
						.getAgentLocationComponent()
						.getAgentLastLocation(agentid);
				int hp = ab.getHp();
				int dmg = ab.getDamage();
				int buriedness = ab.getBuriedness();

				if (!isAgentIgnored(agentid)) {
					StandardEntity se = world.getEntity(location);
					if (se instanceof Building) {
						Building building = (Building) se;
						if (needHelpAgentMap.containsKey(agentid)) {
							BuriedAgentInfo bai = needHelpAgentMap.get(agentid);
							bai.update(
									hp,
									dmg,
									buriedness,
									behavior.getDeathTimeNaive(hp, dmg,
											context.getTimestamp()),
									ab.getTimestamp());

						} else {
							needHelpAgentMap.put(
									agentid,
									new BuriedAgentInfo(agentid, hp, dmg,
											buriedness, building,
											behavior.getDeathTimeNaive(hp, dmg,
													context.getTimestamp()), ab
													.getTimestamp()));
						}
					}
				}
			}

			// Agent Rescued for AT
			if (msg instanceof MAgentRescued) {
				MAgentRescued arm = (MAgentRescued) msg;
				EntityID aid = arm.getAgent();
				if (!isAgentIgnored(aid)) {
					rescuedAgents.add(new Pair<Integer, EntityID>(arm
							.getTimestamp(), aid));
					ignoreAgent(aid);
				}
			}

		}
	}

	private void updateFromVoice() {
		for (IMessage msg : context.getAntCommunicationKit()
				.getVoiceReceiveBox()) {
			// Find Civilian
			if (msg instanceof MFindCivilianInBuilding) {
				MFindCivilianInBuilding fcm = (MFindCivilianInBuilding) msg;
				EntityID cid = fcm.getId();
				EntityID location = fcm.getPosition();
				int hp = fcm.getHp();
				int dmg = fcm.getDamage();
				int buriedness = fcm.getBuriedness();

				if (dmg == 0 && buriedness == 0) { // 忽略可以移动的
					continue;
				}

				updateCivilianInfo(cid, location, hp, dmg, buriedness,
						fcm.getTimestamp());
			}
			if (msg instanceof MFindCivilianOnRoad) {
				MFindCivilianOnRoad fcm = (MFindCivilianOnRoad) msg;
				EntityID cid = fcm.getId();
				EntityID location = fcm.getPosition();
				int hp = fcm.getHp();
				int dmg = fcm.getDamage();
				int buriedness = fcm.getBuriedness();

				if (dmg == 0 && buriedness == 0) { // 忽略可以移动的
					continue;
				}

				updateCivilianInfo(cid, location, hp, dmg, buriedness,
						fcm.getTimestamp());

			}

			// Civilian Rescued
			if (msg instanceof MCivilianRescued) {
				MCivilianRescued cr = (MCivilianRescued) msg;
				EntityID cid = cr.getCivilian();
				if (!isCivilianIgnored(cid)) { // 未被忽略
					rescuedCivilians.add(new Pair<Integer, EntityID>(cr
							.getTimestamp(), cid));
					ignoreCivilian(cid);
				}
			}

			// Agent Buried
			if (msg instanceof IVoiceAgentBuried) {
				IVoiceAgentBuried vab = (IVoiceAgentBuried) msg;
				EntityID agentid = vab.getAgent();
				EntityID location = context.getAntComponentKit()
						.getAgentLocationComponent()
						.getAgentLastLocation(agentid);
				int hp = vab.getHp();
				int dmg = vab.getDamage();
				int buriedness = vab.getBuriedness();

				if (!isAgentIgnored(agentid)) {
					StandardEntity se = world.getEntity(location);
					if (se instanceof Building) {
						Building building = (Building) se;
						if (needHelpAgentMap.containsKey(agentid)) {
							BuriedAgentInfo bai = needHelpAgentMap.get(agentid);
							bai.update(
									hp,
									dmg,
									buriedness,
									behavior.getDeathTimeNaive(hp, dmg,
											context.getTimestamp()),
									msg.getTimestamp());
						} else {
							needHelpAgentMap.put(
									agentid,
									new BuriedAgentInfo(agentid, hp, dmg,
											buriedness, building,
											behavior.getDeathTimeNaive(hp, dmg,
													context.getTimestamp()),
											msg.getTimestamp()));
						}
					}
				}
			}

			// Agent Rescued
			if (msg instanceof MAgentRescued) {
				MAgentRescued arm = (MAgentRescued) msg;
				EntityID aid = arm.getAgent();
				if (!isAgentIgnored(aid)) {
					rescuedAgents.add(new Pair<Integer, EntityID>(arm
							.getTimestamp(), aid));
					ignoreAgent(aid);
				}
			}
		}
	}

	private void sendHumanInfoMsg() {
		updateMsgs();

		List<CivilianInfo> civilianInfos = getUpdatedCivilianInfosToSend();

		for (CivilianInfo next : civilianInfos) {
			EntityID location = next.getPosition();
			StandardEntity se = world.getEntity(location);
			if (se instanceof Building) {
				MFindCivilianInBuilding fcm = new MFindCivilianInBuilding(
						next.getID(), next.getPosition(), next.getHp(),
						next.getDmg(), next.getBuriedness());
				fcm.setTimestamp(next.getLastUpdateTime());
				addVoiceMessage(fcm);
			} else if (se instanceof Road) {
				MFindCivilianOnRoad fcm = new MFindCivilianOnRoad(next.getID(),
						next.getPosition(), next.getHp(), next.getDmg(),
						next.getBuriedness());
				fcm.setTimestamp(next.getLastUpdateTime());
				addVoiceMessage(fcm);
			} else {
				log.error("civilian loaction error!");
			}
		}

		List<BuriedAgentInfo> agentsInfos = getAgentInfoToSend();

		for (BuriedAgentInfo next : agentsInfos) {
			MVoiceAgentBuriedSimple absm = new MVoiceAgentBuriedSimple(
					next.getHp(), next.getDamage(), next.getBuriedness(),
					next.getAgentID(), next.getBuilding().getID());
			addVoiceMessage(absm);
		}

		// rescued
		for (Pair<Integer, EntityID> next : rescuedCivilians) {
			MCivilianRescued mr = new MCivilianRescued(next.second());
			mr.setTimestamp(next.first());
			addVoiceMessage(mr);
		}

		for (Pair<Integer, EntityID> next : rescuedAgents) {
			MAgentRescued arm = new MAgentRescued(next.second());
			arm.setTimestamp(next.first());
			addVoiceMessage(arm);
		}
	}

	private List<CivilianInfo> getUpdatedCivilianInfosToSend() {
		List<CivilianInfo> rescueableCollection = new ArrayList<CivilianInfo>();
		Iterator<Entry<EntityID, CivilianInfo>> civIt = civilianInfoMap
				.entrySet().iterator();
		while (civIt.hasNext()) {
			Entry<EntityID, CivilianInfo> entry = civIt.next();
			CivilianInfo ci = entry.getValue();

			// 检查是否需要忽略
			if (isCivilianIgnored(ci.getID())) {
				continue;
			}

			// 忽略死亡
			if (ci.getHp() <= 0) {
				continue;
			}

			if (ci.getPosition() == null) {
				continue;
			}

			// 忽略在Refuge中的
			if (refugesID.contains(ci.getPosition())) {
				continue;
			}

			// 更新死亡时间
			int deadTime = getDeadTime(ci);

			ci.setTimeToDie(deadTime);

			// 忽略来不及救援的对象
			if (deadTime <= context.getTimestamp() + MIN_RESCUE_TIME) {
				continue;
			}

			// if (!context.getAgentType()
			// .equals(StandardEntityURN.AMBULANCE_TEAM)) {
			// ci.setTimeToDie(deadTime);
			// }

			rescueableCollection.add(ci);
		}

		// 按死亡时间从小到大排序
		Collections.sort(rescueableCollection, new DeathTimeComparator());
		List<CivilianInfo> ret = new ArrayList<CivilianInfo>();
		for (int i = 0; i < rescueableCollection.size()
				&& i < TRANSFER_FINDCIV_MSG_NUM; i++) {
			ret.add(rescueableCollection.get(i));
		}

		return ret;
	}

	private List<BuriedAgentInfo> getAgentInfoToSend() {
		List<BuriedAgentInfo> agentColletion = new ArrayList<BuriedAgentInfo>();
		for (BuriedAgentInfo bai : needHelpAgentMap.values()) {
			EntityID aid = bai.getAgentID();
			if (isAgentIgnored(aid)) {
				continue;
			}

			int deadTime = bai.getDeathTime();

			if (deadTime < context.getTimestamp() + MIN_RESCUE_TIME) {
				continue;
			}

			agentColletion.add(bai);
		}
		Collections.sort(agentColletion, new AgentDeathTimeComparator());
		List<BuriedAgentInfo> ret = new ArrayList<BuriedAgentInfo>();
		for (int i = 0; i < agentColletion.size()
				&& i < TRANSFER_AGENTBURIED_MSG_NUM; i++) {
			ret.add(agentColletion.get(i));
		}

		return ret;
	}

	private int getDeadTime(CivilianInfo ci) {
		ci.getPredictor().step(context.getTimestamp());
		int deadTimeFromInjury;
		int offset = 60 - ci.getBuriedness();
		if (offset < 5) {
			offset = 5;
		} else if (offset > 55) {
			offset = 55;
		}
		deadTimeFromInjury = ci.getPredictor().getDeadTime()[offset];
		return deadTimeFromInjury;
	}

	private void updateMsgs() {
		// Civilian
		Collections.sort(rescuedCivilians, new TimePairComparator());
		List<Pair<Integer, EntityID>> tmp_rescuedCivilians = new ArrayList<Pair<Integer, EntityID>>();
		for (int i = 0; i < TRANSFER_RESCUEDCIVS_MSG_NUM
				&& i < rescuedCivilians.size(); i++) {
			Pair<Integer, EntityID> rescued = rescuedCivilians.get(i);
			if (context.getTimestamp() - rescued.first() >= RESCUEDCIVS_MSG_TIME_THRESHOLD) {
				break;
			}
			tmp_rescuedCivilians.add(rescued);
		}
		rescuedCivilians = tmp_rescuedCivilians;
		// Agent
		Collections.sort(rescuedAgents, new TimePairComparator());
		List<Pair<Integer, EntityID>> tmp_rescuedAgents = new ArrayList<Pair<Integer, EntityID>>();
		for (int i = 0; i < TRANSFER_RESCUEDAGENTS_MSG_NUM
				&& i < rescuedAgents.size(); i++) {
			Pair<Integer, EntityID> rescued = rescuedAgents.get(i);
			if (context.getTimestamp() - rescued.first() >= RESCUEDCIVS_MSG_TIME_THRESHOLD) {
				break;
			}
			tmp_rescuedAgents.add(rescued);
		}
		rescuedAgents = tmp_rescuedAgents;
	}

	public Set<EntityID> getRefugesID() {
		return refugesID;
	}

	public Map<EntityID, CivilianInfo> getCivilianInfoMap() {
		return civilianInfoMap;
	}

	public Map<EntityID, BuriedAgentInfo> getNeedHelpAgentMap() {
		return needHelpAgentMap;
	}

	public Map<EntityID, Set<EntityID>> getBuildingCivilianMap() {
		return buildingCivilianMap;
	}

	public Set<CivilianInfo> getCivIncNow() {
		return this.civIncNow;
	}

	public Set<CivilianInfo> getCivDelNow() {
		return this.civDelNow;
	}

	public void updateCivilianInfo(EntityID cid, EntityID position, int hp,
			int dmg, int buriedness, int timestamp) {
		if (civilianInfoMap.containsKey(cid)) { // update
			CivilianInfo ci = civilianInfoMap.get(cid);
			ci.updatePredictor(hp, dmg, buriedness, context.getTimestamp());
		} else { // add
			CivilianInfo ci = new CivilianInfo(cid, position, hp, dmg,
					buriedness, context.getTimestamp());
			ci.setTimeCostToRefuge(getTimeToNearestRefugeOfMap(cid));
			civilianInfoMap.put(cid, ci);

			civIncNow.add(ci);
		}

		StandardEntity se = world.getEntity(position);
		if (se != null && se instanceof Building && !isCivilianIgnored(cid)) {
			Set<EntityID> cset = buildingCivilianMap.get(se.getID());
			if (cset == null) {
				cset = new HashSet<EntityID>();
				buildingCivilianMap.put(se.getID(), cset);
			}
			cset.add(cid);
		}
	}

	public boolean isCivilianIgnored(EntityID cid) {
		return ignoredCiv.contains(cid);
	}

	public void setNotIgnoreCivilian(EntityID cid) {
		ignoredCiv.remove(cid);
		CivilianInfo ci = civilianInfoMap.get(cid);
		if (ci != null) {
			Set<EntityID> bcset = buildingCivilianMap.get(ci.getPosition());
			if (bcset == null) {
				bcset = new HashSet<EntityID>();
			}
			bcset.add(cid);
		}
	}

	public void ignoreCivilian(EntityID cid) {
		ignoredCiv.add(cid);
		CivilianInfo ci = civilianInfoMap.get(cid);
		if (ci != null) {
			Set<EntityID> bcset = buildingCivilianMap.get(ci.getPosition());
			if (bcset != null) {
				bcset.remove(cid);
			}

			civDelNow.add(ci);
		}
	}

	public boolean isAgentIgnored(EntityID agentId) {
		return ignoredAgents.contains(agentId);
	}

	public void ignoreAgent(EntityID agentId) {
		ignoredAgents.add(agentId);
	}

	public List<CivilianInfo> getUnsavedCivilians() {
		List<CivilianInfo> ret = new ArrayList<CivilianInfo>();
		for (CivilianInfo next : civilianInfoMap.values()) {
			if (isCivilianIgnored(next.getID())) {
				continue;
			}
			ret.add(next);
		}
		return ret;
	}

	public int getTimeToNearestRefugeOfMap(EntityID civLocateArea) {
		Set<Refuge> refuges = context.getAntComponentKit()
				.getAntMapBasisComponent().getRefugeSet();
		StandardEntity se = context.getWorld().getEntity(civLocateArea);
		if (se instanceof Area) {
			Area civloc = (Area) se;
			double dist = Integer.MAX_VALUE;
			for (Refuge refuge : refuges) {
				double tmp = scg.getDistanceToRefuge(civloc, refuge);
				if (tmp < dist) {
					dist = tmp;
				}
			}
			return (int) (dist / AntBehavior.AVERAGE_MOVE_DISTANCE);
		}
		return 5;
	}

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	/**
	 * 
	 * @author Fun Integer值大在前
	 */
	public static class TimePairComparator implements
			Comparator<Pair<Integer, EntityID>> {

		@Override
		public int compare(Pair<Integer, EntityID> arg0,
				Pair<Integer, EntityID> arg1) {
			return arg1.first() - arg0.first();
		}

	}

	public static void main(String[] args) {
		List<Pair<Integer, EntityID>> rescuedCivilians = new ArrayList<Pair<Integer, EntityID>>();
		rescuedCivilians.add(new Pair<Integer, EntityID>(1, new EntityID(1)));
		rescuedCivilians.add(new Pair<Integer, EntityID>(5, new EntityID(5)));
		rescuedCivilians.add(new Pair<Integer, EntityID>(3, new EntityID(3)));
		rescuedCivilians.add(new Pair<Integer, EntityID>(7, new EntityID(7)));
		rescuedCivilians.add(new Pair<Integer, EntityID>(3, new EntityID(3)));
		Collections.sort(rescuedCivilians, new TimePairComparator());
		System.out.println(rescuedCivilians);
	}
}
