package ant.component.universal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractMessageProducerComponent;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.universal.AgentLocationComponent.AgentLocationInfo;
import ant.liaison.RoadCoord;
import ant.liaison.message.IMessage;
import ant.liaison.message.IWho;
import ant.liaison.message.MAgentBuried;
import ant.liaison.message.MStuckInBuilding;
import ant.liaison.message.MStuckInRoad;
import ant.liaison.message.MVoiceAgentBuried;
import ant.liaison.message.MVoiceAgentStuckInBuilding;
import ant.liaison.message.MVoiceAgentStuckInRoad;
import ant.map.component.AntMapBasisComponent;
import ant.utils.CounterSet;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link AntMapBasisComponent} {@link AgentLocationComponent}
 */

// TODO Check myself
// TODO Check Unknown
// TODO 转发
public class JamComponent extends AbstractMessageProducerComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(JamComponent.class);
	private static final int DELIVER_MESSAGES = 5;
	private static final double INSIGHT_RANGE = 5000;
	private static final double REACH_DIST = 2000;
	private static final double STUCK_IN_ROAD_SKIP_DIST = 40000;

	public static enum JamState {
		Unknown, Buried, StuckInBuilding, StuckInRoad, Active
	}

	private StandardWorldModel world;
	private AgentContext context;
	private Map<EntityID, Jam> jmap;

	private AgentLocationComponent alc;
	private AntMapBasisComponent amb;
	private ChangeSetDetailComponent chs;

	@Override
	public String getComponentName() {
		return JamComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.world = world;
		this.context = context;
		this.jmap = new HashMap<EntityID, JamComponent.Jam>();
		this.alc = context.getAntComponentKit().getAgentLocationComponent();
		this.amb = context.getAntComponentKit().getAntMapBasisComponent();
		this.chs = context.getAntComponentKit().getChangeSetDetailComponent();

		CounterSet<EntityID> loccount = new CounterSet<EntityID>();
		for (Human h : amb.getAgentSet()) {
			loccount.inc(h.getPosition());
			jmap.put(h.getID(), new Jam(h));
		}
		for (Human h : amb.getAgentSet()) {
			if (loccount.count(h.getPosition()) > 1) {
				Area t = (Area) world.getEntity(h.getPosition());
				jmap.get(h.getID()).beliefSpawns.addAll(t.getNeighbours());
			}
		}
	}

	@Override
	public void update() {

		// TODO set myself spawn pos

		// Buried, StuckInBuilding, StuckInRoad
		for (IMessage im : context.getAntCommunicationKit().getVoiceReceiveBox()) {
			int time = im.getTimestamp();
			if (time < 0) {
				log.error("Bad Timestamp");
				return;
			}
			if (im instanceof IWho) {
				EntityID who = ((IWho) im).getWho();
				if (who == null || (!jmap.containsKey(who))) {
					log.error("Not found : " + who);
					continue;
				}

				Jam jam = jmap.get(who);
				if (time >= jam.lastUpdatedTimestamp) {
					jam.lastUpdatedTimestamp = time;

					if (im instanceof MVoiceAgentBuried) {
						MVoiceAgentBuried mvab = (MVoiceAgentBuried) im;
						jam.clear(JamState.Buried);
						jam.buriedBuilding = mvab.getBuilding();
						jam.beliefSpawns.clear();
						jam.beliefSpawns.add(jam.buriedBuilding);
					} else if (im instanceof MVoiceAgentStuckInRoad) {
						MVoiceAgentStuckInRoad mvasr = (MVoiceAgentStuckInRoad) im;
						if (mvasr.getRoadCoord() != null) {
							jam.clear(JamState.StuckInRoad);
							jam.stuckRoodCoord = mvasr.getRoadCoord();
						} else {
							log.error("Null rood coord:" + mvasr);
						}
					} else if (im instanceof MVoiceAgentStuckInBuilding) {
						MVoiceAgentStuckInBuilding mvasb = (MVoiceAgentStuckInBuilding) im;
						jam.clear(JamState.StuckInBuilding);
						jam.stuckBuilding = mvasb.getBuilding();
					}
				}
			}
		}

		for (IMessage im : context.getAntCommunicationKit().getRadioReceiveBox()) {
			int time = im.getTimestamp();
			if (time < 0) {
				log.error("Bad Timestamp");
				return;
			}
			EntityID who = im.getSender();
			if (who == null || (!jmap.containsKey(who))) {
				log.error("Not found : " + who);
				continue;
			}
			
			Jam jam = jmap.get(who);
			if (time >= jam.lastUpdatedTimestamp) {
				if (im instanceof MAgentBuried) {
					jam.lastUpdatedTimestamp = time;
					MAgentBuried mab = (MAgentBuried) im;
					jam.clear(JamState.Buried);
					jam.buriedBuilding = mab.getBuriedBuilding();
					jam.beliefSpawns.clear();
					jam.beliefSpawns.add(jam.buriedBuilding);
				} else if (im instanceof MStuckInRoad) {
					jam.lastUpdatedTimestamp = time;
					MStuckInRoad msr = (MStuckInRoad) im;
					if (msr.getRoadCoord() != null) {
						jam.clear(JamState.StuckInRoad);
						jam.stuckRoodCoord = msr.getRoadCoord();
					} else {
						log.error("Null Road Coord: " + msr);
					}
				} else if (im instanceof MStuckInBuilding) {
					jam.lastUpdatedTimestamp = time;
					MStuckInBuilding msb = (MStuckInBuilding) im;
					jam.clear(JamState.StuckInBuilding);
					jam.stuckBuilding = msb.getBuilding();
				}
			}
		}

		// TODO Update MySelf
		if (context.getHuman().isBuriednessDefined()) {
			if (context.getHuman().getBuriedness() > 0) {
				Jam jam = jmap.get(context.getHuman().getID());
				jam.clear(JamState.Buried);
				jam.buriedBuilding = context.getHuman().getPosition();
				jam.lastUpdatedTimestamp = context.getTimestamp();
			}
		}
		// TODO update myself buried

		// 脱离Jam状态
		EntityID imhere = context.getHuman().getPosition();
		Set<EntityID> ais = new HashSet<EntityID>();
		for (Human h : chs.getAgentsInSight()) {
			if (h.isHPDefined()) {
				if (h.getHP() == 0) {
					continue;
				}
			}
			ais.add(h.getID());
		}
		int ax = 0, ay = 0;
		Human human = context.getHuman();
		if (human.isXDefined() && human.isYDefined()) {
			ax = human.getX();
			ay = human.getY();
		} else if (human.isPositionDefined()) {
			Area position = (Area) human.getPosition(context.getWorld());
			ax = position.getX();
			ay = position.getY();
		} else {
			// log.error("human x, y and position is not defined!" + human);
		}

		int insightRange = ConfigKey.getPerceptionMaxDistance(context.getConfig());
		if (insightRange == 0) {
			insightRange = 500;
		} else {
			insightRange /= 1.5;
		}

		for (Entry<EntityID, Jam> entry : jmap.entrySet()) {
			EntityID stuckwho = entry.getKey();
			Jam jam = entry.getValue();
			EntityID stuckAgentNowWhere = alc.getAgentLastLocation(stuckwho);
			boolean active = false;
			if (jam.state == JamState.Buried) {
				if (!jam.buriedBuilding.equals(stuckAgentNowWhere)) {
					active = true;
				} else {
					if (jam.buriedBuilding.equals(imhere)) {
						if (!ais.contains(stuckwho)) {
							active = true;
						}
					}
				}
			} else if (jam.state == JamState.StuckInBuilding) {
				if (jam.getStuckBuilding().equals(imhere)) {
					active = true;
				} else {
					AgentLocationInfo ali = alc.getAgentLastLocationInfo(stuckwho);
					if (ali.getTimestamp() >= jam.getLastUpdatedTimestamp()) {
						StandardEntity se = world.getEntity(jam.getStuckBuilding());
						if (se instanceof Building) {
							Building seb = (Building) se;
							HashSet<EntityID> nb = getBuildings(seb);
							if (!nb.contains(jam.getStuckBuilding())) {
								active = true;
							}
						}
					} else {
						// log.error("ali.getTimestamp() < jam.getLastUpdatedTimestamp()");
					}
				}
			} else if (jam.state == JamState.StuckInRoad) {
				if (!jam.stuckRoodCoord.getRoad().getID().equals(stuckAgentNowWhere)) {
					AgentLocationInfo ali = alc.getAgentLastLocationInfo(stuckwho);
					if (ali.getTimestamp() < jam.getLastUpdatedTimestamp()) {
						log.error("Road: ali.getTimestamp() < jam.getLastUpdatedTimestamp()");
					}
					int stuckagentnowx = 0, stuckagentnowy = 0;
					if (ali.isCoordinateDefined()) {
						stuckagentnowx = ali.getX();
						stuckagentnowy = ali.getY();
					} else {
						Area nowa = (Area) world.getEntity(ali.getLocation());
						stuckagentnowx = nowa.getX();
						stuckagentnowy = nowa.getY();
					}
					int srx = jam.getStuckRoodCoord().getX();
					int sry = jam.getStuckRoodCoord().getY();
					if (GeoUtils.dist(stuckagentnowx, stuckagentnowy, srx, sry) > STUCK_IN_ROAD_SKIP_DIST) {
						active = true;
					} else {
						if (GeoUtils.dist(ax, ay, srx, sry) < INSIGHT_RANGE) {
							if (!ais.contains(stuckwho)) {
								active = true;
							} else {
								if (GeoUtils.dist(ax, ay, srx, sry) < REACH_DIST) {
									active = true;
								}
							}
						}
					}
				} else {
					if (jam.stuckRoodCoord.getRoad().getID().equals(imhere)) {
						if (context.getAgentType() == StandardEntityURN.POLICE_FORCE) {
							// Arrive
							int srx = jam.getStuckRoodCoord().getX();
							int sry = jam.getStuckRoodCoord().getY();
							if (GeoUtils.dist(ax, ay, srx, sry) < INSIGHT_RANGE) {
								if (!ais.contains(stuckwho)) {
									active = true;
								} else {
									if (GeoUtils.dist(ax, ay, srx, sry) < REACH_DIST) {
										active = true;
									}
								}
							}
						}
					}
				}
			} else if (jam.state == JamState.Unknown) {
				if (!jam.getBeliefSpawns().contains(stuckAgentNowWhere)) {
					active = true;
				}
			}

			if (active) {
				jam.clear(JamState.Active);
				jam.lastUpdatedTimestamp = context.getTimestamp();
			}
		}

		deliverMessage();
	}

	private void deliverMessage() {
		List<Jam> jamlist = new ArrayList<Jam>();
		for (Entry<EntityID, Jam> entry : jmap.entrySet()) {
			JamState state = entry.getValue().getState();
			if (state == JamState.StuckInBuilding || state == JamState.StuckInRoad) {
				if (!entry.getKey().equals(context.getAgentID()))
					jamlist.add(entry.getValue());
			}
		}

		Collections.sort(jamlist, new Comparator<Jam>() {
			@Override
			public int compare(Jam o1, Jam o2) {
				return -(o1.lastUpdatedTimestamp - o2.lastUpdatedTimestamp);
			}
		});

		for (int i = 0; i < DELIVER_MESSAGES; i++) {
			if (jamlist.size() <= i) {
				break;
			}
			Jam jam = jamlist.get(i);
			IMessage msg = null;
			if (jam.getState() == JamState.StuckInBuilding) {
				msg = new MVoiceAgentStuckInBuilding(jam.who, jam.getStuckBuilding());
				msg.setTimestamp(jam.getLastUpdatedTimestamp());
			} else if (jam.getState() == JamState.StuckInRoad) {
				msg = new MVoiceAgentStuckInRoad(jam.who, jam.stuckRoodCoord);
				msg.setTimestamp(jam.getLastUpdatedTimestamp());
			}
			if (msg != null) {
				this.addVoiceMessage(msg);
			}
		}
	}

	public void setImStuck(boolean isStuck) {
		if (context.getHuman() instanceof PoliceForce) {
			log.error("Police Force is stuck? are you sure");
			return;
		}
		Jam jam = jmap.get(context.getHuman().getID());
		jam.lastUpdatedTimestamp = context.getTimestamp();
		if (isStuck) {
			Area a = (Area) context.getHuman().getPosition(world);
			if (a instanceof Building) {
				jam.state = JamState.StuckInBuilding;
				jam.stuckBuilding = a.getID();
			} else if (a instanceof Road) {
				jam.state = JamState.StuckInRoad;
				jam.stuckRoodCoord = new RoadCoord((Road) a, context.getHuman().getX(), context
						.getHuman().getY());
			} else {
				log.error("Bad position! " + context.getHuman().getPosition());
			}
		} else {
			jam.clear(JamState.Active);
		}
	}

	public Jam getJam(EntityID human) {
		return this.jmap.get(human);
	}

	public List<EntityID> getActiveAgentsOfType(StandardEntityURN urn) {
		ArrayList<EntityID> rv = new ArrayList<EntityID>();
		for (Jam jam : jmap.values()) {
			if (urn == jam.getUrn() && jam.getState() == JamState.Active) {
				rv.add(jam.getWho());
			}
		}
		return rv;
	}

	private HashSet<EntityID> getBuildings(Building building) {
		HashSet<Building> nearBuildings = new HashSet<Building>();
		HashSet<Building> traversalBuildings = new HashSet<Building>();
		HashSet<EntityID> ret = new HashSet<EntityID>();
		nearBuildings.add(building);
		traversalBuildings.add(building);
		while (true) {
			if (traversalBuildings.isEmpty()) {
				break;
			}
			Iterator<Building> it = traversalBuildings.iterator();
			Building b = it.next();
			it.remove();

			for (EntityID id : b.getNeighbours()) {
				StandardEntity entity = world.getEntity(id);
				if (entity instanceof Building) {
					Building nearBuilding = (Building) entity;
					if (!nearBuildings.contains(nearBuilding)) {
						nearBuildings.add(nearBuilding);
						traversalBuildings.add(nearBuilding);
					}
				}
			}
		}

		for (Building b : nearBuildings) {
			ret.add(b.getID());
		}
		return ret;
	}

	public static class Jam {
		Jam(Human h) {
			urn = h.getStandardURN();
			who = h.getID();
			beliefSpawns = new HashSet<EntityID>();
			spawn = h.getPosition();
			beliefSpawns.add(spawn);
			state = JamState.Unknown;
			lastUpdatedTimestamp = -1;
		}

		void clear(JamState updatedState) {
			buriedBuilding = null;
			stuckBuilding = null;
			stuckRoodCoord = null;
			state = updatedState;
		}

		private StandardEntityURN urn;
		EntityID who;
		JamState state;
		EntityID buriedBuilding;
		EntityID stuckBuilding;
		RoadCoord stuckRoodCoord;

		EntityID spawn;
		Set<EntityID> beliefSpawns;
		int lastUpdatedTimestamp;

		public JamState getState() {
			return state;
		}

		public EntityID getBuriedBuilding() {
			return buriedBuilding;
		}

		public EntityID getStuckBuilding() {
			return stuckBuilding;
		}

		public RoadCoord getStuckRoodCoord() {
			return stuckRoodCoord;
		}

		public EntityID getSpawn() {
			return spawn;
		}

		public Set<EntityID> getBeliefSpawns() {
			return beliefSpawns;
		}

		public int getLastUpdatedTimestamp() {
			return lastUpdatedTimestamp;
		}

		public StandardEntityURN getUrn() {
			return urn;
		}

		public EntityID getWho() {
			return who;
		}

		@Override
		public String toString() {
			switch (this.state) {
			case Active:
				return "Jam [Active]";
			case Buried:
				return "Jam [Buried " + this.buriedBuilding + "]";
			case StuckInBuilding:
				return "Jam [StuckInBuilding " + this.stuckBuilding + "]";
			case StuckInRoad:
				return "Jam [StuckInRoad " + this.stuckRoodCoord.getRoad().getID() + "]";
			case Unknown:
				return "Jam [Unknown]";
			default:
				log.error("Bad case");
				return "Jam [Unknown]";
			}

		}

	}
}
