package ant.component.universal;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.utils.blockade.WrappedRoad;

/**
 * 
 * @author Fun
 * @depend {@link ChangeSetDetailComponent}
 */
public class RoadInfoCacheComponent implements AntComponent{
	private static Log log = LogFactory.getLog(RoadInfoCacheComponent.class);
	
	private ChangeSetDetailComponent  csdc;
	private Map<EntityID, WrappedRoad> wroads;
	private StandardWorldModel world;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		csdc = context.getAntComponentKit().getChangeSetDetailComponent();
		this.world = world;
		wroads = new HashMap<EntityID, WrappedRoad>();
		for(StandardEntity se: world.getAllEntities()){
			if(se instanceof Road){
				wroads.put(se.getID(), new WrappedRoad((Road) se, world));
			}
		}
	}

	@Override
	public void update() {
		Collection<StandardEntity> changed = csdc.getChangedEntities(StandardEntityURN.ROAD).values();
		for(StandardEntity se: changed){
			WrappedRoad wr = new WrappedRoad((Road) se, world);
			wroads.put(se.getID(), wr);
		}
	}
	
	public WrappedRoad getWrappedRoad(EntityID roadId){
		WrappedRoad wr = wroads.get(roadId);
		return wr;
	}

}
