package ant.liaison;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * 
 * @author Ivan
 * 
 */
public class BitArray implements Iterable<Boolean> {

	private int nbits;
	private LinkedList<Boolean> data;

	public BitArray(int nbits) {
		this.nbits = nbits;
		this.data = new LinkedList<Boolean>();
		for (int i = 0; i < nbits; i++)
			this.data.add(false);
	}

	BitArray(List<Boolean> data) {
		this.nbits = data.size();
		this.data = new LinkedList<Boolean>();
		this.data.addAll(data);
	}

	public void clear() {
		this.data.clear();
		this.nbits = 0;
	}

	public int length() {
		return this.nbits;
	}

	public void appendFirst(boolean b) {
		this.nbits++;
		data.addFirst(b);
	}

	public void append(boolean b) {
		this.nbits++;
		data.add(b);
	}
	
	List<Boolean> get() {
		return data;
	}

	public BitArray deepCopy() {
		LinkedList<Boolean> tmp = new LinkedList<Boolean>();
		for (boolean b : data) {
			tmp.add(b);
		}
		return new BitArray(tmp);
	}

	// public ArrayList<Boolean> getArrayCopy() {
	// ArrayList<Boolean> other = new ArrayList<Boolean>();
	// Collections.copy(other, this.data);
	// return other;
	// }

	public void append(BitArray other) {
		this.nbits += other.length();
		for (Boolean b : other.get()) {
			this.data.add(b);
		}
	}

	/**
	 * from 0 to length-1
	 */
	public BitArray truncation(int length) {
		if (length >= nbits)
			length = nbits;
		List<Boolean> list = new LinkedList<Boolean>();
		this.nbits -= length;
		for (int i = 0; i < length; i++) {
			list.add(this.data.poll());
		}

		return new BitArray(list);
	}
	
	public int truncationAndDecode(int length) {
		if (length >= nbits)
			length = nbits;
		
		this.nbits -= length;
		
		int val = 0;
		for (int i = 0; i < length; i++) {
			val += val;
			if (this.data.poll())
				val += 1;
		}
		return val;
	}
	
//	public void startDecode() {
//		this.decodeiter = data.iterator();
//	}
//	
//	public int iterDecode(int length) {
//		int val = 0;
//		for (int i = 0; i < length; i++) {
//			val += val;
//			if (decodeiter.hasNext()) {
//				if (decodeiter.next()) 
//					val += 1;
//			} else {
//				break;
//			}
//		}
//		return val;
//	}
//	
//	public void endDecode() {
//		this.decodeiter = null;
//	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Bits[");
		for (Boolean b : data) {
			if (b) sb.append("1"); else sb.append("0");
		}
		sb.append("]");
		return sb.toString();
	}

	@Override
	public Iterator<Boolean> iterator() {
		return data.iterator();
	}

	public boolean isInvaild() {
		if (nbits < 8) {
			boolean p = getPatch();
			for (Boolean b : data) {
				if (b != p) return false;
			}
			return true;
		}
		return false;
	}
	
	public static boolean getPatch() {
		return true;
	}
}
