package ant.liaison;


/**
 * 
 * @author Ivan, Oliver
 * 
 */
public class BitCodecUtils {
	/**
	 * CODE = 128
	 */
	private static final int CODE = -Byte.MIN_VALUE;

	private BitCodecUtils() {
	}

	/**
	 * unsigned encoding & truncation <br>
	 * accept positive integer <br>
	 * e.g val = 3, nbits = 4 <br>
	 * BitArray = 0011 ( 0 false, 1 true ) <br>
	 * ArrayIndex 0123 <br>
	 */
	public static BitArray encode(int val, int nbits) {
		if (val < 0)
			throw new IllegalArgumentException("val < 0");
		BitArray arr = new BitArray(0);
		for (int i = 0; i < nbits; i++) {
			arr.appendFirst(((val & 0x1) != 0));
			val = val >> 1;
		}
		return arr;
	}

	public static int decode(BitArray arr) {
		int val = 0;
		for (boolean b : arr) {
			val += val;
			if (b)
				val += 1;
		}
		return val;
	}

	/**
	 * @param val
	 * @return val bitlength
	 */
	public static int upperBitLength(int val) {
		if (val < 0)
			throw new IllegalArgumentException("val < 0");

		if (val == 0)
			return 1;
		int ll = 0;
		while (val > 0) {
			ll += 1;
			val = val >> 1;
		}
		return ll;
	}

	/**
	 * Byte -128~127
	 */
	public static BitArray translateToBitArray(byte[] data) {
		BitArray bs = new BitArray(0);
		for (int i = 0; i < data.length; i++) {
			// d = 0 ~ 255
			int d = data[i] + CODE;
			BitArray tmp = encode(d, 8);
			bs.append(tmp);
		}
		return bs;
	}

	public static byte[] translateToByteArray(BitArray bs) {
		int size = bs.length();
		BitArray copy = bs.deepCopy();
		int len = (size - 1) / 8 + 1;
		int patch = (8 - bs.length() % 8) % 8;
		
		boolean patchBoolean = BitArray.getPatch();
		for (int i = 0; i < patch; i++) 
			copy.append(patchBoolean);
		
		byte data[] = new byte[len];
		
		for (int i = 0; i < len; i++) {
			BitArray tmp = copy.truncation(8);
			int k = decode(tmp);
			data[i] = (byte) (k - CODE);
		}
		return data;
	}
	
}
