package ant.liaison;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import ant.AbstractAntCommunicationKit;
import ant.AgentContext;
import ant.ConfigKey;
import ant.agent.command.AntSpeak;
import ant.agent.command.AntSubscribe;
import ant.exception.AntComponentRegisterException;
import ant.liaison.channel.ChannelMannager;
import ant.liaison.channel.VoiceChannel;
import ant.liaison.channel.assign.ChannelAssignResult.BandwidthLevel;
import ant.liaison.channel.assign.ChannelAssigner;
import ant.liaison.channel.detect.ChannelConditionDetector;
import ant.liaison.channel.io.MessageFilter;
import ant.liaison.channel.io.MessageReceiver;
import ant.liaison.channel.io.RadioSender;
import ant.liaison.channel.voice.VoiceSender;
import ant.liaison.message.AbstractCivilianMessage;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Oliver
 * 
 */
public class LiaisonCommunicationKit extends AbstractAntCommunicationKit {
	private static Log log = LogFactory.getLog(LiaisonCommunicationKit.class);

	private ChannelMannager channelMannager;
	private ChannelConditionDetector channelConditionDetector;
	private ChannelAssigner channelAssigner;
	// TODO duplicated msg filter
	private RadioSender radioSender;
	private MessageReceiver messageReceiver;
	private VoiceSender voiceSender;
	private int ignoreTime;

	public LiaisonCommunicationKit() throws AntComponentRegisterException {
		super();
		registerComponents();
		this.channelMannager = (ChannelMannager) getAntComponent(ChannelMannager.class);
		this.channelConditionDetector = (ChannelConditionDetector) getAntComponent(ChannelConditionDetector.class);
		this.channelAssigner = (ChannelAssigner) getAntComponent(ChannelAssigner.class);
		this.radioSender = (RadioSender) getAntComponent(RadioSender.class);
		this.messageReceiver = (MessageReceiver) getAntComponent(MessageReceiver.class);
		this.voiceSender = (VoiceSender) getAntComponent(VoiceSender.class);
	}

	private void registerComponents() throws AntComponentRegisterException {
		registerComponent(ChannelMannager.class);
		registerComponent(ChannelConditionDetector.class);
		registerComponent(ChannelAssigner.class);
		registerComponent(MessageFilter.class);
		registerComponent(RadioSender.class);
		registerComponent(MessageReceiver.class);
		registerComponent(VoiceSender.class);
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		super.initial(context, world);
		this.ignoreTime = ConfigKey.getIgnoreTime(context.getConfig());
		radioSender.setSingleMessageCodec(getSingleMessageCodec());
		voiceSender.setMultiMessageCodec(getMultiMessageCodec());
		messageReceiver.setSingleMessageCodec(getSingleMessageCodec());
		messageReceiver.setMultiMessageCodec(getMultiMessageCodec());
	}

	@Override
	public void update(AgentContext context) {
		super.update(context);
	}

	@Override
	protected void sendRadio(IMessage message) {
		if (message != null) {
			if (message.getTimestamp() < 0) {
				message.setTimestamp(context.getTimestamp());
			}
			radioSender.pushMessage(message);
		} else {
			log.error("SendSpeak null msg!");
		}
	}

	@Override
	protected void sendVoice(IMessage message) {
		if (message != null) {
			if (message.getTimestamp() < 0) {
				message.setTimestamp(context.getTimestamp());
			}
			voiceSender.pushMessage(message);
		} else {
			log.error("SendSpeak null msg!");
		}
	}

	@Override
	public Set<IMessage> getRadioReceiveBox() {
		return messageReceiver.getRadioMessages();
	}

	@Override
	public Set<IMessage> getRadioBoxWithSelf() {
		return messageReceiver.getRadioMessagesWithSelf();
	}

	@Override
	public Set<IMessage> getVoiceReceiveBox() {
		return messageReceiver.getVoiceMessages();
	}

	@Override
	public Set<AbstractCivilianMessage> getMessagesFromCivilian() {
		return messageReceiver.getMessagesFromCivilian();
	}

	@Override
	public List<AntSpeak> pullpreSpeaks() {
		int time = context.getTimestamp();
		if (time < ignoreTime) {
			return new ArrayList<AntSpeak>();
		}

		if (time == ignoreTime) {
			ArrayList<AntSpeak> messages = channelConditionDetector.getMessages();
			if (messages != null) {
				return messages;
			} else {
				log.error("channelConditionDetector.getMessages() == null @ time == ignoreTime!");
			}
		} else if (time == ignoreTime + 1) {
			ArrayList<AntSpeak> messages = channelConditionDetector.getMessages();
			if (messages != null) {
				return messages;
			} else {
				log.error("channelConditionDetector.getMessages() == null @ time == ignoreTime +1 !");
			}
		}
		return super.pullpreSpeaks();
	}

	@Override
	public AntSubscribe pullpreSubscribe() {
		int time = context.getTimestamp();
		if (time == ignoreTime) {
			int[] channels = channelConditionDetector.getSubscribeChannels();
			if (channels != null) {
				return new AntSubscribe(time, channels);
			} else {
				log.error("channelConditionDetector.getSubscribeChannels() == null!");
			}
		}
		return super.pullpreSubscribe();
	}

	@Override
	public List<AntSpeak> pullallSpeak() {
		int time = context.getTimestamp();
		if (time < ignoreTime) {
			return new ArrayList<AntSpeak>();
		}

		ArrayList<AntSpeak> rv = new ArrayList<AntSpeak>();
		if (time > ignoreTime + 1) {
			radioSender.sendAll(time);
			rv.addAll(radioSender.getRadioMessages());
		}
		voiceSender.sendVoiceMessages(time);
		rv.addAll(voiceSender.getAntSpeakMessages());

		return rv;
	}

	@Override
	public AntSubscribe pullSubscribe() {
		int time = context.getTimestamp();
		if (time == ignoreTime + 2) {
			int[] channels = channelAssigner.getSubscribeChannels();
			if (channels != null) {
				return new AntSubscribe(time, channels);
			} else {
				log.error("channelAssigner.getSubscribeChannels() == null!");
			}
		}

		return null;
	}

	@Override
	public boolean isNoRadio() {
		return channelMannager.getRadioChannelCount() == 0;
	}

	public BandwidthLevel getBandwidthLevel() {
		if (isNoRadio()) {
			return BandwidthLevel.None;
		} else if (context.getTimestamp() < ignoreTime + 2) {
			return BandwidthLevel.Unknown;
		} else {
			return channelAssigner.getChannelAssignResult().getBandwidthLevel();
		}
	}

	public VoiceChannel getVoiceChannel() {
		return voiceSender.getVoiceChannel();
	}
}
