package ant.liaison;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import ant.liaison.exception.MessagePropertyDescriptionException;
import ant.liaison.message.IMessage;
import ant.utils.IntrospectorUtils;

/**
 * 
 * @author Ivan
 * 
 */
public class MessagePropertiesOrder {
	private int size = 0;
	private ArrayList<MessageProperty> plist;
	private ArrayList<Method> slist;
	private ArrayList<Method> glist;
	private Class<? extends IMessage> clazz;

	// private int messageLength;

	public MessagePropertiesOrder(Class<? extends IMessage> clazz, int size) {
		this.clazz = clazz;
		this.size = size;
		plist = new ArrayList<MessageProperty>();
		slist = new ArrayList<Method>();
		glist = new ArrayList<Method>();
		for (int i = 0; i < size; i++) {
			plist.add(null);
			slist.add(null);
			glist.add(null);
		}
	}

	public void set(int order, MessageProperty prop, Method setMethod, Method getMethod) {
		plist.set(order, prop);
		slist.set(order, setMethod);
		glist.set(order, getMethod);
	}

	public MessageProperty getBitProperty(int index) {
		return plist.get(index);
	}

	public Method getSetMethod(int index) {
		return slist.get(index);
	}

	public Method getGetMethod(int index) {
		return glist.get(index);
	}

	public Class<? extends IMessage> getClazz() {
		return clazz;
	}

	// public int getMessageLength() {
	// return messageLength;
	// }
	//
	// public void setMessageLength(int messageLength) {
	// this.messageLength = messageLength;
	// }

	public int size() {
		return this.size;
	}

	public static Map<Class<? extends IMessage>, MessagePropertiesOrder> getMessagePropertiesOrderMap(MessagePropertyCodecComponent comp)
			throws MessagePropertyDescriptionException {
		Map<Class<? extends IMessage>, MessagePropertiesOrder> mmap = new HashMap<Class<? extends IMessage>, MessagePropertiesOrder>();
		for (MessageType type : MessageType.values()) {
			Class<? extends IMessage> clazz = type.getBindClazz();
			int size = checkOrder(clazz);
			// log.debug(size);
			MessagePropertiesOrder po = new MessagePropertiesOrder(clazz, size);
			mmap.put(clazz, po);
			clazzAnalysis(clazz, po);
			// int len = 0;
			// len += MessageType.getAllTypeBitLength();
			// for (int i = 0; i < po.size(); i++) {
			// MessageProperty p = po.getBitProperty(i);
			// len += comp.getPropertyBitLength(p);
			// log.info(p + " len:" + comp.getPropertyBitLength(p));
			// }
			// po.setMessageLength(len);
			// log.info("Message: " + type + " bitlength:" + len);
		}
		return mmap;
	}

	private static int checkOrder(Class<? extends IMessage> clazz) throws MessagePropertyDescriptionException {
		int cnt = 0;
		Set<Integer> set = new HashSet<Integer>();
		for (Field field : clazz.getDeclaredFields()) {
			for (Annotation ann : field.getAnnotations()) {
				if (ann.annotationType().equals(MessagePropertyDesc.class)) {
					MessagePropertyDesc info = (MessagePropertyDesc) ann;
					int order = info.order();
					if (set.contains(order) || order < 0) {
						throw new MessagePropertyDescriptionException("Duplicate order:" + clazz.getName());
					} else {
						set.add(order);
					}
					cnt++;
				}
			}
		}
		for (int i = 0; i < cnt; i++) {
			if (!set.contains(i)) {
				throw new MessagePropertyDescriptionException("Invaild order:" + clazz.getName());
			}
		}

		return cnt;
	}

	private static void clazzAnalysis(Class<? extends IMessage> clazz, MessagePropertiesOrder po) throws MessagePropertyDescriptionException {
		for (Field field : clazz.getDeclaredFields()) {
			for (Annotation ann : field.getAnnotations()) {
				if (ann.annotationType().equals(MessagePropertyDesc.class)) {
					MessagePropertyDesc info = (MessagePropertyDesc) ann;
					Method g = IntrospectorUtils.findGetMethod(clazz, field);
					Method s = IntrospectorUtils.findSetMethod(clazz, field);
					MessageProperty p = info.bind();
					if (g == null || s == null || p == null)
						throw new MessagePropertyDescriptionException("Invaild Message Property:" + clazz.getName() + " field: " + field + " not found (" + g + "," + s
								+ "," + p + ")");

					int order = info.order();
					po.set(order, p, s, g);
				}
			}
		}
	}
}