package ant.liaison;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.pf.ObjectiveJigsawMetaComponent;
import ant.liaison.exception.DecodingException;
import ant.liaison.exception.EncodingException;
import ant.map.jigsaw.JigsawMainComponent;
import ant.utils.comparator.EntityIDComparator;

/**
 * 
 * @author Ivan
 * 
 * @depend
 * 
 * @lazy_depend {@link JigsawMainComponent} {@link ObjectiveJigsawMetaComponent}
 */
public class MessagePropertyCodecComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(MessagePropertyCodecComponent.class);

	private AgentContext context;
	private StandardWorldModel world;

	private AreaIDPropertyHandler areaIDPropertyHandler;
	private RoadIDPropertyHandler roadIDPropertyHandler;
	private RoadCoordProeprtyHandler roadCoordPropertyHandler;
	private BuildingIDPropertyHandler buildingIDPropertyHandler;
	private AgentIDPropertyHandler agentIDPropertyHandler;
	private DamagePropertyHandler damagePropertyHandler;
	private HpPropertyHandler hpPropertyHandler;
	private CivilianIDPropertyHandler civilianIDPropertyHandler;
	private RefugeIDPropertyHandler refugeIDPropertyHandler;
	private DTimestampPropertyHandler dtimestampPropertyHandler;
	private FullTimestampPropertyHandler fullTimestampPropertyHandler;
	private JigsawIDPropertyHandler jigsawIDPropertyHandler;
	private ObjectiveJigsawIDPropertyHandler objectiveJigsawIDPropertyHandler;

	@Override
	public String getComponentName() {
		return MessagePropertyCodecComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.areaIDPropertyHandler = new AreaIDPropertyHandler(context);
		this.roadIDPropertyHandler = new RoadIDPropertyHandler(context);
		this.buildingIDPropertyHandler = new BuildingIDPropertyHandler(context);
		this.agentIDPropertyHandler = new AgentIDPropertyHandler(context);
		this.damagePropertyHandler = new DamagePropertyHandler(context);
		this.hpPropertyHandler = new HpPropertyHandler(context);
		this.civilianIDPropertyHandler = new CivilianIDPropertyHandler(context);
		this.roadCoordPropertyHandler = new RoadCoordProeprtyHandler(context,
				this.roadIDPropertyHandler);
		this.refugeIDPropertyHandler = new RefugeIDPropertyHandler(context);
		this.dtimestampPropertyHandler = new DTimestampPropertyHandler(context);
		this.fullTimestampPropertyHandler = new FullTimestampPropertyHandler(context);
		this.jigsawIDPropertyHandler = new JigsawIDPropertyHandler(context);
		this.objectiveJigsawIDPropertyHandler = new ObjectiveJigsawIDPropertyHandler(context);
	}

	@Override
	public void update() {
	}

	public boolean ignoreMessage(MessageType.MessageAcceptor acceptor) {
		switch (acceptor) {
		case All:
			return false;
		case AmbulanceTeam:
			if (context.getAgentType() == StandardEntityURN.AMBULANCE_CENTRE
					|| context.getAgentType() == StandardEntityURN.AMBULANCE_TEAM)
				return false;
			else
				return true;
		case FireBridge:
			if (context.getAgentType() == StandardEntityURN.FIRE_BRIGADE
					|| context.getAgentType() == StandardEntityURN.FIRE_STATION)
				return false;
			else
				return true;
		case PoliceForce:
			if (context.getAgentType() == StandardEntityURN.POLICE_FORCE
					|| context.getAgentType() == StandardEntityURN.POLICE_OFFICE)
				return false;
			else
				return true;
		default:
			return true;
		}
	}

	public int getPropertyBitLength(MessageProperty mp) {
		switch (mp) {
		case AreaID:
			return this.areaIDPropertyHandler.getPropertyBitLength();

		case AgentID:
			return this.agentIDPropertyHandler.getPropertyBitLength();

		case CivilianID:
			return this.civilianIDPropertyHandler.getPropertyBitLength();

		case RoadCoord:
			return this.roadCoordPropertyHandler.getPropertyBitLength();

		case Hp: // {0 - 16383} / mod
			return this.hpPropertyHandler.getPropertyBitLength();

		case Damage: // {0 - 16383} / mod
			return this.damagePropertyHandler.getPropertyBitLength();

		case BuildingID:
			return this.buildingIDPropertyHandler.getPropertyBitLength();

		case RoadID:
			return this.roadIDPropertyHandler.getPropertyBitLength();

		case DTimestamp:
			return this.dtimestampPropertyHandler.getPropertyBitLength();

		case FullTimestamp:
			return this.fullTimestampPropertyHandler.getPropertyBitLength();

		case Boolean:
			return 1;

		case Buriesness: // 0 - 255
			return 8;

		case Temperature: // 0 - 1023
			return 10;

		case Fieryness: // 0 - 15
			return 4;

		case RefugeID:
			return this.refugeIDPropertyHandler.getPropertyBitLength();

		case JigsawID:
			return this.jigsawIDPropertyHandler.getPropertyBitLength();

		case ObjectiveJigsawID:
			return this.objectiveJigsawIDPropertyHandler.getPropertyBitLength();

		default:
			throw new IllegalArgumentException();

		}
	}

	public int encodeProperty(MessageProperty mp, Object obj) throws EncodingException {
		try {
			switch (mp) {
			case Buriesness:
			case Fieryness:
			case Temperature:
				return encodeInt(mp, obj);

			case DTimestamp:
				return dtimestampPropertyHandler.encode(obj);

			case FullTimestamp:
				return this.fullTimestampPropertyHandler.encode(obj);

			case Boolean:
				return encodeBoolean(mp, obj);

			case Hp:
				return hpPropertyHandler.encode(obj);

			case Damage:
				return damagePropertyHandler.encode(obj);

			case AreaID:
				return areaIDPropertyHandler.encode(obj);

			case AgentID:
				return agentIDPropertyHandler.encode(obj);

			case CivilianID:
				return civilianIDPropertyHandler.encode(obj);

			case RoadCoord:
				return roadCoordPropertyHandler.encode(obj);

			case BuildingID:
				return buildingIDPropertyHandler.encode(obj);

			case RoadID:
				return roadIDPropertyHandler.encode(obj);

			case RefugeID:
				return refugeIDPropertyHandler.encode(obj);

			case JigsawID:
				return jigsawIDPropertyHandler.encode(obj);

			case ObjectiveJigsawID:
				return objectiveJigsawIDPropertyHandler.encode(obj);

			default:
				throw new IllegalArgumentException();
			}
		} catch (Exception e) {
			throw new EncodingException(e);
		}
	}

	public Object decodeProperty(MessageProperty mp, int val) throws DecodingException {
		try {
			switch (mp) {
			case Buriesness:
			case Fieryness:
			case Temperature:
				return decodeInt(mp, val);

			case Boolean:
				return decodeBoolean(mp, val);

			case DTimestamp:
				return dtimestampPropertyHandler.decode(val);

			case FullTimestamp:
				return fullTimestampPropertyHandler.decode(val);

			case Hp:
				return hpPropertyHandler.decode(val);

			case Damage:
				return damagePropertyHandler.decode(val);

			case AreaID:
				return areaIDPropertyHandler.decode(val);

			case AgentID:
				return agentIDPropertyHandler.decode(val);

			case CivilianID:
				return civilianIDPropertyHandler.decode(val);

			case RoadCoord:
				return roadCoordPropertyHandler.decode(val);

			case BuildingID:
				return buildingIDPropertyHandler.decode(val);

			case RoadID:
				return roadIDPropertyHandler.decode(val);

			case RefugeID:
				return refugeIDPropertyHandler.decode(val);

			case JigsawID:
				return jigsawIDPropertyHandler.decode(val);

			case ObjectiveJigsawID:
				return objectiveJigsawIDPropertyHandler.decode(val);

			default:
				throw new IllegalArgumentException();
			}
		} catch (Exception e) {
			throw new DecodingException(e);
		}
	}

	private int encodeBoolean(MessageProperty mp, Object obj) {
		boolean b = (Boolean) obj;
		if (b == true)
			return 1;
		else
			return 0;
	}

	private Boolean decodeBoolean(MessageProperty mp, int val) throws DecodingException {
		if (val == 0)
			return false;
		else if (val == 1)
			return true;
		else
			throw new DecodingException("Bad boolean val:" + val);
	}

	private int encodeInt(MessageProperty mp, Object obj) throws EncodingException {
		int val = (Integer) obj;
		if (val < 0)
			throw new EncodingException("neg val:" + mp + ":" + obj);

		int maxVal = (1 << getPropertyBitLength(mp)) - 1;
		if (val > maxVal) {
			// log.warn(mp + " out of range:" + val);
			val = maxVal;
		}
		// log.debug(mp + "," + obj + " -> " + val + " max:" + maxVal);
		return val;
	}

	private Integer decodeInt(MessageProperty mp, int val) throws DecodingException {
		if (val < 0)
			throw new DecodingException("Neg val for " + mp);
		return val;
	}

	public static int round(int value, int precision) {
		int remainder = value % precision;
		value -= remainder;
		if (remainder >= precision / 2) {
			value += precision;
		}
		return value;
	}

	/**
	 * [0, maxVal]
	 */
	public static int getBitLength(int maxVal) {
		int len = 0;

		if (maxVal < 0)
			throw new IllegalArgumentException();

		if (maxVal >= 0 && maxVal <= 1)
			return 1;

		while (maxVal != 0) {
			len++;
			maxVal = maxVal >> 1;
		}

		return len;
	}

	private static abstract class PropertyHandler {
		protected AgentContext context;

		public PropertyHandler(AgentContext context) {
			this.context = context;
		}

		public abstract int encode(Object obj) throws EncodingException;

		public abstract Object decode(int val) throws DecodingException;

		public abstract int getPropertyBitLength();
	}

	private static class RoadCoordProeprtyHandler extends PropertyHandler {
		private int roadCoordPropertyBitLength;
		private RoadIDPropertyHandler rph;
		private StandardWorldModel world;

		public RoadCoordProeprtyHandler(AgentContext context, RoadIDPropertyHandler rph) {
			super(context);
			this.rph = rph;
			this.world = context.getWorld();
			this.roadCoordPropertyBitLength = 4 + rph.getPropertyBitLength();
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			RoadCoord rc = (RoadCoord) obj;
			int x = rc.getX();
			int y = rc.getY();
			Rectangle rect = rc.getRoad().getShape().getBounds();
			double minx = rect.getMinX();
			double miny = rect.getMinY();
			double maxx = rect.getMaxX();
			double maxy = rect.getMaxY();
			double dx = (maxx - minx) / 4;
			double dy = (maxy - miny) / 4;

			boolean b = false;
			int i = 0, j = 0;
			for (i = 0; i < 4; i++) { // x
				for (j = 0; j < 4; j++) { // y
					double t_minx = minx + i * dx;
					double t_miny = miny + j * dy;
					double t_maxx = t_minx + dx;
					double t_maxy = t_miny + dy;
					if (x >= t_minx && x <= t_maxx)
						if (y >= t_miny && y <= t_maxy) {
							b = true;
							break;
						}
				}
				if (b)
					break;
			}

			int e = i * 4 + j;
			// int id = encodeAreaID(MessageProperty.AreaID,
			// rc.getRoad().getID());
			int id = rph.encode(rc.getRoad().getID());
			int ret = id * 16 + e;
			return ret;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			int id = val / 16;
			int e = val % 16;
			int i = e / 4;
			int j = e % 4;
			// EntityID rid = decodeAreaID(MessageProperty.AreaID, id);
			EntityID rid = (EntityID) rph.decode(id);
			Road r = (Road) world.getEntity(rid);
			Rectangle rect = r.getShape().getBounds();
			double minx = rect.getMinX();
			double miny = rect.getMinY();
			double maxx = rect.getMaxX();
			double maxy = rect.getMaxY();
			double dx = (maxx - minx) / 4;
			double dy = (maxy - miny) / 4;
			int x = (int) (minx + i * dx + dx / 2);
			int y = (int) (miny + j * dy + dy / 2);
			return new RoadCoord(r, x, y);
		}

		@Override
		public int getPropertyBitLength() {
			return this.roadCoordPropertyBitLength;
		}
	}

	private static class CivilianIDPropertyHandler extends PropertyHandler {

		private int civilianIDPropertyBitLength;

		public CivilianIDPropertyHandler(AgentContext context) {
			super(context);
			this.civilianIDPropertyBitLength = 18;
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			int maxVal = (1 << this.getPropertyBitLength()) - 1;
			if (id.getValue() > maxVal) {
				throw new IllegalStateException("CivilianID out of range:" + id);
			}
			return id.getValue();
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0)
				throw new DecodingException("Neg civid:" + val);
			return new EntityID(val);
		}

		@Override
		public int getPropertyBitLength() {
			return civilianIDPropertyBitLength;
		}

	}

	private static class HpPropertyHandler extends PropertyHandler {
		private static final Log log = LogFactory
				.getLog(MessagePropertyCodecComponent.HpPropertyHandler.class);

		private int hpPropertyBitLength;
		private int perceptionPrecisionHp;
		private int maxHP;

		public HpPropertyHandler(AgentContext context) {
			super(context);
			this.maxHP = 10000;
			this.perceptionPrecisionHp = ConfigKey.getPerceptionPrecisionHp(context.getConfig());
			int temp = round(maxHP, perceptionPrecisionHp);
			this.hpPropertyBitLength = getBitLength(temp / perceptionPrecisionHp);

			// log.info("HP PropertyBitLength:" + hpPropertyBitLength);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			int hp = (Integer) obj;
			if (hp > maxHP) {
				log.warn("Hp out of range:" + hp);
				hp = maxHP;
			}
			if (hp < 0) {
				throw new IllegalStateException("Hp out of range:" + hp);
			}
			int temp = round(hp, perceptionPrecisionHp);
			// if (temp != hp) {
			// log.warn("Hp perception not equal: act" + hp + " round:" + temp);
			// }
			return temp / perceptionPrecisionHp;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val * perceptionPrecisionHp;
		}

		@Override
		public int getPropertyBitLength() {
			return hpPropertyBitLength;
		}

	}

	private static class DamagePropertyHandler extends PropertyHandler {
		private static final Log log = LogFactory
				.getLog(MessagePropertyCodecComponent.DamagePropertyHandler.class);

		private int damagePropertyBitLength;
		private int perceptionPrecisionDamage;
		private int maxDamage;

		public DamagePropertyHandler(AgentContext context) {
			super(context);

			this.maxDamage = 1000;
			this.perceptionPrecisionDamage = ConfigKey.getPerceptionPrecisionDamage(context
					.getConfig());
			int temp = round(maxDamage, perceptionPrecisionDamage);
			this.damagePropertyBitLength = getBitLength(temp / perceptionPrecisionDamage);

			// log.info("Damage PropertyBitLength:" + damagePropertyBitLength);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			int damage = (Integer) obj;
			if (damage > maxDamage) {
				log.warn("Damage out of range:" + damage);
				damage = maxDamage;
			}
			if (damage < 0) {
				throw new IllegalStateException("Damage out of range:" + damage);
			}
			int temp = round(damage, perceptionPrecisionDamage);
			// if (temp != damage) {
			// log.warn("Damage perception not equal: act" + damage + " round:"
			// + temp);
			// }
			return temp / perceptionPrecisionDamage;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val * perceptionPrecisionDamage;
		}

		@Override
		public int getPropertyBitLength() {
			return damagePropertyBitLength;
		}

	}

	private static class AgentIDPropertyHandler extends PropertyHandler {
		private List<EntityID> agentLabelList;
		private HashMap<EntityID, Integer> agentLabelMap;
		private int agentCount;
		private int agentIDPropertyBitLength;
		private StandardWorldModel world;

		public AgentIDPropertyHandler(AgentContext context) {
			super(context);
			this.world = context.getWorld();
			Set<Human> agentSet = new HashSet<Human>();
			for (final StandardEntity e : world.getEntitiesOfType(StandardEntityURN.POLICE_FORCE,
					StandardEntityURN.FIRE_BRIGADE, StandardEntityURN.AMBULANCE_TEAM)) {

				agentSet.add((Human) e);
			}
			agentSet = Collections.unmodifiableSet(agentSet);

			agentLabelList = new ArrayList<EntityID>();
			agentLabelMap = new HashMap<EntityID, Integer>();

			for (Human human : agentSet) {
				agentLabelList.add(human.getID());
			}
			Collections.sort(agentLabelList, new EntityIDComparator());
			agentLabelList = Collections.unmodifiableList(agentLabelList);

			for (int i = 0; i < agentLabelList.size(); i++) {
				agentLabelMap.put(agentLabelList.get(i), i);
			}

			this.agentCount = agentSet.size();
			this.agentIDPropertyBitLength = getBitLength(agentCount - 1);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			if (agentLabelMap.containsKey(id)) {
				return agentLabelMap.get(id);
			} else {
				throw new EncodingException("Nou found agentID");
			}
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0 || val >= agentLabelList.size()) {
				throw new DecodingException("AgentID out of range:" + val);
			}
			return agentLabelList.get(val);
		}

		@Override
		public int getPropertyBitLength() {
			return agentIDPropertyBitLength;
		}

	}

	private static class BuildingIDPropertyHandler extends PropertyHandler {

		private List<EntityID> buildingLabelList;
		private HashMap<EntityID, Integer> buildingLabelMap;
		private int buildingCount;
		private int buildingIDPropertyBitLength;
		private StandardWorldModel world;

		public BuildingIDPropertyHandler(AgentContext context) {
			super(context);
			this.world = context.getWorld();
			buildingLabelList = new ArrayList<EntityID>();
			buildingLabelMap = new HashMap<EntityID, Integer>();

			HashSet<Building> buildingSet = new HashSet<Building>();
			for (final StandardEntity e : world.getEntitiesOfType(StandardEntityURN.BUILDING,
					StandardEntityURN.AMBULANCE_CENTRE, StandardEntityURN.FIRE_STATION,
					StandardEntityURN.POLICE_OFFICE, StandardEntityURN.REFUGE)) {
				buildingSet.add((Building) e);
			}

			for (Building b : buildingSet) {
				buildingLabelList.add(b.getID());
			}
			Collections.sort(buildingLabelList, new EntityIDComparator());
			buildingLabelList = Collections.unmodifiableList(buildingLabelList);

			for (int i = 0; i < buildingLabelList.size(); i++) {
				buildingLabelMap.put(buildingLabelList.get(i), i);
			}

			this.buildingCount = buildingSet.size();
			this.buildingIDPropertyBitLength = getBitLength(buildingCount - 1);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			if (buildingLabelMap.containsKey(id)) {
				return buildingLabelMap.get(id);
			} else {
				throw new EncodingException("Nou found buildingID:" + id);
			}
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0 || val >= buildingLabelList.size()) {
				throw new DecodingException("BuildingID out of range:" + val);
			}
			return buildingLabelList.get(val);
		}

		@Override
		public int getPropertyBitLength() {
			return this.buildingIDPropertyBitLength;
		}

	}

	private static class RoadIDPropertyHandler extends PropertyHandler {

		private List<EntityID> roadLabelList;
		private HashMap<EntityID, Integer> roadLabelMap;
		private int roadCount;
		private int roadIDPropertyBitLength;
		private StandardWorldModel world;

		public RoadIDPropertyHandler(AgentContext context) {
			super(context);
			this.world = context.getWorld();
			roadLabelList = new ArrayList<EntityID>();
			roadLabelMap = new HashMap<EntityID, Integer>();

			HashSet<Road> roadSet = new HashSet<Road>();
			for (final StandardEntity e : world.getEntitiesOfType(StandardEntityURN.ROAD)) {
				roadSet.add((Road) e);
			}

			for (Road area : roadSet) {
				roadLabelList.add(area.getID());
			}

			Collections.sort(roadLabelList, new EntityIDComparator());
			roadLabelList = Collections.unmodifiableList(roadLabelList);

			for (int i = 0; i < roadLabelList.size(); i++) {
				roadLabelMap.put(roadLabelList.get(i), i);
			}

			this.roadCount = roadSet.size();
			this.roadIDPropertyBitLength = getBitLength(roadCount - 1);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			if (roadLabelMap.containsKey(id)) {
				return roadLabelMap.get(id);
			} else {
				throw new EncodingException("Nou found areaID");
			}
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0 || val >= roadLabelList.size()) {
				throw new DecodingException("AreaID out of range:" + val);
			}
			return roadLabelList.get(val);
		}

		@Override
		public int getPropertyBitLength() {
			return this.roadIDPropertyBitLength;
		}

	}

	private static class RefugeIDPropertyHandler extends PropertyHandler {

		private List<EntityID> refugeLabelList;
		private HashMap<EntityID, Integer> refugeLabelMap;
		private int refugeCount;
		private int refugeIDPropertyBitLength;
		private StandardWorldModel world;

		public RefugeIDPropertyHandler(AgentContext context) {
			super(context);
			this.world = context.getWorld();

			refugeLabelList = new ArrayList<EntityID>();
			refugeLabelMap = new HashMap<EntityID, Integer>();

			HashSet<Refuge> refugeSet = new HashSet<Refuge>();
			for (final StandardEntity e : world.getEntitiesOfType(StandardEntityURN.REFUGE)) {
				refugeSet.add((Refuge) e);
			}

			for (Refuge r : refugeSet) {
				refugeLabelList.add(r.getID());
			}

			Collections.sort(refugeLabelList, new EntityIDComparator());
			refugeLabelList = Collections.unmodifiableList(refugeLabelList);

			for (int i = 0; i < refugeLabelList.size(); i++) {
				refugeLabelMap.put(refugeLabelList.get(i), i);
			}

			this.refugeCount = refugeSet.size();
			if (this.refugeCount == 0)
				this.refugeIDPropertyBitLength = 1;
			else
				this.refugeIDPropertyBitLength = getBitLength(refugeCount - 1);

		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			if (refugeLabelMap.containsKey(id)) {
				return refugeLabelMap.get(id);
			} else {
				throw new EncodingException("Nou found refugeID");
			}
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0 || val >= refugeLabelList.size()) {
				throw new DecodingException("AreaID out of range:" + val);
			}
			return refugeLabelList.get(val);
		}

		@Override
		public int getPropertyBitLength() {
			return refugeIDPropertyBitLength;
		}

	}

	private static class AreaIDPropertyHandler extends PropertyHandler {

		private List<EntityID> areaLabelList;
		private HashMap<EntityID, Integer> areaLabelMap;
		private int areaCount;
		private int areaIDPropertyBitLength;
		private StandardWorldModel world;

		public AreaIDPropertyHandler(AgentContext context) {
			super(context);
			this.world = context.getWorld();
			areaLabelList = new ArrayList<EntityID>();
			areaLabelMap = new HashMap<EntityID, Integer>();

			HashSet<Area> areaSet = new HashSet<Area>();
			for (final StandardEntity e : world.getEntitiesOfType(StandardEntityURN.BUILDING,
					StandardEntityURN.AMBULANCE_CENTRE, StandardEntityURN.FIRE_STATION,
					StandardEntityURN.POLICE_OFFICE, StandardEntityURN.REFUGE,
					StandardEntityURN.ROAD)) {

				areaSet.add((Area) e);
			}

			for (Area area : areaSet) {
				areaLabelList.add(area.getID());
			}
			Collections.sort(areaLabelList, new EntityIDComparator());
			areaLabelList = Collections.unmodifiableList(areaLabelList);

			for (int i = 0; i < areaLabelList.size(); i++) {
				areaLabelMap.put(areaLabelList.get(i), i);
			}

			this.areaCount = areaSet.size();
			this.areaIDPropertyBitLength = getBitLength(areaCount - 1);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			EntityID id = (EntityID) obj;
			if (areaLabelMap.containsKey(id)) {
				return areaLabelMap.get(id);
			} else {
				throw new EncodingException("Nou found areaID");
			}
		}

		@Override
		public Object decode(int val) throws DecodingException {
			if (val < 0 || val >= areaLabelList.size()) {
				throw new DecodingException("AreaID out of range:" + val + " areaCount:"
						+ areaCount + " size:" + areaLabelList.size());
			}
			return areaLabelList.get(val);
		}

		@Override
		public int getPropertyBitLength() {
			return this.areaIDPropertyBitLength;
		}

	}

	private static class DTimestampPropertyHandler extends PropertyHandler {

		public DTimestampPropertyHandler(AgentContext context) {
			super(context);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			int d = (Integer) obj;
			if (d < 0)
				throw new EncodingException("Neg dtime");
			if (d >= 128)
				d = 127;
			return d;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val;
		}

		@Override
		public int getPropertyBitLength() {
			return 7;
		}

	}

	private static class FullTimestampPropertyHandler extends PropertyHandler {

		public FullTimestampPropertyHandler(AgentContext context) {
			super(context);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			int d = (Integer) obj;
			if (d < 0 || d >= 1024)
				throw new EncodingException("Out of bound timestamp");
			return d;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val;
		}

		@Override
		public int getPropertyBitLength() {
			return 10;
		}

	}

	/** Lazy Handler **/
	private static class JigsawIDPropertyHandler extends PropertyHandler {

		private Integer jigsawCount = null;
		private JigsawMainComponent jmc = null;
		private int jbl = 0;

		public JigsawIDPropertyHandler(AgentContext context) {
			super(context);
		}

		private void lazyInit() {
			if (jmc == null) {
				this.jmc = (JigsawMainComponent) context.getAntComponentKit().getAntComponent(
						JigsawMainComponent.class);
				if (jmc != null) {
					this.jigsawCount = jmc.getTotalJigsaws();
					this.jbl = BitCodecUtils.upperBitLength(jigsawCount);
				}
			}
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			lazyInit();
			return (Integer) obj;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val;
		}

		@Override
		public int getPropertyBitLength() {
			lazyInit();
			return jbl;
		}

	}

	/** Lazy Handler **/
	private static class ObjectiveJigsawIDPropertyHandler extends PropertyHandler {
		private ObjectiveJigsawMetaComponent ojmc = null;
		private int count;
		private int jbl;

		public ObjectiveJigsawIDPropertyHandler(AgentContext context) {
			super(context);
		}

		@Override
		public int encode(Object obj) throws EncodingException {
			lazyInit();
			return (Integer) obj;
		}

		@Override
		public Object decode(int val) throws DecodingException {
			return val;
		}

		@Override
		public int getPropertyBitLength() {
			lazyInit();
			return jbl;
		}

		private void lazyInit() {
			if (ojmc == null) {
				this.ojmc = (ObjectiveJigsawMetaComponent) context.getAntComponentKit()
						.getAntComponent(ObjectiveJigsawMetaComponent.class);
				if (ojmc != null) {
					this.count = ojmc.getObjectiveJigsawCount();
					this.jbl = BitCodecUtils.upperBitLength(count);
				}
			}
		}
	}
}
