package ant.liaison;

import ant.liaison.channel.assign.ChannelUserInfo;
import ant.liaison.message.AbstractCivilianMessage;
import ant.liaison.message.IMessage;
import ant.liaison.message.MAgentBuildingLocation;
import ant.liaison.message.MAgentBuried;
import ant.liaison.message.MAgentLocation;
import ant.liaison.message.MAgentRescued;
import ant.liaison.message.MAgentRoadCorLocation;
import ant.liaison.message.MAgentRoadLocation;
import ant.liaison.message.MBuildingBurning;
import ant.liaison.message.MBuildingCivilian;
import ant.liaison.message.MCivilianRescued;
import ant.liaison.message.MFindCivilianInBuilding;
import ant.liaison.message.MFindCivilianOnRoad;
import ant.liaison.message.MJigsawApplication;
import ant.liaison.message.MJigsawDoing;
import ant.liaison.message.MOutOfWater;
import ant.liaison.message.MRescueAgentStart;
import ant.liaison.message.MRescueCivilianStart;
import ant.liaison.message.MRescueNextOrder;
import ant.liaison.message.MRescueOrder;
import ant.liaison.message.MRescueOrderUnreachable;
import ant.liaison.message.MRescueUnreachNear;
import ant.liaison.message.MRescueUnreachable;
import ant.liaison.message.MStuckInBuilding;
import ant.liaison.message.MStuckInRoad;
import ant.liaison.message.MTargeBuildingUnreachable;
import ant.liaison.message.MVoiceAgentBuried;
import ant.liaison.message.MVoiceAgentBuriedSimple;
import ant.liaison.message.MVoiceAgentStuckInBuilding;
import ant.liaison.message.MVoiceAgentStuckInRoad;
import ant.liaison.message.MVoiceConfirmBuilding;
import ant.liaison.message.MVoiceExtinguishBuilding;
import ant.liaison.message.MVoiceGoingToBuilding;
import ant.liaison.message.MVoiceHearCivHelp;
import ant.liaison.message.MVoiceRescuingCivilian;
import ant.liaison.message.MZ_DeprecatedPoliceForceRefugeAcception;
import ant.liaison.message.MZ_DeprecatedPoliceForceRescueAgentAcception;

/**
 * 
 * @author Ivan, Oliver, Fun
 * 
 */
public enum MessageType {
	// -------------------------------------------------------------------------------------------------------
	AgentLocation(MAgentLocation.class, 0, false, MessageAcceptor.All),
	// -------------------------------------------------------------------------------------------------------
	AgentBuildingLocation(MAgentBuildingLocation.class, 0, false, MessageAcceptor.All),
	// -------------------------------------------------------------------------------------------------------
	AgentRoadLocation(MAgentRoadLocation.class, 0, false, MessageAcceptor.All),
	// -------------------------------------------------------------------------------------------------------
	AgentRoadCorLocation(MAgentRoadCorLocation.class, 0, false, MessageAcceptor.All),
	// -------------------------------------------------------------------------------------------------------
	AgentBuried(MAgentBuried.class, 1, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	BuildingBurning(MBuildingBurning.class, 2, false, MessageAcceptor.All),
	// -------------------------------------------------------------------------------------------------------
	FindCivilianInBuilding(MFindCivilianInBuilding.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	FindCivilianOnRoad(MFindCivilianOnRoad.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueUnreachable(MRescueUnreachable.class, 3, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	RescueUnreachNear(MRescueUnreachNear.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueOrderUnreachable(MRescueOrderUnreachable.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueOrder(MRescueOrder.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueNextOrder(MRescueNextOrder.class, 2, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueCivilianStart(MRescueCivilianStart.class, 0, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	RescueAgentStart(MRescueAgentStart.class, 0, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	CivilianRescued(MCivilianRescued.class, 1, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	AgentRescued(MAgentRescued.class, 1, false, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// -------------------------------------------------------------------------------------------------------
	StuckInRoad(MStuckInRoad.class, 1, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	StuckInBuilding(MStuckInBuilding.class, 1, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	TargeBuildingUnreachable(MTargeBuildingUnreachable.class, 3, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	JigsawApplication(MJigsawApplication.class, 0, false, MessageAcceptor.PoliceForce, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	JigsawDoing(MJigsawDoing.class, 1, false, MessageAcceptor.PoliceForce, ChannelUserInfo.POLICE_FORCE),
	// -------------------------------------------------------------------------------------------------------
	OutOfWater(MOutOfWater.class, 0, false, MessageAcceptor.All, ChannelUserInfo.FIRE_BRIGADE),
	// -------------------------------------------------------------------------------------------------------
	BuildingCivilian(MBuildingCivilian.class, 3, false, MessageAcceptor.All),
	
	// -------------------------------------------------------------------------------------------------------
	// ----------------------------------------Deprecated-----------------------------------------------------
	// -------------------------------------------------------------------------------------------------------
	PFRefugeAcception(MZ_DeprecatedPoliceForceRefugeAcception.class, 0, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	PFRescueAgentAcception(MZ_DeprecatedPoliceForceRescueAgentAcception.class, 0, false, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// ------------------------------------------------------------------------------------------
	// -------------------------------------------------------------------------------------------------------
	// -------------------------------------VoiceOnly--------------------------------------------
	// ------------------------------------------------------------------------------------------
	VoiceRescuingCivilian(MVoiceRescuingCivilian.class, 3, true, MessageAcceptor.All, ChannelUserInfo.AMBULANCE_TEAM),
	// --------------------------------------------------------------------------------------------------------
	VoiceExtinguishBuilding(MVoiceExtinguishBuilding.class, 1, true, MessageAcceptor.All, ChannelUserInfo.FIRE_BRIGADE),
	// --------------------------------------------------------------------------------------------------------
	VoiceConfirmBuilding(MVoiceConfirmBuilding.class, 1, true, MessageAcceptor.All, ChannelUserInfo.FIRE_BRIGADE),
	// --------------------------------------------------------------------------------------------------------
	VoiceAgentBuried(MVoiceAgentBuried.class, 0, true, MessageAcceptor.All),
	// --------------------------------------------------------------------------------------------------------
	VoiceAgentBuriedSimple(MVoiceAgentBuriedSimple.class, 0, true, MessageAcceptor.All),
	// --------------------------------------------------------------------------------------------------------
	VoiceAgentStuckInBuilding(MVoiceAgentStuckInBuilding.class, 0, true, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// --------------------------------------------------------------------------------------------------------
	VoiceAgentStuckInRoad(MVoiceAgentStuckInRoad.class, 0, true, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// --------------------------------------------------------------------------------------------------------
	VoiceGoingToBuilding(MVoiceGoingToBuilding.class, 0, true, MessageAcceptor.All, ChannelUserInfo.POLICE_FORCE),
	// --------------------------------------------------------------------------------------------------------
	VoiceHearCivHelp(MVoiceHearCivHelp.class, 0, true, MessageAcceptor.All),
	// --------------------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------------------
	CivilianMessage(AbstractCivilianMessage.class, 3, true, MessageAcceptor.All);
	// --------------------------------------------------------------------------------------------------------
	
	
	public static enum MessageAcceptor {
		PoliceForce, FireBridge, AmbulanceTeam, All
	}

	public static final int MAX_PRIORITY = 5;
	private static Integer atbl = null;
	private static Integer nvtbl = null;
	private static Integer noVoiceOnlyMessageCount = null;

	private Class<? extends IMessage> bindBind;
	private int priority;
	private ChannelUserInfo[] receivers;
	private boolean isVoiceOnly = false;
	private MessageAcceptor acceptor;

	/**
	 * @param clazz
	 *            the message class corresponding to the type
	 * @param priority
	 *            the smaller the value is, the higher the priority is, priority should be between 1 - MAX_PRIORITY
	 * @param receivers
	 *            the receivers of messages in this type, if receivers.length == 0, message is for all users
	 */
	private MessageType(Class<? extends IMessage> clazz, int priority, boolean isVoiceOnly, MessageAcceptor acceptor, ChannelUserInfo... receivers) {
		this.acceptor = acceptor;
		this.bindBind = clazz;
		this.priority = Math.max(1, Math.min(MAX_PRIORITY, priority));
		this.receivers = receivers;
		this.isVoiceOnly = isVoiceOnly;
		if (receivers == null || receivers.length == 0) {
			this.receivers = ChannelUserInfo.values();
		}
	}

	public final Class<? extends IMessage> getBindClazz() {
		return this.bindBind;
	}

	public int getPriority() {
		return priority;
	}

	public ChannelUserInfo[] getReceivers() {
		return receivers;
	}

	public boolean isVoiceOnly() {
		return isVoiceOnly;
	}

	public static final int getNoVoiceTypeBitLength() {
		if (nvtbl == null)
			_calc();

		return nvtbl;
	}

	public static final int getNoVoiceOnlyMessageCount() {
		if (noVoiceOnlyMessageCount == null)
			_calc();

		return noVoiceOnlyMessageCount;
	}

	public static final int getAllTypeBitLength() {
		if (atbl == null)
			_calc();

		return atbl;
	}

	private static final void _calc() {
		atbl = BitCodecUtils.upperBitLength(MessageType.values().length);

		int c = 0;
		for (MessageType mt : MessageType.values()) {
			if (!mt.isVoiceOnly)
				c++;
			else
				break;
		}
		noVoiceOnlyMessageCount = c;
		nvtbl = BitCodecUtils.upperBitLength(c);
	}

	public MessageAcceptor getAcceptor() {
		return acceptor;
	}
}
