package ant.liaison;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import ant.liaison.exception.DecodingException;
import ant.liaison.exception.EncodingException;
import ant.liaison.exception.MessagePropertyDescriptionException;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Ivan
 * 
 */
public class MultiMessageCodec {
	private static final Log log = LogFactory.getLog(MultiMessageCodec.class);
	private SingleMessageCodec smc;
	private MessagePropertyCodecComponent mpcc;

	private int ftimelength;
	private int dtimelength;

	public MultiMessageCodec(MessagePropertyCodecComponent mpcc) throws MessagePropertyDescriptionException {
		this.mpcc = mpcc;
		this.smc = new SingleMessageCodec(mpcc, false);
		this.ftimelength = mpcc.getPropertyBitLength(MessageProperty.FullTimestamp);
		this.dtimelength = mpcc.getPropertyBitLength(MessageProperty.DTimestamp);
	}

	public byte[] encode(List<IMessage> waitingMessages, int maxEncodedBytes, int currentTimestamp) throws EncodingException {
		try {
			BitArray tot = new BitArray(0);
			tot.append(BitCodecUtils.encode(currentTimestamp, mpcc.encodeProperty(MessageProperty.FullTimestamp, ftimelength)));
			Iterator<? extends IMessage> iter = waitingMessages.iterator();
			while (iter.hasNext()) {
				IMessage item = iter.next();
				try {
					if (item.getTimestamp() <= 0) {
						throw new EncodingException("Message:" + item + " timestamp is invalid");
					}

					if (mpcc.ignoreMessage(item.getMessageType().getAcceptor())) {
						throw new EncodingException("Not allow send " + item + " to voice");
					}
					
					BitArray tmp = smc.encode(item);
					int dt = currentTimestamp - item.getTimestamp();
					tmp.append(BitCodecUtils.encode(dt, dtimelength));
					if (tot.length() + tmp.length() > maxEncodedBytes * 8) {
						break;
					} else {
						tot.append(tmp);
					}
				} catch (Exception e) {
					log.error(e);
				}
				iter.remove();
			}
			return BitCodecUtils.translateToByteArray(tot);
		} catch (Exception e) {
			throw new EncodingException(e);
		}
	}

	public List<IMessage> decode(byte[] data) {
		List<IMessage> truck = new ArrayList<IMessage>();
		try {
			BitArray tot = BitCodecUtils.translateToBitArray(data);
			int ftime = (Integer) mpcc.decodeProperty(MessageProperty.FullTimestamp, tot.truncationAndDecode(ftimelength));

			while (true) {
				if (!tot.isInvaild()) {
					try {
						IMessage msg = smc.decode(tot);
						if (msg != null) {
							int dt = (Integer) mpcc.decodeProperty(MessageProperty.DTimestamp, tot.truncationAndDecode(dtimelength));
							msg.setTimestamp(ftime - dt);
							truck.add(msg);
						} else {
							log.error("Bad null message!!!");
							break;
						}
					} catch (DecodingException e) {
						log.error(e);
					}
				} else {
					break;
				}
			}
		} catch (DecodingException e) {
			log.error(e);
		}
		return truck;
	}
}
