package ant.liaison;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.messages.Command;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.standard.messages.AKSpeak;
import rescuecore2.worldmodel.EntityID;
import ant.AbstractAntCommunicationKit;
import ant.AgentContext;
import ant.agent.command.AntSpeak;
import ant.agent.command.AntSubscribe;
import ant.exception.AntComponentRegisterException;
import ant.liaison.exception.DecodingException;
import ant.liaison.exception.EncodingException;
import ant.liaison.message.AbstractCivilianMessage;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Ivan
 * 
 */
public class SimpleAntCommunicationKit extends AbstractAntCommunicationKit {
	private static final Log log = LogFactory.getLog(SimpleAntCommunicationKit.class);
	protected int[] detect;
	protected List<AntSpeak> currentSpeak;
	protected Set<IMessage> recbox;

	public SimpleAntCommunicationKit() throws AntComponentRegisterException {
		super();
		currentSpeak = new ArrayList<AntSpeak>();
		detect = new int[1];
		recbox = new HashSet<IMessage>();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		super.initial(context, world);
		this.detect = detectChannel();
	}

	private int[] detectChannel() {
		return new int[] { 1 };
	}

	/**
	 * need first update
	 */
	@Override
	public void update(AgentContext context) {
		super.update(context);

		recbox.clear();
		Collection<Command> cc = context.getNaiveHearing();
		for (Command c : cc) {
			if (c instanceof AKSpeak) {
				AKSpeak message = (AKSpeak) c;
				EntityID sender = message.getAgentID();
				// TODO civ id?
				StandardEntity entity = world.getEntity(sender);
				if (entity == null) {
					continue;
				} else if (entity instanceof Civilian) {
					// TODO
					// String content = new String(message.getContent());
					// if (content.equals(BitCivilianSendHelp.HELP_CONTENT)) {
					// mc.addMessage(new BitCivilianSendHelp(sender));
					// } else if
					// (content.equals(BitCivilianSendOuch.OUCH_CONTENT)) {
					// mc.addMessage(new BitCivilianSendOuch(sender));
					// }
					continue;
				}

				try {
					byte[] bytes = message.getContent();
					if (bytes.length == 0) {
						log.debug("Sender: " + message.getAgentID() + "'s Message dropout");
					} else {
						IMessage msg = getSingleMessageCodec().decode(BitCodecUtils.translateToBitArray(bytes));
						if (msg != null) {
							msg.setSender(sender);
							msg.setTimestamp(context.getTimestamp() - 1);
							recbox.add(msg);
						}
					}
				} catch (DecodingException e) {
					log.error(e);
				}

			}
		}

	}
	
	@Override
	protected void sendVoice(IMessage message) {
		// TODO Auto-generated method stub
	}

	@Override
	protected void sendRadio(IMessage message) {
		if (message == null)
			log.warn("Null speak");
		else {
			message.setTimestamp(context.getTimestamp());
			try {
				// log.debug(message);
				byte[] data = BitCodecUtils.translateToByteArray(getSingleMessageCodec().encode(message));
				currentSpeak.add(new AntSpeak(context.getTimestamp(), 1, data));
			} catch (EncodingException e) {
				log.error(e);
			}
		}
	}

	/**
	 * always detect channel 1
	 */
	@Override
	public AntSubscribe pullSubscribe() {
		return new AntSubscribe(context.getTimestamp(), detect);
	}

	@Override
	public List<AntSpeak> pullallSpeak() {
		List<AntSpeak> tmp = new ArrayList<AntSpeak>();
		for (AntSpeak speak : currentSpeak) {
			tmp.add(speak);
		}
		currentSpeak.clear();
		return tmp;
	}

	@Override
	public Set<IMessage> getRadioReceiveBox() {
		return recbox;
	}

	@Override
	public Set<AbstractCivilianMessage> getMessagesFromCivilian() {
		return Collections.emptySet();
	}

	@Override
	public Set<IMessage> getRadioBoxWithSelf() {
		return recbox;
	}

	@Override
	public Set<IMessage> getVoiceReceiveBox() {
		// TODO Auto-generated method stub
		return Collections.EMPTY_SET;
	}

	// @Override
	// public void sendSubscribe(AntSubscribe subscribe) {
	// if (subscribe == null) {
	// log.warn("Null subscribe");
	// return;
	// }
	// if (currentSubscribe != null) {
	// log.warn("multi-sendSubscribe:" + subscribe);
	// }
	// // replace
	// currentSubscribe = subscribe;
	// }
}
