package ant.liaison.channel;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.config.Config;
import rescuecore2.standard.entities.StandardWorldModel;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;

/**
 * 
 * @author Oliver
 * 
 */
public class ChannelMannager implements AntComponent {
	private static Log log = LogFactory.getLog(ChannelMannager.class);
	private HashMap<Integer, RadioChannel> radioChannels;
	private HashMap<Integer, VoiceChannel> voiceChannels;
	private int channelCount;
	private int channelCentreMax;
	private int channelPlatoonMax;

	@Override
	public String getComponentName() {
		return this.getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		Config config = context.getConfig();

		this.radioChannels = new HashMap<Integer, RadioChannel>();
		this.voiceChannels = new HashMap<Integer, VoiceChannel>();
		this.channelCount = ConfigKey.getChannelCount(config);
		for (int i = 0; i < channelCount; i++) {
			if (ConfigKey.getChannelType(config, i).equals(ConfigKey.CHANNEL_VOICE)) {
				int messageSize = ConfigKey.getChannelMessageSize(config, i);
				int messageMax = ConfigKey.getChannelMessageMax(config, i);
				this.voiceChannels.put(i, new VoiceChannel(i, messageSize, messageMax));
			} else if (ConfigKey.getChannelType(config, i).equals(ConfigKey.CHANNEL_RADIO)) {
				int bandWidth = ConfigKey.getChannelBandWidth(config, i);
				this.radioChannels.put(i, new RadioChannel(i, bandWidth));
			} else {
				log.error("Unkown Channel Type Detected!");
			}
		}

		// in case there is not enough channels
		this.channelCentreMax = Math.min(ConfigKey.getChannelCentreMax(config),
				radioChannels.size());
		this.channelPlatoonMax = Math.min(ConfigKey.getChannelPlatoonMax(config),
				radioChannels.size());
	}

	@Override
	public void update() {
		// empty
	}

	public Map<Integer, RadioChannel> getRadioChannels() {
		return radioChannels;
	}

	public int getRadioChannelCount() {
		return radioChannels.size();
	}

	public Map<Integer, VoiceChannel> getVoiceChannels() {
		return voiceChannels;
	}

	public int getVoiceChannelCount() {
		return voiceChannels.size();
	}

	/**
	 * 
	 * @param id
	 * @return the according channel, null if there is no such channel
	 */
	public AbstractChannel getChannel(Integer id) {
		AbstractChannel rv;
		rv = voiceChannels.get(id);
		if (rv == null) {
			rv = radioChannels.get(id);
		}

		return rv;
	}

	public int getChannelCount() {
		return channelCount;
	}

	public int getChannelCentreMax() {
		return channelCentreMax;
	}

	public int getChannelPlatoonMax() {
		return channelPlatoonMax;
	}
}
