package ant.liaison.channel.assign;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import ant.liaison.channel.RadioChannel;

/**
 * 
 * @author Oliver
 * 
 */
public class ChannelAssignResult {
	private static Log log = LogFactory.getLog(ChannelAssignResult.class);

	private ArrayList<RadioChannel> radioChannelList;
	/**
	 * line for channel, row for ChannelUserInfo; </br> row[0] for
	 * ChannelUserInfo.AMBULANCE_TEAM, </br> row[1] for
	 * ChannelUserInfo.FIRE_BRIGADE,</br> row[2] for
	 * ChannelUserInfo.POLICE_FORCE
	 */
	private int[][] assignTable;
	private int channelWaste;
	private HashMap<ChannelUserInfo, HashMap<RadioChannel, Integer>> userToChannelsMap;
	private HashMap<RadioChannel, HashMap<ChannelUserInfo, Integer>> channelToUsersMap;

	private int atIdealQuota;
	private int fbIdealQuota;
	private int pfIdealQuota;

	private int atActualQuota = 0;
	private int fbActualQuota = 0;
	private int pfActualQuota = 0;

	private int channelPlatoonMax;

	public enum BandwidthLevel {
		Unknown, None, Poor, Low, Mid, High
	}
	private double bandwidthLevelValue;
	private BandwidthLevel bandwidthLevel;

	public ChannelAssignResult(ArrayList<RadioChannel> radioChannelList, int[][] assignTable,
			int channelWaste, int atIdealQuota, int fbIdealQuota, int pfIdealQuota,
			double bandwidthLevelValue, int channelPlatoonMax) {
		if (assignTable.length != radioChannelList.size()) {
			log.error("assignment.length != radioChannelList.size()");
			return;
		}
		this.radioChannelList = new ArrayList<RadioChannel>(radioChannelList);
		this.assignTable = assignTable.clone();
		this.channelWaste = channelWaste;
		this.atIdealQuota = atIdealQuota;
		this.fbIdealQuota = fbIdealQuota;
		this.pfIdealQuota = pfIdealQuota;
		this.channelPlatoonMax = channelPlatoonMax;
		this.bandwidthLevelValue = bandwidthLevelValue;
		calBandwidthLevel();
		handleAssignment();
	}

	private void calBandwidthLevel() {
		if (bandwidthLevelValue <= 0) {
			bandwidthLevel = BandwidthLevel.None;
		} else if (bandwidthLevelValue < 0.15) {
			bandwidthLevel = BandwidthLevel.Poor;
		} else if (bandwidthLevelValue < 0.7) {
			bandwidthLevel = BandwidthLevel.Low;
		} else if (bandwidthLevelValue < 1.8) {
			bandwidthLevel = BandwidthLevel.Mid;
		} else {
			bandwidthLevel = BandwidthLevel.High;
		}
	}

	private void handleAssignment() {
		this.userToChannelsMap = new HashMap<ChannelUserInfo, HashMap<RadioChannel, Integer>>();
		HashMap<RadioChannel, Integer> map;
		map = new HashMap<RadioChannel, Integer>();
		for (int i = 0; i < radioChannelList.size(); i++) {
			map.put(radioChannelList.get(i), assignTable[i][0]);
			atActualQuota += assignTable[i][0];
		}
		userToChannelsMap.put(ChannelUserInfo.AMBULANCE_TEAM, map);
		map = new HashMap<RadioChannel, Integer>();
		for (int i = 0; i < radioChannelList.size(); i++) {
			map.put(radioChannelList.get(i), assignTable[i][1]);
			fbActualQuota += assignTable[i][1];
		}
		userToChannelsMap.put(ChannelUserInfo.FIRE_BRIGADE, map);
		map = new HashMap<RadioChannel, Integer>();
		for (int i = 0; i < radioChannelList.size(); i++) {
			map.put(radioChannelList.get(i), assignTable[i][2]);
			pfActualQuota += assignTable[i][2];
		}
		userToChannelsMap.put(ChannelUserInfo.POLICE_FORCE, map);

		this.channelToUsersMap = new HashMap<RadioChannel, HashMap<ChannelUserInfo, Integer>>();
		for (int i = 0; i < radioChannelList.size(); i++) {
			HashMap<ChannelUserInfo, Integer> rv = new HashMap<ChannelUserInfo, Integer>();
			rv.put(ChannelUserInfo.AMBULANCE_TEAM, assignTable[i][0]);
			rv.put(ChannelUserInfo.FIRE_BRIGADE, assignTable[i][1]);
			rv.put(ChannelUserInfo.POLICE_FORCE, assignTable[i][2]);

			channelToUsersMap.put(radioChannelList.get(i), rv);
		}
	}

	/**
	 * FIXME take care on using this, the map does not necessary contain all
	 * channel
	 * 
	 * @param cui
	 * @return
	 */
	public Map<RadioChannel, Integer> getUserSubscribeChannels(ChannelUserInfo cui) {
		return userToChannelsMap.get(cui);
	}

	public Map<ChannelUserInfo, Integer> getChannelSubscribedUsers(RadioChannel radioChannel) {
		HashMap<ChannelUserInfo, Integer> rv = channelToUsersMap.get(radioChannel);
		if (rv == null) {
			log.error("Unexpected radioChannel used!");
			throw new RuntimeException();
		}
		return rv;
	}

	public int getChannelWaste() {
		return channelWaste;
	}

	public int getIdealQuota(ChannelUserInfo cui) {
		if (cui == ChannelUserInfo.AMBULANCE_TEAM) {
			return atIdealQuota;
		} else if (cui == ChannelUserInfo.FIRE_BRIGADE) {
			return fbIdealQuota;
		} else {
			return pfIdealQuota;
		}
	}

	public int getActualQuota(ChannelUserInfo cui) {
		if (cui == ChannelUserInfo.AMBULANCE_TEAM) {
			return atActualQuota;
		} else if (cui == ChannelUserInfo.FIRE_BRIGADE) {
			return fbActualQuota;
		} else {
			return pfActualQuota;
		}
	}

	public int[] getSubstribeChannels(ChannelUserInfo cui) {
		int[] rv = new int[channelPlatoonMax];
		int index = 0;
		for (Iterator<Entry<RadioChannel, Integer>> it = userToChannelsMap.get(cui).entrySet()
				.iterator(); it.hasNext();) {
			Entry<RadioChannel, Integer> entry = it.next();
			if (entry.getValue() > 0) {
				rv[index++] = entry.getKey().getId();
			}
		}
		return rv;
	}

	public double getBandwidthLevelValue() {
		return bandwidthLevelValue;
	}

	public BandwidthLevel getBandwidthLevel() {
		return bandwidthLevel;
	}

	public ArrayList<RadioChannel> getRadioChannels() {
		return radioChannelList;
	}

	public void print() {
		String str = "\nchannelWaste:" + channelWaste;
		for (ChannelUserInfo cui : ChannelUserInfo.values()) {
			str += "\n" + cui + ":";
			for (Iterator<Entry<RadioChannel, Integer>> in_it = userToChannelsMap.get(cui)
					.entrySet().iterator(); in_it.hasNext();) {
				Entry<RadioChannel, Integer> in_entry = in_it.next();
				str += "[" + in_entry.getKey().getId() + ","
						+ String.format("%4d", in_entry.getValue()) + "] ";
			}
		}
		log.info(str);
	}
}
