package ant.liaison.channel.assign;

import java.util.ArrayList;
import java.util.Collections;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.config.Config;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.liaison.channel.ChannelMannager;
import ant.liaison.channel.RadioChannel;
import ant.utils.CombinationTraversal;

/**
 * 
 * @author Oliver
 * 
 */
public class ChannelAssigner implements AntComponent {
	private static Log log = LogFactory.getLog(ChannelAssigner.class);

	private static final float SPILL_WASTE_COEFFICIENT = 0.2f;
	private static final int MAX_TRAIL_TIME_COUT = 10000;
	private static final int CHANNEL_WASTER_TOLERANCE = 20;

	private ArrayList<RadioChannel> radioChannelList;
	private int channelPlatoonMax;

	private ChannelAssignResult channelAssignResult;
	private Config config;

	private int atIdealQuota;
	private int fbIdealQuota;
	private int pfIdealQuota;

	private double bandWidthLevel;

	private int[] subscribeChannels;

	private AgentContext context;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.config = context.getConfig();
		this.context = context;
	}

	@Override
	public void update() {
		if (context.getTimestamp() == ConfigKey.getIgnoreTime(config) + 2) {
			// right after channel is detected
			ChannelMannager channelMannager = (ChannelMannager) context.getAntCommunicationKit()
					.getAntComponent(ChannelMannager.class);
			this.channelPlatoonMax = channelMannager.getChannelPlatoonMax();
			this.radioChannelList = new ArrayList<RadioChannel>(channelMannager.getRadioChannels()
					.values());
			Collections.sort(radioChannelList, new ChannelComparatorForAssign());
			// for
			// debug////////////////////////////////////////////////////////////////////////////////
			String str = "sorted radioChannelList:";
			for (RadioChannel rc : radioChannelList) {
				str += "(" + rc.getId() + "," + rc.getBandWidth() + ","
						+ String.format("%.2f", rc.getSuccessRate()) + "),";
				if (rc.getSuccessRate() == Double.NaN) {
					log.error("rc.getSuccessRate() == Double.NaN!!! " + rc);
				} else if (rc.getSuccessRate() == 0) {
					log.error("rc.getSuccessRate() == 0 " + rc);
				}
			}
			log.info(str);
			// /////////////////////////////////////////////////////////////////////////////////////////
			if (channelPlatoonMax * 3 < radioChannelList.size()) {
				radioChannelList = new ArrayList<RadioChannel>(radioChannelList.subList(0,
						channelPlatoonMax * 3));
			}

			long timeCost = System.currentTimeMillis();
			setChannelUserInfos();
			assign();
			timeCost = (System.currentTimeMillis() - timeCost);
			if (timeCost > 500) {
				log.info("assgin cost : " + timeCost);
			}

			this.subscribeChannels = new int[channelPlatoonMax];
			if (context.getAgentType() == StandardEntityURN.AMBULANCE_TEAM) {
				subscribeChannels = channelAssignResult
						.getSubstribeChannels(ChannelUserInfo.AMBULANCE_TEAM);
			} else if (context.getAgentType() == StandardEntityURN.FIRE_BRIGADE) {
				subscribeChannels = channelAssignResult
						.getSubstribeChannels(ChannelUserInfo.FIRE_BRIGADE);
			} else if (context.getAgentType() == StandardEntityURN.POLICE_FORCE) {
				subscribeChannels = channelAssignResult
						.getSubstribeChannels(ChannelUserInfo.POLICE_FORCE);
			} else {
				log.error("Unexpected agent type:" + context.getAgentType());
			}

			// ((TempAntCommunicationKit) context.getAntCommunicationKit())
			// .sendSubscribe(new AntSubscribe(context.getTimestamp(),
			// subscribeChannels));
		}
	}

	private void setChannelUserInfos() {
		int atNeed = ChannelUserInfo.AMBULANCE_TEAM.getChannelCostPerAgent()
				* ConfigKey.getTotalATs(config);
		int fbNeed = ChannelUserInfo.FIRE_BRIGADE.getChannelCostPerAgent()
				* ConfigKey.getTotalFBs(config);
		int pfNeed = ChannelUserInfo.POLICE_FORCE.getChannelCostPerAgent()
				* ConfigKey.getTotalPFs(config);
		int totalActualBandWidth = 0;
		for (RadioChannel rc : radioChannelList) {
			totalActualBandWidth += rc.getBandWidth() * rc.getSuccessRate();
		}
		int totalNeed = atNeed + fbNeed + pfNeed;
		this.atIdealQuota = totalActualBandWidth * atNeed / totalNeed;
		this.fbIdealQuota = totalActualBandWidth * fbNeed / totalNeed;
		this.pfIdealQuota = totalActualBandWidth * pfNeed / totalNeed;
		this.bandWidthLevel = totalActualBandWidth * 1.0 / totalNeed;
		log.info("atIdealQuota:" + atIdealQuota + " fbIdealQuota:" + fbIdealQuota
				+ " pfIdealQuota:" + pfIdealQuota);
	}

	private void assign() {
		ArrayList<int[]> combinations = CombinationTraversal.getCombinations(
				radioChannelList.size(), channelPlatoonMax);
		// TODO make combinations in random order may be better
		int minChannelWaste = Integer.MAX_VALUE;
		int radioChannelListSize = radioChannelList.size();
		int[][] bestAssignment = new int[radioChannelList.size()][3];
		int[][] currentAssignment = new int[radioChannelList.size()][3];
		int[] cuter = new int[radioChannelListSize];

		int trailTimeCount = 0;
		TRAIL: {
			for (int[] atChannels : combinations) {
				for (int[] fbChannels : combinations) {
					for (int[] pfChannels : combinations) {
						if (atChannels[0] > 0 && fbChannels[0] > 0 && pfChannels[0] > 0) {
							break;
						}
						int atActual = 0;
						int fbActual = 0;
						int pfActual = 0;

						// for (int i = 0; i < radioChannelListSize; i++) {
						// cuter[i] = 0;
						// }
						// for (int i = 0; i < atChannels.length; i++){
						// cuter[atChannels[i]] = 1;
						// }
						// for (int i = 0; i < pfChannels.length; i++){
						// cuter[pfChannels[i]] = 1;
						// }
						// for (int i = 0; i < fbChannels.length; i++){
						// cuter[fbChannels[i]] = 1;
						// }
						// int maxc = -1;
						// for (int i = radioChannelListSize - 1; i >= 0; i--) {
						// if (cuter[i] != 0) {
						// maxc = i;
						// break;
						// }
						// }
						// boolean b = false;
						// for (int i = maxc - 1; i >= 0; i--) {
						// if (cuter[i] == 0) {
						// b = true;
						// break;
						// }
						// }
						// if (b)
						// break;

						for (int i = 0; i < currentAssignment.length; i++) {
							// clear currentAssignment
							for (int j = 0; j < 3; j++) {
								currentAssignment[i][j] = 0;
							}
							RadioChannel rc = radioChannelList.get(i);
							int denominator = 0;
							for (int j = 0; j < atChannels.length; j++) {
								if (atChannels[j] == i) {
									currentAssignment[i][0] = -1;
									denominator += atIdealQuota;
									break;
								}
							}
							for (int j = 0; j < fbChannels.length; j++) {
								if (fbChannels[j] == i) {
									currentAssignment[i][1] = -1;
									denominator += fbIdealQuota;
									break;
								}
							}
							for (int j = 0; j < pfChannels.length; j++) {
								if (pfChannels[j] == i) {
									currentAssignment[i][2] = -1;
									denominator += pfIdealQuota;
									break;
								}
							}
							if (currentAssignment[i][0] == -1) {
								currentAssignment[i][0] = (int) (rc.getBandWidth()
										* rc.getSuccessRate() * atIdealQuota / denominator);
								atActual += currentAssignment[i][0];
							}
							if (currentAssignment[i][1] == -1) {
								currentAssignment[i][1] = (int) (rc.getBandWidth()
										* rc.getSuccessRate() * fbIdealQuota / denominator);
								fbActual += currentAssignment[i][1];
							}
							if (currentAssignment[i][2] == -1) {
								currentAssignment[i][2] = (int) (rc.getBandWidth()
										* rc.getSuccessRate() * pfIdealQuota / denominator);
								pfActual += currentAssignment[i][2];
							}
						}
						int channelWaste = 0;
						if (atIdealQuota > atActual) {
							channelWaste += atIdealQuota - atActual;
						} else {
							channelWaste += (atIdealQuota - atActual) * SPILL_WASTE_COEFFICIENT;
						}
						if (fbIdealQuota > fbActual) {
							channelWaste += fbIdealQuota - fbActual;
						} else {
							channelWaste += (fbIdealQuota - fbActual) * SPILL_WASTE_COEFFICIENT;
						}
						if (pfIdealQuota > pfActual) {
							channelWaste += pfIdealQuota - pfActual;
						} else {
							channelWaste += (pfIdealQuota - pfActual) * SPILL_WASTE_COEFFICIENT;
						}

						if (minChannelWaste > channelWaste + CHANNEL_WASTER_TOLERANCE) {
							minChannelWaste = channelWaste;
							for (int i = 0; i < radioChannelList.size(); i++) {
								for (int j = 0; j < 3; j++) {
									bestAssignment[i][j] = currentAssignment[i][j];
								}
							}
						}

						if (trailTimeCount++ > MAX_TRAIL_TIME_COUT) {
							log.info("take " + MAX_TRAIL_TIME_COUT
									+ " times trails for channel assign");
							break TRAIL;
						}
					}
				}
			}
		}
		this.channelAssignResult = new ChannelAssignResult(radioChannelList, bestAssignment,
				minChannelWaste, atIdealQuota, fbIdealQuota, pfIdealQuota, bandWidthLevel,
				this.channelPlatoonMax);
		log.info("BandwidthLevelValue:" + channelAssignResult.getBandwidthLevelValue() + ", level:"
				+ channelAssignResult.getBandwidthLevel());
	}

	public ChannelAssignResult getChannelAssignResult() {
		return channelAssignResult;
	}

	public int[] getSubscribeChannels() {
		return subscribeChannels;
	}
}
