package ant.liaison.channel.detect;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.worldmodel.EntityID;
import ant.liaison.channel.ChannelComparator;
import ant.liaison.channel.RadioChannel;
import ant.utils.comparator.EntityIDComparator;

/**
 * 
 * @author Oliver
 * 
 */
public class DetectorAssigner {
	private static Log log = LogFactory.getLog(DetectorAssigner.class);

	private ArrayList<EntityID> agents;
	private ArrayList<RadioChannel> radioChannels;
	private int subscribeMax;
	private HashMap<EntityID, ArrayList<RadioChannel>> agentToChannelsMap;
	private HashMap<RadioChannel, ArrayList<EntityID>> channelToAgentsMap;

	public DetectorAssigner(ArrayList<EntityID> agents, Map<Integer, RadioChannel> radioChannels,
			int subscribeMax) {
		this(agents, new ArrayList<RadioChannel>(radioChannels.values()), subscribeMax);
	}

	public DetectorAssigner(ArrayList<EntityID> agents, ArrayList<RadioChannel> radioChannels,
			int subscribeMax) {
		this.agents = agents;
		this.radioChannels = radioChannels;
		this.subscribeMax = subscribeMax;

		if (agents.size() * subscribeMax < radioChannels.size()) {
			log.warn("Poor channel Condition for DetectorAssigner with agent count:"
					+ agents.size() + ", subscribeMax:" + subscribeMax + ", radioChannels:"
					+ radioChannels.size());
		}

		assign();
	}

	private void assign() {
		Collections.sort(this.agents, new EntityIDComparator());
		Collections.sort(this.radioChannels, new ChannelComparator());
		this.agentToChannelsMap = new HashMap<EntityID, ArrayList<RadioChannel>>();
		for (EntityID id : agents) {
			agentToChannelsMap.put(id, new ArrayList<RadioChannel>());
		}
		this.channelToAgentsMap = new HashMap<RadioChannel, ArrayList<EntityID>>();
		for (RadioChannel channel : radioChannels) {
			channelToAgentsMap.put(channel, new ArrayList<EntityID>());
		}

		if (subscribeMax != 0 && radioChannels.size() != 0) {
			for (int i = 0; i < agents.size(); i++) {
				EntityID id = agents.get(i);
				ArrayList<RadioChannel> atcs = agentToChannelsMap.get(id);
				int index = (i * subscribeMax) % radioChannels.size();
				for (int j = 0; j < subscribeMax; j++) {
					RadioChannel radioChannel = radioChannels.get((index + j)
							% radioChannels.size());
					atcs.add(radioChannel);
					channelToAgentsMap.get(radioChannel).add(id);
				}
			}
		}
		for (EntityID id : agents) {
			Collections.sort(agentToChannelsMap.get(id), new ChannelComparator());
		}
	}

	public HashMap<EntityID, ArrayList<RadioChannel>> getAgentToChannelsMap() {
		return agentToChannelsMap;
	}

	public HashMap<RadioChannel, ArrayList<EntityID>> getChannelToAgentsMap() {
		return channelToAgentsMap;
	}

	public ArrayList<RadioChannel> getSortedRadioChannels() {
		return radioChannels;
	}

	public void print() {
		String str = "DetectorAssigner:";
		for (EntityID id : agents) {
			str += "\nAgent " + id + ":";
			ArrayList<RadioChannel> al = agentToChannelsMap.get(id);
			for (RadioChannel ch : al) {
				str += " " + ch.getId();
			}
		}

		str += "\n*********************************";
		for (RadioChannel channel : radioChannels) {
			str += "\nChannel" + channel + ":";
			ArrayList<EntityID> al = channelToAgentsMap.get(channel);
			for (EntityID id : al) {
				str += " " + id;
			}
		}
		log.info(str);
	}

	public static void main(String[] args) {
		ArrayList<EntityID> ids = new ArrayList<EntityID>();
		Random r = new Random();
		for (int i = 0; i < 50; i++) {
			ids.add(new EntityID(i));
		}
		ArrayList<RadioChannel> chs = new ArrayList<RadioChannel>();
		for (int i = 0; i < 10; i++) {
			chs.add(new RadioChannel(i, Math.abs(r.nextInt() % 10000)));
		}

		DetectorAssigner da = new DetectorAssigner(ids, chs, 3);
		da.print();
	}
}