package ant.liaison.channel.io;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.liaison.message.IMessage;
import ant.liaison.message.MBuildingBurning;
import ant.liaison.message.MFindCivilianInBuilding;

/**
 * 
 * @author Oliver
 * 
 */
public class MessageFilter implements AntComponent {
	private static Log log = LogFactory.getLog(MessageFilter.class);

	private static final int BUILDING_BURNING_NO_REPEAT_TIME = 1;
	private static final int FIND_CIVILIAN_NO_REPEAT_TIME = 1;

	private AgentContext context;

	private HashMap<EntityID, MessageSimpleRecoder> timeMap;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.timeMap = new HashMap<EntityID, MessageSimpleRecoder>();
	}

	@Override
	public void update() {
		if (context.getHuman().getID().getValue() < 0) {
			// in case viewer launch component
			return;
		}
		updateFromMessage();
	}

	private void updateFromMessage() {
		for (IMessage msg : context.getAntCommunicationKit().getRadioReceiveBox()) {
			if (msg instanceof MBuildingBurning) {
				MBuildingBurning bb = (MBuildingBurning) msg;
				if (timeMap.containsKey(bb.getBuilding())) {
					timeMap.get(bb.getBuilding()).setReceiveTime(context.getTimestamp() - 1);
				} else {
					timeMap.put(bb.getBuilding(), new MessageSimpleRecoder(
							context.getTimestamp() - 1));
				}
			} else if (msg instanceof MFindCivilianInBuilding) {
				MFindCivilianInBuilding fc = (MFindCivilianInBuilding) msg;
				if (timeMap.containsKey(fc.getId())) {
					timeMap.get(fc.getSender()).setReceiveTime(context.getTimestamp() - 1);
				} else {
					timeMap.put(fc.getSender(),
							new MessageSimpleRecoder(context.getTimestamp() - 1));
				}
			}
			// TODO else if(other case)
		}
	}

	public void sendMessageAbout(IMessage msg) {
		MessageSimpleRecoder msr = null;
		if (msg instanceof MBuildingBurning) {
			msr = timeMap.get(((MBuildingBurning) msg).getBuilding());
		} else if (msg instanceof MFindCivilianInBuilding) {
			msr = timeMap.get(((MFindCivilianInBuilding) msg).getId());
		}
		// TODO else if(other case)

		if (msr == null) {
			msr = new MessageSimpleRecoder(MessageSimpleRecoder.INVALID_TIME);
		}
		msr.setSendTime(context.getTimestamp());
	}

	public boolean shouldSend(IMessage msg) {
		if (msg instanceof MBuildingBurning) {
			return shouldSendMessageAbout(((MBuildingBurning) msg).getBuilding(),
					BUILDING_BURNING_NO_REPEAT_TIME);
		} else if (msg instanceof MFindCivilianInBuilding) {
			return shouldSendMessageAbout(((MFindCivilianInBuilding) msg).getId(),
					FIND_CIVILIAN_NO_REPEAT_TIME);
		} else {
			return true;
		}
		// TODO other case
	}

	private boolean shouldSendMessageAbout(EntityID id, int noRepeatTime) {
		MessageSimpleRecoder msr = timeMap.get(id);
		if (msr == null) {
			return true;
		}
		int now = context.getTimestamp();
		if (now - msr.getReceiveTime() >= noRepeatTime && now - msr.getSendTime() > noRepeatTime) {
			return true;
		} else {
			return false;
		}
	}

	private static class MessageSimpleRecoder {
		private final static int INVALID_TIME = -1;
		private int receiveTime;
		private int sendTime;

		public MessageSimpleRecoder(int receiveTime) {
			this.receiveTime = receiveTime;
		}

		public int getReceiveTime() {
			return receiveTime;
		}

		public void setReceiveTime(int receiveTime) {
			this.receiveTime = receiveTime;
		}

		public int getSendTime() {
			return sendTime;
		}

		public void setSendTime(int sendTime) {
			this.sendTime = sendTime;
		}
	}

}
