package ant.liaison.channel.io;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.PriorityQueue;
import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.agent.command.AntSpeak;
import ant.liaison.BitCodecUtils;
import ant.liaison.MessageType;
import ant.liaison.SingleMessageCodec;
import ant.liaison.channel.RadioChannel;
import ant.liaison.channel.assign.ChannelAssignResult;
import ant.liaison.channel.assign.ChannelAssigner;
import ant.liaison.channel.assign.ChannelUserInfo;
import ant.liaison.exception.EncodingException;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Oliver
 * 
 */
public class RadioSender implements AntComponent {
	private static Log log = LogFactory.getLog(RadioSender.class);

	/**
	 * MessageType.MAX_PRIORITY matters
	 */
	private static final int[][] MAX_REAPEATE_TIME = { { 1, 1, 1, 1, 1 }, { 2, 2, 1, 1, 1 },
			{ 3, 2, 2, 1, 1 }, { 3, 3, 2, 2, 1 } };
	private static final double[] BAND_WIDTH_LEVEL_DIVIDER = { 0.3, 0.8, 1.2, Double.MAX_VALUE };
	private static final double MIN_QUATO = 5.0d;
	private static final double QUATO_MARGIN_COEFFICIENT = 1.3d;
	private static final double MIN_MSG_RECEIVE_RATE = 0.95d;
	private static final int INITIAL_MSG_QUEUE_CAPACITY = 30;
	private static final long RANDOM_SEED = 0;
	private boolean isDebugOn = false;
	private ChannelAssignResult car;
	private SingleMessageCodec messageCodec;
	private int[] maxRepeateTime;
	private int agentsCount;

	private PriorityQueue<MessageInfo> msgInfoQueue;
	private HashMap<Integer, HashSet<RadioChannel>> commonChannelMap;
	private HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> initialQuotaMap;
	private Random random;
	private ArrayList<AntSpeak> antSpeakMessages;
	private AgentContext context;

	private MessageFilter messageFilter;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		if (messageCodec == null) {
			messageCodec = context.getAntCommunicationKit().getSingleMessageCodec();
		}
		this.msgInfoQueue = new PriorityQueue<MessageInfo>(INITIAL_MSG_QUEUE_CAPACITY,
				new MessageInfoComparator());
		this.random = new Random(RANDOM_SEED);

		this.agentsCount = ConfigKey.getTotalATs(context.getConfig())
				+ ConfigKey.getTotalFBs(context.getConfig())
				+ ConfigKey.getTotalPFs(context.getConfig());

		this.antSpeakMessages = new ArrayList<AntSpeak>();
		this.messageFilter = (MessageFilter) context.getAntCommunicationKit().getAntComponent(
				MessageFilter.class);
	}

	public void setSingleMessageCodec(SingleMessageCodec messageCodec) {
		this.messageCodec = messageCodec;
	}

	private void setMaxRepeateTime(double bandWidthLevel) {
		maxRepeateTime = new int[MessageType.MAX_PRIORITY];
		if (BAND_WIDTH_LEVEL_DIVIDER.length != MAX_REAPEATE_TIME.length) {
			log.error("BAND_WIDTH_LEVEL_DIVIDER.length != MAX_REAPEATE_TIME.length");
		}
		for (int i = 0; i < BAND_WIDTH_LEVEL_DIVIDER.length; i++) {
			if (MessageType.MAX_PRIORITY != MAX_REAPEATE_TIME[i].length) {
				log.error("MessageType.MAX_PRIORITY != MAX_REAPEATE_TIME[i].length");
			}
			if (bandWidthLevel < BAND_WIDTH_LEVEL_DIVIDER[i]) {
				for (int j = 0; j < MessageType.MAX_PRIORITY; j++) {
					maxRepeateTime[j] = MAX_REAPEATE_TIME[i][j];
				}
				break;
			}
		}
		log.info("bandWidthLevel:" + bandWidthLevel);
	}

	private void setChannels() {
		this.commonChannelMap = new HashMap<Integer, HashSet<RadioChannel>>();
		commonChannelMap.put(getIndex(ChannelUserInfo.AMBULANCE_TEAM),
				findCommonChannels(ChannelUserInfo.AMBULANCE_TEAM));

		commonChannelMap.put(getIndex(ChannelUserInfo.FIRE_BRIGADE),
				findCommonChannels(ChannelUserInfo.FIRE_BRIGADE));

		commonChannelMap.put(getIndex(ChannelUserInfo.POLICE_FORCE),
				findCommonChannels(ChannelUserInfo.POLICE_FORCE));

		commonChannelMap.put(
				getIndex(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.FIRE_BRIGADE),
				findCommonChannels(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.FIRE_BRIGADE));

		commonChannelMap.put(
				getIndex(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.POLICE_FORCE),
				findCommonChannels(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.POLICE_FORCE));

		commonChannelMap.put(getIndex(ChannelUserInfo.FIRE_BRIGADE, ChannelUserInfo.POLICE_FORCE),
				findCommonChannels(ChannelUserInfo.FIRE_BRIGADE, ChannelUserInfo.POLICE_FORCE));

		commonChannelMap.put(
				getIndex(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.FIRE_BRIGADE,
						ChannelUserInfo.POLICE_FORCE),
				findCommonChannels(ChannelUserInfo.AMBULANCE_TEAM, ChannelUserInfo.FIRE_BRIGADE,
						ChannelUserInfo.POLICE_FORCE));
	}

	private int getIndex(ChannelUserInfo... infos) {
		int rv = 0;
		for (ChannelUserInfo cui : infos) {
			if (cui == ChannelUserInfo.AMBULANCE_TEAM) {
				rv += 1;
			} else if (cui == ChannelUserInfo.FIRE_BRIGADE) {
				rv += 2;
			} else if (cui == ChannelUserInfo.POLICE_FORCE) {
				rv += 4;
			} else {
				log.error("unknown ChannelUserInfo of " + cui);
			}
		}
		return rv;
	}

	private HashSet<RadioChannel> findCommonChannels(ChannelUserInfo... infos) {
		if (infos.length == 0) {
			log.error("infos.length == 0");
		}
		HashSet<RadioChannel> rv = new HashSet<RadioChannel>();
		for (Iterator<Entry<RadioChannel, Integer>> it = car.getUserSubscribeChannels(infos[0])
				.entrySet().iterator(); it.hasNext();) {
			Entry<RadioChannel, Integer> entry = it.next();
			if (entry.getValue() > 0) {
				rv.add(entry.getKey());
			}
		}
		for (int i = 1; i < infos.length; i++) {
			Map<RadioChannel, Integer> map = car.getUserSubscribeChannels(infos[i]);
			HashSet<RadioChannel> rvCopy = new HashSet<RadioChannel>(rv);
			for (RadioChannel rc : rv) {
				Integer value = map.get(rc);
				if (value == null || value == 0) {
					rvCopy.remove(rc);
				}
			}
			rv = rvCopy;
		}
		// log.info(rv);
		return rv;
	}

	private void setInitialQuotaMap() {
		this.initialQuotaMap = new HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>>();
		for (RadioChannel rc : car.getRadioChannels()) {
			Map<ChannelUserInfo, Integer> csu = car.getChannelSubscribedUsers(rc);
			HashMap<ChannelUserInfo, Double> map = new HashMap<ChannelUserInfo, Double>();
			for (ChannelUserInfo cui : ChannelUserInfo.values()) {
				Integer quota = csu.get(cui);
				map.put(cui,
						Math.max(quota / rc.getSuccessRate() / agentsCount
								* QUATO_MARGIN_COEFFICIENT, quota > 0 ? MIN_QUATO : 0));
			}
			initialQuotaMap.put(rc, map);
		}

		printQuotaMap(initialQuotaMap);
	}

	private void printQuotaMap(HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> map) {
		String str = "\nQuotaMap:";
		for (Iterator<Entry<RadioChannel, HashMap<ChannelUserInfo, Double>>> it = map.entrySet()
				.iterator(); it.hasNext();) {
			Entry<RadioChannel, HashMap<ChannelUserInfo, Double>> entry = it.next();
			str += "\nchannel" + entry.getKey().getId() + ":";
			for (ChannelUserInfo cui : ChannelUserInfo.values()) {
				str += String.format("%.2f", entry.getValue().get(cui)) + " ";
			}
		}
		log.info(str);
	}

	@Override
	public void update() {
		if (context.getTimestamp() < ConfigKey.getIgnoreTime(context.getConfig()) + 2) {
			return;
		}
		if (context.getTimestamp() == ConfigKey.getIgnoreTime(context.getConfig()) + 2) {
			this.car = ((ChannelAssigner) context.getAntCommunicationKit().getAntComponent(
					ChannelAssigner.class)).getChannelAssignResult();
			setMaxRepeateTime(car.getBandwidthLevelValue());
			setChannels();
			setInitialQuotaMap();
		}

		// if (context.getTimestamp() >=
		// ConfigKey.getIgnoreTime(context.getConfig()) + 2) {
		// msgInfoQueue.clear();
		// 目前会发送前两个周期缓存的信息
		antSpeakMessages.clear();
		// }
	}

	public void sendAll(int time) {
		HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> quotaMap = copyQuatoMap(initialQuotaMap);
		int msgDropCount = 0;
		int msgReSendCount = 0;
		while (true) {
			if (msgInfoQueue.isEmpty()) {
				break;
			}

			MessageInfo msgInfo = msgInfoQueue.poll();
			if (!messageFilter.shouldSend(msgInfo.getMessage())) {
				continue;
			}
			ArrayList<RadioChannel> channelToSend = new ArrayList<RadioChannel>();
			ChannelUserInfo[] receivers = msgInfo.getMessageType().getReceivers();

			RadioChannel rc = sendMessage(quotaMap, commonChannelMap.get(getIndex(receivers)),
					msgInfo);
			if (rc != null) {
				channelToSend.add(rc);
			} else {
				if (receivers.length == 1) {
					// msg dropped
				} else if (receivers.length == 2) {
					// try send separately
					RadioChannel rc0 = sendMessage(quotaMap,
							commonChannelMap.get(getIndex(receivers[0])), msgInfo, receivers[0]);
					if (rc0 != null) {
						channelToSend.add(rc0);
					}
					RadioChannel rc1 = sendMessage(quotaMap,
							commonChannelMap.get(getIndex(receivers[1])), msgInfo, receivers[1]);
					if (rc1 != null) {
						channelToSend.add(rc1);
					}
				} else if (receivers.length == 3) {
					// FIXME there are better ways
					RadioChannel rc0 = sendMessage(quotaMap,
							commonChannelMap.get(getIndex(receivers[0])), msgInfo, receivers[0]);
					if (rc0 != null) {
						channelToSend.add(rc0);
					}
					RadioChannel rc12 = sendMessage(quotaMap,
							commonChannelMap.get(getIndex(receivers[1], receivers[2])), msgInfo,
							receivers[1], receivers[2]);
					if (rc12 != null) {
						channelToSend.add(rc12);
					} else {
						RadioChannel rc1 = sendMessage(quotaMap,
								commonChannelMap.get(getIndex(receivers[1])), msgInfo, receivers[1]);
						if (rc1 != null) {
							channelToSend.add(rc1);
						}
						RadioChannel rc2 = sendMessage(quotaMap,
								commonChannelMap.get(getIndex(receivers[2])), msgInfo, receivers[2]);
						if (rc2 != null) {
							channelToSend.add(rc2);
						}
					}
				} else {
					log.error("Unexpected receivers.length:" + receivers.length);
				}
			}

			if (channelToSend.size() != 0) {
				double successRate = 0;
				for (RadioChannel radioChannel : channelToSend) {
					successRate += radioChannel.getSuccessRate();
					antSpeakMessages.add(new AntSpeak(time, radioChannel.getId(), msgInfo
							.getMsgContent()));
				}
				msgInfo.sendMessage(successRate / channelToSend.size());
				if (msgInfo.getRepeateTime() < maxRepeateTime[msgInfo.getMessageType()
						.getPriority() - 1] && msgInfo.getReceiveRate() < MIN_MSG_RECEIVE_RATE) {
					msgInfoQueue.add(msgInfo);
					msgReSendCount++;
				}
			} else {
				msgDropCount++;
			}
		}
		if (isDebugOn) {
			log.info(msgDropCount + " msg dropped and " + msgReSendCount + " msg resend tried @"
					+ time);
		}
	}

	private HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> copyQuatoMap(
			HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> quatoMap) {
		if (quatoMap == null) {
			log.error("quatoMap:" + quatoMap);
			return new HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>>();
		}
		HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> rv = new HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>>(
				quatoMap);
		for (Iterator<Entry<RadioChannel, HashMap<ChannelUserInfo, Double>>> it = quatoMap
				.entrySet().iterator(); it.hasNext();) {
			Entry<RadioChannel, HashMap<ChannelUserInfo, Double>> entry = it.next();
			rv.put(entry.getKey(), new HashMap<ChannelUserInfo, Double>((entry.getValue())));
		}
		return rv;
	}

	private RadioChannel getRandomChannel(ArrayList<RadioChannel> channelsForRandom,
			ArrayList<Double> quotaLeftForRandom) {
		if (channelsForRandom.size() != quotaLeftForRandom.size()) {
			log.error(channelsForRandom.size() != quotaLeftForRandom.size());
		}
		if (channelsForRandom.size() == 1) {
			return channelsForRandom.get(0);
		}
		double totalQuotaLeft = 0;
		for (Double d : quotaLeftForRandom) {
			totalQuotaLeft += d;
		}
		double value = random.nextInt((int) Math.ceil(totalQuotaLeft));
		totalQuotaLeft = 0;
		for (int i = 0; i < channelsForRandom.size(); i++) {
			totalQuotaLeft += quotaLeftForRandom.get(i);
			if (value < totalQuotaLeft) {
				return channelsForRandom.get(i);
			}
		}
		log.error("Random failed!");
		return null;
	}

	/**
	 * 
	 * @param quotaMap
	 * @param commonChannel
	 * @param msgInfo
	 * @param receivers
	 * @return null if it wasn't sent, the RadioChannel msg should be sent
	 */
	private RadioChannel sendMessage(
			HashMap<RadioChannel, HashMap<ChannelUserInfo, Double>> quotaMap,
			HashSet<RadioChannel> commonChannel, MessageInfo msgInfo, ChannelUserInfo... receivers) {
		if (receivers.length == 0) {
			receivers = msgInfo.getMessageType().getReceivers();
		}
		if (commonChannel.size() == 0) {
			return null;
		}
		ArrayList<RadioChannel> channelsForRandom = new ArrayList<RadioChannel>();
		ArrayList<Double> quotaLeftForRandom = new ArrayList<Double>();
		for (RadioChannel rc : commonChannel) {
			boolean flag = false;
			double totoalQuotaLeft = 0d;
			for (ChannelUserInfo cui : receivers) {
				Double quotaLeft = quotaMap.get(rc).get(cui);
				if (quotaLeft > 0) {
					flag = true;
					totoalQuotaLeft += quotaLeft;
				}
			}
			if (flag) {
				channelsForRandom.add(rc);
				quotaLeftForRandom.add(totoalQuotaLeft);
			}
		}

		if (channelsForRandom.size() == 0) {
			// msg dropped
			return null;
		} else {
			// get a random channel to send msg
			RadioChannel rc = getRandomChannel(channelsForRandom, quotaLeftForRandom);
			HashMap<ChannelUserInfo, Double> map = quotaMap.get(rc);
			// get the actual receivers num
			int actualReceiverCount = 0;
			for (ChannelUserInfo cui : receivers) {
				if (map.get(cui) > 0) {
					actualReceiverCount++;
				}
			}
			// update quota for the channel
			for (ChannelUserInfo cui : receivers) {
				map.put(cui, map.get(cui) - msgInfo.getMsgContent().length * 1.0
						/ actualReceiverCount);
			}
			return rc;
		}
	}

	public void pushMessage(IMessage message) {
		try {
			msgInfoQueue.add(new MessageInfo(message, BitCodecUtils
					.translateToByteArray(messageCodec.encode(message))));
		} catch (EncodingException e) {
			// FIXME
			log.error("pushMessage fail:" + message);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Get the messages to be sent in this cycle
	 * 
	 * @return the messages of type AntSpeak
	 */
	public ArrayList<AntSpeak> getRadioMessages() {
		return antSpeakMessages;
	}
}
