package ant.liaison.channel.voice;

import java.util.Comparator;

import ant.liaison.MessageType;
import ant.liaison.message.IMessage;

/**
 * 
 * @author Oliver
 * 
 */
public class VoiceMessageComparator implements Comparator<IMessage> {

	private static final int VOICE_ONLY_IMPORTANCE = 1000;
	private static final int TIME_COEFFICIENT = 2;
	private static final int PRIORITY_COEFFICIENT = 50;
	private static final int RECEIVERS_LENGTH_COEFFICIENT = 1;

	@Override
	public int compare(IMessage o1, IMessage o2) {
		return getMessageImportance(o2) - getMessageImportance(o1);
	}

	/**
	 * The high the return value, the more importance the message is
	 * 
	 * @param msg
	 * @return the importance value of msg
	 */
	private int getMessageImportance(IMessage msg) {
		int rv = 0;

		MessageType type = msg.getMessageType();

		if (type.isVoiceOnly()) {
			rv += VOICE_ONLY_IMPORTANCE;
		}

		// TODO RECENT MSG is more important
		rv += TIME_COEFFICIENT * msg.getTimestamp();

		rv += PRIORITY_COEFFICIENT * type.getPriority();

		rv += RECEIVERS_LENGTH_COEFFICIENT * type.getReceivers().length;

		return rv;
	}
}
