package ant.liaison.channel.voice;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardWorldModel;
import ant.AgentContext;
import ant.AntComponent;
import ant.agent.command.AntSpeak;
import ant.liaison.MultiMessageCodec;
import ant.liaison.channel.ChannelMannager;
import ant.liaison.channel.VoiceChannel;
import ant.liaison.exception.EncodingException;
import ant.liaison.message.IMessage;

/**
 * @author Oliver
 * @depend {@link ChannelMannager}
 */
public class VoiceSender implements AntComponent {
	private static Log log = LogFactory.getLog(VoiceSender.class);
	private static final int INITIAL_MSG_QUEUE_CAPACITY = 100;

	private List<IMessage> messges;
	private VoiceMessageComparator vmc;
	private VoiceChannel voiceChannel;
	private MultiMessageCodec messageCodec;
	private ArrayList<AntSpeak> antSpeakMessages;
	private AgentContext context;

	@Override
	public String getComponentName() {
		return getClass().getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.antSpeakMessages = new ArrayList<AntSpeak>();
		this.messges = new ArrayList<IMessage>(INITIAL_MSG_QUEUE_CAPACITY);
		this.vmc = new VoiceMessageComparator();

		this.voiceChannel = pickVoiceChannel((ChannelMannager) context.getAntCommunicationKit()
				.getAntComponent(ChannelMannager.class));
	}

	/**
	 * 若以后版本出现了多个voice信道，需要修改此处voice只使用最好的那个的问题
	 * 
	 * @param channelMannager
	 * 
	 * @return the VoiceChannel with max ability
	 */
	private VoiceChannel pickVoiceChannel(ChannelMannager channelMannager) {
		int maxSize = Integer.MIN_VALUE;
		VoiceChannel rv = null;
		Collection<VoiceChannel> vcs = channelMannager.getVoiceChannels().values();
		if (vcs.size() != 1) {
			log.error("" + vcs);
		}
		for (VoiceChannel vc : vcs) {
			int size = vc.getMessageMax() * vc.getMessageSize();
			if (size > maxSize) {
				rv = vc;
			}
		}
		// FIXME what if rv == null
		return rv;
	}

	@Override
	public void update() {
		if (voiceChannel == null) {
			return;
		}
		antSpeakMessages.clear();
		messges.clear();
		// TODO add messages to msgQueue
	}

	public void pushMessage(IMessage message) {
		// filter for duplicated messages
		int now = context.getTimestamp();
		if (message.getTimestamp() < 0) {
			// FIXME
			message.setTimestamp(now);
		} else if (message.getTimestamp() > now) {
			log.error("Send Message with invalid timestamp:" + message);
			return;
		}
		messges.add(message);
	}

	public void sendVoiceMessages(int time) {
		if (voiceChannel == null) {
			return;
		}

		if (messges.isEmpty()) {
			return;
		}
		Collections.sort(messges, vmc);

		@SuppressWarnings("unchecked")
		List<IMessage> useMessages = Collections.EMPTY_LIST;
		boolean repeatSend = false;
		for (int i = 0; i < voiceChannel.getMessageMax(); i++) {
			if (useMessages.isEmpty()) {
				if (i != 0) {
					repeatSend = true;
				}
				useMessages = new ArrayList<IMessage>(messges);
			}

			byte[] data;
			try {
				data = messageCodec.encode(useMessages, voiceChannel.getMessageSize(), time);
				if (useMessages.size() == messges.size()) {
					log.warn("No voice msg encoded!");
					break;
				}
				antSpeakMessages.add(new AntSpeak(time, voiceChannel.getId(), data));
				if (!useMessages.isEmpty() && !repeatSend && i == voiceChannel.getMessageMax() - 1) {
					StringBuilder sb = new StringBuilder();
					sb.append("Too many voice messages to send, the following "
							+ useMessages.size() + " messages are left unsent:");
					for (IMessage msg : useMessages) {
						sb.append("\n" + msg);
					}
					log.info(new String(sb));
				}
			} catch (EncodingException e) {
				log.error(e);
			}
		}
	}

	public void setMultiMessageCodec(MultiMessageCodec messageCodec) {
		this.messageCodec = messageCodec;
	}

	/**
	 * Get the messages to be sent in this cycle
	 * 
	 * @return the messages of type AntSpeak
	 */
	public ArrayList<AntSpeak> getAntSpeakMessages() {
		return antSpeakMessages;
	}
	
	public VoiceChannel getVoiceChannel() {
		return voiceChannel;
	}
}
