package ant.liaison.message;

import rescuecore2.standard.entities.Road;
import rescuecore2.worldmodel.EntityID;
import ant.liaison.MessageProperty;
import ant.liaison.MessagePropertyDesc;
import ant.liaison.MessageType;
import ant.liaison.RoadCoord;

/**
 * 
 * @author Oliver
 * 
 */
public class MAgentRoadCorLocation extends AbstractMessage implements IAgentLocationMessage {

	@MessagePropertyDesc(order = 0, bind = MessageProperty.RoadCoord)
	private RoadCoord roadCoord;

	@MessagePropertyDesc(order = 1, bind = MessageProperty.Boolean)
	private boolean isFireSeenAround;

	@MessagePropertyDesc(order = 2, bind = MessageProperty.Boolean)
	private boolean isBlockadeSeenOnTheRoad;

	@MessagePropertyDesc(order = 3, bind = MessageProperty.Boolean)
	private boolean isNoCivilianInSight;

	public MAgentRoadCorLocation() {
	}

	public MAgentRoadCorLocation(Road r, int x, int y, boolean isFireSeenAround,
			boolean isBlockadeSeenOnTheRoad, boolean isNoCivilianInSight) {
		this.roadCoord = new RoadCoord(r, x, y);
		this.isBlockadeSeenOnTheRoad = isBlockadeSeenOnTheRoad;
		this.isFireSeenAround = isFireSeenAround;
		this.isNoCivilianInSight = isNoCivilianInSight;
	}

	public RoadCoord getRoadCoord() {
		return roadCoord;
	}

	public void setRoadCoord(RoadCoord roadCoord) {
		this.roadCoord = roadCoord;
	}

	public int getX() {
		if (roadCoord != null) {
			return roadCoord.getX();
		} else {
			return -1;
		}
	}

	public int getY() {
		if (roadCoord != null) {
			return roadCoord.getY();
		} else {
			return -1;
		}
	}

	@Override
	public EntityID getLocation() {
		if (roadCoord != null) {
			return roadCoord.getRoad().getID();
		} else {
			return null;
		}
	}

	public void setFireSeenAround(boolean isFireSeenAround) {
		this.isFireSeenAround = isFireSeenAround;
	}

	/**
	 * only if return false, can be used to guide around condition
	 * 
	 * @return whether fire can be seen around
	 */
	public boolean isFireSeenAround() {
		return isFireSeenAround;
	}

	public boolean isBlockadeSeenOnTheRoad() {
		return isBlockadeSeenOnTheRoad;
	}

	public void setBlockadeSeenOnTheRoad(boolean isBlockadeSeenOnTheRoad) {
		this.isBlockadeSeenOnTheRoad = isBlockadeSeenOnTheRoad;
	}

	public boolean isNoCivilianInSight() {
		return isNoCivilianInSight;
	}

	public void setNoCivilianInSight(boolean isNoCivilianInSight) {
		this.isNoCivilianInSight = isNoCivilianInSight;
	}

	@Override
	public MessageType getMessageType() {
		return MessageType.AgentRoadCorLocation;
	}

	@Override
	public String toString() {
		return "#" + timestamp + " " + "MAgentRoadCorLocation [roadCoord=" + roadCoord
				+ ", isFireSeenAround=" + isFireSeenAround + ", isBlockadeSeenOnTheRoad="
				+ isBlockadeSeenOnTheRoad + ", isNoCivilianInSight=" + isNoCivilianInSight + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + (isBlockadeSeenOnTheRoad ? 1231 : 1237);
		result = prime * result + (isFireSeenAround ? 1231 : 1237);
		result = prime * result + (isNoCivilianInSight ? 1231 : 1237);
		result = prime * result + ((roadCoord == null) ? 0 : roadCoord.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		MAgentRoadCorLocation other = (MAgentRoadCorLocation) obj;
		if (isBlockadeSeenOnTheRoad != other.isBlockadeSeenOnTheRoad)
			return false;
		if (isFireSeenAround != other.isFireSeenAround)
			return false;
		if (isNoCivilianInSight != other.isNoCivilianInSight)
			return false;
		if (roadCoord == null) {
			if (other.roadCoord != null)
				return false;
		} else if (!roadCoord.equals(other.roadCoord))
			return false;
		return true;
	}

}
