package ant.liaison.message;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Road;
import rescuecore2.worldmodel.EntityID;
import ant.liaison.MessageProperty;
import ant.liaison.MessagePropertyDesc;
import ant.liaison.MessageType;
import ant.liaison.RoadCoord;

/**
 * 
 * @author Ivan
 * 
 */
public class MVoiceAgentStuckInRoad extends AbstractMessage implements IWho, IAgentLocationMessage {
	private static Log log = LogFactory.getLog(MVoiceAgentStuckInRoad.class);

	@MessagePropertyDesc(order = 0, bind = MessageProperty.AgentID)
	private EntityID who;

	@MessagePropertyDesc(order = 1, bind = MessageProperty.RoadCoord)
	private RoadCoord roadCoord;

	public MVoiceAgentStuckInRoad() {
	}

	public MVoiceAgentStuckInRoad(EntityID who, RoadCoord roadCoord) {
		this.roadCoord = roadCoord;
		this.who = who;
	}

	public MVoiceAgentStuckInRoad(EntityID who, Road r, int x, int y) {
		this.roadCoord = new RoadCoord(r, x, y);
		this.who = who;
	}

	@Override
	public EntityID getWho() {
		return who;
	}

	public void setWho(EntityID who) {
		this.who = who;
	}

	public RoadCoord getRoadCoord() {
		return roadCoord;
	}

	public void setRoadCoord(RoadCoord roadCoord) {
		this.roadCoord = roadCoord;
	}

	@Override
	public EntityID getLocation() {
		if (roadCoord == null) {
			log.error("roadCoord:" + roadCoord);
			return null;
		}
		return roadCoord.getRoad().getID();
	}

	@Override
	public MessageType getMessageType() {
		return MessageType.VoiceAgentStuckInRoad;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((roadCoord == null) ? 0 : roadCoord.hashCode());
		result = prime * result + ((who == null) ? 0 : who.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		MVoiceAgentStuckInRoad other = (MVoiceAgentStuckInRoad) obj;
		if (roadCoord == null) {
			if (other.roadCoord != null)
				return false;
		} else if (!roadCoord.equals(other.roadCoord))
			return false;
		if (who == null) {
			if (other.who != null)
				return false;
		} else if (!who.equals(other.who))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "#" + timestamp + " MVoiceAgentStuckInRoad [who=" + who + ", roadCoord=" + roadCoord
				+ "]";
	}
}
