package ant.map.component;

import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.component.universal.ChangeSetDetailComponent;
import ant.liaison.message.IMessage;
import ant.map.objects.RouteResult;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link ChangeSetDetailComponent}
 */
public abstract class AbstractRouteComponent implements AntComponent {

	protected AgentContext context;
	protected StandardWorldModel world;
	protected ChangeSetDetailComponent changeSetDetailComponent;

	@Override
	public final void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		this.changeSetDetailComponent = context.getAntComponentKit().getChangeSetDetailComponent();
		initialInternal();
	}

	@Override
	public final void update() {
		if (context.getTimestamp() < ConfigKey.getIgnoreTime(context.getConfig())) {
			return;
		}
		// First Message
		preupdate();
		for (IMessage msg : context.getAntCommunicationKit().getRadioReceiveBox()) {
			updateMessage(msg);
		}
		// Second ChangeSet
//		for (StandardEntity r : changeSetDetailComponent.getChangedEntities(StandardEntityURN.ROAD).values()) {
//			updateInsightRoad((Road) r);
//		}
		for (StandardEntity r : changeSetDetailComponent.getEntitiesInSight(StandardEntityURN.ROAD)) {
			updateInsightRoad((Road) r);
		}
		laterUpdate();
	}

	protected void preupdate() {
	}
	
	protected void laterUpdate() {
	}
	
	public abstract void initialInternal();

	public abstract RouteResult search(EntityID from, int fromx, int fromy, EntityID to);

	@Deprecated
	public abstract RouteResult search(EntityID from, EntityID to);

	public abstract void updateInsightRoad(Road r);

	public abstract void updateMessage(IMessage message);

	// public abstract Set<MessageType> getInterestMessageTypes();

	
}
