package ant.map.component;

import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;

/**
 * 
 * @author Ivan
 * @depend {@link AntMapBasisComponent}
 */
public class NaiveLaneComponent implements AntComponent {
	private Set<EntityID> laneSet;
	private StandardWorldModel world;
	private AntMapBasisComponent amb;

	@Override
	public String getComponentName() {
		return NaiveLaneComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.world = world;
		this.laneSet = new HashSet<EntityID>();
		this.amb = context.getAntComponentKit().getAntMapBasisComponent();
		initLane();
	}

	@Override
	public void update() {
	}
	
	public boolean isLane(EntityID roadID) {
		return laneSet.contains(roadID);
	}
	
	private void initLane() {
		for (Road road : amb.getRoadSet()) {
			if (!laneSet.contains(road.getID())) {
				int b = 0;
				int r = 0;
				for (Edge edge : road.getEdges()) {
					if (edge.isPassable()) {
						EntityID nei = edge.getNeighbour();
						StandardEntity entity = world.getEntity(nei);
						if (entity instanceof Building) {
							b++;
						}
						if (entity instanceof Road) {
							r++;
						}
					}
				}
				if (b >= 1 && r == 1) {
					laneSet.add(road.getID());
				}
			}
		}
	}

}