package ant.map.component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.objects.RoadSide;
import ant.map.objects.RoadSidePair;
import ant.utils.LazyMap;

/**
 * provide {@link Passage} {@link RoadSide} {@link RoadSidePair}
 * 
 * @author Ivan
 * 
 * @depend {@link AntMapBasisComponent} {@link WorldMaintainComponent}
 */
public class RoadPassObjectsComponent implements AntComponent {
	private static Log log = LogFactory.getLog(RoadPassObjectsComponent.class);

	private AgentContext context;
	private StandardWorldModel world;
	private Map<EntityID, List<RoadSide>> sideMap;
	private Map<EntityID, List<RoadSidePair>> pairMap;
	private Map<EntityID, Map<EntityID, RoadSidePair>> rrPairMap;

	// private Map<EntityID, List<Passage>> passageMap;

	@Override
	public String getComponentName() {
		return RoadPassObjectsComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.context = context;
		this.world = world;
		initialRoadPassableEdge();
		initialRoadPassableEdgePair();
	}

	public List<RoadSide> getSides(EntityID roadId) {
		return this.sideMap.get(roadId);
	}

	public List<RoadSidePair> getSidePairs(EntityID roadId) {
		return this.pairMap.get(roadId);
	}

	public RoadSidePair getSidePair(EntityID road0, EntityID road1) {
		return rrPairMap.get(road0).get(road1);
	}

	private void initialRoadPassableEdgePair() {
		
		this.pairMap = new LazyMap<EntityID, List<RoadSidePair>>() {
			@Override
			public List<RoadSidePair> createValue() {
				return new ArrayList<RoadSidePair>();
			}
		};
		
		this.rrPairMap = new LazyMap<EntityID, Map<EntityID, RoadSidePair>>() {
			@Override
			public Map<EntityID, RoadSidePair> createValue() {
				return new HashMap<EntityID, RoadSidePair>();
			}
		};
		
		for (Road r : context.getAntComponentKit().getAntMapBasisComponent().getRoadSet()) {
			EntityID from = r.getID(); 
			for (RoadSide side : sideMap.get(from)) {
				if (side.isRoadNeighbour()) {
					EntityID to = side.getNeighbourID();
					for (RoadSide other : sideMap.get(to)) {
						if (other.getNeighbourID().equals(from)) {
							RoadSidePair rsp = new RoadSidePair(side, other);
							rrPairMap.get(from).put(to, rsp);
							rrPairMap.get(to).put(from, rsp);
						}
					}
				}
			}
		}
		
		for (Road r : context.getAntComponentKit().getAntMapBasisComponent().getRoadSet()) {
//			for (RoadSidePair pair : rrPairMap.get(r.getID()).values()) {
//				pairMap.get(r.getID()).add(pair);
//				pair.getLeft().setPair(pair);
//				pair.getRight().setPair(pair);
//			}
			for (RoadSide side : sideMap.get(r.getID())) {
				if (side.isRoadNeighbour()) {
					EntityID id = side.getNeighbourID();
					for (RoadSide other : sideMap.get(id)) {
						if (other.getNeighbourID().equals(r.getID())) {
							RoadSidePair pair = rrPairMap.get(r.getID()).get(id);
							if (pair == null) {
								throw new IllegalStateException();
							} else {
//								if (r.getID().getValue() == 4323 || r.getID().getValue() == 32855 || id.getValue() == 32855 || id.getValue() == 4323) {
//									log.info("");
//									log.info(side + " | " + other);
//									log.info(pair.getLeft().getRoad().getID() + "," + pair.getRight().getRoad().getID() + " | " + r.getID() + "," + id);
//									log.info(pair);
//								}
								side.setPair(pair);
							}
						}
					}
				}
			}
		}
	}

	private void initialRoadPassableEdge() {
		sideMap = new LazyMap<EntityID, List<RoadSide>>() {
			@Override
			public List<RoadSide> createValue() {
				return new ArrayList<RoadSide>();
			}
		};

		for (Road r : context.getAntComponentKit().getAntMapBasisComponent().getRoadSet()) {
			for (Edge e : r.getEdges()) {
				if (e.isPassable()) {
					RoadSide rpe = new RoadSide(world, r, e);
					sideMap.get(r.getID()).add(rpe);
				}
			}
		}

		for (EntityID id : sideMap.keySet()) {
			sideMap.put(id, Collections.unmodifiableList(sideMap.get(id)));
		}
	}

	// private void initialPassage() {
	// this.passageMap = new LazyMap<EntityID, List<Passage>>() {
	// @Override
	// public List<Passage> createValue() {
	// return new ArrayList<Passage>();
	// }
	// };
	// int c = 0;
	// for (Road r :
	// context.getAntComponentKit().getAntWorldComponent().getRoadSet()) {
	// int size = r.getEdges().size();
	// for (int i = 0; i < size - 1; i++) {
	// if (r.getEdges().get(i).isPassable()) {
	// for (int j = i + 1; j < size; j++) {
	// if (r.getEdges().get(j).isPassable()) {
	// passageMap.get(r.getID()).add(new Passage(r, i, j));
	// c++;
	// }
	// }
	// }
	// }
	// }
	// log.info("PassageCount:" + c);
	// }

	@Override
	public void update() {
	}

}
