package ant.map.component;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;

/**
 * 
 * @author Ivan
 * 
 * @depend null
 */
public class WorldMaintainComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(WorldMaintainComponent.class);
	
	private StandardWorldModel world;

	@Override
	public String getComponentName() {
		return WorldMaintainComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.world = world;
		maintainEdge();
	}

	private void maintainEdge() {
		for (final StandardEntity entity : world.getEntitiesOfType(StandardEntityURN.BUILDING,
				StandardEntityURN.AMBULANCE_CENTRE, StandardEntityURN.FIRE_STATION, StandardEntityURN.POLICE_OFFICE,
				StandardEntityURN.REFUGE, StandardEntityURN.ROAD)) {
			Set<EntityID> to = new HashSet<EntityID>();
			Area area = (Area) entity;
			Iterator<Edge> iter = area.getEdges().iterator();
			boolean maintain = false;
			while (iter.hasNext()) {
				Edge edge = iter.next();
				if (edge.isPassable()) {
					if (to.contains(edge.getNeighbour())) {
						// log.warn("Area:" + area.getID() + " to:" + edge.getNeighbour() + " 有多重边");
						maintain = true;
					} else {
						to.add(edge.getNeighbour());
					}
				}
			}
			if (maintain) {
				to.clear();
				List<Edge> edges = new ArrayList<Edge>();
				for (Edge edge : area.getEdges()) {
					if (edge.isPassable()) {
						if (to.contains(edge.getNeighbour())) {
							//log.warn("Area:" + area.getID() + " to:" + edge.getNeighbour() + " 多重边, 去掉Passable属性");
							edges.add(new Edge(edge.getStart(), edge.getEnd()));
						} else {
							edges.add(edge);
							to.add(edge.getNeighbour());
						}
					} else {
						edges.add(edge);
					}
				}
				area.setEdges(edges);
			}
		}
	}

	@Override
	public void update() {
	}

}
