package ant.map.jigsaw;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Road;

/**
 * contains Buildings and Entrances
 * 
 * @author Ivan
 * 
 */
public class JigsawAttach extends JigsawObject {
	private static final Log log = LogFactory.getLog(JigsawAttach.class);
	
	private Set<Building> buildings;
	private Set<Road> entrances;
	private HashSet<JigsawObject> enclosed;
	private JigsawObject fixed;
	
	public JigsawAttach(int jigsawID, Set<Area> raw) {
		super(jigsawID, raw);
		
		this.buildings = new HashSet<Building>();
		this.entrances = new HashSet<Road>();
		this.enclosed = new HashSet<JigsawObject>();
		
		for (Area a : raw) {
			if (a instanceof Building) {
				buildings.add((Building) a);
			} else if (a instanceof Road) {
				entrances.add((Road) a);
			}
		}
		
		this.fixed = null;
	}
	
	@Override
	public JigsawType getType() {
		return JigsawType.Attach;
	}
	
	@Override
	public String toString() {
		return "[JAT " + rawAreaIDs + "]";
	}
	
	public void addEnclosed(JigsawObject jo) {
		if (jo.getType() == JigsawType.Avenue || jo.getType() == JigsawType.Cross) {
			this.enclosed.add(jo);
			
			if (fixed == null) {
				fixed = jo;
			} else {
				if (jo.getJigsawSize() > fixed.getJigsawSize()) {
					fixed = jo;
				}
			}
		} else {
			log.error("Bad enclosed object");
		}
	}
	
	public HashSet<JigsawObject> getEncloseds() {
		return enclosed;
	}
	
	public JigsawObject getFixedEnclosed() {
		return this.fixed;
	}
}
