package ant.map.jigsaw;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.map.component.AntMapBasisComponent;

/**
 * 
 * @author Ivan, Sarah
 * 
 * @depend {@link AntMapBasisComponent}
 */

// TODO check all area index
public class JigsawMarkerComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(JigsawMarkerComponent.class);
	private static final int AGENT_RADIUS = 500;
	private static final double IGNORE_SCALE = 1.0;

	private Map<EntityID, RoadMark> mmap;
	private Set<EntityID> entranceSet;
	private Set<EntityID> crossSet;
	private Set<EntityID> avenueSet;

	private StandardWorldModel world;

	private AntMapBasisComponent comp;

	public enum RoadMark {
		Entrance, Cross, Avenue
	};

	public JigsawMarkerComponent() {
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		this.mmap = new HashMap<EntityID, RoadMark>();
		this.entranceSet = new HashSet<EntityID>();
		this.crossSet = new HashSet<EntityID>();
		this.avenueSet = new HashSet<EntityID>();
		this.world = world;
		this.comp = context.getAntComponentKit().getAntMapBasisComponent();
		// entranceMark4();
		// entranceMark2();
		// entranceMark3();
		// crossMark2();
		try {
			makeGraph();
			entranceMark0();
			entranceMark1();
			entranceMark11();
			entranceMark2();
			entranceMark3();
			entranceMark3();
			entranceMark4();
			entranceMark5();
			entranceMark6();
			avenueMark();
			crossMark();
		} catch (Exception e) {
			log.error(e);
		}
	}

	/** mark all nei-building **/
	private void entranceMark0() {
		for (Road road : comp.getRoadSet()) {
			boolean isEntrance = true;
			for (EntityID nei : road.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Road) {
					isEntrance = false;
					break;
				}
			}

			if (isEntrance) {
				mmap.put(road.getID(), RoadMark.Entrance);
				entranceSet.add(road.getID());
			}
		}
	}

	private void entranceMark1() {
		for (Road road : comp.getRoadSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : road.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (r == 0)
						left = (Area) entity;
					else
						right = (Area) entity;
					r++;
				}
			}

			// r == 1
			if (b >= 1 && r == 1) {
				mmap.put(road.getID(), RoadMark.Entrance);
				entranceSet.add(road.getID());
			}
			if (b >= 1 && r == 2) {
				// left - right
				// left - mid - right
				if (left.getEdgeTo(right.getID()) != null && right.getEdgeTo(left.getID()) != null) {
					mmap.put(road.getID(), RoadMark.Entrance);
					entranceSet.add(road.getID());
				} else {
					Area mid = null;
					for (EntityID tmp : left.getNeighbours()) {
						if (right.getEdgeTo(tmp) != null) {
							if (!tmp.equals(road.getID())) {
								mid = (Area) world.getEntity(tmp);
								break;
							}
						}
					}
					if (mid != null) {

						Edge roadToLeft = road.getEdgeTo(left.getID());
						Edge roadToRight = road.getEdgeTo(right.getID());
						Edge leftToMid = left.getEdgeTo(mid.getID());
						Edge rightToMid = right.getEdgeTo(mid.getID());

						double roadToLeftLength = Math.hypot(
								roadToLeft.getEndX() - roadToLeft.getStartX(), roadToLeft.getEndY()
										- roadToLeft.getStartY());
						double roadToRightLength = Math.hypot(
								roadToRight.getEndX() - roadToRight.getStartX(),
								roadToRight.getEndY() - roadToRight.getStartY());
						double leftToMidLength = Math.hypot(
								leftToMid.getEndX() - leftToMid.getStartX(), leftToMid.getEndY()
										- leftToMid.getStartY());
						double rightToMidLength = Math.hypot(
								rightToMid.getEndX() - rightToMid.getStartX(), rightToMid.getEndY()
										- rightToMid.getStartY());
						if (roadToLeftLength < leftToMidLength
								|| roadToRightLength < rightToMidLength) {
							mmap.put(road.getID(), RoadMark.Entrance);
							entranceSet.add(road.getID());
						}
					}
				}
			}

		}
	}

	private void entranceMark11() {
		for (Road road : comp.getRoadSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : road.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (r == 0)
						left = (Area) entity;
					else
						right = (Area) entity;
					r++;
				}
			}

			// r == 1
			if (b >= 1 && r == 1) {
				mmap.put(road.getID(), RoadMark.Entrance);
				entranceSet.add(road.getID());
			}
			if (b >= 1 && r == 2) {
				// left - right
				// left - mid - right
				if (left.getEdgeTo(right.getID()) != null && right.getEdgeTo(left.getID()) != null) {
					mmap.put(road.getID(), RoadMark.Entrance);
					entranceSet.add(road.getID());
				} else {
					for (EntityID midLeft : left.getNeighbours()) {
						Area midareaLeft = (Area) world.getEntity(midLeft);

						if (midareaLeft instanceof Building)
							continue;
						if (midLeft.equals(left.getID()))
							continue;
						if (midLeft.equals(right.getID()))
							continue;
						if (midLeft.equals(road.getID()))
							continue;

						for (EntityID midRight : right.getNeighbours()) {
							Area midareaRight = (Area) world.getEntity(midRight);

							if (midareaRight instanceof Building)
								continue;
							if (midRight.equals(right.getID()))
								continue;
							if (midRight.equals(left.getID()))
								continue;
							if (midRight.equals(road.getID()))
								continue;

							if (midareaLeft.getEdgeTo(midRight) != null) {
								Edge roadToLeft = road.getEdgeTo(left.getID());
								Edge roadToRight = road.getEdgeTo(right.getID());
								Edge leftToMidLeft = left.getEdgeTo(midLeft);
								Edge rightToMidRight = right.getEdgeTo(midRight);

								double roadToLeftLength = Math.hypot(roadToLeft.getEndX()
										- roadToLeft.getStartX(),
										roadToLeft.getEndY() - roadToLeft.getStartY());
								double roadToRightLength = Math.hypot(roadToRight.getEndX()
										- roadToRight.getStartX(), roadToRight.getEndY()
										- roadToRight.getStartY());
								double leftToMidLeftLength = Math.hypot(leftToMidLeft.getEndX()
										- leftToMidLeft.getStartX(), leftToMidLeft.getEndY()
										- leftToMidLeft.getStartY());
								double rightToMidRightLength = Math.hypot(rightToMidRight.getEndX()
										- rightToMidRight.getStartX(), rightToMidRight.getEndY()
										- rightToMidRight.getStartY());
								if (roadToLeftLength < leftToMidLeftLength
										|| roadToRightLength < rightToMidRightLength) {
									mmap.put(road.getID(), RoadMark.Entrance);
									entranceSet.add(road.getID());
								}

							}
						}
					}
				}

			}

		}
	}

	private void entranceMark2() {
		while (true) {
			int c = 0;
			for (Road road : comp.getRoadSet()) {
				if (!entranceSet.contains(road.getID())) {
					int b = 0;
					int r = 0;
					Area left = null, right = null;
					for (EntityID nei : road.getNeighbours()) {
						StandardEntity entity = world.getEntity(nei);
						if (entity instanceof Building) {
							b++;
						}
						if (entity instanceof Road) {
							if (r == 0)
								left = (Area) entity;
							else
								right = (Area) entity;
							r++;
						}
					}
					if (r == 2) {
						if (entranceSet.contains(left.getID())
								|| entranceSet.contains(right.getID())) {
							mmap.put(road.getID(), RoadMark.Entrance);
							entranceSet.add(road.getID());
							c++;
						}
					}
				}
			}
			if (c == 0) {
				break;
			}
		}
	}

	private void entranceMark3() {
		for (Road road : comp.getRoadSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : road.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (!entranceSet.contains(nei)) {
						if (r == 0)
							left = (Area) entity;
						else
							right = (Area) entity;
						r++;
					}
				}
			}
			// r == 1
			if (b >= 1 && r == 1) {
				mmap.put(road.getID(), RoadMark.Entrance);
				entranceSet.add(road.getID());
			}
			if (b >= 1 && r == 2) {
				// left - right
				// left - mid - right
				if (left.getEdgeTo(right.getID()) != null && right.getEdgeTo(left.getID()) != null) {
					mmap.put(road.getID(), RoadMark.Entrance);
					entranceSet.add(road.getID());
				} else {
					Area mid = null;
					for (EntityID tmp : left.getNeighbours()) {
						if (right.getEdgeTo(tmp) != null) {
							if (!tmp.equals(road.getID())) {
								mid = (Area) world.getEntity(tmp);
								break;
							}
						}
					}
					if (mid != null) {
						Edge roadToLeft = road.getEdgeTo(left.getID());
						Edge roadToRight = road.getEdgeTo(right.getID());
						Edge leftToMid = left.getEdgeTo(mid.getID());
						Edge rightToMid = right.getEdgeTo(mid.getID());

						double roadToLeftLength = Math.hypot(
								roadToLeft.getEndX() - roadToLeft.getStartX(), roadToLeft.getEndY()
										- roadToLeft.getStartY());
						double roadToRightLength = Math.hypot(
								roadToRight.getEndX() - roadToRight.getStartX(),
								roadToRight.getEndY() - roadToRight.getStartY());
						double leftToMidLength = Math.hypot(
								leftToMid.getEndX() - leftToMid.getStartX(), leftToMid.getEndY()
										- leftToMid.getStartY());
						double rightToMidLength = Math.hypot(
								rightToMid.getEndX() - rightToMid.getStartX(), rightToMid.getEndY()
										- rightToMid.getStartY());
						if (roadToLeftLength < leftToMidLength
								|| roadToRightLength < rightToMidLength) {
							mmap.put(road.getID(), RoadMark.Entrance);
							entranceSet.add(road.getID());
						}
					}
				}
			}

		}
	}

	private void entranceMark4() {
		for (Building building : comp.getBuildingSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : building.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (!entranceSet.contains(nei)) {
						if (r == 0)
							left = (Area) entity;
						else
							right = (Area) entity;
						r++;
					}
				}
			}

			if (r != 2)
				continue;

			if (avenueSet.contains(left.getID()) || avenueSet.contains(right.getID())) {
				continue;
			}

			if (left.getEdgeTo(right.getID()) == null || right.getEdgeTo(left.getID()) == null) {
				continue;
			}

			Road leftNextRoad = getOnlyNeighborRoad(left, right.getID());
			Road rightNextRoad = getOnlyNeighborRoad(right, left.getID());

			if (leftNextRoad == null || rightNextRoad == null)
				continue;

			if (leftNextRoad.getEdgeTo(rightNextRoad.getID()) != null) {
				mmap.put(left.getID(), RoadMark.Entrance);
				entranceSet.add(left.getID());

				mmap.put(right.getID(), RoadMark.Entrance);
				entranceSet.add(right.getID());
			} else {
				for (EntityID mid : leftNextRoad.getNeighbours()) {
					Area midarea = (Area) world.getEntity(mid);
					if (midarea instanceof Building)
						continue;
					if (mid.equals(left.getID()))
						continue;
					if (mid.equals(right.getID()))
						continue;

					if (rightNextRoad.getEdgeTo(mid) != null) {
						Edge leftToLeftNext = left.getEdgeTo(leftNextRoad.getID());
						Edge rightToRightNext = right.getEdgeTo(rightNextRoad.getID());
						Edge leftNextToMid = leftNextRoad.getEdgeTo(mid);
						Edge rightNextToMid = rightNextRoad.getEdgeTo(mid);

						double leftToLeftNextLength = Math.hypot(leftToLeftNext.getEndX()
								- leftToLeftNext.getStartX(), leftToLeftNext.getEndY()
								- leftToLeftNext.getStartY());
						double rightToRightNextLength = Math.hypot(rightToRightNext.getEndX()
								- rightToRightNext.getStartX(), rightToRightNext.getEndY()
								- rightToRightNext.getStartY());
						double leftNextToMidLength = Math.hypot(leftNextToMid.getEndX()
								- leftNextToMid.getStartX(), leftNextToMid.getEndY()
								- leftNextToMid.getStartY());
						double rightNextToMidLength = Math.hypot(rightNextToMid.getEndX()
								- rightNextToMid.getStartX(), rightNextToMid.getEndY()
								- rightNextToMid.getStartY());
						if (leftToLeftNextLength < leftNextToMidLength
								|| rightToRightNextLength < rightNextToMidLength) {
							mmap.put(left.getID(), RoadMark.Entrance);
							entranceSet.add(left.getID());

							mmap.put(right.getID(), RoadMark.Entrance);
							entranceSet.add(right.getID());
						}

					}
				}
			}
		}
	}

	private void entranceMark5() {
		for (Building building : comp.getBuildingSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : building.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (!entranceSet.contains(nei)) {
						if (r == 0)
							left = (Area) entity;
						else
							right = (Area) entity;
						r++;
					}
				}
			}

			if (r != 2)
				continue;

			if (avenueSet.contains(left.getID()) || avenueSet.contains(right.getID())) {
				continue;
			}

			if (left.getEdgeTo(right.getID()) == null || right.getEdgeTo(left.getID()) == null) {
				continue;
			}

			Road leftNextRoad = getOnlyNeighborRoadWithoutEntrance(left, right.getID());
			Road rightNextRoad = getOnlyNeighborRoadWithoutEntrance(right, left.getID());

			if (leftNextRoad == null || rightNextRoad == null)
				continue;

			if (leftNextRoad.getEdgeTo(rightNextRoad.getID()) != null) {
				mmap.put(left.getID(), RoadMark.Entrance);
				entranceSet.add(left.getID());

				mmap.put(right.getID(), RoadMark.Entrance);
				entranceSet.add(right.getID());
			} else {
				for (EntityID mid : leftNextRoad.getNeighbours()) {
					Area midarea = (Area) world.getEntity(mid);
					if (midarea instanceof Building)
						continue;
					if (mid.equals(left.getID()))
						continue;
					if (mid.equals(right.getID()))
						continue;

					if (rightNextRoad.getEdgeTo(mid) != null) {
						Edge leftToLeftNext = left.getEdgeTo(leftNextRoad.getID());
						Edge rightToRightNext = right.getEdgeTo(rightNextRoad.getID());
						Edge leftNextToMid = leftNextRoad.getEdgeTo(mid);
						Edge rightNextToMid = rightNextRoad.getEdgeTo(mid);

						double leftToLeftNextLength = Math.hypot(leftToLeftNext.getEndX()
								- leftToLeftNext.getStartX(), leftToLeftNext.getEndY()
								- leftToLeftNext.getStartY());
						double rightToRightNextLength = Math.hypot(rightToRightNext.getEndX()
								- rightToRightNext.getStartX(), rightToRightNext.getEndY()
								- rightToRightNext.getStartY());
						double leftNextToMidLength = Math.hypot(leftNextToMid.getEndX()
								- leftNextToMid.getStartX(), leftNextToMid.getEndY()
								- leftNextToMid.getStartY());
						double rightNextToMidLength = Math.hypot(rightNextToMid.getEndX()
								- rightNextToMid.getStartX(), rightNextToMid.getEndY()
								- rightNextToMid.getStartY());
						if (leftToLeftNextLength < leftNextToMidLength
								|| rightToRightNextLength < rightNextToMidLength) {
							mmap.put(left.getID(), RoadMark.Entrance);
							entranceSet.add(left.getID());

							mmap.put(right.getID(), RoadMark.Entrance);
							entranceSet.add(right.getID());
						}

					}
				}
			}
		}
	}

	private void entranceMark6() {
		for (Building building : comp.getBuildingSet()) {
			int b = 0;
			int r = 0;
			Area left = null, right = null;
			for (EntityID nei : building.getNeighbours()) {
				StandardEntity entity = world.getEntity(nei);
				if (entity instanceof Building) {
					b++;
				}
				if (entity instanceof Road) {
					if (!entranceSet.contains(nei)) {
						if (r == 0)
							left = (Area) entity;
						else
							right = (Area) entity;
						r++;
					}
				}
			}

			if (r != 2)
				continue;

			if (avenueSet.contains(left.getID()) || avenueSet.contains(right.getID())) {
				continue;
			}

			if (left.getEdgeTo(right.getID()) == null || right.getEdgeTo(left.getID()) == null) {
				continue;
			}

			Road leftNextRoad = getOnlyNeighborRoadWithoutEntrance(left, right.getID());
			Road rightNextRoad = getOnlyNeighborRoadWithoutEntrance(right, left.getID());

			if (leftNextRoad == null || rightNextRoad == null)
				continue;

			if (leftNextRoad.getEdgeTo(rightNextRoad.getID()) != null) {
				mmap.put(left.getID(), RoadMark.Entrance);
				entranceSet.add(left.getID());

				mmap.put(right.getID(), RoadMark.Entrance);
				entranceSet.add(right.getID());
			} else {
				for (EntityID midLeft : leftNextRoad.getNeighbours()) {
					Area midareaLeft = (Area) world.getEntity(midLeft);

					if (midareaLeft instanceof Building)
						continue;
					if (midLeft.equals(leftNextRoad.getID()))
						continue;
					if (midLeft.equals(rightNextRoad.getID()))
						continue;
					if (midLeft.equals(building.getID()))
						continue;

					for (EntityID midRight : rightNextRoad.getNeighbours()) {
						Area midareaRight = (Area) world.getEntity(midRight);

						if (midareaRight instanceof Building)
							continue;
						if (midRight.equals(rightNextRoad.getID()))
							continue;
						if (midRight.equals(leftNextRoad.getID()))
							continue;
						if (midRight.equals(building.getID()))
							continue;

						if (midareaLeft.getEdgeTo(midRight) != null) {
							Edge leftToLeftNext = left.getEdgeTo(leftNextRoad.getID());
							Edge rightToRightNext = right.getEdgeTo(rightNextRoad.getID());
							Edge leftNextToLeftMid = leftNextRoad.getEdgeTo(midLeft);
							Edge rightNextToRightMid = rightNextRoad.getEdgeTo(midRight);

							double leftToLeftNextLength = Math.hypot(leftToLeftNext.getEndX()
									- leftToLeftNext.getStartX(), leftToLeftNext.getEndY()
									- leftToLeftNext.getStartY());
							double rightToRightNextLength = Math.hypot(rightToRightNext.getEndX()
									- rightToRightNext.getStartX(), rightToRightNext.getEndY()
									- rightToRightNext.getStartY());
							double leftNextToMidLength = Math.hypot(leftNextToLeftMid.getEndX()
									- leftNextToLeftMid.getStartX(), leftNextToLeftMid.getEndY()
									- leftNextToLeftMid.getStartY());
							double rightNextToMidLength = Math
									.hypot(rightNextToRightMid.getEndX()
											- rightNextToRightMid.getStartX(),
											rightNextToRightMid.getEndY()
													- rightNextToRightMid.getStartY());
							if (leftToLeftNextLength < leftNextToMidLength
									|| rightToRightNextLength < rightNextToMidLength) {
								mmap.put(left.getID(), RoadMark.Entrance);
								entranceSet.add(left.getID());

								mmap.put(right.getID(), RoadMark.Entrance);
								entranceSet.add(right.getID());
							}

						}
					}
				}
			}
		}
	}

	private Road getOnlyNeighborRoad(Area from, EntityID ignore) {
		Road ret = null;
		for (EntityID id : from.getNeighbours()) {
			if (id.equals(ignore)) {
				continue;
			}
			StandardEntity entity = world.getEntity(id);
			if (entity instanceof Road) {
				if (ret == null) {
					ret = (Road) entity;
				} else {
					// not only one
					return null;
				}
			}
		}
		return ret;
	}

	private Road getOnlyNeighborRoadWithoutEntrance(Area from, EntityID ignore) {
		Road ret = null;
		for (EntityID id : from.getNeighbours()) {
			if (id.equals(ignore)) {
				continue;
			}
			if (entranceSet.contains(id)) {
				continue;
			}
			StandardEntity entity = world.getEntity(id);
			if (entity instanceof Road) {
				if (ret == null) {
					ret = (Road) entity;
				} else {
					// not only one
					return null;
				}
			}
		}
		return ret;
	}

	private void makeGraph() {
	}

	// private void entranceMark() {
	// for (Road road : comp.getRoadSet()) {
	// int b = 0;
	// int r = 0;
	// Area left = null, right = null;
	// for (EntityID nei : road.getNeighbours()) {
	// StandardEntity entity = world.getEntity(nei);
	// if (entity instanceof Building) {
	// b++;
	// }
	// if (entity instanceof Road) {
	// if (r == 0)
	// left = (Area) entity;
	// else
	// right = (Area) entity;
	// r++;
	// }
	// }
	// // b >= 1
	// if (b >= 1 && r == 1) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// }
	// if (b >= 1 && r == 2) {
	// if (left.getEdgeTo(right.getID()) != null &&
	// right.getEdgeTo(left.getID()) != null) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// } else {
	// // Case 1
	// Set<EntityID> leftNei = new HashSet<EntityID>(left.getNeighbours());
	// Set<EntityID> rightNei = new HashSet<EntityID>(right.getNeighbours());
	// Set<EntityID> common = new HashSet<EntityID>();
	// for (EntityID id : rightNei) {
	// if (leftNei.contains(id)) {
	// common.add(id);
	// }
	// }
	// if (common.size() == 2 && common.contains(road.getID())) {
	// common.remove(road.getID());
	// StandardEntity entity = world.getEntity(common.iterator().next());
	// if (entity instanceof Road) {
	// Road center = (Road) entity;
	// double d1 = GeoUtils.manhattanDistance(left, center) +
	// GeoUtils.manhattanDistance(center, right);
	// double d2 = GeoUtils.manhattanDistance(left, road) +
	// GeoUtils.manhattanDistance(road, right);
	// if (d1 <= d2) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// } else {
	// // for left
	// boolean flag = false;
	// for (EntityID nei : left.getNeighbours()) {
	// if (world.getEntity(nei) instanceof Building) {
	// flag = true;
	// break;
	// }
	// }
	// if (flag) {
	// d1 = GeoUtils.manhattanDistance(right, (Road) entity);
	// d2 = GeoUtils.manhattanDistance(right, road) +
	// GeoUtils.manhattanDistance(road, left) +
	// GeoUtils.manhattanDistance(left, (Road) entity);
	// if (d1 < d2) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// }
	//
	// } else {
	// // for right
	// for (EntityID nei : right.getNeighbours()) {
	// if (world.getEntity(nei) instanceof Building) {
	// flag = true;
	// break;
	// }
	// }
	//
	// if (flag) {
	// d1 = GeoUtils.manhattanDistance(left, (Road) entity);
	// d2 = GeoUtils.manhattanDistance(left, road) +
	// GeoUtils.manhattanDistance(road, right) +
	// GeoUtils.manhattanDistance(right, (Road) entity);
	// if (d1 < d2) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// }
	// }
	// }
	// }
	// }
	// }
	// }
	// }
	// }
	// }
	//
	// private void entranceMark2() {
	// for (Road road : comp.getRoadSet()) {
	// int b = 0;
	// int nr = 0;
	// Area left = null, right = null;
	// for (EntityID nei : road.getNeighbours()) {
	// StandardEntity entity = world.getEntity(nei);
	// if (entity instanceof Building) {
	// b++;
	// }
	// if (entity instanceof Road) {
	// if (!entranceSet.contains(entity.getID())) {
	// if (nr == 0)
	// left = (Area) entity;
	// else
	// right = (Area) entity;
	// nr++;
	// }
	// }
	// }
	// // b >= 1
	// if (b >= 1 && nr == 1) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// } else if (b >= 1 && nr == 2) {
	// if (left.getEdgeTo(right.getID()) != null &&
	// right.getEdgeTo(left.getID()) != null) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	// } else {
	// // boolean pass = false;
	// // for (EntityID nei : left.getNeighbours()) {
	// // Area tmp = (Area) world.getEntity(nei);
	// // if (tmp instanceof Building) {
	// // pass = true;
	// // break;
	// // }
	// // }
	// // if (pass) {
	// for (EntityID nei : left.getNeighbours()) {
	// if (!nei.equals(road.getID())) {
	// if (right.getEdgeTo(nei) != null) {
	// mmap.put(road.getID(), RoadMark.Entrance);
	// entranceSet.add(road.getID());
	//
	// }
	// }
	// }
	// // }
	// }
	// }
	//
	// }
	// }
	//
	// private void entranceMark3() {
	// for (Road road : comp.getRoadSet()) {
	// if (!entranceSet.contains(road.getID())) {
	// int b = 0;
	// int r = 0;
	// int nr = 0;
	// Area left = null, right = null;
	// for (Edge edge : road.getEdges()) {
	// if (edge.isPassable()) {
	// double len = GeoUtils.dist(edge.getStartX(), edge.getStartY(),
	// edge.getEndX(), edge.getEndY());
	//
	// EntityID nei = edge.getNeighbour();
	// StandardEntity entity = world.getEntity(nei);
	// if (entity instanceof Building) {
	// b++;
	// }
	// if (entity instanceof Road) {
	// if (len > AGENT_RADIUS * IGNORE_SCALE) {
	// if (r == 0)
	// left = (Area) entity;
	// else
	// right = (Area) entity;
	// r++;
	// } else {
	// nr++;
	// }
	// }
	//
	// }
	// }
	// // b >= 1
	// if (b >= 1 && r == 1) {
	// // mmap.put(road.getID(), RoadMark.Entrance);
	// // entranceSet.add(road.getID());
	// mmap.put(road.getID(), RoadMark.Cross);
	// crossSet.add(road.getID());
	// }
	// if (b >= 1 && r == 2) {
	// if (left.getEdgeTo(right.getID()) != null &&
	// right.getEdgeTo(left.getID()) != null) {
	// // mmap.put(road.getID(), RoadMark.Entrance);
	// // entranceSet.add(road.getID());
	// mmap.put(road.getID(), RoadMark.Cross);
	// crossSet.add(road.getID());
	// }
	// }
	// if (b >= 1 && r == 0 && nr >= 1) {
	// mmap.put(road.getID(), RoadMark.Cross);
	// crossSet.add(road.getID());
	// }
	// }
	// }
	// }
	//
	private void avenueMark() {
		for (Road road : comp.getRoadSet()) {
			if (!entranceSet.contains(road.getID())) {
				int b = 0;
				int nr = 0;
				Area left = null, right = null;
				for (EntityID nei : new HashSet<EntityID>(road.getNeighbours())) {
					StandardEntity entity = world.getEntity(nei);
					if (entity instanceof Building) {
						b++;
					}
					if (entity instanceof Road) {
						if (!entranceSet.contains(entity.getID())) {
							if (nr == 0) {
								left = (Area) entity;
							} else {
								right = (Area) entity;
							}
							nr++;
						}
					}
				}

				if (nr <= 2) {
					if (nr < 2) {
						mmap.put(road.getID(), RoadMark.Avenue);
						avenueSet.add(road.getID());
					} else {
						if (left.getEdgeTo(right.getID()) == null
								&& right.getEdgeTo(left.getID()) == null) {
							mmap.put(road.getID(), RoadMark.Avenue);
							avenueSet.add(road.getID());
						}
					}
				}
			}
		}

	}

	//
	private void crossMark() {
		for (Road road : comp.getRoadSet()) {
			if (!entranceSet.contains(road.getID()) && !avenueSet.contains(road.getID())) {
				mmap.put(road.getID(), RoadMark.Cross);
				crossSet.add(road.getID());
			}
		}
		// for (Road road : comp.getRoadSet()) {
		// if (avenueSet.contains(road.getID())) {
		// boolean isCross = true;
		// for (EntityID nei : road.getNeighbours()) {
		// if (!crossSet.contains(nei)) {
		// isCross = false;
		// break;
		// }
		// }
		// if (isCross) {
		// mmap.put(road.getID(), RoadMark.Cross);
		// avenueSet.remove(road.getID());
		// crossSet.add(road.getID());
		// }
		// }
		// }
	}

	//
	// private void crossMark2() {
	// for (Road road : comp.getRoadSet()) {
	// if (avenueSet.contains(road.getID())) {
	// int t = 0;
	// for (EntityID nei : road.getNeighbours()) {
	// StandardEntity entity = world.getEntity(nei);
	// if (entity instanceof Road) {
	// if (!entranceSet.contains(entity.getID())) {
	// t++;
	// }
	// }
	// }
	// if (t <= 1) {
	// EntityID id = road.getID();
	// avenueSet.remove(id);
	// crossSet.add(id);
	// mmap.put(id, RoadMark.Cross);
	// }
	// }
	// }
	// }

	@Override
	public void update() {
	}

	public RoadMark getMark(EntityID id) {
		return mmap.get(id);
	}

	@Override
	public String getComponentName() {
		return JigsawMarkerComponent.class.getName();
	}
}
