package ant.map.jigsaw;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Road;
import rescuecore2.worldmodel.EntityID;
import ant.utils.IPositioning;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 */
public abstract class JigsawObject implements IPositioning {
	private static final Log log = LogFactory.getLog(JigsawObject.class);

	private int x;
	private int y;
	private Area center;
	private int jigsawID;
	private Set<JigsawAttach> attaches;
	private Set<Area> rawAreas;
	protected Set<EntityID> rawAreaIDs;
	private int roadCount = 0;
	private int jigsawSize = 0;
	
	// TODO 判定Raw是否为空
	public JigsawObject(int jigsawID, Set<Area> raw) {
		if (raw.size() == 0)
			throw new IllegalArgumentException();
		
		this.jigsawID = jigsawID;
		this.attaches = new HashSet<JigsawAttach>();
		this.rawAreas = new HashSet<Area>();
		this.rawAreaIDs = new HashSet<EntityID>();
		this.roadCount = 0;
		
		for (Area a : raw) {
			rawAreas.add(a);
			rawAreaIDs.add(a.getID());
			jigsawSize++;
			
			if (a instanceof Road) {
				roadCount++;
			}
		}
		
		calcXY(raw);
		calcCenter(raw);
	}

	private void calcXY(Set<Area> raw) {
		long sumx = 0;
		long sumy = 0;
		
		for (Area a : raw) {
			sumx += a.getX();
			sumy += a.getY();
		}
		
		x = (int) (sumx / raw.size());
		y = (int) (sumy / raw.size());
	}

	private void calcCenter(Set<Area> raw) {
		double dist = Double.MAX_VALUE;
		Area pick = null;
		for (Area a : raw) {
			double temp = GeoUtils.manhattanDistance(a, x, y);
			if (temp < dist) {
				dist = temp;
				pick = a;
			}
		}
		center = pick;
	}

	@Override
	public final int getX() {
		return x;
	}

	@Override
	public final int getY() {
		return y;
	}

	public final int getJigsawID() {
		return jigsawID;
	}

	public final Set<JigsawAttach> getAttaches() {
		return attaches;
	}

	protected void addAttach(JigsawAttach attach) {
		attaches.add(attach);
	}

	public final Area getCenter() {
		return this.center;
	}
	
	public final int getRoadCount() {
		return this.roadCount;
	}
	
	public final int getJigsawSize() {
		return jigsawSize;
	}
	
	public abstract JigsawType getType();

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + jigsawID;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JigsawObject other = (JigsawObject) obj;
		if (jigsawID != other.jigsawID)
			return false;
		return true;
	}
	
	public final boolean contains(Area a) {
		return this.rawAreaIDs.contains(a.getID());
	}
	
	public final boolean contains(EntityID id) {
		return this.rawAreaIDs.contains(id);
	}
	
	public final Set<Area> getObjects() {
		return this.rawAreas;
	}
}
