package ant.map.objects;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rescuecore2.standard.entities.Area;

/**
 * 
 * @author Ivan
 * 
 */
public class AreaGraph implements IGraph<Area, StandardGraphEdge<Area>> {

	private Set<Area> vSet;
	private Map<Area, List<StandardGraphEdge<Area>>> adjacency;
	private Map<Area, Map<Area, StandardGraphEdge<Area>>> connectivity;

	// private Map<EntityID, Set<EntityID>> neighbor;

	public AreaGraph() {
		this.adjacency = new HashMap<Area, List<StandardGraphEdge<Area>>>();
		this.vSet = new HashSet<Area>();
		this.connectivity = new HashMap<Area, Map<Area, StandardGraphEdge<Area>>>();
		// this.neighbor = new HashMap<EntityID, Set<EntityID>>();
	}

	@Override
	public void addVertex(Area v) {
		if (!vSet.contains(v)) {
			vSet.add(v);
			adjacency.put(v, new ArrayList<StandardGraphEdge<Area>>());
			connectivity.put(v, new HashMap<Area, StandardGraphEdge<Area>>());
			// neighbor.put(v, new HashSet<EntityID>());
		}
	}

	@Override
	public boolean containsEdge(Area source, Area target) {
		if (!vSet.contains(source) || !vSet.contains(target)) {
			throw new IllegalArgumentException("Vertex not found");
		}
		return connectivity.get(source).containsKey(target);
	}

	@Override
	public boolean containsVertex(Area v) {
		return vSet.contains(v);
	}

	@Override
	public Collection<StandardGraphEdge<Area>> edgesOf(Area v) {
		if (!vSet.contains(v)) {
			throw new IllegalArgumentException("Vertex not found");
		}
		return adjacency.get(v);
	}

	@Override
	public StandardGraphEdge<Area> getEdge(Area source, Area target) {
		if (!vSet.contains(source) || !vSet.contains(target)) {
			throw new IllegalArgumentException("Vertex not found");
		}
		return connectivity.get(source).get(target);
	}

	@Override
	public Set<Area> getNeighbours(Area v) {
		if (!vSet.contains(v)) {
			throw new IllegalArgumentException("Vertex not found");
		}
		// TODO keyset or new field?
		return this.connectivity.get(v).keySet();
	}

	@Override
	public Set<Area> getVertexSet() {
		return vSet;
	}

	@Override
	public void removeEdge(Area source, Area target) {
		if (!vSet.contains(source) || !vSet.contains(target)) {
			throw new IllegalArgumentException("Vertex not found");
		}

		connectivity.get(source).remove(target);
		Iterator<StandardGraphEdge<Area>> iter = adjacency.get(source).iterator();
		while (iter.hasNext()) {
			if (iter.next().getTarget().equals(target)) {
				iter.remove();
				break;
			}
		}
	}

	@Override
	public void setEdge(Area source, Area target, StandardGraphEdge<Area> edge) {
		if (!vSet.contains(source) || !vSet.contains(target)) {
			throw new IllegalArgumentException("Vertex not found");
		}

		if (edge == null) {
			throw new IllegalArgumentException("Null edge");
		}

		if (!edge.getSource().equals(source) || !edge.getTarget().equals(target)) {
			throw new IllegalArgumentException("Source/Target not corresponded");
		}
		
		adjacency.get(source).add(edge);
		connectivity.get(source).put(target, edge);
	}
	
}
