package ant.map.objects;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.utils.IPositioning;

/**
 * 
 * @author Ivan
 * 
 */
public class RoadSide implements Comparable<RoadSide>, IPositioning {

	private Road road;
	private int index;
	private Edge edge;
	private int x, y;
	private EntityID neighbourId;
	private boolean isRoadNeighbour;
	private boolean isBuildingNeighbour;
	private RoadSidePair pair;
	
	public RoadSide(StandardWorldModel world, Road road, Edge edge) {
		if (!edge.isPassable())
			throw new IllegalArgumentException("Non-passable edge! road:" + road + " edge:" + edge);

		this.x = (edge.getStartX() + edge.getEndX()) / 2;
		this.y = (edge.getStartY() + edge.getEndY()) / 2;

		boolean find = false;
		for (int i = 0; i < road.getEdges().size(); i++) {
			if (road.getEdges().get(i).equals(edge)) {
				this.road = road;
				this.index = i;
				this.edge = edge;
				find = true;
				break;
			}
		}
		if (!find) {
			throw new IllegalArgumentException("Bad edge! road:" + road + " edge:" + edge);
		}
		this.neighbourId = edge.getNeighbour();
		StandardEntity entity = world.getEntity(neighbourId);
		if (entity instanceof Road) {
			this.isRoadNeighbour = true;
			this.isBuildingNeighbour = false;
		} else if (entity instanceof Building) {
			this.isRoadNeighbour = false;
			this.isBuildingNeighbour = true;
		} else {
			throw new IllegalStateException();
		}
	}

	public Road getRoad() {
		return road;
	}

	public Edge getEdge() {
		return edge;
	}

	public int getEdgeIndex() {
		return index;
	}

	public boolean isRoadNeighbour() {
		return isRoadNeighbour;
	}

	public boolean isBuildingNeighbour() {
		return isBuildingNeighbour;
	}
	
	public RoadSidePair getPair() {
		if (isRoadNeighbour) {
			return pair;
		} else {
			throw new IllegalArgumentException();
		}
	}

	public void setPair(RoadSidePair pair) {
		if (this.isRoadNeighbour) {
			if (pair.getLeft().equals(this) || pair.getRight().equals(this)) {
				this.pair = pair;
			} else {
				throw new IllegalArgumentException();
			}
		} else {
			throw new IllegalArgumentException();
		}
	}
	
	@Override
	public int getX() {
		return x;
	}

	@Override
	public int getY() {
		return y;
	}

	public EntityID getNeighbourID() {
		return neighbourId;
	}

	@Override
	public String toString() {
		return "Side [road=" + road.getID() + ", To=" + edge.getNeighbour() + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + index;
		result = prime * result + ((road == null) ? 0 : road.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RoadSide other = (RoadSide) obj;
		if (index != other.index)
			return false;
		if (road == null) {
			if (other.road != null)
				return false;
		} else if (!road.equals(other.road))
			return false;
		return true;
	}

	@Override
	public int compareTo(RoadSide o) {
		if (road.getID().getValue() != o.road.getID().getValue()) {
			return road.getID().getValue() - o.road.getID().getValue();
		} else {
			return index - o.index;
		}
	}

}
