package ant.map.objects;

import ant.utils.IPositioning;

/**
 * 
 * @author Ivan
 *
 */
public class RoadSidePair implements Comparable<RoadSidePair>, IPositioning {
	private final RoadSide left, right;
	private final int x, y;
	
	public RoadSidePair(RoadSide a, RoadSide b) {
		if (!check(a, b)) {
			throw new IllegalArgumentException("Non-Pair");
		}
		if (a.compareTo(b) <= 0) {
			left = a;
			right = b;
		} else {
			left = b;
			right = a;
		}
		x = left.getX();
		y = left.getY();
	}
	
	public RoadSide getLeft() {
		return left;
	}

	public RoadSide getRight() {
		return right;
	}
	
	private boolean check(RoadSide a, RoadSide b) {
		if (a.getRoad().getID().equals(b.getRoad().getID())) {
			return false;
		}
		if (!a.getEdge().getNeighbour().equals(b.getRoad().getID())) {
			return false;
		}
		if (!b.getEdge().getNeighbour().equals(a.getRoad().getID())) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(RoadSidePair o) {
		return this.left.compareTo(o.left);
	}

	@Override
	public int getX() {
		return this.x;
	}

	@Override
	public int getY() {
		return this.y;
	}

	@Override
	public String toString() {
		return "Pair [left=" + left + ", right=" + right + "]";
	}
	
}
