package ant.map.objects;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.worldmodel.EntityID;
import ant.utils.geo.Point;

/**
 * 
 * @author Ivan
 * 
 */
public class RouteResult {

	private EntityID from;
	private EntityID to;
	private double distance;
	private boolean hasAcceptableRoute;
	private List<EntityID> moveRoute;
	private List<Point> suitablePositions;

	public static class RouteResultFactory {

		private boolean acceptable;
		private EntityID to;
		private EntityID from;
		private List<Point> suitablePositions;
		private List<EntityID> moveRoute;
		private double distance = Double.POSITIVE_INFINITY;

		public RouteResultFactory(boolean acceptable) {
			this.suitablePositions = new ArrayList<Point>();
			this.moveRoute = new ArrayList<EntityID>();
			this.acceptable = acceptable;
		}
		
		public void setTo(EntityID to) {
			this.to = to;
		}

		public void setFrom(EntityID from) {
			this.from = from;
		}

		public void setDistance(double distance) {
			this.distance = distance;
		}

		public void setAcceptable(boolean acceptable) {
			this.acceptable = acceptable;
		}

		public void addSuitableDestPoint(Point p) {
			this.suitablePositions.add(p);
		}
		
		public void addSuitableDestPoint(int x, int y) {
			this.suitablePositions.add(new Point(x, y));
		}
		
		public void setRawRoute(List<EntityID> rawRoute) {
			if (rawRoute == null) {
				this.acceptable = false;
				return;
			}
			if (rawRoute.size() == 0) {
				throw new IllegalArgumentException();
			}
			if (rawRoute.size() > 1) {
				for (int i = 1; i < rawRoute.size(); i++) {
					this.moveRoute.add(rawRoute.get(i));
				}
			} else {
				for (int i = 0; i < rawRoute.size(); i++) {
					this.moveRoute.add(rawRoute.get(i));
				}
			}
		}
		
		public void setRawRouteEntityID(Route<EntityID> rawRoute) {
			if (rawRoute == null) {
				this.acceptable = false;
				return;
			}
			setRawRoute(rawRoute.getRoute());
			this.from = rawRoute.getFrom();
			this.to = rawRoute.getTo();
			this.distance = rawRoute.getDistance();
		}

		public void setRawRouteLabeledArea(Route<LabeledArea> rawRoute) {
			if (rawRoute == null) {
				this.acceptable = false;
				return;
			}
			if (rawRoute.getRoute().size() == 0) {
				throw new IllegalArgumentException();
			}
			if (rawRoute.getRoute().size() > 1) {
				for (int i = 1; i < rawRoute.getRoute().size(); i++) {
					this.moveRoute.add(rawRoute.getRoute().get(i).getID());
				}
			} else {
				for (int i = 0; i < rawRoute.getRoute().size(); i++) {
					this.moveRoute.add(rawRoute.getRoute().get(i).getID());
				}
			}
			this.from = rawRoute.getFrom().getID();
			this.to = rawRoute.getTo().getID();
			this.distance = rawRoute.getDistance();
		}

		public RouteResult newInstance() {
			RouteResult rr = new RouteResult();
			rr.setAcceptable(acceptable);
			rr.setFrom(from);
			rr.setTo(to);
			rr.setSuitablePositions(suitablePositions);
			rr.setDistance(distance);
			rr.setMoveRoute(moveRoute);
			return rr;
		}
	}

	private RouteResult() {
	}

	public boolean hasAcceptableRoute() {
		return this.hasAcceptableRoute;
	}

	public double getDistance() {
		return this.distance;
	}

	public List<EntityID> getMoveRoute() {
		return this.moveRoute;
	}

	public EntityID getFrom() {
		return this.from;
	}

	public EntityID getTo() {
		return this.to;
	}

	private void setFrom(EntityID from) {
		this.from = from;
	}

	private void setTo(EntityID to) {
		this.to = to;
	}

	private void setAcceptable(boolean b) {
		this.hasAcceptableRoute = b;
	}

	public List<Point> getSuitablePositions() {
		return suitablePositions;
	}

	private void setSuitablePositions(List<Point> suitablePositions) {
		this.suitablePositions = suitablePositions;
	}

	private void setDistance(double distance) {
		this.distance = distance;
	}

	private void setMoveRoute(List<EntityID> moveRoute) {
		this.moveRoute = moveRoute;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((from == null) ? 0 : from.hashCode());
		result = prime * result + (hasAcceptableRoute ? 1231 : 1237);
		result = prime * result + ((moveRoute == null) ? 0 : moveRoute.hashCode());
		result = prime * result + ((to == null) ? 0 : to.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RouteResult other = (RouteResult) obj;
		if (from == null) {
			if (other.from != null)
				return false;
		} else if (!from.equals(other.from))
			return false;
		if (hasAcceptableRoute != other.hasAcceptableRoute)
			return false;
		if (moveRoute == null) {
			if (other.moveRoute != null)
				return false;
		} else if (!moveRoute.equals(other.moveRoute))
			return false;
		if (to == null) {
			if (other.to != null)
				return false;
		} else if (!to.equals(other.to))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "RouteResult [from=" + from + ", to=" + to + ", hasAcceptableRoute=" + hasAcceptableRoute + ", moveRoute=" + moveRoute + "]";
	}
	
	
}
