package ant.map.partition;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import ant.AgentContext;
import ant.map.component.AntMapBasisComponent;
import ant.utils.RandomSelector;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link AntMapBasisComponent}
 */
public class BasicBuildingPartitionAlgorithm extends AbstractPartitionAlgorithm {
	private static final Log log = LogFactory.getLog(BasicBuildingPartitionAlgorithm.class);

	private static final double CONV_DISTANCE = 0;
	
	private int partCount;
	private AntMapBasisComponent comp;
	private Set<Building> buildings;
	private Set<PartitionCluster> pcs;
	private Map<Integer, Partition> partmap;
	private ArrayList<Partition> parts;
	private int actpart;
	
	public BasicBuildingPartitionAlgorithm(AgentContext context, int partCount) {
		super(context);
		this.partCount = partCount;
		this.comp = context.getAntComponentKit().getAntMapBasisComponent();
		this.partmap = new HashMap<Integer, Partition>();
		this.parts = new ArrayList<Partition>();
	}

	@Override
	public void partition() {
		this.pcs = new HashSet<PartitionCluster>();
		this.buildings = comp.getBuildingSet();
		Set<Building> inits = RandomSelector.randomSelect(buildings, partCount, context.getUniformRandom());
		int id = 0;
		for (Building b : inits) {
			PartitionCluster pc = new PartitionCluster(id);
			pc.addArea(b);
			pc.convCalc();
			pcs.add(pc);
			id++;
			// System.out.println(pc.getX() + "," + pc.getY() + "," + pc.getID());
		}
		
		int repeat = 0;
		while (true) {
			boolean conv = true;
			repeat++;
			for (Building b : buildings) {
				double min = Double.MAX_VALUE;
				PartitionCluster pick = null;
				for (PartitionCluster pc : pcs) {
					double temp = GeoUtils.manhattanDistance(pc, b);
					if (temp < min) {
						pick = pc;
						min = temp;
					}
				}
				pick.addArea(b);
			}
			
			for (PartitionCluster pc : pcs) {
				double c = pc.convCalc();
				conv = conv && (c <= CONV_DISTANCE);
				// log.info(pc.getID() + "," + c);
			}
			
			if (repeat != 1 && conv) break;
		}
		
		log.info("Conv.Repeat: " + repeat);

		for (Building b : buildings) {
			double min = Double.MAX_VALUE;
			PartitionCluster pick = null;
			for (PartitionCluster pc : pcs) {
				double temp = GeoUtils.manhattanDistance(pc, b);
				if (temp < min) {
					pick = pc;
					min = temp;
				}
			}
			if (!this.partmap.containsKey(pick.getID()))
				this.partmap.put(pick.getID(), new Partition(pick.getID()));
			this.partmap.get(pick.getID()).addArea(b);
		}
		
		this.actpart = 0;
		
		for (int i = 0; i < partCount; i++) {
			Partition p = this.partmap.get(i);
			if (p != null) {
				p.convCalc();
				p.setID(actpart);
				this.parts.add(p);
				actpart++;
			}
		}
		
		log.info("actual part count:" + actpart);
	}

	@Override
	public int getPartitionCount() {
		return actpart;
	}

	@Override
	public ArrayList<Partition> getPartitions() {
		return this.parts;
	}

}
