package ant.map.partition;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.worldmodel.EntityID;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Ivan
 *
 */
public class Partition extends PartitionCluster {

	private Set<EntityID> idset;
	private Set<Area> areaset;
	private Area center = null;
	
	public Partition(int id) {
		super(id);
		this.idset = new HashSet<EntityID>();
		this.areaset = new HashSet<Area>();
	}
	
	public Area getCenter() {
		if (center == null) {
			calcCenter();
		}
		
		return center;
	}
	private void calcCenter() {
		double x = 0, y = 0;
		for (Area a : areaset) {
			x += a.getX();
			y += a.getY();
		}
		if (this.areaset.size() != 0) {
			x = x / areaset.size();
			y = y / areaset.size();
		}
		
		int ix = (int) x;
		int iy = (int) y;
		double mindist = Double.MAX_VALUE;
		for (Area a : areaset) {
			double temp = GeoUtils.manhattanDistance(a, ix, iy);
			if (temp < mindist) {
				mindist = temp;
				center = a;
			}
		}
	}

	@Override
	public void addArea(Area a) {
		super.addArea(a);
		this.areaset.add(a);
		this.idset.add(a.getID());
	}
	
	public void addAll(Collection<Area> input) {
		for (Area a : input) {
			super.addArea(a);
			this.areaset.add(a);
			this.idset.add(a.getID());
		}
	}
	
	@Override
	public int count() {
		return idset.size();
	}
	
	public boolean isLocated(Area a) {
		return idset.contains(a.getID());
	}
	
	public boolean isLocated(EntityID a) {
		return this.idset.contains(a);
	}
	
	public Set<Area> getAreaSet() {
		return areaset;
	}
	
	public Set<EntityID> getAreaIDSet() {
		return idset;
	}
	
	public int size() {
		return this.idset.size();
	}
	
	@Override
	public double convCalc() {
		if (this.idset.isEmpty()) {
			this.x = 0;
			this.y = 0;
			this.count = 0;
		} else {
			this.count = this.idset.size();
			int tx = 0, ty = 0;
			for (Area a : areaset) {
				tx += a.getX();
				ty += a.getY();
			}
			this.x = tx / this.count;
			this.y = ty / this.count;
		}
		return 0;
	}
	
	
}
