package ant.map.partition;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.AntComponent;
import ant.ConfigKey;
import ant.map.component.AntMapBasisComponent;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link AntMapBasisComponent}
 */
public class PartitionAreaComponent implements AntComponent {
	private static final Log log = LogFactory.getLog(PartitionAreaComponent.class);

	private FastCellPartitionAlgorithm alg;
	private int expectedPartitionCount;
	private int actualPartitionCount;
	private ArrayList<Partition> parts;

	private int pfCount;
	private int atCount;
	private int fbCount;

	@Override
	public String getComponentName() {
		return PartitionAreaComponent.class.getName();
	}

	@Override
	public void initial(AgentContext context, StandardWorldModel world) {
		// TODO
		// for (String key : context.getConfig().getAllKeys()) {
		// log.info(key);
		// }

		try {
			int pfCount2 = world.getEntitiesOfType(StandardEntityURN.POLICE_FORCE).size();
			int atCount2 = world.getEntitiesOfType(StandardEntityURN.AMBULANCE_TEAM).size();
			int fbCount2 = world.getEntitiesOfType(StandardEntityURN.FIRE_BRIGADE).size();
			this.pfCount = ConfigKey.getTotalPFs(context.getConfig());
			this.atCount = ConfigKey.getTotalATs(context.getConfig());
			this.fbCount = ConfigKey.getTotalFBs(context.getConfig());
			// log.debug("PF:" + pfCount + " AT:" + atCount + " FB:" + fbCount);
			// log.debug("PF:" + pfCount2 + " AT:" + atCount2 + " FB:" + fbCount2);

		} catch (Exception e) {
			this.pfCount = this.atCount = this.fbCount = 30;
			log.error(e);
		}

		if (context.getAgentType() == StandardEntityURN.FIRE_BRIGADE) {
			if (fbCount <= 1)
				expectedPartitionCount = 1;
			else
				expectedPartitionCount = fbCount / 2;
		} else if (context.getAgentType() == StandardEntityURN.POLICE_FORCE) {
			if (pfCount <= 1)
				expectedPartitionCount = 1;
			else {
				if (pfCount <= 9)
					expectedPartitionCount = (int) Math.sqrt(pfCount);
				else
					expectedPartitionCount = (int) (pfCount / 2.5);
			}

		} else if (context.getAgentType() == StandardEntityURN.AMBULANCE_TEAM) {
			if (atCount <= 1)
				expectedPartitionCount = 1;
			else
				expectedPartitionCount = atCount / 2;

		} else {
			expectedPartitionCount = 12;
		}

		// this.alg = new SimplePartitionAlg(context, expectedPartitionCount);
		// this.alg = new BasicBuildingPartitionAlgorithm(context,
		// expectedPartitionCount);
		// this.alg = new BasicAreaPartitionAlgorithm(context,
		// expectedPartitionCount);
		// this.alg = new RefPartitionAlgorithm(context);
		log.debug("partition count: " + expectedPartitionCount);
		this.alg = new FastCellPartitionAlgorithm(context, expectedPartitionCount);

		this.alg.partition();
		this.parts = this.alg.getPartitions();
		this.actualPartitionCount = this.parts.size();
		
		// this.index = new HashMap<EntityID, Integer>();
		// for (int i = 0; i < this.actualPartitionCount; i++) {
		// this.parts.get(i).setID(i);
		// Set<Area> areaSet = this.parts.get(i).getAreaSet();
		// for (Area a : areaSet) {
		// index.put(a.getID(), i);
		// }
		// }
	}

	public int getPartitionCount() {
		return this.actualPartitionCount;
	}

	public int findPartitionIndex(EntityID areaId) {
		return this.alg.findPartitionIndex(areaId);
	}

	public ArrayList<Partition> getPartitions() {
		return this.parts;
	}

	@Override
	public void update() {
	}

}
