package ant.map.search;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.map.jigsaw.JigsawMarkerComponent;
import ant.map.jigsaw.JigsawMarkerComponent.RoadMark;
import ant.map.objects.AreaGraph;
import ant.map.objects.Route;
import ant.map.objects.StandardGraphEdge;

/**
 * TODO test, and deploy in trails
 * 
 * @author Ivan
 * 
 */
public class RangeStepBFSSearch extends AbstractShortestPathAlgorithm<Area, StandardGraphEdge<Area>> {

	private int rangestep;
	private StandardWorldModel world;
	private LinkedList<EntityID> open;
	private HashMap<EntityID, VertexInfo> ancestors;
	private JigsawMarkerComponent jmc;

	public RangeStepBFSSearch(JigsawMarkerComponent jmc, AreaGraph graph, StandardWorldModel world, int rangestep) {
		super(graph);
		this.world = world;
		this.rangestep = rangestep;
		this.open = new LinkedList<EntityID>();
		this.ancestors = new HashMap<EntityID, VertexInfo>();
		this.jmc = jmc;
	}

	@Override
	public Route<Area> spSearch(Area source, Area target) {
		open.clear();
		ancestors.clear();
		// System.out.println("IN");
		List<Area> tmp = breadthFirstSearch(source.getID(), target.getID());
		// System.out.println("OUT");
		if (tmp == null) return null;
		Route<Area> sp = new Route<Area>();
		sp.setFrom(source);
		sp.setTo(target);
		sp.setRoute(tmp);
		return sp;
	}

	@Override
	public AbstractSSSPTree<Area> ssspSearch(Area source) {
		open.clear();
		ancestors.clear();
		sssp(source.getID());
		return new RangeStepBFSSSSPTree(source, ancestors, world);
	}

	private List<Area> breadthFirstSearch(EntityID start, EntityID goal) {
		// System.out.println(start + "," + goal);
		open.add(start);
		EntityID next = null;
		boolean found = false;
		ancestors.put(start, new VertexInfo(start));
		do {
			next = open.remove(0);
			Area nextArea = (Area) world.getEntity(next);
			VertexInfo item = ancestors.get(next);
			if (goal.equals(next)) {
				found = true;
				break;
			}
			if (item.step >= rangestep)
				continue;
			
			Collection<Area> neighbours = graph.getNeighbours(nextArea);
			if (neighbours.isEmpty()) {
				continue;
			}
			if (jmc == null) {
				for (Area neighbour : neighbours) {
					if (!ancestors.containsKey(neighbour.getID())) {
						open.add(neighbour.getID());
						ancestors.put(neighbour.getID(), new VertexInfo(neighbour.getID(), next, item.step + 1));
					}
				}
			} else {
				for (Area neighbour : neighbours) {
					if (jmc.getMark(neighbour.getID()) != RoadMark.Entrance) {
						if (!ancestors.containsKey(neighbour.getID())) {
							open.add(neighbour.getID());
							ancestors.put(neighbour.getID(), new VertexInfo(neighbour.getID(), next, item.step + 1));
						}
					}
				}
				for (Area neighbour : neighbours) {
					if (jmc.getMark(neighbour.getID()) == RoadMark.Entrance) {
						if (!ancestors.containsKey(neighbour.getID())) {
							open.add(neighbour.getID());
							ancestors.put(neighbour.getID(), new VertexInfo(neighbour.getID(), next, item.step + 1));
						}
					}
				}
			}
		} while (!found && !open.isEmpty());

		if (!found) {
			return null;
		}
		// System.out.println("MID search from " + start + " to " + goal);
		EntityID current = next;
		List<Area> path = new LinkedList<Area>();
		do {
			Area tmp = (Area) world.getEntity(current);
			path.add(0, tmp);
			// System.out.print(current + ", ");
			current = ancestors.get(current).pre;
			if (current == null) {
				// System.out.println(start);
				// System.out.println(goal);
				// System.out.println(current);
				// throw new RuntimeException("Found a node with no ancestor! Something is broken.");
				break;
			}
		} while (current != start);
		return path;
	}

	private void sssp(EntityID start) {
		open.add(start);
		EntityID next = null;
		boolean found = false;
		ancestors.put(start, new VertexInfo(start));
		do {
			next = open.remove(0);
			Area nextArea = (Area) world.getEntity(next);
			VertexInfo item = ancestors.get(next);
			if (item.step >= rangestep)
				continue;
			Collection<Area> neighbours = graph.getNeighbours(nextArea);
			if (neighbours.isEmpty()) {
				continue;
			}
			for (Area neighbour : neighbours) {
				if (!ancestors.containsKey(neighbour)) {
					open.add(neighbour.getID());
					ancestors.put(neighbour.getID(), new VertexInfo(neighbour.getID(), next, item.step + 1));
				}
			}
		} while (!found && !open.isEmpty());
	}

	private class VertexInfo {
		EntityID v;
		EntityID pre;
		int step;

		public VertexInfo(EntityID v) {
			this.v = v;
			this.pre = null;
			this.step = 0;
		}

		public VertexInfo(EntityID v, EntityID pre, int step) {
			this.v = v;
			this.pre = pre;
			this.step = step;
		}
	}

	private static class RangeStepBFSSSSPTree extends AbstractSSSPTree<Area> {

		private HashMap<EntityID, ant.map.search.RangeStepBFSSearch.VertexInfo> ancestors;
		private StandardWorldModel world;

		public RangeStepBFSSSSPTree(Area source, HashMap<EntityID, VertexInfo> ancestors, StandardWorldModel world) {
			super(source);
			this.ancestors = ancestors;
			this.world = world;
		}

		@Override
		public double getDistance(Area target) {
			return 0;
		}

		@Override
		public Area getSPFather(Area target) {
			if (ancestors.containsKey(target.getID())) {
				return (Area) world.getEntity(ancestors.get(target.getID()).pre);
			} else {
				return null;
			}
		}
	}
}
