package ant.map.search;

import java.util.LinkedList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import ant.map.objects.LabeledArea;
import ant.map.objects.LabeledGraph;
import ant.map.objects.Route;
import ant.map.objects.StandardGraphEdge;

/**
 * 
 * @author Ivan
 * 
 */
public class StandardLabeledSPFASearch extends AbstractShortestPathAlgorithm<LabeledArea, StandardGraphEdge<LabeledArea>> {
	private static final double INFINITY = Double.POSITIVE_INFINITY;
	
	private static final Log log = LogFactory.getLog(StandardLabeledSPFASearch.class);

	private int size;
	private LabeledGraph<LabeledArea, StandardGraphEdge<LabeledArea>> g;
	private VertexInfo[] arr;
	private LinkedList<Integer> queue;

	public StandardLabeledSPFASearch(LabeledGraph<LabeledArea, StandardGraphEdge<LabeledArea>> graph) {
		super(graph);
		this.g = graph;
		this.size = g.getVertexSize();
		this.queue = new LinkedList<Integer>();
		this.arr = new VertexInfo[this.size];
	}

	private void initial() {
		for (int i = 0; i < this.size; i++) {
			this.arr[i] = new VertexInfo(this.g.getVertex(i));
			this.arr[i].pre = -1;
		}
		queue.clear();
	}

	// TODO optimize
	private void spSearchInternal(int source, int target) {
		arr[source].inQueue = true;
		arr[source].sp = 0;
		queue.add(source);
		VertexInfo s, t;
		while (!queue.isEmpty()) {
			s = arr[queue.pollFirst()];
			s.inQueue = false;
			for (StandardGraphEdge<LabeledArea> e : g.edgesOf(s.v)) {
				t = arr[e.getTarget().getLabel()];
				if (s.sp + e.getWeight() < t.sp) {
					t.sp = s.sp + e.getWeight();
					t.pre = s.label;

					if (!t.inQueue) {
						t.inQueue = true;

						// queue.addLast(t.label);

						// SLF optimize

						if (!queue.isEmpty()) {
							if (t.sp < arr[queue.getFirst()].sp) {
								queue.addFirst(t.label);
							} else {
								queue.addLast(t.label);
							}
						} else {
							queue.addLast(t.label);
						}

					}
				}
			}
		}
	}

	private void ssspSearchInternal(int source) {
		arr[source].inQueue = true;
		arr[source].sp = 0;
		queue.add(source);
		VertexInfo s, t;
		while (!queue.isEmpty()) {
			s = arr[queue.pollFirst()];
			s.inQueue = false;
			for (StandardGraphEdge<LabeledArea> e : g.edgesOf(s.v)) {
				t = arr[e.getTarget().getLabel()];
				if (s.sp + e.getWeight() < t.sp) {
					t.sp = s.sp + e.getWeight();
					t.pre = s.label;

					if (!t.inQueue) {
						t.inQueue = true;

						// queue.addLast(t.label);

						// SLF optimize

						if (!queue.isEmpty()) {
							if (t.sp < arr[queue.getFirst()].sp) {
								queue.addFirst(t.label);
							} else {
								queue.addLast(t.label);
							}
						} else {
							queue.addLast(t.label);
						}

					}
				}
			}
		}
	}

	@Override
	public Route<LabeledArea> spSearch(LabeledArea source, LabeledArea target) {
		initial();
		spSearchInternal(source.getLabel(), target.getLabel());
		return makePath(source, target);
	}

	@Override
	public AbstractSSSPTree<LabeledArea> ssspSearch(LabeledArea source) {
		initial();
		ssspSearchInternal(source.getLabel());
		return new SimpleSSSPTree(source, this.arr);
	}

	private Route<LabeledArea> makePath(LabeledArea source, LabeledArea target) {
		Route<LabeledArea> sp = new Route<LabeledArea>();
		sp.setFrom(source);
		sp.setTo(target);
		sp.setDistance(arr[target.getLabel()].sp);

		if (Double.isInfinite(sp.getDistance()))
			return null;

		LinkedList<LabeledArea> route = new LinkedList<LabeledArea>();
		LabeledArea current = target;
		while (true) {
			route.addFirst(current);
			if (current.equals(source) || current == null) {
				break;
			} else {
				current = arr[arr[current.getLabel()].pre].v;
			}
		}
		sp.setRoute(route);
		return sp;
	}

	private class VertexInfo {
		LabeledArea v;
		int label = -1;
		boolean inQueue = false;
		double sp = INFINITY;
		int pre;

		public VertexInfo(LabeledArea v) {
			this.v = v;
			this.label = v.getLabel();
		}
	}

	private class SimpleSSSPTree extends AbstractSSSPTree<LabeledArea> {

		private VertexInfo[] arr;

		public SimpleSSSPTree(LabeledArea source, VertexInfo[] arr) {
			super(source);
			this.arr = arr;
		}

		@Override
		public double getDistance(LabeledArea target) {
			return arr[target.getLabel()].sp;
		}

		@Override
		public LabeledArea getSPFather(LabeledArea target) {
			if (arr[target.getLabel()].pre == -1)
				return null;
			return arr[arr[target.getLabel()].pre].v;
		}

	}
}
