package ant.map.search.astar;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.worldmodel.EntityID;
import ant.map.objects.IGraph;
import ant.utils.LazyMap;

/**
 * 
 * @author Ivan
 * TODO label optimize
 */
public class AStarGraph implements IGraph<AStarGraphNode, AStarGraphEdge> {
	private static final Log log = LogFactory.getLog(AStarGraph.class);
	private Set<AStarGraphNode> vertexSet;
	private Map<AStarGraphNode, Map<AStarGraphNode, AStarGraphEdge>> mmap;
	private AbstractAStarFunction costFunction;
	private ArrayList<AStarGraphNode> nodelist; // nodelist == vertexset
	private Map<EntityID, List<AStarGraphNode>> areaNodeMap;
	
	public AStarGraph(AbstractAStarFunction costFunction) {
		this.vertexSet = new HashSet<AStarGraphNode>();
		this.mmap = new LazyMap<AStarGraphNode, Map<AStarGraphNode, AStarGraphEdge>>() {
			@Override
			public Map<AStarGraphNode, AStarGraphEdge> createValue() {
				return new HashMap<AStarGraphNode, AStarGraphEdge>();
			}
		};
		this.costFunction = costFunction;
	}

	@Override
	public void addVertex(AStarGraphNode v) {
		vertexSet.add(v);
	}

	@Override
	public boolean containsEdge(AStarGraphNode source, AStarGraphNode target) {
		return mmap.get(source).containsKey(target);
	}

	@Override
	public boolean containsVertex(AStarGraphNode v) {
		return vertexSet.contains(v);
	}

	@Override
	public Collection<AStarGraphEdge> edgesOf(AStarGraphNode v) {
		return mmap.get(v).values();
	}

	@Override
	public AStarGraphEdge getEdge(AStarGraphNode source, AStarGraphNode target) {
		return mmap.get(source).get(target);
	}

	@Override
	public Set<AStarGraphNode> getNeighbours(AStarGraphNode v) {
		return mmap.get(v).keySet();
	}

	@Override
	public Set<AStarGraphNode> getVertexSet() {
		return vertexSet;
	}

	@Override
	public void removeEdge(AStarGraphNode source, AStarGraphNode target) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void setEdge(AStarGraphNode source, AStarGraphNode target, AStarGraphEdge edge) {
		mmap.get(source).put(target, edge);
	}

	public void setEdge(AStarGraphNode source, AStarGraphNode target) {
		double visibility = this.costFunction.initialVisibility(source, target);
		double weight = this.costFunction.initialCost(source, target);
		AStarGraphEdge edge = new AStarGraphEdge(source, target, weight);
		edge.setVisibility(visibility);
		setEdge(source, target, edge);
	}
	
	public ArrayList<AStarGraphNode> getNodelist() {
		return nodelist;
	}

	public void setNodelist(ArrayList<AStarGraphNode> nodelist) {
		this.nodelist = nodelist;
	}

	public Map<EntityID, List<AStarGraphNode>> getAreaNodeMap() {
		return areaNodeMap;
	}

	public void setAreaNodeMap(Map<EntityID, List<AStarGraphNode>> areaNodeMap) {
		this.areaNodeMap = areaNodeMap;
	}
	
}
