package ant.map.search.astar;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.map.objects.Route;
import ant.map.search.astar.AStarGraphNode.InnerType;
import ant.map.search.astar.AStarGraphNode.SearchInfo;
import ant.utils.FastHeap;
import ant.utils.geo.Point;

/**
 * 
 * @author Ivan
 * 
 */
public class AStarSearch {
	private static final Log log = LogFactory.getLog(AStarSearch.class);
	private static final boolean BAN_TWIST = false;
	
	private AStarGraph graph;
	private AbstractAStarFunction function;

	private Area originArea;
	private Point originPoint;
	private Area destArea;
	private Point destPoint;
	private AgentContext context;
	private FastHeap heap;

	private String debugMsg = null;

	public AStarSearch(AgentContext context, AStarGraph graph, AbstractAStarFunction function) {
		this.context = context;
		this.graph = graph;
		this.function = function;
		this.heap = new FastHeap(graph.getVertexSet().size() + 1);
	}

	/*
	 * public void setOrigin(AStarGraphNode origin) { }
	 * 
	 * public void setDestination(AStarGraphNode destination) { }
	 */

	public void setOrigin(Area originArea, Point originPoint) {
		this.originArea = originArea;
		this.originPoint = originPoint;
	}

	public void setDestination(Area destArea, Point destPoint) {
		this.destArea = destArea;
		this.destPoint = destPoint;
	}

	public Route<AStarGraphNode> search() {
		initialSearch();
		AStarGraphNode x = searchInternal();
		
		if (x == null)
			debugMsg = "NULL";
		else
			debugMsg = x.toString() + " " + x.searchInfo.toString();

		if (x != null) {
			Route<AStarGraphNode> route = new Route<AStarGraphNode>();
			route.setDistance(x.searchInfo.g + function.costEnd(x, destArea, destPoint));
			ArrayList<AStarGraphNode> list = new ArrayList<AStarGraphNode>();
			while (true) {
				list.add(x);
				if (x.searchInfo.pre != null) {
					x = x.searchInfo.pre;
				} else {
					break;
				}
			}
			List<AStarGraphNode> ret = new ArrayList<AStarGraphNode>();
			for (int i = list.size() - 1; i >= 0; i--) {
				ret.add(list.get(i));
			}
			route.setFrom(ret.get(0));
			route.setTo(ret.get(list.size() - 1));
			route.setRoute(ret);
			return route;
		} else {
			return null;
		}
	}

	private AStarGraphNode searchInternal() {
		while (!heap.isEmpty()) {
			AStarGraphNode x = ((SearchInfo) heap.pop()).node;
			x.searchInfo.inOpen = false;
			x.searchInfo.inClose = true;

			if (function.finish(x, destArea, destPoint)) {
				return x;
			}

			for (AStarGraphEdge edge : graph.edgesOf(x)) {
				AStarGraphNode y = edge.getTarget();
				
				if (BAN_TWIST) {
					if (!(x.getType() == InnerType.Building || y.getType() == InnerType.Building)) {
						if (x.searchInfo.pre == null) {
							// First Node
							EntityID sr = AbstractAStarRouteComponent.getSameRoad(x, y);
							if (sr != null) {
								if (originArea.getID().equals(sr)) {
									// GIVEUP
									continue;
								}
							}
						} else {
							AStarGraphNode z = x.searchInfo.pre;
							if (z.getType() != InnerType.Building) {
								// z - x - y
								EntityID a = AbstractAStarRouteComponent.getSameRoad(z, x);
								EntityID b = AbstractAStarRouteComponent.getSameRoad(x, y);
								if (a != null && b != null) {
									if (a.equals(b)) {
										// GIVEUP
										continue;
									}
								}
							}
						}
					}
				}
				
				double ng = x.searchInfo.g + edge.getWeight();
				
				// log.info("(" + x.searchInfo.g + ">" + ng + ")" + x + " -->" + y);

				if (!Double.isInfinite(ng)) {
					SearchInfo yinfo = y.searchInfo;
					if ((!yinfo.inClose) && (!yinfo.inOpen)) {
						yinfo.g = ng;
						yinfo.inOpen = true;
						yinfo.pre = x;
						heap.add(yinfo);
					} else if (yinfo.inOpen) {
						if (yinfo.g > ng) {
							yinfo.g = ng;
							yinfo.pre = x;
							heap.decadj(yinfo);
						}
					} else if (yinfo.inClose) {
						if (yinfo.g > ng) {
							yinfo.inClose = false;
							yinfo.inOpen = true;
							yinfo.g = ng;
							yinfo.pre = x;
							heap.add(yinfo);
						}
					}
				}
			} // for
				// log.info("HeapSize:" + heap.size());
		}

		return null;
	}

	private void initialSearch() {
		// Process Queue
		this.heap.clear();
		// Process Origin & Dest
		Map<AStarGraphNode, Double> origins = new HashMap<AStarGraphNode, Double>();

		for (AStarGraphNode originNode : graph.getAreaNodeMap().get(originArea.getID())) {
			// if (originNode.getType() == InnerType.Building) {
			// origins.put(originNode, 0.0);
			//
			// } else if (originNode.getType() == InnerType.SidePair) {
			// if (originNode.getSidePair().getLeft().getRoad().getID().equals(this.originArea.getID())) {
			// double cost = this.function.costStart(originPoint, originNode.getSidePair().getLeft());
			// origins.put(originNode, cost);
			// } else if (originNode.getSidePair().getRight().getRoad().getID().equals(this.originArea.getID())) {
			// double cost = this.function.costStart(originPoint, originNode.getSidePair().getRight());
			// origins.put(originNode, cost);
			// } else {
			// throw new IllegalStateException();
			// }
			//
			// } else if (originNode.getType() == InnerType.Side) {
			// if (originNode.getSingleSide().getRoad().getID().equals(this.originArea.getID())) {
			// double cost = this.function.costStart(originPoint, originNode.getSingleSide());
			// origins.put(originNode, cost);
			// } else {
			// throw new IllegalStateException();
			// }
			// }
			double c = function.costStart(originNode, originArea, originPoint);
			if (!Double.isInfinite(c))
				origins.put(originNode, c);
		}

		// for (Entry<AStarGraphNode, Double> entry : origins.entrySet()) {
		// log.debug("Origins:" + entry.getKey() + ", cost:" + entry.getValue());
		// }

		// Process graph
		for (AStarGraphNode vertex : this.graph.getVertexSet()) {
			vertex.searchInfo.initial();
			vertex.searchInfo.h = this.function.heristric(vertex, destArea, destPoint);
			if (origins.containsKey(vertex)) {
				vertex.searchInfo.g = origins.get(vertex);
				vertex.searchInfo.inOpen = true;
				heap.add(vertex.searchInfo);
			} else {
				vertex.searchInfo.g = Double.POSITIVE_INFINITY;
			}
		}
	}
	
	public void printDebug() {
		System.out.println("Standard:" + debugMsg);
	}
}
