package ant.map.search.astar;

import java.util.ArrayList;
import java.util.LinkedList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Road;
import rescuecore2.worldmodel.EntityID;
import ant.map.component.AbstractRouteComponent;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.IGraphComponent;
import ant.map.component.RoadPassObjectsComponent;
import ant.map.objects.IGraph;
import ant.map.objects.RoadSide;
import ant.map.objects.Route;
import ant.map.objects.RouteResult;
import ant.map.objects.RouteResult.RouteResultFactory;
import ant.map.search.astar.AStarGraphNode.InnerType;
import ant.utils.geo.Point;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link RoadPassObjectsComponent} {@link AntMapBasisComponent}
 */
public abstract class AbstractAStarRouteComponent extends AbstractRouteComponent implements
		IGraphComponent<AStarGraphNode, AStarGraphEdge> {
	private static final Log log = LogFactory.getLog(AbstractAStarRouteComponent.class);
	private static final boolean SHOW_ASTAR_TIME = false;

	protected AStarGraph graph;
	// protected AStarSearch algorithm;
	private AStarAdvancedSearch advalg;

	// private long pertc1, pertc2, tc;

	@Override
	public IGraph<AStarGraphNode, AStarGraphEdge> getGraph() {
		return this.graph;
	}

	@Override
	public void initialInternal() {
		AStarGraphBuilder builder = new AStarGraphBuilder(context, getAStarFunction());
		this.graph = builder.getGraph();
		// this.algorithm = new AStarSearch(context, graph, getAStarFunction());
		this.advalg = new AStarAdvancedSearch(context, builder.getGraph(), getAStarFunction());
	}

	@Override
	public final RouteResult search(EntityID from, EntityID to) {
		Area source = (Area) world.getEntity(from);
		Area target = (Area) world.getEntity(to);
		return search(source, source.getX(), source.getY(), target);
	}

	@Override
	public final RouteResult search(EntityID from, int fromx, int fromy, EntityID to) {
		Area source = (Area) world.getEntity(from);
		Area target = (Area) world.getEntity(to);
		return search(source, fromx, fromy, target);
	}

	@Override
	protected void preupdate() {
		super.preupdate();
		this.advalg.resetFailedCache();
		// if (context.getTimestamp() % 10 == 0) {
		// // TODO remove
		// // log.info("SearchCount:" + tc + " ALG1:" + pertc1 + " ALG2:" +
		// pertc2);
		// pertc1 = 0;
		// pertc2 = 0;
		// tc = 0;
		//
		// //
		// viewer.Bundle.getInstance().printAStarStat(context.getTimestamp());
		// }
	}

	private final RouteResult search(Area source, int fromx, int fromy, Area target) {
		this.advalg.setOrigin(source, new Point(fromx, fromy));
		this.advalg.setDestination(target, new Point(target.getX(), target.getY()));

		long _time_st = System.currentTimeMillis();
		Route<AStarGraphNode> raw2 = this.advalg.search();
		long _time_ed = System.currentTimeMillis();
		long _dTime = _time_ed - _time_st;
		if (SHOW_ASTAR_TIME && _dTime > 10) {
			log.info("AbstractAStarRouteComponent.search cost: " + _dTime + " milliseconds");
			advalg.printDebug();
		}
//		System.out.println("--------------------------------------------");
//		log.info("AStarSearch:" + source + "," + fromx + "," + fromy + "-->" + target);
//		
//		advalg.printDebug();
//		for (AStarGraphNode asgn : raw2.getRoute()) {
//			log.info(asgn);
//			if (asgn.getLabel() == 38) {
//				System.out.println("=======");
//				for (AStarGraphEdge asge : graph.edgesOf(asgn)) {
//					System.out.println(asge);
//				}
//				System.out.println("=======");
//			}
//		}
//		System.out.println("--------------------------------------------");
		
		if (raw2 == null) {
			return new RouteResultFactory(false).newInstance();
		} else {
			RouteResult ret = transToMoveRoute(raw2, source, target);
			return ret;
		}
	}

	// private final RouteResult search(Area source, int fromx, int fromy, Area
	// target) {
	// // TODO target point
	// this.algorithm.setOrigin(source, new Point(fromx, fromy));
	// this.algorithm.setDestination(target, new Point(target.getX(),
	// target.getY()));
	//
	// this.advalg.setOrigin(source, new Point(fromx, fromy));
	// this.advalg.setDestination(target, new Point(target.getX(),
	// target.getY()));
	//
	// // TODO
	// // tc += 1;
	// long _time_st = System.currentTimeMillis();
	// Route<AStarGraphNode> raw = this.algorithm.search();
	// long _time_ed = System.currentTimeMillis();
	// pertc1 += _time_ed - _time_st;
	//
	// _time_st = System.currentTimeMillis();
	// Route<AStarGraphNode> raw2 = this.advalg.search();
	// _time_ed = System.currentTimeMillis();
	// pertc2 = _time_ed - _time_st;
	//
	// if (raw == null) {
	// if (raw2 != null) {
	// log.error("Not Match 2-search algorithm");
	// log.error("Agent: " + context.getAgentID() + " From:" + source + " to" +
	// target);
	// algorithm.printDebug();
	// advalg.printDebug();
	// // throw new RuntimeException();
	// }
	//
	// return new RouteResultFactory(false).newInstance();
	// } else {
	// RouteResult ret = transToMoveRoute(raw, source, target);
	// if (raw2 == null) {
	// log.error("Not Match 2-search algorithm, raw != null but raw2 == null");
	// log.error("Agent: " + context.getAgentID() + " From:" + source + " to" +
	// target);
	// algorithm.printDebug();
	// advalg.printDebug();
	// // throw new RuntimeException();
	// // return ret;
	// }
	//
	// RouteResult ret2 = transToMoveRoute(raw2, source, target);
	//
	// if (!ret.equals(ret2)) {
	// if (ret.getDistance() != ret2.getDistance()) {
	// log.error("Not Match 2-search algorithm");
	// log.error("Agent: " + context.getAgentID() + " From:" + source + " to" +
	// target);
	// algorithm.printDebug();
	// advalg.printDebug();
	// log.info("PATH1:" + ret.getMoveRoute());
	// log.info("PATH2:" + ret2.getMoveRoute());
	// // throw new RuntimeException();
	// }
	// }
	//
	// checkInnerRoadTwist(raw2, source.getID());
	// return ret;
	// }
	// }

	// // 在同一个Area里面E-E-E这种case
	// private void checkInnerRoadTwist(Route<AStarGraphNode> raw2, EntityID
	// start) {
	// int th2 = 0;
	// int t2 = 0;
	// int t3 = 0;
	// List<AStarGraphNode> list = raw2.getRoute();
	// if (list.size() >= 2) {
	// AStarGraphNode a0, a1;
	// a0 = list.get(0);
	// a1 = list.get(1);
	// if (!(a0.getType() == InnerType.Building || a1.getType() ==
	// InnerType.Building)) {
	// EntityID a = getSameRoad(a0, a1);
	// if (a.equals(start)) {
	// th2++;
	// t2++;
	// }
	// }
	// }
	//
	// for (int i = 0; i < list.size() - 2; i++) {
	// AStarGraphNode a0, a1, a2, a3 = null;
	// a0 = list.get(i);
	// a1 = list.get(i + 1);
	// a2 = list.get(i + 2);
	// if (i != list.size() - 3)
	// a3 = list.get(i + 3);
	//
	// // FOR t2
	// if (!(a0.getType() == InnerType.Building || a1.getType() ==
	// InnerType.Building || a2.getType() == InnerType.Building)) {
	// EntityID a = getSameRoad(a0, a1);
	// EntityID b = getSameRoad(a1, a2);
	// if (a == null || b == null)
	// throw new RuntimeException("BADASTARNODE");
	// if (a.equals(b)) {
	// if (i == 0)
	// t3++;
	// else
	// t2++;
	// }
	// }
	//
	// if (a3 != null) {
	// if (!(a0.getType() == InnerType.Building || a1.getType() ==
	// InnerType.Building
	// || a2.getType() == InnerType.Building || a3.getType() ==
	// InnerType.Building)) {
	// EntityID a = getSameRoad(a0, a1);
	// EntityID b = getSameRoad(a1, a2);
	// EntityID c = getSameRoad(a2, a3);
	// if (a == null || b == null || c == null)
	// throw new RuntimeException("BADASTARNODE");
	// if (a.equals(b) && b.equals(c))
	// t3++;
	// }
	// }
	// }
	//
	// viewer.Bundle.getInstance().astarStat(th2, t2, t3, list.size());
	// }

	public static EntityID getSameRoad(AStarGraphNode a0, AStarGraphNode a1) {
		if (!(a0.getType() == InnerType.Building || a1.getType() == InnerType.Building)) {
			if (a0.getType() == InnerType.SidePair) {
				if (a1.getType() == InnerType.Side) {
					Road r = a1.getSingleSide().getRoad();
					RoadSide rs = a0.getInnerSide(r);
					if (rs == null)
						return null;
					else
						return rs.getRoad().getID();
				} else {
					EntityID preleft = a0.getSidePair().getLeft().getRoad().getID();
					EntityID preright = a0.getSidePair().getRight().getRoad().getID();
					EntityID curleft = a1.getSidePair().getLeft().getRoad().getID();
					EntityID curright = a1.getSidePair().getRight().getRoad().getID();
					if (curleft.equals(preleft)) {
						return curleft;
					} else if (curleft.equals(preright)) {
						return curleft;
					} else if (curright.equals(preleft)) {
						return curright;
					} else if (curright.equals(preright)) {
						return curright;
					} else {
						return null;
					}
				}
			} else {
				Road r = a0.getSingleSide().getRoad();
				RoadSide rs = a1.getInnerSide(r);
				if (rs == null)
					return null;
				else
					return rs.getRoad().getID();
			}
		} else {
			return null;
		}
	}

	private RouteResult transToMoveRoute(Route<AStarGraphNode> raw, Area source, Area target) {
		// log.debug(raw) ;
		LinkedList<EntityID> tmp = new LinkedList<EntityID>();
		tmp.add(source.getID());
		AStarGraphNode pre = null;

		// TODO check pre
		for (AStarGraphNode node : raw.getRoute()) {
			if (node.getType() == InnerType.Building) {
				tmp.add(node.getBuilding().getID());
			} else if (node.getType() == InnerType.Side) {
				tmp.add(node.getSingleSide().getRoad().getID());
			} else if (node.getType() == InnerType.SidePair) {
				// if
				// (node.getSidePair().getLeft().getRoad().getID().equals(tmp.getLast()))
				// {
				// tmp.add(node.getSidePair().getLeft().getRoad().getID());
				// tmp.add(node.getSidePair().getRight().getRoad().getID());
				// } else if
				// (node.getSidePair().getRight().getRoad().getID().equals(tmp.getLast()))
				// {
				// tmp.add(node.getSidePair().getRight().getRoad().getID());
				// tmp.add(node.getSidePair().getLeft().getRoad().getID());
				// } else {
				// throw new IllegalStateException();
				// }
				if (pre != null) {
					if (pre.getType() == InnerType.Building)
						throw new IllegalStateException();
					if (pre.getType() == InnerType.Side) {
						boolean a = node.getSidePair().getLeft().getRoad().getID()
								.equals(pre.getSingleSide().getRoad().getID());
						boolean b = node.getSidePair().getRight().getRoad().getID()
								.equals(pre.getSingleSide().getRoad().getID());
						if (a == false && b == false) {
							throw new IllegalStateException();
						}
						tmp.add(pre.getSingleSide().getRoad().getID());
					} else {
						EntityID preleft = pre.getSidePair().getLeft().getRoad().getID();
						EntityID preright = pre.getSidePair().getRight().getRoad().getID();
						EntityID curleft = node.getSidePair().getLeft().getRoad().getID();
						EntityID curright = node.getSidePair().getRight().getRoad().getID();
						if (curleft.equals(preleft)) {
							tmp.add(curleft);
						} else if (curleft.equals(preright)) {
							tmp.add(curleft);
						} else if (curright.equals(preleft)) {
							tmp.add(curright);
						} else if (curright.equals(preright)) {
							tmp.add(curright);
						} else {
							throw new IllegalStateException();
						}
					}
				} else {
					boolean a = node.getSidePair().getLeft().getRoad().getID()
							.equals(tmp.getLast());
					boolean b = node.getSidePair().getRight().getRoad().getID()
							.equals(tmp.getLast());
					if (a == false && b == false) {
						throw new IllegalStateException();
					}
					tmp.add(tmp.getLast());
				}
			}
			pre = node;
		}
		if (pre.getType() == InnerType.SidePair) {
			boolean a = pre.getSidePair().getLeft().getRoad().getID().equals(target.getID());
			boolean b = pre.getSidePair().getRight().getRoad().getID().equals(target.getID());
			if (a == false && b == false) {
				throw new IllegalStateException();
			} else {
				tmp.add(target.getID());
			}
		}
		ArrayList<EntityID> inner = new ArrayList<EntityID>();
		while (!tmp.isEmpty()) {
			EntityID id = tmp.pollFirst();
			if (inner.isEmpty()) {
				inner.add(id);
			} else {
				if (!inner.get(inner.size() - 1).equals(id)) {
					inner.add(id);
				}
			}
		}
		RouteResultFactory factory = new RouteResultFactory(true);
		factory.setDistance(raw.getDistance());
		factory.setRawRoute(inner);
		factory.setFrom(source.getID());
		factory.setTo(target.getID());
		factory.addSuitableDestPoint(target.getX(), target.getY());
		return factory.newInstance();
	}

	public abstract AbstractAStarFunction getAStarFunction();
}
