package ant.map.search.astar;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Road;
import ant.liaison.message.IMessage;
import ant.map.component.AntMapBasisComponent;
import ant.map.component.RoadPassObjectsComponent;
import ant.map.objects.RoadSide;
import ant.map.search.astar.AStarGraphNode.InnerType;
import ant.utils.geo.GeoUtils;
import ant.utils.geo.Point;

/**
 * 
 * @author Ivan
 * 
 * @depend {@link RoadPassObjectsComponent} {@link AntMapBasisComponent}
 */
public class StaticAStarRouteComponent extends AbstractAStarRouteComponent {
	private StaticAStarFunction function = null;

	@Override
	public String getComponentName() {
		return StaticAStarRouteComponent.class.getName();
	}

	@Override
	public AbstractAStarFunction getAStarFunction() {
		if (function == null) {
			function = new StaticAStarFunction();
		}
		return function;
	}

	@Override
	public void updateInsightRoad(Road r) {
	}

	@Override
	public void updateMessage(IMessage message) {
	}

//	@Override
//	public Set<MessageType> getInterestMessageTypes() {
//		return Collections.EMPTY_SET;
//	}

//	@Override
//	public void laterUpdate() {
//	}

	/**
	 * Use L1 distance
	 * 
	 * @author Ivan
	 * 
	 */
	public static class StaticAStarFunction extends AbstractAStarFunction {
		private static final Log log = LogFactory.getLog(StaticAStarRouteComponent.StaticAStarFunction.class);
		
//		private Point getAStarGraphNodePoint(AStarGraphNode source) {
//			if (source.getType() == InnerType.Building) {
//				return new Point(source.getBuilding().getX(), source.getBuilding().getY());
//			} else if (source.getType() == InnerType.Side) {
//				int x = (source.getSingleSide().getEdge().getStartX() + source.getSingleSide().getEdge().getEndX()) / 2;
//				int y = (source.getSingleSide().getEdge().getStartY() + source.getSingleSide().getEdge().getEndY()) / 2;
//				return new Point(x, y);
//			} else {
//				int x = source.getSidePair().getLeft().getRoad().getX();
//				int y = source.getSidePair().getLeft().getRoad().getY();
//				return new Point(x, y);
//			}
//		}
		
		@Override
		public double heuristic(AStarGraphNode source, AStarGraphNode end) {
			return GeoUtils.manhattanDistance(source, end);
		}
		
		@Override
		public double heristric(AStarGraphNode source, Area end, Point endPoint) {
			return GeoUtils.manhattanDistance(source, endPoint);
		}

		@Override
		public boolean accept(AStarGraphNode source, Area end, Point endPoint) {
//			if (source.getType() == InnerType.Building) {
//				return source.getBuilding().getID().equals(end.getID());
//			} else if (source.getType() == InnerType.Side) {
//				return source.getSingleSide().getRoad().getID().equals(end.getID());
//			} else {
//				return (source.getSidePair().getLeft().getRoad().getID().equals(end.getID()) || source.getSidePair()
//						.getRight().getRoad().getID().equals(end.getID()));
//			}
			
			if (end instanceof Road) {
				RoadSide rs = source.getInnerSide((Road) end);
				if (rs == null) {
					log.warn("Bad call");
					return false;
				}
				return true;
				
			} else {
				if (source.getType() == InnerType.Building) {
					if (source.getBuilding().getID().equals(end.getID())) {
						return true;
					} else {
						log.warn("Bad call");
					}
				}
				return false;
			}
			
		}

		@Override
		public double costEnd(AStarGraphNode source, Area end, Point endPoint) {
			if (accept(source, end, endPoint)) {
				return GeoUtils.manhattanDistance(source, endPoint);
			} else {
				throw new IllegalArgumentException();
			}
		}

//		@Override
//		public double cost(RoadSide edge1, RoadSide edge2) {
//			int x1 = (edge1.getEdge().getStartX() + edge1.getEdge().getEndX()) / 2;
//			int y1 = (edge1.getEdge().getStartY() + edge1.getEdge().getEndY()) / 2;
//
//			int x2 = (edge2.getEdge().getStartX() + edge2.getEdge().getEndX()) / 2;
//			int y2 = (edge2.getEdge().getStartY() + edge2.getEdge().getEndY()) / 2;
//
//			return GeoUtils.manhattanDistance(x1, y1, x2, y2);
//		}
//
//		@Override
//		public double cost(RoadSide edge, Building b) {
//			int x = (edge.getEdge().getStartX() + edge.getEdge().getEndX()) / 2;
//			int y = (edge.getEdge().getStartY() + edge.getEdge().getEndY()) / 2;
//			return GeoUtils.manhattanDistance(x, y, b.getX(), b.getY());
//		}
//
//		@Override
//		public double cost(Building b1, Building b2) {
//			return GeoUtils.manhattanDistance(b1.getX(), b1.getY(), b2.getX(), b2.getY());
//		}

		@Override
		public double costStart(AStarGraphNode source, Area end, Point endPoint) {
			if (accept(source, end, endPoint)) {
				return GeoUtils.manhattanDistance(source, endPoint);
			} else {
				throw new IllegalArgumentException();
			}
		}

		@Override
		public double initialCost(AStarGraphNode n1, AStarGraphNode n2) {
			// TODO check nei?
			return GeoUtils.manhattanDistance(n1, n2);
		}

		@Override
		public boolean finish(AStarGraphNode source, Area end, Point point) {
			if (end instanceof Road) {
				RoadSide rs = source.getInnerSide((Road) end);
				if (rs == null) {
					return false;
				}
				return true;
				
			} else {
				if (source.getType() == InnerType.Building) {
					if (source.getBuilding().getID().equals(end.getID())) {
						return true;
					}
				}
				return false;
			}
		}

	}
}
