package ant.utils;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import ant.utils.geo.Point;

public class CoordinatesInArea {
	private static final boolean APEX_ON = true;
	private static final boolean EDGE_ON = true;
	/**
	 * POSITION_XY_TO_CENTER is the POSITION_XY weight, and (1-
	 * POSITION_XY_TO_CENTER) is center weight
	 */
	private static double POSITION_XY_TO_CENTER = 0.80;

	public static List<Point> getPositionCoordinate(Area position) {
		ArrayList<Point> rv = new ArrayList<Point>();
		if (position instanceof Building) {
			rv.add(new Point(position.getX(), position.getY()));
		} else if (position instanceof Road) {
			getRoadPositionCoordinate((Road) position, rv);
		} else {
			// never happen
		}
		return rv;
	}

	private static void getRoadPositionCoordinate(Road position, List<Point> coordinateList) {
		if (APEX_ON) {
			getRoadPositionCoordinateBasedOnApex(position, coordinateList);
		}
		if (EDGE_ON) {
			getRoadPositionCoordinateBasedOnEdge(position, coordinateList);
		}
	}

	private static void getRoadPositionCoordinateBasedOnApex(Road position,
			List<Point> coordinateList) {
		/**
		 * search apex
		 */
		for (int i = 0; i < position.getApexList().length - 1; i += 2) {
			Point point = new Point(
					(int) (position.getApexList()[i] * POSITION_XY_TO_CENTER + position.getX()
							* (1 - POSITION_XY_TO_CENTER)),
					(int) (position.getApexList()[i + 1] * POSITION_XY_TO_CENTER + position.getY()
							* (1 - POSITION_XY_TO_CENTER)));
			coordinateList.add(point);
		}
	}

	private static void getRoadPositionCoordinateBasedOnEdge(Road position,
			List<Point> coordinateList) {
		/**
		 * search edge midpoint
		 */
		for (Edge edge : position.getEdges()) {
			Point point = new Point(
					(int) ((edge.getStartX() + edge.getEndX()) / 2 * POSITION_XY_TO_CENTER + position.getX()
							* (1 - POSITION_XY_TO_CENTER)),
					(int) ((edge.getStartY() + edge.getEndY()) / 2 * POSITION_XY_TO_CENTER + position
							.getY() * (1 - POSITION_XY_TO_CENTER)));
			coordinateList.add(point);
		}
	}
}
