package ant.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author Ivan
 *
 */
public class FastHeap {
	private static final Log log = LogFactory.getLog(FastHeap.class);
	
	private int[] locator;
	private int size;
	private int maxSize;
	private FastHeapNode[] heap;
	
	public FastHeap(int maxSize) {
		this.maxSize = maxSize;
		this.size = 0;
		this.locator = new int[maxSize + 1];
		this.heap = new FastHeapNode[maxSize + 1];
		for (int i = 0; i <= maxSize; i++) {
			locator[i] = -1;
			heap[i] = null;
		}
	}
	
	public void add(FastHeapNode node) {
		if (size >= maxSize) {
			throw new IllegalStateException("Out of capacity");
		} else {
			if (node.getLabel() >= maxSize || node.getLabel() < 0)
				throw new IllegalArgumentException("Bad label");
			
			if (locator[node.getLabel()] == -1) {
				size++;
				locator[node.getLabel()] = size;
				heap[size] = node;
				up();
			} else {
				log.error(node + " already in heap");
				throw new IllegalStateException("Already in heap");
			}
		}
	}
	
	/**
	 * @return null if heap is empty
	 */
	public FastHeapNode pop() {
		if (size > 0) {
			FastHeapNode node = heap[1];
			locator[node.getLabel()] = -1;
			if (size != 1) {
				heap[1] = heap[size];
				locator[heap[1].getLabel()] = 1;
			}
			heap[size] = null;
			size--;
			if (size != 0)
				down();
			return node;
		} else {
			return null;
		}
	}
	
	public FastHeapNode top() {
		if (size > 0) {
			return heap[1];
		} else {
			return null;
		}
	}
	public boolean isEmpty() {
		return size == 0;
	}
	
	public int size() {
		return this.size;
	}
	
	public void clear() {
		for (int i = 0; i <= maxSize; i++) {
			heap[i] = null;
			locator[i] = -1;
		}
		size = 0;
	}
	
	public int getMaxSize() {
		return this.maxSize;
	}
	
	private void up() {
		int i = size;
		FastHeapNode x = heap[i];
		int j = i >>> 1;
		while (j > 0 && x.getValue() < heap[j].getValue()) {
			heap[i] = heap[j];
			locator[heap[i].getLabel()] = i;
			i = j;
			j = i >>> 1;
		}
		heap[i] = x;
		locator[x.getLabel()] = i;
	}
	
	private void up(int loc) {
		int i = loc;
		FastHeapNode x = heap[i];
		int j = i >>> 1;
		while (j > 0 && x.getValue() < heap[j].getValue()) {
			heap[i] = heap[j];
			locator[heap[i].getLabel()] = i;
			i = j;
			j = i >>> 1;
		}
		heap[i] = x;
		locator[x.getLabel()] = i;
	}
	
	private void down() {
		int i = 1;
		FastHeapNode x = heap[1];
		int j = i << 1;
		int k = j + 1;
		
		if (k <= size && heap[k].getValue() < heap[j].getValue()) {
			j = k;
		}
		
		while (j <= size && x.getValue() > heap[j].getValue()) {
			heap[i] = heap[j];
			locator[heap[i].getLabel()] = i;
			i = j;
			j = i << 1;
			k = j + 1;
			if (k <= size && heap[k].getValue() < heap[j].getValue()) {
				j = k;
			}
		}
		heap[i] = x;
		locator[x.getLabel()] = i;
	}
	
	public void decadj(FastHeapNode node) {
		up(locator[node.getLabel()]);
	}

	
	
}
