package ant.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author Ivan
 *
 * @param <A>
 * @param <B>
 */
public class IdempotentMatch<A, B> {
	private static final Log log = LogFactory.getLog(IdempotentMatch.class);
	
	private List<A> left;
	private List<B> right;
	private DualisticFunction<A, B> func;
	private Map<A, B> abmap;
	private Map<B, A> bamap;
	
	public IdempotentMatch(List<A> left, List<B> right, DualisticFunction<A, B> func) {
		this.left = left;
		this.right = right;
		this.func = func;
		this.abmap = new HashMap<A, B>();
		this.bamap = new HashMap<B, A>();
		if (this.left.size() != this.right.size())
			throw new RuntimeException("Not Idempotent");
	}
	
	/** greedy match, O(n^3) **/
	public void minMatch() {
		long _time_st = System.currentTimeMillis();
		List<A> l = new ArrayList<A>();
		List<B> r = new ArrayList<B>();
		for (A a : left) l.add(a);
		for (B b : right) r.add(b);
		int n = l.size();
		for (int i = 0; i < n; i++) {
			A ma = null;
			B mb = null;
			double min = Double.MAX_VALUE;
			for (A a : l)
				for (B b : r) {
					double val = this.func.invoke(a, b);
					if (val < min) {
						min = val;
						ma = a;
						mb = b;
					}
				}
			abmap.put(ma, mb);
			bamap.put(mb, ma);
			l.remove(ma);
			r.remove(mb);
		}
		
		// TODO switch~
		
		l = new ArrayList<A>();
		r = new ArrayList<B>();
		for (A a : left) l.add(a);
		for (B b : right) r.add(b);
		
		while (true) {
			if (!swaponce(l, n)) {
				break;
			}
		}
		
		long _time_ed = System.currentTimeMillis();
		log.info("IdempotentMatch.match cost:" + (_time_ed - _time_st) + " milliseconds");
	}

	private boolean swaponce(List<A> l, int n) {
		for (int i = 0; i < n - 1; i++) {
			for (int j = i + 1; j < n; j++) {
				A a1 = l.get(i);
				A a2 = l.get(j);
				B b1 = abmap.get(a1);
				B b2 = abmap.get(a2);
				double now = func.invoke(a1, b1) + func.invoke(a2, b2);
				double swap = func.invoke(a1, b2) + func.invoke(a2, b1);
				if (now > swap) {
					abmap.put(a1, b2);
					abmap.put(a2, b1);
					return true;
				}
			}
		}
		return false;
	}
	
	public B getAmatch(A a) {
		return abmap.get(a);
	}
	
	public A getBmatch(B b) {
		return bamap.get(b);
	}
	
	public static void main(String[] args) {
		List<Integer> a = new ArrayList<Integer>();
		for (int i = 0; i < 3; i++) a.add(i);
		// final int[][] edge = {{7,6,3}, {4,1,3}, {3,2,3}};
		final int[][] edge = {{1, 2, 3}, {2, 10, 20}, {3, 10, 20}};
		IdempotentMatch<Integer, Integer> im = new IdempotentMatch<Integer, Integer>(a, a, new DualisticFunction<Integer, Integer>() {
			@Override
			public double invoke(Integer t1, Integer t2) {
				return edge[t1][t2];
			}
		});
		im.minMatch();
		for (int i = 0; i < 3; i++) System.out.println(im.getAmatch(i));
	}
}
