package ant.utils;

import static java.util.Locale.ENGLISH;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/**
 * 
 * @author Ivan, Oliver
 * 
 */
public class IntrospectorUtils {

	public static Method findGetMethod(Class<?> clazz, Field field) {
		Class<?> type = field.getType();
		String getname;
		String fieldName = field.getName();
		if (type == boolean.class || type == null) {
			if (fieldName.startsWith("is")) {
				getname = fieldName;
			} else {
				getname = "is" + capitalize(fieldName);
			}
		} else {
			getname = "get" + capitalize(fieldName);
		}
		return internalFindMethod(clazz, getname, 0, null);
	}

	public static Method findSetMethod(Class<?> clazz, Field field) {
		String fieldName = field.getName();
		String setname;
		if (fieldName.startsWith("is")) {
			setname = "set" + capitalize(fieldName.substring(2));
		} else {
			setname = "set" + capitalize(fieldName);
		}
		return internalFindMethod(clazz, setname, 1, null);
	}

	public static String capitalize(String name) {
		if (name == null || name.length() == 0) {
			return name;
		}
		return name.substring(0, 1).toUpperCase(ENGLISH) + name.substring(1);
	}

	/**
	 * ref java.beans.Introspector, java.beans.PropertyDescriptor
	 */
	private static Method internalFindMethod(Class<?> start, String methodName, int argCount,
			Class<?> args[]) {
		// For overriden methods we need to find the most derived version.
		// So we start with the given class and walk up the superclass chain.

		Method method = null;

		for (Class<?> cl = start; cl != null; cl = cl.getSuperclass()) {
			Method methods[] = getPublicDeclaredMethods(cl);
			for (int i = 0; i < methods.length; i++) {
				method = methods[i];
				if (method == null) {
					continue;
				}

				// make sure method signature matches.
				Class<?> params[] = method.getParameterTypes();
				if (method.getName().equals(methodName) && params.length == argCount) {
					if (args != null) {
						boolean different = false;
						if (argCount > 0) {
							for (int j = 0; j < argCount; j++) {
								if (params[j] != args[j]) {
									different = true;
									continue;
								}
							}
							if (different) {
								continue;
							}
						}
					}
					return method;
				}
			}
		}
		method = null;

		// Now check any inherited interfaces. This is necessary both when
		// the argument class is itself an interface, and when the argument
		// class is an abstract class.
		Class<?> ifcs[] = start.getInterfaces();
		for (int i = 0; i < ifcs.length; i++) {
			// Note: The original implementation had both methods calling
			// the 3 arg method. This is preserved but perhaps it should
			// pass the args array instead of null.
			method = internalFindMethod(ifcs[i], methodName, argCount, null);
			if (method != null) {
				break;
			}
		}
		return method;
	}

	private static Method[] getPublicDeclaredMethods(Class<?> cl) {
		Method[] result = cl.getDeclaredMethods();
		for (int i = 0; i < result.length; i++) {
			Method method = result[i];
			int mods = method.getModifiers();
			if (!Modifier.isPublic(mods)) {
				result[i] = null;
			}
		}
		return result;
	}

}
