package ant.utils;

import java.util.HashMap;
import java.util.Map.Entry;

/**
 * 
 * @author Ivan
 *
 */
public class LRUMap<K, V> {
	
	private int maxsize;
	private HashMap<K, V> map;
	private HashMap<K, Integer> timemap;
	
	public LRUMap(int maxsize) {
		this.maxsize = maxsize;
		this.map = new HashMap<K, V>(maxsize);
	}
	
	public boolean contains(K key) {
		return map.containsKey(key);
	}
	
	public void put(K key, V val, int timestamp) {
		this.map.put(key, val);
		this.timemap.put(key, timestamp);
		if (map.size() > maxsize) adjust(); 
	}

	public V get(K key, int timestamp) {
		if (map.containsKey(key)) {
			timemap.put(key, timestamp);
			return map.get(key);
		} else {
			return null;
		}
	}
	
	public boolean isEmpty() {
		return this.map.isEmpty();
	}
	
	public int size() {
		return map.size();
	}
	
	private void adjust() {
		K pick = null;
		int low = Integer.MAX_VALUE;
		for (Entry<K, Integer> entry : timemap.entrySet()) {
			if (entry.getValue() < low) {
				low = entry.getValue();
				pick = entry.getKey();
			}
		}
		if (pick != null) {
			map.remove(pick);
			timemap.remove(pick);
		}
	}
}
