package ant.utils;

import java.util.Collection;
import java.util.Comparator;

/**
 * Top(MAX-N) set
 * 可以用作capacity=N 的优先队列实现, 丢弃优先级较低的
 * 
 * @author Ivan
 * @param <T> sho {@link Comparable}
 * @NonThreadSafe
 */

// TODO UnitTest
public class MaxNSet<T> {
	// min-heap impl
	protected Object[] heap;
	protected int size;
	protected int N;
	protected Comparator<? super T> comparator = null;
	
	public MaxNSet(int N, Comparator<? super T> comparator) {
		this.N = N;
		this.comparator = comparator;
		
		heap = new Object[N + 1];
		for (int i = 0; i <= N; i++) 
			heap[i] = null;
		heap[0] = null;
		size = 0;
	}
	
	public void add(T t) {
		if (size >= N) {
			// t > min
			// if (((Comparable) t).compareTo((Comparable) heap[1]) > 0) {
			if (comparator.compare(t, (T) heap[1]) > 0) {
				heap[1] = t;
				down();
			}
		} else {
			size++;
			heap[size] = t;
			up();
		}
	}
	
	public void add(Collection<T> container) {
		for (T t : container) {
			add(t);
		}
	}
	
	// unsorted
	public T pop() {
		if (size > 0) {
			Object ret = heap[1];
			heap[1] = heap[size];
			heap[size] = null;
			size--;
			down();
			return (T) ret;
		} else {
			return null;
		}
	}
	
	public int size() {
		return size;
	}
	
	public boolean isEmpty() {
		return size == 0;
	}
	
	public void clear() {
		for (int i = 0; i <= size; i++) {
			heap[i] = null;
		}
		size = 0;
	}
	
	protected void up() {
		int i = size;
		Object obj = heap[i];
		int j = i >>> 1;
		// obj < heap[j]
		// while (j > 0 && ((Comparable) obj).compareTo((Comparable) heap[j]) < 0) {
		while (j > 0 && (comparator.compare((T) obj, (T) heap[j]) < 0)) {
			heap[i] = heap[j]; // shift father down
			i = j;
			j = j >>> 1;
		}
		heap[i] = obj;
	}
	
	protected void down() {
		int i = 1;
		Object obj = heap[1];
		int j = i << 1;
		int k = j + 1;
		
		// j = min(.k, .j)
		// if (k <= size && ((Comparable) heap[k]).compareTo((Comparable) heap[j]) < 0) {
		if (k <= size && (comparator.compare((T) heap[k], (T) heap[j]) < 0)) {
			j = k;
		}
		
		// obj > heap[j]
		// while (j <= size && ((Comparable) obj).compareTo((Comparable) heap[j]) > 0) {
		while (j <= size && (comparator.compare((T) obj, (T) heap[j]) < 0)) {
			heap[i] = heap[j]; // shift child up
			i = j;
			j = i << 1;
			k = j + 1;
			// if (k <= size && ((Comparable) heap[k]).compareTo((Comparable) heap[j]) < 0) {
			while (k <= size && (comparator.compare((T) heap[k], (T) heap[j]) < 0)) {
				j = k;
			}
		}
		heap[i] = obj;
	}
	
	public Object[] toSortedSet() {
		int x = size;
		if (size >= N) x = N;
		Object[] ret = new Object[x];
		
		for (int i = x - 1; i >= 0; i--)
			ret[i] = pop();
		return ret;
	}
}
