package ant.utils.agent;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.Constants;
import rescuecore2.components.Component;
import rescuecore2.components.ComponentConnectionException;
import rescuecore2.components.ComponentLauncher;
import rescuecore2.components.TCPComponentLauncher;
import rescuecore2.config.Config;
import rescuecore2.config.ConfigException;
import rescuecore2.connection.ConnectionException;
import rescuecore2.log.Logger;
import rescuecore2.misc.CommandLineOptions;
import rescuecore2.registry.Registry;
import rescuecore2.standard.entities.StandardEntityFactory;
import rescuecore2.standard.entities.StandardPropertyFactory;
import rescuecore2.standard.messages.StandardMessageFactory;
import ant.AbstractAntAgent;
import ant.agent.CommonAmbulanceTeam;
import ant.agent.CommonFireBrigade;
import ant.agent.CommonPoliceForce;

/**
 * 
 * @author Ivan
 * 
 */
public class AntStandardLaunchTool {
	private static final Log log = LogFactory.getLog(AntStandardLaunchTool.class);

	private static final String FIRE_BRIGADE_FLAG = "-fb";
	private static final String POLICE_FORCE_FLAG = "-pf";
	private static final String AMBULANCE_TEAM_FLAG = "-at";
	private static final String FIRE_STATION_FLAG = "-fs";
	private static final String POLICE_OFFICE_FLAG = "-po";
	private static final String AMBULANCE_CENTER_FLAG = "-ac";
	private static final String CIVILIAN_FLAG = "-cv";

	private ArrayList<Component> views = new ArrayList<Component>();
	private Class<? extends AbstractAntAgent> pfclazz = null;
	private Class<? extends AbstractAntAgent> atclazz = null;
	private Class<? extends AbstractAntAgent> fbclazz = null;
	private Class<? extends AbstractAntAgent> centreclazz = null;
	private boolean noagent = false;
	private String sethost = null;
	private int fbs = -1;
	private int pfs = -1;
	private int ats = -1;
	private int cs = -1;
	private boolean lazyView = false;

	public void addView(Component view) {
		this.views.add(view);
	}

	public void setPoliceForceAnt(Class<? extends AbstractAntAgent> clazz) {
		this.pfclazz = clazz;
	}

	public void setAmbulanceTeamAnt(Class<? extends AbstractAntAgent> clazz) {
		this.atclazz = clazz;
	}

	public void setFireBrigadeAnt(Class<? extends AbstractAntAgent> clazz) {
		this.fbclazz = clazz;
	}

	// public void setCentre(Class<? extends AbstractAntAgent> clazz) {
	// this.centreclazz = clazz;
	// }

	public void setNoAgent(boolean flag) {
		this.noagent = flag;
	}

	public void setHost(String host) {
		this.sethost = host;
	}

	public void setAgentCount(int fbs, int pfs, int ats, int cs) {
		this.fbs = fbs;
		this.pfs = pfs;
		this.ats = ats;
		this.cs = cs;
	}

	/**
	 * Launch 'em!
	 * 
	 * @param args
	 *            The following arguments are understood: -p <port>, -h
	 *            <hostname>, -fb <fire brigades>, -pf <police forces>, -at
	 *            <ambulance teams>
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public void launch(String[] args) throws InstantiationException, IllegalAccessException {
		Logger.setLogContext("ANT");
		try {
			Registry.SYSTEM_REGISTRY.registerEntityFactory(StandardEntityFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY.registerMessageFactory(StandardMessageFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY.registerPropertyFactory(StandardPropertyFactory.INSTANCE);

			Config config = new Config();
			args = CommandLineOptions.processArgs(args, config);

			// TODO FIXME remove
			config.setValue("random.seed", "1");
			int port = config.getIntValue(Constants.KERNEL_PORT_NUMBER_KEY,
					Constants.DEFAULT_KERNEL_PORT_NUMBER);
			String host = config.getValue(Constants.KERNEL_HOST_NAME_KEY,
					Constants.DEFAULT_KERNEL_HOST_NAME);

			if (sethost != null)
				host = sethost;

			int fb = fbs;
			int pf = pfs;
			int at = ats;
			int fs = -1;
			int po = -1;
			int ac = -1;
			// CHECKSTYLE:OFF:ModifiedControlVariable
			for (int i = 0; i < args.length; ++i) {
				if (args[i].equals(FIRE_BRIGADE_FLAG)) {
					fb = Integer.parseInt(args[++i]);
				} else if (args[i].equals(POLICE_FORCE_FLAG)) {
					pf = Integer.parseInt(args[++i]);
				} else if (args[i].equals(AMBULANCE_TEAM_FLAG)) {
					at = Integer.parseInt(args[++i]);
				} else if (args[i].equals(FIRE_STATION_FLAG)) {
					fs = Integer.parseInt(args[++i]);
				} else if (args[i].equals(POLICE_OFFICE_FLAG)) {
					po = Integer.parseInt(args[++i]);
				} else if (args[i].equals(AMBULANCE_CENTER_FLAG)) {
					ac = Integer.parseInt(args[++i]);
				} else {
					Logger.warn("Unrecognised option: " + args[i]);
				}
			}
			// CHECKSTYLE:ON:ModifiedControlVariable
			ComponentLauncher launcher = new TCPComponentLauncher(host, port, config);
			System.out.println("launching fb " + fb + " pf " + pf + " at " + at + " fs " + fs
					+ " po" + po + " ac " + ac);
			connect(launcher, fb, pf, at, fs, po, ac, config);
		} catch (IOException e) {
			Logger.error("Error connecting agents", e);
		} catch (ConfigException e) {
			Logger.error("Configuration error", e);
		} catch (ConnectionException e) {
			Logger.error("Error connecting agents", e);
		} catch (InterruptedException e) {
			Logger.error("Error connecting agents", e);
		}
	}

	private void connect(ComponentLauncher launcher, int fb, int pf, int at, int fs, int po,
			int ac, Config config) throws InterruptedException, ConnectionException,
			InstantiationException, IllegalAccessException {
		int i = 0;

		if (noagent)
			return;

		try {
			while (fb-- != 0) {
				Logger.info("Connecting fire brigade " + (i++) + "...");
				System.out.println("Connecting fire brigade " + i + "...");
				if (fbclazz == null) {
					launcher.connect(new EmptyFireBrigade());
				} else {
					CommonFireBrigade cfb = new CommonFireBrigade(fbclazz.newInstance());
					launcher.connect(cfb);
					if (i % 5 == 0)
						gc();
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (pf-- != 0) {
				Logger.info("Connecting police force " + (i++) + "...");
				System.out.println("Connecting police force " + i + "...");
				if (pfclazz == null) {
					launcher.connect(new EmptyPoliceForce());
				} else {
					CommonPoliceForce cpf = new CommonPoliceForce(pfclazz.newInstance());
					launcher.connect(cpf);
					if (i % 5 == 0)
						gc();
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (at-- != 0) {
				Logger.info("Connecting ambulance team " + (i++) + "...");
				System.out.println("Connecting ambulance team " + i + "...");
				if (atclazz == null) {
					launcher.connect(new EmptyAmbulanceTeam());
				} else {
					CommonAmbulanceTeam cat = new CommonAmbulanceTeam(atclazz.newInstance());
					launcher.connect(cat);
					if (i % 5 == 0)
						gc();
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}

		try {
			while (fs-- != 0) {
				Logger.info("Connecting firestation " + (i++) + "...");
				System.out.println("Connecting firestation " + i + "...");
				launcher.connect(new EmptyCentre());
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (po-- != 0) {
				Logger.info("Connecting police office " + (i++) + "...");
				System.out.println("Connecting police office " + i + "...");
				launcher.connect(new EmptyCentre());
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (ac-- != 0) {
				Logger.info("Connecting ambulance center " + (i++) + "...");
				System.out.println("Connecting ambulance center " + i + "...");
				launcher.connect(new EmptyCentre());
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}

	}

	private void gc() {
		// long _time_st = System.currentTimeMillis();
		// long _st_usable = Runtime.getRuntime().maxMemory() -
		// Runtime.getRuntime().totalMemory() +
		// Runtime.getRuntime().freeMemory();
		// System.gc();
		// long _ed_usable = Runtime.getRuntime().maxMemory() -
		// Runtime.getRuntime().totalMemory() +
		// Runtime.getRuntime().freeMemory();
		// long _time_ed = System.currentTimeMillis();
		// log.info("GC cost: " + (_time_ed - _time_st) +
		// " milliseconds , cost " + (_st_usable - _ed_usable) / 1000000.0 +
		// "MB memory");
	}
}
