package ant.utils.agent;

import java.io.IOException;
import java.util.ArrayList;

import rescuecore2.Constants;
import rescuecore2.components.Component;
import rescuecore2.components.ComponentConnectionException;
import rescuecore2.components.ComponentLauncher;
import rescuecore2.components.TCPComponentLauncher;
import rescuecore2.config.Config;
import rescuecore2.config.ConfigException;
import rescuecore2.connection.ConnectionException;
import rescuecore2.log.Logger;
import rescuecore2.misc.CommandLineOptions;
import rescuecore2.registry.Registry;
import rescuecore2.standard.entities.StandardEntityFactory;
import rescuecore2.standard.entities.StandardPropertyFactory;
import rescuecore2.standard.messages.StandardMessageFactory;

/**
 * 
 * @author Ivan
 * 
 */
public class EmptyAgentLaunchTool {

	private static final String FIRE_BRIGADE_FLAG = "-fb";
	private static final String POLICE_FORCE_FLAG = "-pf";
	private static final String AMBULANCE_TEAM_FLAG = "-at";
	private static final String CIVILIAN_FLAG = "-cv";

	private ArrayList<Component> views = new ArrayList<Component>();
	private Class<? extends Component> pfclazz = null;
	private Class<? extends Component> atclazz = null;
	private Class<? extends Component> fbclazz = null;
	private Class<? extends Component> centreclazz = null;
	private boolean noagent = false;
	private String sethost = null;

	public void addView(Component view) {
		this.views.add(view);
	}

	public void setPoliceForce(Class<? extends Component> clazz) {
		this.pfclazz = clazz;
	}

	public void setAmbulanceTeam(Class<? extends Component> clazz) {
		this.atclazz = clazz;
	}

	public void setFireBrigade(Class<? extends Component> clazz) {
		this.fbclazz = clazz;
	}

	public void setCentre(Class<? extends Component> clazz) {
		this.centreclazz = clazz;
	}

	public void setNoAgent(boolean flag) {
		this.noagent = flag;
	}

	public void setHost(String host) {
		this.sethost = host;
	}

	/**
	 * Launch 'em!
	 * 
	 * @param args
	 *            The following arguments are understood: -p <port>, -h
	 *            <hostname>, -fb <fire brigades>, -pf <police forces>, -at
	 *            <ambulance teams>
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public void launch(String[] args) throws InstantiationException, IllegalAccessException {
		Logger.setLogContext("sample");
		try {
			Registry.SYSTEM_REGISTRY.registerEntityFactory(StandardEntityFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY.registerMessageFactory(StandardMessageFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY.registerPropertyFactory(StandardPropertyFactory.INSTANCE);
			Config config = new Config();
			args = CommandLineOptions.processArgs(args, config);
			// TODO FIXME remove
			config.setValue("random.seed", "1");
			int port = config.getIntValue(Constants.KERNEL_PORT_NUMBER_KEY, Constants.DEFAULT_KERNEL_PORT_NUMBER);
			String host = config.getValue(Constants.KERNEL_HOST_NAME_KEY, Constants.DEFAULT_KERNEL_HOST_NAME);

			if (sethost != null)
				host = sethost;

			int fb = -1;
			int pf = -1;
			int at = -1;
			// CHECKSTYLE:OFF:ModifiedControlVariable
			for (int i = 0; i < args.length; ++i) {
				if (args[i].equals(FIRE_BRIGADE_FLAG)) {
					fb = Integer.parseInt(args[++i]);
				} else if (args[i].equals(POLICE_FORCE_FLAG)) {
					pf = Integer.parseInt(args[++i]);
				} else if (args[i].equals(AMBULANCE_TEAM_FLAG)) {
					at = Integer.parseInt(args[++i]);
				} else {
					Logger.warn("Unrecognised option: " + args[i]);
				}
			}
			// CHECKSTYLE:ON:ModifiedControlVariable
			ComponentLauncher launcher = new TCPComponentLauncher(host, port, config);
			System.out.println("launching fb " + fb + " pf " + pf + " at " + at);
			connect(launcher, fb, pf, at, config);
		} catch (IOException e) {
			Logger.error("Error connecting agents", e);
		} catch (ConfigException e) {
			Logger.error("Configuration error", e);
		} catch (ConnectionException e) {
			Logger.error("Error connecting agents", e);
		} catch (InterruptedException e) {
			Logger.error("Error connecting agents", e);
		}
	}

	private void connect(ComponentLauncher launcher, int fb, int pf, int at, Config config) throws InterruptedException,
			ConnectionException, InstantiationException, IllegalAccessException {
		int i = 0;

		for (Component v : views) {
			try {
				Logger.info("Connecting viewer ...");
				launcher.connect(v);
				Logger.info("success");
			} catch (ComponentConnectionException e) {
				Logger.info("failed: " + e.getMessage());
			}
		}

		if (noagent)
			return;

		try {
			while (fb-- != 0) {
				Logger.info("Connecting fire brigade " + (i++) + "...");
				System.out.println("Connecting fire brigade " + i + "...");
				if (fbclazz == null) {
					launcher.connect(new EmptyFireBrigade());
				} else {
					launcher.connect(fbclazz.newInstance());
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (pf-- != 0) {
				Logger.info("Connecting police force " + (i++) + "...");
				System.out.println("Connecting police force " + i + "...");
				if (pfclazz == null) {
					launcher.connect(new EmptyFireBrigade());
				} else {
					launcher.connect(pfclazz.newInstance());
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (at-- != 0) {
				Logger.info("Connecting ambulance team " + (i++) + "...");
				System.out.println("Connecting ambulance team " + i + "...");
				if (atclazz == null) {
					launcher.connect(new EmptyFireBrigade());
				} else {
					launcher.connect(atclazz.newInstance());
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (true) {
				Logger.info("Connecting centre " + (i++) + "...");
				System.out.println("Connecting center " + i + "...");
				if (centreclazz == null) {
					launcher.connect(new EmptyFireBrigade());
				} else {
					launcher.connect(centreclazz.newInstance());
				}
				Logger.info("success");
				System.out.println("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
		try {
			while (true) {
				Logger.info("Connecting dummy agent " + (i++) + "...");
				launcher.connect(new DummyAgent());
				Logger.info("success");
			}
		} catch (ComponentConnectionException e) {
			Logger.info("failed: " + e.getMessage());
		}
	}

}
