package ant.utils.blockade;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.utils.geo.Converter;
import ant.utils.geo.GeoUtils;
import ant.utils.geo.Line;
import ant.utils.geo.Point;
import ant.utils.geo.Polygon;
import ant.utils.geo.Segment;
import ant.utils.geo.acc.AccUtils;

/**
 * 
 * @author Fun
 * 
 */
public class AccJudger {
	private static Log log = LogFactory.getLog(AccJudger.class);

	public static final int EXTEND_DISTANCE = 1000;

//	public static boolean isPassableE2E(Edge from,Area through, Edge to, StandardWorldModel world, int agentRadius){
//		if(!(through instanceof Road)){
//			return true;
//		}
//		WrappedRoad road = new WrappedRoad((Road)through, world);
//		return isPassableE2E(from, road, to, agentRadius);
//	}
	
	private static boolean edgeCheckNaive(Edge from, WrappedRoad through, Edge to, int agentRadius){
		boolean from1,from2,to1,to2;
		
		for (Blockade blockade : through.getBlockEntities()) {
			from1=from2=to1=to2=false;
			int[] exp = AccUtils.expandPolygon(blockade.getApexes(), agentRadius, true);
			if(!from1){
				if(GeoUtils.inPolygon(exp,from.getStartX(),from.getStartY())){
					from1=true;
				}
			}
			if(!from2){
				if(GeoUtils.inPolygon(exp,from.getStartX(),from.getStartY())){
					from2=true;
				}
			}
			if(from1&&from2){
				return false;
			}
			if(!to1){
				if(GeoUtils.inPolygon(exp,from.getStartX(),from.getStartY())){
					to1=true;
				}
			}
			if(!to2){
				if(GeoUtils.inPolygon(exp,from.getStartX(),from.getStartY())){
					to2=true;
				}
			}
			if(to1&&to2){
				return false;
			}
		}
		
		return true;
	}
	
	private static boolean edgeCheck(Edge from, WrappedRoad through, Edge to, int agentRadius){
		Segment start = Converter.generateSegment(from);
		Segment target = Converter.generateSegment(to);
		List<Polygon> blockades = through.getExpBlockades();
		
		for(Polygon blockade: blockades){
			if(GeoUtils.inPolygon(blockade.getApexes(),start.getX1(),start.getY1())){
				if(GeoUtils.inPolygon(blockade.getApexes(), start.getX2(), start.getY2())){
					return false;
				}
				for(Segment seg: blockade.getEdges()){
					if(GeoUtils.isSegmentIntersect(seg, start)){
						Point origin = GeoUtils.getIntersectionPoint(new Line(
								seg.getOrigin(), seg.getEnd()),
								new Line(start.getOrigin(), start.getEnd()));
						if(origin != null){
							start = new Segment(origin,start.getEnd());
						}
					}
				}
			}
		}
		
		for(Polygon blockade: blockades){
			if(GeoUtils.inPolygon(blockade.getApexes(), start.getX2(), start.getY2())){
				if(GeoUtils.inPolygon(blockade.getApexes(), start.getX1(), start.getY1())){
					return false;
				}
			}
		}
		
		
		for(Polygon blockade: blockades){
			if(GeoUtils.inPolygon(blockade.getApexes(),target.getX1(),target.getY1())){
				if(GeoUtils.inPolygon(blockade.getApexes(), target.getX2(), target.getY2())){
					return false;
				}
				for(Segment seg: blockade.getEdges()){
					if(GeoUtils.isSegmentIntersect(seg, target)){
						Point origin = GeoUtils.getIntersectionPoint(new Line(
								seg.getOrigin(), seg.getEnd()),
								new Line(target.getOrigin(), target.getEnd()));
						if(origin != null){
							target = new Segment(origin,target.getEnd());
						}
					}
				}
			}
		}
		
		for(Polygon blockade: blockades){
			if(GeoUtils.inPolygon(blockade.getApexes(), target.getX2(), target.getY2())){
				if(GeoUtils.inPolygon(blockade.getApexes(), target.getX1(), target.getY1())){
					return false;
				}
			}
		}
		return true;
	}
	
	public static boolean e2eOld = false;
	
	public static boolean isPassableE2E(Edge from, WrappedRoad through, Edge to, int agentRadius) {

		if (!through.isBlockadesDefined()) {
			return true;
		}
		
		if (!(from.isPassable()&&to.isPassable())){
			return false;
		}
		
		if(!e2eOld){
			if(!edgeCheck(from,through,to,agentRadius)){
				return false;
			}
		}
		
		
		List<Blockade> blockadelist = through.getBlockEntities();

		// get blockades
		ArrayList<WrappedBlockade> blockades = new ArrayList<WrappedBlockade>();
		Map<WrappedBlockade, Boolean> maps = new HashMap<WrappedBlockade, Boolean>();
		for (Blockade next : blockadelist) {
			WrappedBlockade wp = new WrappedBlockade(next);
			blockades.add(wp);
			maps.put(wp, false);
		}

		// divide edges into two parts
		ArrayList<Edge> sideOne = new ArrayList<Edge>();
		ArrayList<Edge> sideTwo = new ArrayList<Edge>();

		ArrayList<Edge> edges = new ArrayList<Edge>();

		for (Edge edge : through.getEdges()) {
			if (edge.getStart().equals(edge.getEnd())) {
				continue;
			}
			edges.add(edge);
		}

		for (int i = 0; i < edges.size(); i++) {
			Edge edge = edges.get(i);
			if ((edge.getStart().equals(from.getStart()) && edge.getEnd().equals(from.getEnd()))
					|| (edge.getStart().equals(to.getStart()) && edge.getEnd().equals(to.getEnd()))) {
				edges.remove(edge);
			}
		}

		Point2D point = from.getStart();
		int flag = 0;
		while (!(point.equals(to.getStart()) || point.equals(to.getEnd()))) {
			for (int i = 0; i < edges.size(); i++) {
				Edge edge = edges.get(i);
				if (point.equals(edge.getStart()) || point.equals(edge.getEnd())) {
					sideOne.add(edge);
					edges.remove(edge);
					if (point.equals(edge.getStart())) {
						point = edge.getEnd();
					} else {
						point = edge.getStart();
					}
					break;
				}
			}
			if (flag++ > 40) {
				break;
			}
		}

		if (flag > 40) {
			boolean res1 = isPoint2EdgeBlocked(through, new Point(through.getX(), through.getY()), to, agentRadius);
			boolean res2 = isPoint2EdgeBlocked(through, new Point(through.getX(), through.getY()), from, agentRadius);
			if (res1 && res2) {
				return true;
			}
		}

		point = from.getEnd();
		flag = 0;
		while (!(point.equals(to.getStart()) || point.equals(to.getEnd()))) {
			for (int i = 0; i < edges.size(); i++) {
				Edge edge = edges.get(i);
				if (point.equals(edge.getStart()) || point.equals(edge.getEnd())) {
					sideTwo.add(edge);
					edges.remove(edge);
					if (point.equals(edge.getStart())) {
						point = edge.getEnd();
					} else {
						point = edge.getStart();
					}
					break;
				}
			}
			if (flag++ > 20) {
				break;
			}
		}

		if (flag > 20) {
			boolean res1 = isPoint2EdgeBlocked(through, new Point(through.getX(), through.getY()), to, agentRadius);
			boolean res2 = isPoint2EdgeBlocked(through, new Point(through.getX(), through.getY()), from, agentRadius);
			if (res1 && res2) {
				return true;
			}
		}

		// side one
		boolean passSideOne = true;

		if (sideOne.size() == 0) {
			for (int i = 0; i < blockades.size(); i++) {
				WrappedBlockade b = blockades.get(i);
				if (b.getShortestDistanceToPoint(from.getStart()) < 2 * agentRadius) {
					passSideOne = false;
					maps.put(b, true);
					// TODO
				}
			}
		}
		for (Edge sideEdge : sideOne) {
			for (int i = 0; i < blockades.size(); i++) {
				WrappedBlockade b = blockades.get(i);
				Line2D segment = b.getShortestSegmetToEdge(sideEdge);
				if (segment == null) {
					continue;
				}
				double distance = GeometryTools2D.getDistance(segment.getOrigin(), segment.getEndPoint());
				if (distance < 2 * agentRadius) {
					passSideOne = false;
					maps.put(b, true);
					break;
				}
			}
		}

		if (passSideOne) {
			return true;
		}

		// side two
		boolean passSideTwo = true;

		if (sideTwo.size() == 0) {
			for (int i = 0; i < blockades.size(); i++) {
				WrappedBlockade b = blockades.get(i);
				if (b.getShortestDistanceToPoint(from.getEnd()) < 2 * agentRadius) {
					passSideTwo = false;
					if (maps.get(b)) {
						return false;
					}
				}
			}
		}

		for (Edge sideEdge : sideTwo) {
			for (int i = 0; i < blockades.size(); i++) {
				WrappedBlockade b = blockades.get(i);
				Line2D segment = b.getShortestSegmetToEdge(sideEdge);
				if (segment == null) {
					continue;
				}
				double distance = GeometryTools2D.getDistance(segment.getOrigin(), segment.getEndPoint());
				if (distance < 2 * agentRadius) {
					passSideTwo = false;
					if (maps.get(b)) {
						return false;
					}
				}
			}
		}

		if (passSideTwo) {
			return true;
		}

		// added 2011/8/8
		boolean isBlockaded = false;

		for (Blockade next : blockadelist) {
			if (BlockJudger.blockP2E(next, new Point((int) (from.getStartX() + from.getEndX()) / 2,
					(int) (from.getStartY() + from.getEndY()) / 2), to, agentRadius)) {
				isBlockaded = true;
			}
		}
		if (!isBlockaded) {
			return true;
		}

		// between blockades
		if (blockades.size() < 2) {
			return false;
		}

		while (blockades.size() > 1) {
			WrappedBlockade b1 = blockades.get(blockades.size() - 1);
			for (int i = blockades.size() - 2; i >= 0; i--) {
				WrappedBlockade b2 = blockades.get(i);
				// compare with closest Blockade
				List<Point2D> pointsOfB = b1.getPoints();
				List<Point2D> pointsOfCB = b2.getPoints();
				Point2D origin = null, end = null;
				double shortestDistance = Double.MAX_VALUE;
				// N*N
				for (Point2D p1 : pointsOfB) {
					for (Point2D p2 : pointsOfCB) {
						double distance = GeometryTools2D.getDistance(p1, p2);
						if (distance < shortestDistance) {
							shortestDistance = distance;
							origin = p1;
							end = p2;
						}
					}
				}

				if (shortestDistance < 2 * agentRadius) {
					return false;
				}

				// find lines near origin and end
				for (Line2D line : b1.getLines()) {
					Point2D p1 = line.getOrigin();
					Point2D p2 = line.getEndPoint();
					if ((p1.getX() == origin.getX() && p1.getY() == origin.getY())
							|| (p2.getX() == origin.getX() && p2.getY() == origin.getY())) { // line
																								// next
																								// to
																								// origin
						Point2D p = GeoUtils.getClosestPointOnSegment(line, end);
						if (p == null) {
							continue;
						}
						double distance = GeometryTools2D.getDistance(end, p);
						if (distance < 2 * agentRadius) {
							return false;
						}
					}
				}

				for (Line2D line : b2.getLines()) {
					Point2D p1 = line.getOrigin();
					Point2D p2 = line.getEndPoint();
					if ((p1.getX() == end.getX() && p1.getY() == end.getY())
							|| (p2.getX() == end.getX() && p2.getY() == end.getY())) { // line
																						// next
																						// to
																						// end
						Point2D p = GeoUtils.getClosestPointOnSegment(line, origin);
						if (p == null) {
							continue;
						}
						double distance = GeometryTools2D.getDistance(origin, p);
						if (distance < 2 * agentRadius) {
							return false;
						}
					}
				}

			}
			blockades.remove(b1);
		}

		return true;
	}

	/**
	 * is passable from point to edge? use expended points
	 * 
	 * @param wroad
	 * @param from
	 * @param to
	 * @param world
	 * @param agentRadius
	 * @return
	 */
	@Deprecated
	public static boolean isPassableP2E_EP(Area area, Point from, Edge to, StandardWorldModel world, int agentRadius) {
		if (area.isBlockadesDefined() && area instanceof Road) {
			Road road = (Road) area;
			WrappedRoad wroad = new WrappedRoad(road, world);
			return isPassableP2E_EPT(wroad, from, to, agentRadius);
		} else {
			return true;
		}

	}

	public static boolean isPassableP2E_EPT(WrappedRoad wroad, Point from, Edge to, int agentRadius) {
		if (wroad.isBlockadesDefined()) {
			List<EntityID> bIds = wroad.getBlockades();
			if (bIds.size() == 0) {
				return true;
			}
			List<Point2D> extendPoints = null;
			List<Blockade> blockades = wroad.getBlockEntities();
			for (Blockade b : blockades) {
				// 判断点是否在b内 最小外接矩形 求交 奇内偶外
				if (b.isApexesDefined()) {
					if (GeoUtils.inPolygon(b.getApexes(), (int) from.x(), (int) from.y())) {
						return false;
					}
				}

				if (BlockJudger.blockP2E(b, from, to, agentRadius)) {
					extendPoints = getExtendPoints(b, EXTEND_DISTANCE);
					break;
				}
			}
			if (extendPoints != null && extendPoints.size() > 0) {
				int startX = (int) from.x();
				int startY = (int) from.y();
				int thirdX = (int) ((to.getStartX() + to.getEndX()) / 2);
				int thirdY = (int) ((to.getStartY() + to.getEndY()) / 2);
				for (Point2D epoint : extendPoints) {
					if (!GeoUtils.inPolygon(wroad.getApexList(), (int) epoint.getX(), (int) epoint.getY())) {
						continue;
					}
					boolean startToSecond = true;
					boolean secondToThird = true;
					for (Blockade b : blockades) {
						if (b.isApexesDefined()) {
							if (GeoUtils.inPolygon(b.getApexes(), (int) epoint.getX(), (int) epoint.getY())) {
								startToSecond = false;
								secondToThird = false;
								break;
							}
						}
						if (isOnSegment(b, (int) epoint.getX(), (int) epoint.getY(), startX, startY)) {
							startToSecond = false;
							break;
						}
						if (isOnSegment(b, (int) epoint.getX(), (int) epoint.getY(), thirdX, thirdY)) {
							secondToThird = false;
							break;
						}

					}
					// System.out.println((int) epoint.getX()+","+(int)
					// epoint.getY());
					// System.out.println("third:\t"+thirdX+","+thirdY);
					// for (Blockade b : blockades) {
					// SenceMaker.printArray(b.getApexes());
					// System.out.println(isOnSegment(b, (int) epoint.getY(),
					// (int) epoint.getY(), thirdX, thirdY));
					// }
					if (startToSecond && secondToThird) {
						// log.warn("2");
						return true;
					}

				}
				return false;
			}
		}
		return true;
	}

	/**
	 * dose the point to edge path be blocked?
	 * 
	 * @param area
	 * @param from
	 * @param to
	 * @return if be blocked returns true
	 */
	public static boolean isPoint2EdgeBlocked(WrappedRoad road, Point from, Edge to, int agentRadius) {
		if (road.isBlockadesDefined()) {
			List<EntityID> bIds = road.getBlockades();
			if (bIds.size() == 0) {
				return false;
			}
			for(Blockade blockade: road.getBlockEntities()){
				// 判断点是否在b内 最小外接矩形 求交 奇内偶外
				if (blockade.isApexesDefined()) {
					if (GeoUtils.inPolygon(blockade.getApexes(), (int) from.x(), (int) from.y())) {
						return true;
					}
				}

				if (BlockJudger.blockP2E(blockade, from, to, agentRadius)) {
					return true;
				}
			}
		}
		return false;
	}
	/*
	public static boolean isPoint2EdgeBlocked(Area area, Point from, Edge to, StandardWorldModel world, int agentRadius) {

		if (area.isBlockadesDefined()) {
			List<EntityID> bIds = area.getBlockades();
			if (bIds.size() == 0) {
				return false;
			}
			for (EntityID next : bIds) {
				Blockade blockade = (Blockade) world.getEntity(next);
				// 判断点是否在b内 最小外接矩形 求交 奇内偶外
				if (blockade.isApexesDefined()) {
					if (GeoUtils.inPolygon(blockade.getApexes(), (int) from.x(), (int) from.y())) {
						return true;
					}
				}

				if (BlockJudger.blockP2E(blockade, from, to, agentRadius)) {
					return true;
				}
			}
		}
		return false;
	}
	*/
	public static boolean isOnSegment(Blockade b, int x1, int y1, int x2, int y2) {
		Segment s1 = new Segment(x1, y1, x2, y2);
		if (!b.isApexesDefined()) {
			return false;
		}

		List<Segment> ss = Converter.generateSegments(b.getApexes());

		for (Segment segment : ss) {
			if (GeoUtils.isSegmentIntersect(s1, segment)) {
				return true;
			}
		}
		return false;
	}
	
	// -----------------------------------------------------------

	private static List<Point2D> getExtendPoints(Blockade blocade, int expDistance) {
		List<Point2D> ret = new ArrayList<Point2D>();
		int x = blocade.getX();
		int y = blocade.getY();
		WrappedBlockade wb = new WrappedBlockade(blocade);
		List<Point2D> points = wb.getPoints();
		for (Point2D point : points) {
			double divx = point.getX() - x;
			double divy = point.getY() - y;
			boolean isXnearlyZero = GeometryTools2D.nearlyZero(divx);
			boolean isYnearlyZero = GeometryTools2D.nearlyZero(divy);
			if (isXnearlyZero && isYnearlyZero) {
				continue;
			} else if (isXnearlyZero) {
				ret.add(new Point2D(point.getX(), point.getY() + expDistance));
			} else if (isYnearlyZero) {
				ret.add(new Point2D(point.getX() + expDistance, point.getY()));
			} else {
				double incX = (divx / Math.sqrt(divx * divx + divy * divy)) * expDistance;
				double incY = (divy / Math.sqrt(divx * divx + divy * divy)) * expDistance;
				ret.add(new Point2D(point.getX() + incX, point.getY() + incY));
			}
		}
		return ret;
	}
}
