package ant.utils.blockade;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import ant.AgentContext;
import ant.ConfigKey;
import ant.component.universal.ChangeSetDetailComponent;
import ant.component.universal.RoadInfoCacheComponent;
import ant.component.universal.TrafficStepSimComponent;
import ant.utils.geo.Converter;
import ant.utils.geo.GeoUtils;
import ant.utils.geo.Point;
import ant.utils.geo.Polygon;
import ant.utils.geo.acc.VGraph;
import ant.utils.traffic.TrafficMove;
import ant.utils.traffic.TrafficStepSimulator.TrafficSimRes;

/**
 * 
 * @author Fun, Ivan
 * 
 */
public class RoadUtils {
	private static Log log = LogFactory.getLog(RoadUtils.class);
	private static final int SIM_MOVE_ACCEPT_LEN = 1500;

	public static final int RESCUE_AGENT_RADIUS = 500;

	public static boolean isPassable(RoadInfoCacheComponent ricc, Edge from,
			Area through, Edge to, PassableJudgeType type) {
		WrappedRoad wroad = ricc.getWrappedRoad(through.getID());
		if (wroad == null) {
			log.error("Area is not a Road!");
			return true;
		}

		if (type.equals(PassableJudgeType.VGraph)) {
			Polygon boundary = new Polygon(through.getApexList());
			List<Polygon> blockades = new ArrayList<Polygon>();
			if (through.isBlockadesDefined()) {
				for (Blockade blockade : wroad.getBlockEntities()) {
					blockades.add(new Polygon(blockade.getApexes()));
				}
			}
			return VGraph.isAccessible(Converter.generateSegment(from),
					Converter.generateSegment(to), blockades, boundary,
					RESCUE_AGENT_RADIUS);
		} else {
			return AccJudger
					.isPassableE2E(from, wroad, to, RESCUE_AGENT_RADIUS);
		}
	}

	public static boolean isPassable(RoadInfoCacheComponent ricc, Area area,
			Point fromPoint, Edge toEdge, PassableJudgeType type) {
		WrappedRoad wroad = ricc.getWrappedRoad(area.getID());
		if (wroad == null) {
			log.error("Area is not a Road!");
			return true;
		}

		if (type.equals(PassableJudgeType.VGraph)) {
			Polygon boundary = new Polygon(area.getApexList());
			List<Polygon> blockades = new ArrayList<Polygon>();
			if (area.isBlockadesDefined()) {
				for (Blockade blockade : wroad.getBlockEntities()) {
					blockades.add(new Polygon(blockade.getApexes()));
				}
			}
			return VGraph.isAccessible(fromPoint,
					new Point((toEdge.getStartX() + toEdge.getEndX()) / 2,
							(toEdge.getStartY() + toEdge.getEndY()) / 2),
					blockades, boundary, RESCUE_AGENT_RADIUS);
		} else {
			return AccJudger.isPassableP2E_EPT(wroad, fromPoint, toEdge,
					RESCUE_AGENT_RADIUS);
		}
	}

	// 超时
	@Deprecated
	public static boolean isPassable(AgentContext context, Area area,
			Point from, EntityID toNeighbour) {
		Edge to = area.getEdgeTo(toNeighbour);
		if (to == null) {
			return false;
		}
		if (AccJudger.isPassableP2E_EP(area, from, to, context.getWorld(),
				RESCUE_AGENT_RADIUS)) {
			TrafficStepSimComponent sim = (TrafficStepSimComponent) context
					.getAntComponentKit().getAntComponent(
							TrafficStepSimComponent.class);
			List<EntityID> path = new ArrayList<EntityID>();
			path.add(toNeighbour);
			// AKMove move = new AKMove(context.getAgentID(),
			// context.getTimestamp(), path);
			TrafficMove move = new TrafficMove(path, from.getX(), from.getY());
			sim.simulate(move);
			TrafficSimRes result = sim.getResult();
			if (result != null) {
				if (result.getLocation().equals(toNeighbour)) {
					return true;
				} else {
					double x = result.getX();
					double y = result.getY();
					if (Math.hypot(x - from.getX(), y - from.getY()) > SIM_MOVE_ACCEPT_LEN) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * get nearest blockade in area for public
	 * 
	 * @param context
	 * @param area
	 * @return
	 */
	public static EntityID getNearestBlockadeInArea(StandardWorldModel world,
			int x, int y, Area area) {
		EntityID res = null;
		int dis = Integer.MAX_VALUE;
		if (area instanceof Road) {
			if (area.isBlockadesDefined()) {
				List<EntityID> blockades = area.getBlockades();

				for (int i = 0; i < blockades.size(); i++) {
					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							((Blockade) world.getEntity(blockades.get(i)))
									.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
					if (dis > tmp) {
						dis = tmp;
						res = blockades.get(i);
					}
				}

			}
		}

		return res;
	}

	// for clean
	// public static EntityID getBlockade(AgentContext context) {
	// int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());
	// Area here = (Area) context.getWorld().getEntity(
	// context.getHuman().getPosition());
	// EntityID res = null;
	// int x = context.getHuman().getX();
	// int y = context.getHuman().getY();
	//
	// int dis = Integer.MAX_VALUE;
	//
	// if (here instanceof Road) {
	// if (here.isBlockadesDefined()) {
	// List<EntityID> blockades = here.getBlockades();
	//
	// for (int i = 0; i < blockades.size(); i++) {
	// Point p = Polygon.getNearestEdgeOfPolygon(
	// x,
	// y,
	// ((Blockade) context.getWorld().getEntity(
	// blockades.get(i))).getApexes());
	// int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
	// if (dis > tmp) {
	// dis = tmp;
	// res = blockades.get(i);
	// }
	// }
	//
	// }
	// }
	//
	// if (dis < maxdist) {
	// return res;
	// }
	//
	// dis = Integer.MAX_VALUE;
	// for (EntityID areaid : here.getNeighbours()) {
	// Area neighbour = (Area) context.getWorld().getEntity(areaid);
	// if (neighbour.isBlockadesDefined()) {
	// List<EntityID> ids = neighbour.getBlockades();
	// for (EntityID bid : ids) {
	// Blockade bb = (Blockade) context.getWorld().getEntity(bid);
	// Point p = Polygon.getNearestEdgeOfPolygon(x, y,
	// bb.getApexes());
	// int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
	// if (dis > tmp) {
	// dis = tmp;
	// res = bb.getID();
	// }
	// }
	// }
	// }
	//
	// if (dis < maxdist) {
	// return res;
	// } else {
	// return null;
	// }
	// }

	private static Set<WrappedRoad> getWRoadsInSight(AgentContext context) {
		ChangeSetDetailComponent csdc = context.getAntComponentKit()
				.getChangeSetDetailComponent();
		RoadInfoCacheComponent rcc = context.getAntComponentKit()
				.getRoadInfoCacheComponent();
		Set<WrappedRoad> roadsInSight = new HashSet<WrappedRoad>();
		for (StandardEntity se : csdc
				.getEntitiesInSight(StandardEntityURN.ROAD)) {
			
			WrappedRoad temp = rcc.getWrappedRoad(se.getID());
			roadsInSight.add(temp);
			
		}
		
		return roadsInSight;
	}

	public static Set<EntityID> getRoadIDsInSight(AgentContext context) {
		ChangeSetDetailComponent csdc = context.getAntComponentKit()
				.getChangeSetDetailComponent();
		Set<EntityID> roadsInSight = new HashSet<EntityID>();
		for (StandardEntity se : csdc
				.getEntitiesInSight(StandardEntityURN.ROAD)) {
			roadsInSight.add(se.getID());
		}
		return roadsInSight;
	}

	private static List<StandardEntity> getRoadsInSight(AgentContext context) {
		ChangeSetDetailComponent csdc = context.getAntComponentKit()
				.getChangeSetDetailComponent();
		return csdc.getEntitiesInSight(StandardEntityURN.ROAD);
	}

	public static EntityID getBlockadeInSight(AgentContext context) {
		int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());

		Area here = (Area) context.getWorld().getEntity(
				context.getHuman().getPosition());

		List<StandardEntity> roadsInSight = getRoadsInSight(context);

		EntityID res = null;
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();

		int dis = Integer.MAX_VALUE;

		if (here instanceof Road) {
			if (here.isBlockadesDefined()) {
				List<EntityID> blockades = here.getBlockades();

				for (int i = 0; i < blockades.size(); i++) {
					Point p = Polygon.getNearestEdgeOfPolygon(
							x,
							y,
							((Blockade) context.getWorld().getEntity(
									blockades.get(i))).getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
					if (dis > tmp) {
						dis = tmp;
						res = blockades.get(i);
					}
				}

			}
		}

		if (dis < maxdist) {
			return res;
		}

		dis = Integer.MAX_VALUE;

		for (StandardEntity se : roadsInSight) {
			Area neighbour = (Area) se;
			if (neighbour.isBlockadesDefined()) {
				List<EntityID> ids = neighbour.getBlockades();
				for (EntityID bid : ids) {
					Blockade bb = (Blockade) context.getWorld().getEntity(bid);
					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							bb.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
					if (dis > tmp) {
						dis = tmp;
						res = bb.getID();
					}
				}
			}
		}

		if (dis < maxdist) {
			return res;
		} else {
			return null;
		}
	}

	public static EntityID getBlockadeInArea(AgentContext context,
			CleanType ctype) {
		int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());
		Area here = (Area) context.getWorld().getEntity(
				context.getHuman().getPosition());
		EntityID ret = null;
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();

		if (ctype.equals(CleanType.Partial)) {
			int dis = Integer.MAX_VALUE;
			if (here.isBlockadesDefined()) {
				List<EntityID> blockades = new ArrayList<EntityID>();
				for (EntityID id : here.getBlockades()) {
					blockades.add(id);
				}

				for (int i = 0; i < blockades.size(); i++) {

					Blockade blockade = (Blockade) context.getWorld()
							.getEntity(blockades.get(i));
					if (BlockJudger.blockArea(here, blockade,
							RESCUE_AGENT_RADIUS) == false) {
						continue;
					}
					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
					if (dis > tmp) {
						dis = tmp;
						ret = blockades.get(i);
					}
				}

				if (dis < maxdist)
					return ret;
			}

			if (dis < maxdist)
				return ret;
			else
				return null;
		} else if (ctype.equals(CleanType.Most)) {
			// TODO
			log.error("TODO getBlockadeInArea CleanType.Most");
		} else {
			int dis = Integer.MAX_VALUE;
			if (here instanceof Road) {
				if (here.isBlockadesDefined()) {
					List<EntityID> blockades = new ArrayList<EntityID>();
					for (EntityID id : here.getBlockades()) {
						blockades.add(id);
					}

					for (int i = 0; i < blockades.size(); i++) {
						Point p = Polygon.getNearestEdgeOfPolygon(
								x,
								y,
								((Blockade) context.getWorld().getEntity(
										blockades.get(i))).getApexes());
						int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
						if (dis > tmp) {
							dis = tmp;
							ret = blockades.get(i);
						}
					}

					if (dis < maxdist) {
						return ret;
					}
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * 
	 * @param context
	 * @param ctype
	 * @param path
	 * @return null when no blockade to clean
	 */
	public static EntityID getBlockadeOnPath(AgentContext context,
			CleanType ctype, List<EntityID> path) {
		
		if(context.getTimestamp()<6){
			return RoadUtils.getBlockadeInSight(context);
		}
		// 模拟
		TrafficStepSimComponent tssc = (TrafficStepSimComponent) context
				.getAntComponentKit().getAntComponent(
						TrafficStepSimComponent.class);
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();
		TrafficMove move = new TrafficMove(path, x, y);
		
		long _time_st = System.currentTimeMillis();
		tssc.simulate(move);
		long _time_ed = System.currentTimeMillis();
		if (_time_ed - _time_st > 75) {
			log.info("RoadUtils.getBlockadeOnPath Simulate cost: " + (_time_ed - _time_st) + " milliseconds");
		}
		
		TrafficSimRes res = tssc.getResult();

		if (ctype.equals(CleanType.Partial)) {
			boolean canMove = canMove(res, context.getHuman(), path);
			
			EntityID ret = getBlockadeOnPathPartialInSight(context, res, path,
					canMove);
			
			// log.info("CanMove:" + canMove);
			if(!canMove && ret == null) {
				// log.info("This Case");
				ret = RoadUtils.getBlockadeInRoadNearest(context, path); 
			}
			
			return ret;

		} else if (ctype.equals(CleanType.Medium)) {
			return getBlockadeOnPathPartial(context, path, true);
		} else if (ctype.equals(CleanType.Most)) {
			// TODO
		} else {
			EntityID ret = RoadUtils.getBlockadeInSight(context);
			return ret;
		}
		return null;
	}

	private static boolean canMove(TrafficSimRes res, Human human,
			List<EntityID> path) {
		int x = human.getX();
		int y = human.getY();

		if (!res.getLocation().equals(human.getPosition())) {
			if (path.contains(res.getLocation())) {
				if (GeoUtils.distSquareL(new Point(x, y),
						new Point((int) res.getX(), (int) res.getY())) > 250000) {
					return true;
				}
			}
		} else {
			if (GeoUtils.distSquareL(new Point(x, y),
					new Point((int) res.getX(), (int) res.getY())) > 25000000) {
				return true;
			}
		}
		return false;
	}

	// TODO
	private static EntityID getBlockadeOnPathPartial(AgentContext context,
			List<EntityID> path, boolean chere2center) {
		int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());
		Area here = (Area) context.getWorld().getEntity(
				context.getHuman().getPosition());
		EntityID ret = null;
		StandardWorldModel world = context.getWorld();
		RoadInfoCacheComponent ricc = context.getAntComponentKit()
				.getRoadInfoCacheComponent();
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();

		int dis = Integer.MAX_VALUE;

		// from here to next area
		List<EntityID> blockades = new ArrayList<EntityID>();
		if (here.isBlockadesDefined()) {
			for (EntityID id : here.getBlockades()) {
				blockades.add(id);
			}
		}

		for (int i = 0; i < blockades.size(); i++) {
			Blockade blockade = (Blockade) world.getEntity(blockades.get(i));
			if (!BlockJudger.blockAreaP2E(here, blockade, new Point(x, y),
					here.getEdgeTo(path.get(0)),
					(int) (RESCUE_AGENT_RADIUS * 1.25))) {

				continue;
			}
			Point p = Polygon.getNearestEdgeOfPolygon(x, y,
					blockade.getApexes());
			int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

			if (dis > tmp) {
				dis = tmp;
				ret = blockades.get(i);
			}
		}

		if (dis < maxdist) {
			return ret;
		}

		// FIXME
		if (chere2center) {
			Area area = (Area) world.getEntity(path.get(0));
			List<EntityID> nbs = new ArrayList<EntityID>();
			if (area.isBlockadesDefined()) {
				for (EntityID id : area.getBlockades()) {
					nbs.add(id);
				}
			}
			for (int i = 0; i < nbs.size(); i++) {
				Blockade blockade = (Blockade) world.getEntity(nbs.get(i));

				if (!BlockJudger.isOnP2PSegment(blockade, x, y, area.getX(),
						area.getY())) {
					continue;
				}
				Point p = Polygon.getNearestEdgeOfPolygon(x, y,
						blockade.getApexes());
				int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
				if (dis > tmp) {
					dis = tmp;
					ret = nbs.get(i);
				}
			}
			if (dis < maxdist) {
				return ret;
			}
		}

		// test
		if (ret == null) {
			if (!AccJudger.isPassableP2E_EP(here, new Point(x, y),
					here.getEdgeTo(path.get(0)), world,
					(int) (RESCUE_AGENT_RADIUS * 1.25))) {
				for (EntityID next : blockades) {
					Blockade blockade = (Blockade) world.getEntity(next);
					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

					if (dis > tmp) {
						dis = tmp;
						ret = next;
					}
				}
				if (dis < maxdist) {
					return ret;
				}
			}
		}

		// on path area
		dis = Integer.MAX_VALUE;
		EntityID fromArea, toArea;
		// TODO reduce checked size
		for (int i = 0; i < path.size(); i++) {
			Area area = (Area) world.getEntity(path.get(i));
			if (!area.isBlockadesDefined()) {
				continue;
			}

			if (i == 0) {
				fromArea = here.getID();
			} else {
				fromArea = path.get(i - 1);
			}
			if (i < path.size() - 1) {
				toArea = path.get(i + 1);
			} else {
				toArea = null;
			}

			List<EntityID> ids = area.getBlockades();

			if (ids.size() <= 0) {
				continue;
			}

			if (toArea != null) {

				boolean isEveryoneNoblocked = true;
				for (EntityID next : ids) {

					Blockade blockade = (Blockade) world.getEntity(next);
					if (!BlockJudger.blockAreaE2E(area, blockade,
							area.getEdgeTo(fromArea), area.getEdgeTo(toArea),
							(int) (RESCUE_AGENT_RADIUS * 1.25))) {
						continue;
					}

					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

					if (dis > tmp) {
						dis = tmp;
						ret = next;
					}

					isEveryoneNoblocked = false;
				}
				if (isEveryoneNoblocked) {
					if (i < 2) {
						boolean isPassable = true;
						WrappedRoad wroad = ricc.getWrappedRoad(area.getID());
						if (wroad != null) {
							isPassable = AccJudger.isPassableE2E(
									area.getEdgeTo(fromArea), wroad,
									area.getEdgeTo(toArea),
									(int) (RESCUE_AGENT_RADIUS * 1.25));
						}

						if (!isPassable) {
							Blockade blockade = (Blockade) world.getEntity(ids
									.get(0));
							Point p = Polygon.getNearestEdgeOfPolygon(x, y,
									blockade.getApexes());
							int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

							if (dis > tmp) {
								dis = tmp;
								ret = ids.get(0);
							}
						}
					}
				}
			} else {
				// last
				for (EntityID next : ids) {
					Blockade blockade = (Blockade) world.getEntity(next);

					if (!BlockJudger.blockArea(area, blockade,
							RESCUE_AGENT_RADIUS)) {
						continue;
					}

					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

					if (dis > tmp) {
						dis = tmp;
						ret = next;
					}
				}
			}
		}

		if (dis < maxdist) {
			return ret;
		} else {
			return null;
		}
	}

	// ----------------------------------------------------------
	private static EntityID getBlockadeOnPathPartialInSight(
			AgentContext context, TrafficSimRes res, List<EntityID> path,
			boolean canMove) {
		int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());

		Area here = (Area) context.getWorld().getEntity(
				context.getHuman().getPosition());

		EntityID ret = null;
		StandardWorldModel world = context.getWorld();
		RoadInfoCacheComponent ricc = context.getAntComponentKit()
				.getRoadInfoCacheComponent();
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();

		//Set<WrappedRoad> roadsInSight = getWRoadsInSight(context);		
		Set<EntityID> roadIDsInSight = getRoadIDsInSight(context);
		
		if (!canMove) {
			ret = getBlockadeInRoadNearSimuResInSight(world, ricc, res, here.getID(),
					x, y, path, roadIDsInSight, maxdist);
			if (ret != null) {
				// log.info("ReturnThis");
				return ret;
			}
		}		
		
		int dis = Integer.MAX_VALUE;

		EntityID fromArea, toArea;

		for (int i = 0; i < path.size() && i < 5; i++) {
			if (!roadIDsInSight.contains(path.get(i))) {
				continue;
			}
			Area area = (Area) world.getEntity(path.get(i));
			if (!area.isBlockadesDefined()) {
				continue;
			}

			if (i == 0) {
				fromArea = here.getID();
			} else {
				fromArea = path.get(i - 1);
			}
			if (i < path.size() - 1) {
				toArea = path.get(i + 1);
			} else {
				toArea = null;
			}

			List<EntityID> ids = area.getBlockades();

			if (ids.size() <= 0) {
				continue;
			}

			if (toArea != null) {

				boolean isEveryoneNoblocked = true;
				Edge fromEdge = area.getEdgeTo(fromArea);
				Edge toEdge = area.getEdgeTo(toArea);
				for (EntityID next : ids) {

					Blockade blockade = (Blockade) world.getEntity(next);
					if(i==0){ // 下一块
						if (!BlockJudger.blockAreaE2E(area, blockade,
								area.getEdgeTo(fromArea), area.getEdgeTo(toArea),
								(int) (RESCUE_AGENT_RADIUS * 1.25))) {
							continue;
						}
					}else{ 
						if (!isOnSegment(blockade,
								(fromEdge.getStartX() + fromEdge.getEndX()) / 2,
								(fromEdge.getStartY() + fromEdge.getEndY()) / 2,
								(toEdge.getStartX() + toEdge.getEndX()) / 2,
								(toEdge.getStartY() + toEdge.getEndY()) / 2)) {
							continue;
						}
					}
					
					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

					if (dis > tmp) {
						dis = tmp;
						ret = next;
					}

					isEveryoneNoblocked = false;
				}
				if (isEveryoneNoblocked) {
					if (i < 2) {
						boolean isPassable = true;
						WrappedRoad wroad = ricc.getWrappedRoad(area.getID());
						if (wroad != null) {
							isPassable = AccJudger.isPassableE2E(
									area.getEdgeTo(fromArea), wroad,
									area.getEdgeTo(toArea),
									(int) (RESCUE_AGENT_RADIUS * 1.25));
						}

						if (!isPassable) {
							Blockade blockade = (Blockade) world.getEntity(ids
									.get(0));
							Point p = Polygon.getNearestEdgeOfPolygon(x, y,
									blockade.getApexes());
							int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

							if (dis > tmp) {
								dis = tmp;
								ret = ids.get(0);
							}
						}
					}
				}
			} else {
				// last
				for (EntityID next : ids) {
					Blockade blockade = (Blockade) world.getEntity(next);

					if (!BlockJudger.blockArea(area, blockade,
							RESCUE_AGENT_RADIUS)) {
						continue;
					}

					Point p = Polygon.getNearestEdgeOfPolygon(x, y,
							blockade.getApexes());
					int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());

					if (dis > tmp) {
						dis = tmp;
						ret = next;
					}
				}
			}
		}

		if (dis < maxdist) {
			return ret;
		}
		
		
		
		return null;
	}
	
	
	/** SimFailed, clean nearest **/
	private static EntityID getBlockadeInRoadNearest(AgentContext context, List<EntityID> path) {
		EntityID ret = null;
		int dis = Integer.MAX_VALUE;
		int maxdist = ConfigKey.getClearRepairDistance(context.getConfig());
		EntityID here = context.getHuman().getPosition();
		int x = context.getHuman().getX();
		int y = context.getHuman().getY();
		Set<WrappedRoad> roadsInSight = getWRoadsInSight(context);	
		
		int tarx, tary;
		Area loc = (Area) context.getHuman().getPosition(context.getWorld());
		Edge taredge = loc.getEdgeTo(path.get(0));
		if (taredge == null) {
			tarx = x;
			tary = y;
		} else {
			tarx = (taredge.getStartX() + taredge.getEndX()) / 2;
			tary = (taredge.getStartY() + taredge.getEndY()) / 2;
		}
		
		for (WrappedRoad road : roadsInSight) {
			// log.info(road);
			Road r = (Road) context.getWorld().getEntity(road.getID());
			// log.info(r.getBlockades());
//			if (!road.isBlockadesDefined()) {
//				continue;
//			}
	
			if (!(here.equals(road.getID()) || path.contains(road.getID()))) {
				continue;
			}
			
//			if (!(here.equals(road.getID()))) {
//				continue;
//			}
				
			for (Blockade blockade : road.getBlockEntities()) {
				
//				Point p = Polygon.getNearestEdgeOfPolygon(x, y,
//						blockade.getApexes());
//				
//				int tmp = (int) GeoUtils.dist(x, y, p.x(), p.y());
//				
//				if (tmp >= maxdist)
//					continue;
				// log.info(blockade.getID());
				
				if (!isInClearRange(x, y, blockade, maxdist)) 
					continue;
				
				Point p1 = Polygon.getNearestEdgeOfPolygon(x, y, blockade.getApexes());
				Point p2 = Polygon.getNearestEdgeOfPolygon(tarx, tary, blockade.getApexes());
				int tmp = (int) (GeoUtils.dist(x, y, p1.x(), p1.y()) + GeoUtils.dist(tarx, tary, p2.x(), p2.y()));
				
				if (dis > tmp) {
					dis = tmp;
					ret = blockade.getID();
				}
			}
		}

		return ret;
	}

	private static EntityID getBlockadeInRoadNearSimuResInSight(
			StandardWorldModel world, RoadInfoCacheComponent ricc,
			TrafficSimRes res, EntityID here, int x, int y,
			List<EntityID> path, Set<EntityID> roadIDsInSight, int maxdist) {

		int dis = Integer.MAX_VALUE;
		int resX = (int) res.getX();
		int resY = (int) res.getY();
		EntityID resPosition = res.getLocation();
		EntityID ret = null;
		int bX = Integer.MAX_VALUE;
		int bY = Integer.MAX_VALUE;

		Area p1 = null;
		Area p2 = null;
		if (resPosition.equals(here)) { // 卡在原Road
			p1 = (Area) world.getEntity(here);
			if (path.size() > 0) {
				if(roadIDsInSight.contains(path.get(0))){
					p2 = (Area) world.getEntity(path.get(0));
				}
			}
		} else {
			for (int i = 0; i < path.size() && i < 10; i++) {
				EntityID tmp = path.get(i);
				if(!roadIDsInSight.contains(tmp)){
					continue;
				}
				if (tmp.equals(resPosition) && (i + 1) < path.size()) {
					p1 = (Area) world.getEntity(tmp);
					p2 = (Area) world.getEntity(path.get(i + 1));
					break;
				}
			}
		}

		if (p1 != null && p2 != null) {
			Edge edge = p1.getEdgeTo(p2.getID());
			if (p1.isBlockadesDefined()) {
				Point start = new Point(x, y);
				WrappedRoad wroad1 = ricc.getWrappedRoad(p1.getID());
				for (Blockade blockade : wroad1.getBlockEntities()) {
					if (BlockJudger.blockAreaP2E(p1, blockade, start, edge,
							(int) (RESCUE_AGENT_RADIUS * 1.25))) {
						Point p = Polygon.getNearestEdgeOfPolygon(resX, resY,
								blockade.getApexes());
						int tmp = (int) GeoUtils.dist(resX, resY, p.x(), p.y());
						if (dis > tmp) {
							dis = tmp;
							bX = p.x();
							bY = p.y();
							ret = blockade.getID();
						}
					}
				}
			}
			if (ret == null && p2.isBlockadesDefined()) {
				WrappedRoad wroad2 = ricc.getWrappedRoad(p2.getID());
				Point start = new Point(wroad2.getX(), wroad2.getY());
				for (Blockade blockade : wroad2.getBlockEntities()) {
					if (BlockJudger.blockAreaP2E(p2, blockade, start, edge,
							(int) (RESCUE_AGENT_RADIUS * 1.25))) {
						Point p = Polygon.getNearestEdgeOfPolygon(resX, resY,
								blockade.getApexes());
						int tmp = (int) GeoUtils.dist(resX, resY, p.x(), p.y());
						if (dis > tmp) {
							dis = tmp;
							bX = p.x();
							bY = p.y();
							ret = blockade.getID();
						}
					}
				}
			}

			if (ret != null) {
				dis = (int) GeoUtils.dist(x, y, bX, bY);
				if (dis < maxdist) {

					return ret;
				}
				
				if(!resPosition.equals(here)){
					return null;
				}
				dis = bX = bY = Integer.MAX_VALUE;
				ret = null;
			}
		}

		return null;
	}

	public static boolean isOnSegment(Blockade b, int x1, int y1, int x2, int y2) {
		return AccJudger.isOnSegment(b, x1, y1, x2, y2);
	}
	
	public static boolean isInClearRange(int pfx, int pfy, Blockade b, int cleanRange) {
		Point2D agentLocation = new Point2D(pfx, pfy);
        double bestDistance = Double.MAX_VALUE;
        for (Line2D line : GeometryTools2D.pointsToLines(GeometryTools2D.vertexArrayToPoints(b.getApexes()), true)) {
            Point2D closest = GeometryTools2D.getClosestPointOnSegment(line, agentLocation);
            double distance = GeometryTools2D.getDistance(agentLocation, closest);
            if (distance < cleanRange) {
                return true;
            }
            if (bestDistance > distance) {
                bestDistance = distance;
            }
        }
        return false;
	}
}
