package ant.utils.blockade;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;

/**
 * 
 * @author Fun
 * 
 */
public class SenceMaker {
	public static String printBlockade(Blockade blockade) {
		StringBuffer sb = new StringBuffer();
		for (int i : blockade.getApexes()) {
			sb.append(i + ",");
		}
		String desc = "" + blockade.getID() + "," + blockade.getPosition() + "#" + blockade.getX() + "," + blockade.getY()
				+ "#" + sb.toString() + "#" + blockade.getRepairCost();

		return desc;
	}

	public static void printArea(Area area, StandardWorldModel world) {
		StringBuffer sb = new StringBuffer();
		sb.append(area.getID() + "$");
		sb.append(area.getX() + "," + area.getY() + "$");
		if (area.isEdgesDefined()) {
			for (Edge next : area.getEdges()) {
				if (next.getNeighbour() == null) {
					sb.append(next.getStartX() + "," + next.getStartY() + "," + next.getEndX() + "," + next.getEndY()
							+ ",-1,");
				} else {
					sb.append(next.getStartX() + "," + next.getStartY() + "," + next.getEndX() + "," + next.getEndY() + ","
							+ next.getNeighbour() + ",");
				}
			}
		}
		sb.append("$");

		if (area.isBlockadesDefined()) {
			for (EntityID next : area.getBlockades()) {
				Blockade blockade = (Blockade) world.getEntity(next);
				sb.append(printBlockade(blockade) + "$");
			}
		}

		System.out.println(sb.toString());
	}

	public static void printWrappedRoad(WrappedRoad area) {
		StringBuffer sb = new StringBuffer();
		sb.append(area.getID() + "$");
		sb.append(area.getX() + "," + area.getY() + "$");
		if (area.isEdgesDefined()) {
			for (Edge next : area.getEdges()) {
				if (next.getNeighbour() == null) {
					sb.append(next.getStartX() + "," + next.getStartY() + "," + next.getEndX() + "," + next.getEndY()
							+ ",-1,");
				} else {
					sb.append(next.getStartX() + "," + next.getStartY() + "," + next.getEndX() + "," + next.getEndY() + ","
							+ next.getNeighbour() + ",");
				}
			}

		}
		sb.append("$");

		if (area.isBlockadesDefined()) {
			for (Blockade next : area.getBlockEntities()) {
				sb.append(printBlockade(next) + "$");
			}
		}

		System.out.println(sb.toString());
	}

	public static WrappedRoad makeWrappeRoad(String desc) {
		String[] a0 = desc.split("\\$");
		int id = Integer.parseInt(a0[0]);

		String[] a1 = a0[1].split(",");
		int x = Integer.parseInt(a1[0]);
		int y = Integer.parseInt(a1[1]);

		String[] a2 = a0[2].split(",");
		List<Edge> edges = new ArrayList<Edge>();
		for (int i = 0; i + 4 < a2.length; i += 5) {
			int sx = Integer.parseInt(a2[i]);
			int sy = Integer.parseInt(a2[i + 1]);
			int ex = Integer.parseInt(a2[i + 2]);
			int ey = Integer.parseInt(a2[i + 3]);
			int nid = Integer.parseInt(a2[i + 4]);
			if (nid < 0) {
				edges.add(new Edge(sx, sy, ex, ey));
			} else {
				edges.add(new Edge(sx, sy, ex, ey, new EntityID(nid)));
			}

		}
		List<EntityID> blockades = new ArrayList<EntityID>();
		List<Blockade> Eblockades = new ArrayList<Blockade>();
		for (int i = 3; i < a0.length; i++) {
			Blockade blockade = makeBlockde(a0[i]);
			Eblockades.add(blockade);
			blockades.add(blockade.getID());
		}

		WrappedRoad road = new WrappedRoad(new EntityID(id), Eblockades);
		road.setX(x);
		road.setY(y);
		road.setEdges(edges);
		road.setBlockades(blockades);
		return road;
	}

	public static Blockade makeBlockde(String desc) {
		String[] array = desc.split("#");

		String[] a1 = array[0].split(",");
		int id = Integer.parseInt(a1[0]);
		int position = Integer.parseInt(a1[1]);

		String[] a2 = array[1].split(",");
		int x = Integer.parseInt(a2[0]);
		int y = Integer.parseInt(a2[1]);

		String[] a3 = array[2].split(",");
		int[] apexes = new int[a3.length];
		for (int i = 0; i < a3.length; i++) {
			apexes[i] = Integer.parseInt(a3[i]);
		}

		int cost = Integer.parseInt(array[3]);

		Blockade ret = new Blockade(new EntityID(id));
		ret.setApexes(apexes);
		ret.setPosition(new EntityID(position));
		ret.setRepairCost(cost);
		ret.setX(x);
		ret.setY(y);

		return ret;
	}

	public static int[] scale(double scale, int[] array) {
		int[] ret = new int[array.length];
		for (int i = 0; i < array.length; i++) {
			ret[i] = (int) (array[i] * scale);
		}
		return ret;
	}

	public static int minx(int[] array, double scale) {
		int ret = Integer.MAX_VALUE;
		for (int i = 0; i < array.length - 1; i += 2) {
			if (ret > array[i]) {
				ret = array[i];
			}
		}
		return (int) (ret * scale);
	}

	public static int miny(int[] array, double scale) {
		int ret = Integer.MAX_VALUE;
		for (int i = 0; i < array.length - 1; i += 2) {
			if (ret > array[i + 1]) {
				ret = array[i + 1];
			}
		}
		return (int) (ret * scale);
	}

	public static void printArray(int[] array) {
		System.out.print("[");
		for (int i : array) {
			System.out.print(i + ",");
		}
		System.out.print("]");
	}

//	public static void main(String[] args) {
//
//		String tmp1 = "971$89544,19925$93000,23160,72000,23160,-1,72000,23160,70000,19000,970,70000,19000,70000,16840,-1,70000,16840,110000,16840,-1,110000,16840,106840,23160,279,106840,23160,98000,23160,-1,98000,23160,93000,23160,899,$1033,971#72487,19400#70000,16840,70000,19000,72000,23160,72405,23160,73385,22409,74142,21434,74631,20282,74804,19000,74804,16840,#23$1034,971#78589,22795#78590,22216,77034,22465,75681,23160,81498,23160,80145,22465,#3$";
//
//		WrappedRoad road = makeWrappeRoad(tmp1);
//
//		// int x = 73783;
//		// int y = 22710;
//
//		int x = 73100;
//		int y = 23159;
//		Blockade blockade = null;
//
//		for (Blockade next : road.getBlockEntities()) {
//			if (next.getID().getValue() == 1033) {
//				blockade = next;
//				break;
//			}
//		}
//
//		System.out.println(printBlockade(blockade));
//		System.out.println(BlockJudger.blockAreaP2E(road, blockade, new Point(x, y), road.getEdgeTo(new EntityID(970)),
//				(int) (RoadUtils.RESCUE_AGENT_RADIUS * 1.25)));
//
//		GeoFrame frame = new GeoFrame("Main");
//		frame.setSize(800, 800);
//		frame.addSegment(new Segment(705, 200, 733, 217));
//		frame.addSegment(new Segment(715, 221, 743, 237));
//
//		frame.addSegment(new Segment(700, 168, 700, 190));
//		frame.addSegment(new Segment(700, 190, 720, 231));
//		frame.addSegment(new Segment(720, 231, 724, 231));
//		frame.addSegment(new Segment(724, 231, 733, 224));
//		frame.addSegment(new Segment(733, 224, 741, 214));
//		frame.addSegment(new Segment(741, 214, 746, 202));
//		frame.addSegment(new Segment(746, 202, 748, 190));
//		frame.addSegment(new Segment(748, 190, 748, 168));
//		frame.addSegment(new Segment(748, 168, 700, 168));
//
//		frame.setStartPoint(700, 150);
//	}

}
