package ant.utils.blockade;

import java.util.List;

import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.worldmodel.EntityID;
import ant.utils.geo.Converter;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Fun
 * 
 */
public class WrappedBlockade extends Blockade {

	private List<Point2D> points;
	private List<Line2D> lines;

	public WrappedBlockade(EntityID id) {
		super(id);
	}

	public WrappedBlockade(Blockade other) {
		super(other);
	}

	public List<Point2D> getPoints() {
		if (points == null) {
			points = Converter.generatePoint2Ds(this.getApexes());
		}
		return this.points;
	}

	public List<Line2D> getLines() {
		if (lines == null) {
			lines = Converter.generateLine2Ds(this.getApexes());
		}
		return lines;
	}

	/**
	 * 
	 * @param edge
	 * @return the closest line Segment from Blockade to edge. Line's origin is
	 *         on Blockade and its end is on edge.
	 */
	public Line2D getShortestSegmetToEdge(Edge edge) {
		Line2D e = edge.getLine();
		
		Point2D closestPoint = null;
		Point2D closestPointOnEdge = null;
		double shortestDistance = Double.MAX_VALUE;
		boolean valid = false;
		for(Point2D point: this.getPoints()){
			Point2D pointOnEdge = GeoUtils.getClosestPointOnSegment(e, point);
			
			if(pointOnEdge == null){
				continue;
			}
			double distance = GeometryTools2D.getDistance(point, pointOnEdge);
			if(distance>=Double.MAX_VALUE){
				continue;
			}
			
			if(distance<shortestDistance){
				valid = true;
				shortestDistance = distance;
				closestPointOnEdge = pointOnEdge;
				closestPoint = point;
			}
		}
		if(valid){
			return new Line2D(closestPoint,closestPointOnEdge);
		}else{
			return null;
		}
	}
	
	public double getShortestDistanceToPoint(Point2D point){
		double shortestDistance = Double.MAX_VALUE;
		for(Point2D next: this.getPoints()){
			double distance = GeometryTools2D.getDistance(point, next);
			if(distance<shortestDistance){
				shortestDistance = distance;
			}
		}
		return shortestDistance;
	}
	
}
