package ant.utils.civilian;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.worldmodel.EntityID;
import ant.agent.behavior.ATBehavior;
import ant.component.at.AmbulanceMemComponent;
import ant.utils.comparator.DeathTimeComparator;
import ant.utils.geo.GeoUtils;

/**
 * 
 * @author Fun
 *
 */
public class AmbulanceCommuless {
	private static Log log = LogFactory.getLog(AmbulanceCommuless.class);
	
	private static int TIMECOSTONWAY = 3;	// TODO
//	private static int TIMECOSTONTOREFUGE = 3;
	private static final int DEFAULT_TIME = 300;
	
	private static final int RC_DEADTIME_THRESHOLD = 500;
	
	private List<CivilianInfo> rescueableCollection = null;
	private EntityID lastAssignCivilian = null;
	private AmbulanceMemComponent amc;
	private ATBehavior behavior;
	private Map<EntityID,IntValue> civAtnumMap;
	private EntityID [] assignHistory;
	
	public AmbulanceCommuless(AmbulanceMemComponent amc,ATBehavior behavior){
		this.amc = amc;
		this.behavior = behavior;
		this.assignHistory = new EntityID[500];
	}
	
	public EntityID getTarget(){
		//showCivilianInfo();
		//showIgnorInfo();
		EntityID restCiv = checkRestCivilianAround();
		if(restCiv != null){
			assignHistory[amc.getCurrentTime()] = restCiv;
			return restCiv;
		}
		showCivilianMinusIgnorInfo();
//		assumeAssign();
		findSubsetToResuce();
		showRescuableCollection();
		assignTarget();
		showTarget();
		assignHistory[amc.getCurrentTime()] = lastAssignCivilian;
		return lastAssignCivilian;
	}
	
	private void showCivilianMinusIgnorInfo(){
		String str = amc.getCurrentTime()+" CMI:";
		for(CivilianInfo next:amc.getCivilianInfoMap().values()){
			if(amc.isCivilianIgnored(next.getID())){
				continue;
			}
			str += ("("+next.getID()+","+next.getTimeToDie()+")");
		}
		log.info(str);
	}
	
	private void showCivilianInfo(){
		String str = amc.getCurrentTime()+" CI:";
		for(CivilianInfo next:amc.getCivilianInfoMap().values()){
			str += ("("+next.getID()+","+next.getTimeToDie()+")");
		}
		log.info(str);
	}
//	private void showIgnorInfo(){
//		String str = amc.getCurrentTime()+" IG:";
//		for(EntityID next:amc.getIgnoredCiv()){
//			str += (next+",");
//		}
//		log.info(str);
//	}
	private void showRescuableCollection(){
		String str=amc.getCurrentTime()+" RC:";
		for(CivilianInfo next:rescueableCollection){
			str += ("("+next.getID()+","+next.getTimeToDie()+"), ");
		}
		log.info(str);
	}
	
	private void showTarget(){
		log.info(amc.getCurrentTime()+" Target:"+lastAssignCivilian);
	}
	private void findSubsetToResuce(){
		rescueableCollection = new ArrayList<CivilianInfo>();
		EntityID lastRescueTarget = amc.getLastRescueTarget();
		for(CivilianInfo ci: amc.getCivilianInfoMap().values()){
			if(amc.isCivilianIgnored(ci.getID())){
				continue;
			}
			if(amc.isUnreachableCiv(ci.getID())){
				continue;
			}
			if(ci.getHp()<=0){
				continue;
			}
			if(ci.getPosition()==null){
				continue;
			}
			Area position = behavior.getArea(ci.getPosition());
			if(position == null){
				continue;
			}
			if(!(position instanceof Building)){
				continue;
			}
			if(position instanceof Refuge){
				amc.ignoreCivlian(ci.getID());
				continue;
			}
			// 距离条件 TODO
		
			updateDeadTime(ci);

			if(ci.getTimeToDie() <=RC_DEADTIME_THRESHOLD && ci.getTimeToDie() > amc.getCurrentTime()+2){
				if(lastRescueTarget != null && lastRescueTarget.equals(ci.getID())){
					rescueableCollection.add(ci);
				}else{
					if(!isEnoughAT(ci)){
						rescueableCollection.add(ci);
					}else{
//						System.err.println(ci.getID() +"'s ATs is enough!!!!!");
					}
				}
				
			}
		}
		Collections.sort(rescueableCollection,new DeathTimeComparator());
		showRescuableCollection();
		int predicatedTime = amc.getCurrentTime();
		ArrayList<CivilianInfo> savedCivilian = new ArrayList<CivilianInfo>();
		for(CivilianInfo ci: rescueableCollection){
			int rescuetime = getTimeNeedToSave(ci);
			predicatedTime+= rescuetime;
			if(predicatedTime > DEFAULT_TIME || predicatedTime > ci.getTimeToDie()){
//				System.err.println(ci.getID()+" removed. Dead Time:"+ci.getTimeToDie()+" Save Time:"+predicatedTime);
				predicatedTime -= rescuetime;
				continue;
			}
			savedCivilian.add(ci);
		}
		rescueableCollection.clear();
		rescueableCollection.addAll(savedCivilian);
		// When has no refuge ....
	}
	
	private EntityID checkRestCivilianAround(){
		List<Civilian> civs=behavior.getNearbyCivilian();
		int atnum=behavior.getNearbyTeammates().size()+1;
		for(Civilian next: civs){
			if(!behavior.canRescueHuman(next.getID())){
				continue;
			}
			CivilianInfo ci = amc.getCivilianInfoMap().get(next.getID());
			updateDeadTime(ci);
			int rescueTime = getResuceTime(ci, atnum);
			if(rescueTime <8 && rescueTime+ci.getTimeCostToRefuge() < ci.getTimeToDie()){
				return next.getID();
			}
			//TODO
		}
		return null;
	}
	
	private void assumeAssign(){
		// TODO
		civAtnumMap = new HashMap<EntityID, IntValue>();
		for(EntityID at:amc.getAvailableATs()){
			EntityID ATposition = amc.getAmbulanceTeamPosition(at);
			int [] atxy = behavior.getAreaXY(ATposition);
			long [] beas = new long [2];
			EntityID [] cis = new EntityID [2];
			beas[0] = Integer.MAX_VALUE;
			beas[1] = Integer.MAX_VALUE;
			for(CivilianInfo ci: amc.getCivilianInfoMap().values()){
				if(amc.isCivilianIgnored(ci.getID())){
					continue;
				}
				EntityID civpositoin = ci.getPosition();
				int [] civxy = behavior.getAreaXY(civpositoin);
				long dist = (long) GeoUtils.manhattanDistance(atxy[0], atxy[1], civxy[0], civxy[1]);
				if(dist<beas[0]&&dist<beas[1]){
					if(beas[0]>beas[1]){
						beas[0] = dist;
						cis[0] = ci.getID();
					}else{
						beas[1] = dist;
						cis[1] = ci.getID();
					}
				}else if(dist<beas[0]){
					beas[0] = dist;
					cis[0] = ci.getID();
				}else if(dist<beas[1]){
					beas[1] = dist;
					cis[1] = ci.getID();
				}
			}
			if(cis[0] != null){
				if(civAtnumMap.containsKey(cis[0])){
					civAtnumMap.get(cis[0]).value++;
				}else{
					civAtnumMap.put(cis[0], new IntValue(1));
				}
			}
			if(cis[1] != null){
				if(civAtnumMap.containsKey(cis[1])){
					civAtnumMap.get(cis[1]).value++;
				}else{
					civAtnumMap.put(cis[1], new IntValue(1));
				}
			}
		}
	}
	
	private void assignTarget(){
		if(rescueableCollection.isEmpty()){
			lastAssignCivilian = null;
			return;
		}
		if(checkLastAssign()){
			return;
		}
//		System.err.println("ASSIGN:");
		int time_urgent = 140;
		long dist_half_map = Math.max(amc.getWidth()/2, amc.getHeight()/2);
		dist_half_map *= 1.5;
		double coef = dist_half_map/((double)time_urgent);
		EntityID bestTarget = null;
		long minDistBest = Integer.MAX_VALUE;
		long dist; 
		
		for(CivilianInfo ci:rescueableCollection){
			dist = behavior.manhattanDistanceToArea(ci.getPosition());
			if(ci.getTimeToDie() - amc.getCurrentTime()> time_urgent){
				dist += dist_half_map;
			}else{
				double a = coef*((double)(ci.getTimeToDie()-amc.getCurrentTime()));
				dist = dist+(int)a;
			}
			
			if(dist < minDistBest){
				minDistBest = dist;
				bestTarget = ci.getID();
			}
		}
		if(minDistBest != Integer.MAX_VALUE){
			lastAssignCivilian = bestTarget;
//			System.err.println("bestTarget:"+bestTarget+" dist:"+minDistBest);
		}
	}
	
	private boolean checkLastAssign(){
		CivilianInfo ci = amc.getCivilianInfoMap().get(lastAssignCivilian);
		if(ci == null){
			return false;
		}
		if(rescueableCollection.contains(ci)){
			return true;
		}else{
			return false;
		}
	}
	
	private void updateDeadTime(CivilianInfo ci){
		if(ci.isLocked()){
			// TODO
			ci.unlock();
			ci.getPredictor().setParticlesNeedResample();
		}
		ci.getPredictor().step(amc.getCurrentTime());
		
		int deadTimeFromInjury = 500,deadTimeFromFire = 500;
		int offset = 60 - ci.getBuriedness();
		if(offset <5){
			offset = 5;
		}else if(offset>55){
			offset =55;
		}
		deadTimeFromInjury = ci.getPredictor().getDeadTime()[offset];
		BuildingInfo nearestBuildingInfo = null;
		if(nearestBuildingInfo == null){
			deadTimeFromFire = 500;
			ci.setDeadTimeFromFire(500);
			ci.setNearestBurningBuilding(null);
		}else if(nearestBuildingInfo.equals(ci.getNearestBurningBuilding())){
			deadTimeFromFire = ci.getDeadTimeFromFire();
		}else{
			//ci.setNearestBurningBuilding(nearestBurningBuilding);
			// TODO
		}
		
		if(isCivilianBurned(ci)){
			ci.lock();
			if(ci.getBuriedness() > 20){
				ci.setTimeToDie(amc.getCurrentTime());
				ci.setSecondDeadTime(amc.getCurrentTime());
			}else{
				ci.setTimeToDie(amc.getCurrentTime()+20);
				ci.setSecondDeadTime(amc.getCurrentTime()+20);
			}
		}else{
			ci.setTimeToDie(Math.min(deadTimeFromInjury, deadTimeFromFire));
			ci.setSecondDeadTime(Math.min(ci.getPredictor().getDeadTime()[30], deadTimeFromFire));
		}
	}
	
	private int getResuceTime(CivilianInfo ci, int atnum){
		atnum= atnum>2?atnum:2;
		return ci.getBuriedness()/atnum;
	}
	
	private boolean isEnoughAT(CivilianInfo ci){
		if(ci.getResuceATSet().size()==0){
			return false;
		}
		int timeneeded = ci.getTimeCostToRefuge() + ci.getBuriedness()/ci.getResuceATSet().size();
		int savedTime = amc.getCurrentTime()+timeneeded;
		if(savedTime < (ci.getTimeToDie() - 5)){
			return true;
		}else{
			return false;
		}
	}
	
	private int getTimeNeedToSave(CivilianInfo ci){
		int tmp = behavior.getNearbyTeammates().size()+1;
//		for(EntityID atid:amc.getAmbulanceTeams()){
//			EntityID atPosition = amc.getAmbulanceTeamPosition(atid);
//			if(atPosition.equals(ci.getPosition())){
//				tmp++;
//			}
//		}
//		IntValue num = civAtnumMap.get(ci.getID());
//		if(num != null){
//			tmp = num.value;
//		}
		tmp = tmp>2?tmp:2;
//		tmp = tmp>amc.getAvailableATnum()?tmp:amc.getAvailableATnum();
		return ci.getTimeCostToRefuge()+TIMECOSTONWAY+ci.getBuriedness()/tmp;
	}
	

	
	private EntityID getNearestBurningBuilding(){
		// TODO
		return null;
	}
	
	private boolean isCivilianBurned(CivilianInfo ci){
		// TODO
		return false;
	}

}
