package ant.utils.civilian;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import ant.component.at.AmbulanceKernelComponent;
import ant.component.at.AmbulanceMemComponent;
import ant.utils.comparator.DeathTimeComparator;
import ant.utils.comparator.EntityIDComparator;

/**
 * 
 * @author Fun
 * 
 */
public class AmbulanceKernelSimple {
	private static Log log = LogFactory.getLog(AmbulanceKernelSimple.class);
	private static final int LONG_LIFE_TIME = 500;
	private static final int MIN_RESCUE_TIME = 5;
	// 救援集合大小下限
	private static final int MIN_RESCUEABLE_COLLECTTION_SIZE = 3;
	// 模拟时救援队列最大容量
	private static final int CURRENT_TASK_QUEUE_SIZE = 40;
	// 默认AT移动到救援对象所花费的时间 TODO
	public static final int TIMECOSTONWAY = 10;
	
	// FIXME
	private static final int DEFAULT_TIME = 500;

	private List<CivilianInfo> rescueableCollection = null;
	/*
	 * <正在救援的Civilian的ID,已经在施救的AT数目> 仅用于模拟
	 */
	private Map<EntityID, Integer> nowRescuePerformance; 
	private Map<EntityID, Integer> rescueMargin;

	private Map<EntityID, EntityID> greedyAssign;
	private Map<EntityID, EntityID> finalAssign; // 有效
	private Map<EntityID, EntityID> lastAssign;
	private AmbulanceMemComponent amc;
	private AmbulanceKernelComponent akc;
	private Map<EntityID, EntityID> secondFinalAssign = null; // 有效

	public AmbulanceKernelSimple(AmbulanceMemComponent amc, AmbulanceKernelComponent akc) {
		this.amc = amc;
		this.akc = akc;
		
		this.rescueMargin = new HashMap<EntityID, Integer>();
		this.greedyAssign = new HashMap<EntityID, EntityID>();
		this.lastAssign = new HashMap<EntityID, EntityID>();
		this.secondFinalAssign = new HashMap<EntityID, EntityID>();
	}

	public void perform() {
		findSubsetToRescue();
//		showRescuableCollection();
		assignTarget();
		assignSecondTarget();
		sendOrderMsg();
	}

	// -------------------------------------------
	private void findSubsetToRescue() {
		List<CivilianInfo> longLifeTimeCiv = new ArrayList<CivilianInfo>();

		nowRescuePerformance = new HashMap<EntityID, Integer>();
		rescueableCollection = new ArrayList<CivilianInfo>();

		Iterator<Entry<EntityID, CivilianInfo>> civIt = amc
				.getCivilianInfoMap().entrySet().iterator();

		while (civIt.hasNext()) {
			Entry<EntityID, CivilianInfo> entry = civIt.next();
			CivilianInfo ci = entry.getValue();

			// 检查是否需要忽略
			if (amc.isCivilianIgnored(ci.getID())) {
				continue;
			}
			
			// 检查是否可到
			if (amc.isUnreachableCiv(ci.getID())){
				continue;
			}
			
			// 忽略死亡
			if (ci.getHp() <= 0) {
				amc.ignoreCivlian(ci.getID());
				continue;
			}
			BuildingInfo bi = amc.getBuildingInfoMap().get(ci.getPosition());
			// 忽略不在Building中的
			if (bi == null) {
				continue;
			}
			// 忽略在Refuge中的
			if (amc.getRefugesID().contains(ci.getPosition())) {
				continue;
			}

			// 更新死亡时间
			updateDeadTime(ci);

			// 忽略来不及救援的对象
			if (ci.getTimeToDie() <= amc.getCurrentTime() + MIN_RESCUE_TIME) {
				continue;
			}

			// 暂时忽略生命过长的救援对象
			if (ci.getTimeToDie() > LONG_LIFE_TIME) {
				longLifeTimeCiv.add(ci);
				continue;
			}

			// 有效
			rescueableCollection.add(ci);
			// 已有AT救援
			if (ci.getResuceATSet().size() > 0) {
				nowRescuePerformance
						.put(ci.getID(), ci.getResuceATSet().size());
			}

		}

		// 如果救援集合大小小于下限，则加入长时间不会死亡的救援对象
		if (rescueableCollection.size() < MIN_RESCUEABLE_COLLECTTION_SIZE) {
			rescueableCollection.addAll(longLifeTimeCiv);
		}
		// 按死亡时间从小到大排序
		Collections.sort(rescueableCollection, new DeathTimeComparator());
		
		while (!rescueableCollection.isEmpty()) {
			rescueMargin = new HashMap<EntityID, Integer>();
			// 模拟中已救援的集合
			Set<EntityID> savedCiv = new HashSet<EntityID>();
			BinaryMinHeap currTaskQueue = new BinaryMinHeap(
					CURRENT_TASK_QUEUE_SIZE);

			int forcePower = amc.getAvailableATs().size(); // TODO 被困？死亡？
			int index = 0;
			int time = amc.getCurrentTime();
			while (forcePower > 0 && index < rescueableCollection.size()) {
				// 忽略模拟已救援对象
				if (savedCiv.contains(rescueableCollection.get(index).getID())) {
					index++;
					continue;
				}
				// 得到下一个模拟对象
				SimulationElement se = null;

				EntityID cid = rescueableCollection.get(index).getID();

				if (currTaskQueue.containsKey(cid.getValue())) {
					se = (SimulationElement) currTaskQueue.get(cid.getValue());
				} else {
					se = new SimulationElement(cid.getValue(), amc);
				}

				int numATNeeded = se.getNumATNeeded(time + TIMECOSTONWAY);

				if (numATNeeded <= forcePower) { // 剩余AT数目满足
					forcePower -= numATNeeded;
					se.addATSkillful(numATNeeded, time + TIMECOSTONWAY);
					if (currTaskQueue.containsKey(se.getHashKey())) { // 以CID为Key，被救时间为权
						currTaskQueue.decreaseKey(se, se.getPriority());
					} else {
						currTaskQueue.put(se);
					}
					index++;
					savedCiv.add(new EntityID(se.getCid()));
				} else {// 剩余AT数目不足
					se.addATSkillful(forcePower, time + TIMECOSTONWAY);
					forcePower = 0;
					if (currTaskQueue.containsKey(se.getHashKey())) {
						currTaskQueue.decreaseKey(se, se.getPriority());
					} else {
						currTaskQueue.put(se);
					}
				}
			}

			boolean flag = true; // ??

			while (!currTaskQueue.isEmpty()) {
				SimulationElement seSaved = (SimulationElement) currTaskQueue
						.deleteMin();
				// 被救却无法活着到达Refuge的情况
				if (seSaved.getTimeSaved() + seSaved.getTimeToRefuge() > seSaved
						.getTimeDead()) {
					int savedCid = seSaved.getCid();
					int maxBuryIndex = 0;
					int maxBury = -1000;
					// 剔除一个4周期后Buriedness最大的元素
					for (int i = 0; i < rescueableCollection.size(); i++) {
						CivilianInfo ci = rescueableCollection.get(i);
						if (ci.getBuriedness() - 4 * ci.getResuceATSet().size() > maxBury) {
							maxBury = ci.getBuriedness() - 4
									* ci.getResuceATSet().size();
							maxBuryIndex = i;
						}
						if (ci.getID().getValue() == savedCid) {
							break;
						}
					}
					// 剔除
					rescueableCollection.remove(maxBuryIndex);
					// ???
					flag = false;
					break;
				} else {// 正常情况
					EntityID savedCid = new EntityID(seSaved.getCid());

					rescueMargin.put(savedCid,
							seSaved.getTimeDead() - seSaved.getTimeSaved()
									- seSaved.getTimeToRefuge());
					//
					savedCiv.add(new EntityID(seSaved.getCid()));
					// ??????????????????
					while (forcePower > 0
							&& index < rescueableCollection.size()) {
						// 剔除被救集合
						while (index < rescueableCollection.size()
								&& savedCiv.contains(rescueableCollection.get(
										index).getID())) {
							index++;
						}
						// 越界
						if (index >= rescueableCollection.size()) {
							break;
						}
						// 得到下一个模拟对象
						SimulationElement se = null;
						EntityID cid = rescueableCollection.get(index).getID();
						if (currTaskQueue.containsKey(cid.getValue())) {
							se = (SimulationElement) currTaskQueue.get(cid
									.getValue());
						} else {
							se = new SimulationElement(cid.getValue(), amc);
						}

						int numATNeeded = se.getNumATNeeded(seSaved
								.getTimeSaved() + TIMECOSTONWAY);

						if (numATNeeded <= forcePower) {// 剩余的AT数目满足
							forcePower -= numATNeeded;
							se.addAT(numATNeeded, seSaved.getTimeSaved()
									+ TIMECOSTONWAY);
							if (currTaskQueue.containsKey(se.getCid())) {
								currTaskQueue.decreaseKey(se, se.getPriority());
							} else {
								currTaskQueue.put(se);
							}
							index++;
							savedCiv.add(new EntityID(se.getCid()));
						} else { // 剩余AT数目不足
							se.addAT(forcePower, seSaved.getTimeSaved()
									+ TIMECOSTONWAY);
							forcePower = 0;
							if (currTaskQueue.containsKey(se.getCid())) {
								currTaskQueue.decreaseKey(se, se.getPriority());
							} else {
								currTaskQueue.put(se);
							}
						}
					}
				}
			}
			if (flag) {
				for (int i = 0; i < rescueableCollection.size(); i++) {// 根据死亡时间设定营救的优先级
					rescueableCollection.get(i).setRescuePriority(i);
				}
				return;
			}
		}
	}

	private void assignTarget() {
//		int mapdist = (amc.getHeight()+amc.getWidth())/2;
//		int assumeCostOnWay = (int)((1000*mapdist)/220663);
//		System.err.println("assume Cost On Way:"+assumeCostOnWay);
		// 可救援集合为空
		if (rescueableCollection.isEmpty()) {
			finalAssign = new HashMap<EntityID, EntityID>();
			lastAssign = new HashMap<EntityID, EntityID>();
			log.warn("Rescueable Collection is empty");
			return;
		}
		// 重设死亡时间
		for (CivilianInfo ci : rescueableCollection) {
			ci.setTimeToDie(ci.getSecondDeadTime());
		}
		// 分配信息
		AssignInfoSimple assignInfo = new AssignInfoSimple(rescueableCollection, amc);
		
		for (EntityID atId : amc.getAvailableATs()) {
			if (checkLastAssign(atId, lastAssign.get(atId))) {
				assignInfo.assign(atId, lastAssign.get(atId));
			} 
			else if (checkSecondAssign(atId)) {
				assignInfo.assign(atId, secondFinalAssign.get(atId));
			} 
			else {
				EntityID newCiv = assignCivilianGreedy(atId, assignInfo);
				assignInfo.assign(atId, newCiv);
			}
		}
		//showRescuableCollection();
//		showAssignInfo(assignInfo.getAssignation());
		// Greedy assign
		greedyAssign.putAll(assignInfo.getAssignation());
		finalAssign = greedyAssign; // for test
		lastAssign.putAll(assignInfo.getAssignation());// for test		
		
	}

	private long[] loooooooooong(EntityID atID, Collection<EntityID> oneCivCol,
			Set<EntityID> anotherCivCol) {
		long[] resultAll = new long[2];
		long[] resultBest = new long[2];
		long minDistBest = Integer.MAX_VALUE;
		long minDistAll = Integer.MAX_VALUE;
		long dist;

		for (Iterator<EntityID> it = oneCivCol.iterator(); it.hasNext();) {
			EntityID idTarget = it.next();

			dist = amc.manhattanDistance(atID,
					StandardEntityURN.AMBULANCE_TEAM, idTarget,
					StandardEntityURN.CIVILIAN);
			if (dist < minDistAll) {
				minDistAll = dist;
				resultAll[0] = idTarget.getValue();
				resultAll[1] = dist;
			}

			if (!anotherCivCol.contains(idTarget)) {
				if (dist < minDistBest) {
					minDistBest = dist;
					resultBest[0] = idTarget.getValue();
					resultBest[1] = dist;
				}
			}
		}
		if (minDistBest != Integer.MAX_VALUE)
			return resultBest;
		else
			return resultAll;
	}

	private void assignSecondTarget() {
		secondFinalAssign = new HashMap<EntityID, EntityID>();
		List<EntityID> atsNeedSecondAssign = findAtWhoNeedSecondAssign();
		AssignInfoSimple assigInfo = new AssignInfoSimple(rescueableCollection, amc);
		if (rescueableCollection.isEmpty()) {
			return;
		}
		for (Entry<EntityID, EntityID> next : finalAssign.entrySet()) {
			if (!atsNeedSecondAssign.contains(next.getKey())) {
				assigInfo.assign(next.getKey(), next.getValue());
			}
		}
		for (EntityID atId : atsNeedSecondAssign) {
			EntityID newCiv = assignCivilianGreedy(atId, assigInfo);
			secondFinalAssign.put(atId, newCiv);
			assigInfo.assign(atId, newCiv);
		}
	}

	/**
	 * 发送分配命令
	 */
	private void sendOrderMsg() {
		akc.sendOrderForKernel(finalAssign, secondFinalAssign,rescueableCollection);
	}

	private void showAssignInfo(Map<EntityID, EntityID> assign) {
		String str = amc.getCurrentTime() + " Assign:";
		for (Entry<EntityID, EntityID> next : assign.entrySet()) {
			str += ("[" + next.getKey() + "," + next.getValue() + "],");
		}
//		Set<EntityID> civs = new HashSet<EntityID>(assign.values());
//		for(EntityID next: civs){
//			str += (next+",");
//		}
		System.out.println(str);
	}

	private void showRescuableCollection() {
		String str = amc.getCurrentTime() + " RC:";
		for (CivilianInfo next : rescueableCollection) {
			str += ("(" + next.getID() + "," + next.getTimeToDie() + "), ");
		}
		System.err.println(str);
	}

	// ----------------------------------------
	private void updateDeadTime(CivilianInfo ci) {
		if (ci.isLocked()) {
			BuildingInfo bi = amc.getBuildingInfoMap().get(ci.getPosition());
			if (bi == null) {
				return;
			}
			if (bi.isOnFire()) { // TODO???
				return;
			}
			ci.unlock();
			ci.getPredictor().setParticlesNeedResample();
		}

		ci.getPredictor().step(amc.getCurrentTime());

		int deadTimeFromInjury, deadTimeFromFire;
		int offset = 60 - ci.getBuriedness();
		if (offset < 5) {
			offset = 5;
		} else if (offset > 55) {
			offset = 55;
		}
		deadTimeFromInjury = ci.getPredictor().getDeadTime()[offset];
		Building nearestBurningBuilding = null; // FIXME
												// getNearsetBurningBuilding(c);
		if (nearestBurningBuilding == null) {
			deadTimeFromFire = 500;
			ci.setDeadTimeFromFire(500);
			ci.setNearestBurningBuilding(null);
		} else if (nearestBurningBuilding
				.equals(ci.getNearestBurningBuilding())) {
			deadTimeFromFire = ci.getDeadTimeFromFire();
		} else {
			ci.setNearestBurningBuilding(nearestBurningBuilding);
			long dist = amc.distance(ci.getID(), StandardEntityURN.CIVILIAN,
					nearestBurningBuilding.getID(), StandardEntityURN.BUILDING);
			int additionalTime = 10; // FIXME 火模拟
			if (additionalTime < 0) {
				additionalTime = 0;
			}
			deadTimeFromFire = (int) (amc.getCurrentTime() + dist / 1000 + additionalTime);
			deadTimeFromFire += 20;
			ci.setDeadTimeFromFire(deadTimeFromFire);
		}

		if (isCivilianBurning(ci)) {
			ci.lock();
			if (ci.getBuriedness() > 20) { // 如果Civilian所在建筑着火，且Buriedness超过20，视为死亡
				ci.setTimeToDie(amc.getCurrentTime());
				ci.setSecondDeadTime(amc.getCurrentTime());
			} else {
				ci.setTimeToDie(amc.getCurrentTime() + 20); // TODO 20 过大？
				ci.setSecondDeadTime(amc.getCurrentTime() + 20);
			}
		} else {
			ci.setTimeToDie(Math.min(deadTimeFromInjury, deadTimeFromFire));
			ci.setSecondDeadTime(Math.min(ci.getPredictor().getDeadTime()[30],
					deadTimeFromFire));
		}
	}

	private boolean isCivilianBurning(CivilianInfo ci) {
		BuildingInfo bi = amc.getBuildingInfoMap().get(ci.getPosition());
		if (bi == null) {
			return false;
		}
		if (bi.isOnFire()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 检查lastAssign中分配对象距离、是否属于可救援集合、忽略集合
	 * 
	 * @param atId
	 * @return
	 */
	private boolean checkLastAssign(EntityID atId, EntityID cid) {
		if (cid == null) {
			return false;
		}
		// TODO
		long dist_at_civ = amc.manhattanDistance(atId,
				StandardEntityURN.AMBULANCE_TEAM, cid,
				StandardEntityURN.CIVILIAN);
		if (dist_at_civ >= Integer.MAX_VALUE / 2000) {
			return false;
		}
		if (!rescueableCollection.contains(amc.getCivilianInfoMap().get(cid))) {
			return false;
		}
		if (amc.isCivilianIgnored(cid)) {
			return false;
		}
		return true;
	}

	/**
	 * 检查secondAssign中分配对象是否属于可救援集合
	 * 
	 * @param atId
	 * @return
	 */
	private boolean checkSecondAssign(EntityID atId) {
		if (!secondFinalAssign.containsKey(atId)) {
			return false;
		}
		EntityID cid = secondFinalAssign.get(atId);
		CivilianInfo ci = amc.getCivilianInfoMap().get(cid);
		if (!rescueableCollection.contains(ci)) {
			return false;
		}
		return true;
	}

	private EntityID assignCivilianGreedy(EntityID atId, AssignInfoSimple ainfo) {
		int time_urgent = 140;
		int dist_half_map = Math.max(amc.getWidth() / 2,
				amc.getHeight() / 2);
		dist_half_map *= 1.5;
		double coef = ((double) dist_half_map) / ((double) time_urgent);
		Set<EntityID> savedCiv = ainfo.getSavedCiv();

		EntityID resAllCiv = null;
		EntityID resBestCiv = null;

		long minDistBest = Integer.MAX_VALUE;
		long minDistAll = Integer.MAX_VALUE;

		for (CivilianInfo ci : rescueableCollection) {
			long dist = amc.manhattanDistance(atId,
					StandardEntityURN.AMBULANCE_TEAM, ci.getID(),
					StandardEntityURN.CIVILIAN); 
			if (ci.getTimeToDie() - amc.getCurrentTime() > time_urgent) {
				dist += dist_half_map;
			} else {
				double a = coef * (ci.getTimeToDie() - amc.getCurrentTime());
				dist = dist + (int) a;
			}
			if (dist < minDistAll) {
				minDistAll = dist;
				resAllCiv = ci.getID();
			}

			if (!savedCiv.contains(ci.getID())) {
				if (dist < minDistBest) {
					minDistBest = dist;
					resBestCiv = ci.getID();
				}
			}
		}

		if (minDistBest != Integer.MAX_VALUE) {
			return resBestCiv;
		} else {
			return resAllCiv;
		}
	}

	private List<EntityID> findAtWhoNeedSecondAssign() {
		Map<EntityID, List<EntityID>> civilianNeedAt = new HashMap<EntityID, List<EntityID>>();
		List<EntityID> atsNeedSecondAssign = new ArrayList<EntityID>();
		for (EntityID cid : finalAssign.values()) {
			if (civilianNeedAt.containsKey(cid)) {
				continue;
			}
			ArrayList<EntityID> atList = new ArrayList<EntityID>();
			for (Entry<EntityID, EntityID> entry : finalAssign.entrySet()) {
				if (entry.getValue().equals(cid)) {
					atList.add(entry.getKey());
				}
			}
			civilianNeedAt.put(cid, atList);
			CivilianInfo ci = amc.getCivilianInfoMap().get(cid);
			if (ci.getResuceATSet().size() * 2 >= ci.getBuriedness()) {// 已参与救援的AT能在2个周期内救出此对象
				for (EntityID atId : atList) {
					atsNeedSecondAssign.add(atId); // 为这些AT分配第二目标
					rescueableCollection.remove(ci); // ？？？？从可救援列表删除
				}
			}
		}
		Collections.sort(atsNeedSecondAssign, new EntityIDComparator());
		return atsNeedSecondAssign;
	}
}
