package ant.utils.civilian;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author Fun
 *
 */
// TODO 泛型
public class BinaryMinHeap {
private static Log log = LogFactory.getLog(BinaryMinHeap.class);
	
	private int m_size = 0;

	private Prioritized[] m_array;
	private HashMap<Integer,PositionProperty> m_posMap;
	
	public BinaryMinHeap(int capacity)
	{
		m_array = new Prioritized[capacity];
		m_posMap = new HashMap<Integer,PositionProperty> ();
	}
	
	public int size(){
		return m_size;
	}
	
	public boolean isEmpty(){
		return (m_size==0);
	}

	private void doubleArray() {
        Prioritized[] newArray = new Prioritized[m_array.length * 2];
        System.arraycopy( m_array, 0, newArray, 0, m_array.length );
        m_array = newArray;
    }
	
	public void put(Prioritized p)
	{
		if(m_size +1 >= m_array.length)
			doubleArray();
		int hole = m_size + 1;
		m_array[0] = p;
		for(;p.getPriority() < m_array[hole/2].getPriority();hole/=2)
		{
			m_array[hole] = m_array[hole/2];
			m_posMap.get(m_array[hole].getHashKey()).m_pos = hole;
		}
		m_array[hole] = p;
		m_posMap.put(p.getHashKey(), new PositionProperty(hole));
		m_size++;
	}
	
	public boolean contains(Prioritized p)
	{
		return m_posMap.containsKey(p.getHashKey());
	}
	
	public boolean containsKey(int key)
	{
		return m_posMap.containsKey(key);
	}
	
	public Prioritized get(int key)
	{
		return m_array[m_posMap.get(key).m_pos];
	}
	
	public int findPos(Prioritized p)
	{
		PositionProperty pos = m_posMap.get(p.getHashKey());
		if(pos!=null)
			return pos.m_pos;
		else
			return -1;
	}
	
	public Prioritized getMin(){
		return m_array[1];
	}
	
	private void percolateDown(int hole){
		int child;
        Prioritized tmp = m_array[hole];

        for (; (hole * 2) <= m_size; hole = child) {
            child = hole * 2;

            if ((child != m_size) && m_array[child+1].getPriority() < m_array[child].getPriority())
            {
                child++;
            }

            if (m_array[child].getPriority() < tmp.getPriority()) {
                m_array[hole] = m_array[child];
                m_posMap.get(m_array[hole].getHashKey()).m_pos = hole;
            } else {
                break;
            }
        }

        m_array[hole] = tmp;
        m_posMap.get(tmp.getHashKey()).m_pos = hole;
	}
	
	public Prioritized deleteMin()
	{
		Prioritized minItem = getMin();
		m_posMap.remove(minItem.getHashKey());
		
		m_array[1] = m_array[m_size--];
		if(m_size==0)
			return minItem;
		m_posMap.get(m_array[1].getHashKey()).m_pos = 1;
		percolateDown(1);
		return minItem;
	}
	
	public void decreaseKey(Prioritized p , double newKey)
	{
		p.setPriority(newKey);
		int hole = m_posMap.get(p.getHashKey()).m_pos;
		while(hole>1 && p.getPriority()<m_array[hole/2].getPriority())
		{
			m_array[hole] = m_array[hole/2];
			m_posMap.get(m_array[hole].getHashKey()).m_pos = hole;
			hole/=2;
		}
		
		m_array[hole] = p;
		m_posMap.get(p.getHashKey()).m_pos = hole;
	}

	private class PositionProperty{
		public int m_pos = -1;
		public PositionProperty(int pos){
			m_pos = pos;
		}
	}

}
