package ant.utils.civilian;

import java.util.HashSet;
import java.util.Set;

import rescuecore2.standard.entities.Building;
import rescuecore2.worldmodel.EntityID;

/**
 * 
 * @author Fun
 * 
 */
public class CivilianInfo {

	private EntityID id;
	private EntityID position;
	private int hp;
	private int dmg;
	private int buriedness;
	private ParticleFilter predictor;

	private int timeToDie = 299;
	private int secondDeadTime = 299;

	private int rescuePriority = -1;

	private int discoverTime = -1;

	private boolean lifeTimeLocker = false;
	private Building nearestBurningBuilding = null;
	private int deadTimeFromFire = 500;

	private Set<EntityID> resuceATSet;

	private int timeCostToRefuge = 5;

	private int lastUpdateTime = -1;

	public CivilianInfo(EntityID id, EntityID position, int hp, int dmg,
			int buriedness, int time) {
		this.id = id;
		this.position = position;
		this.hp = hp;
		this.dmg = dmg;
		this.buriedness = buriedness;
		this.lastUpdateTime = time;

		this.resuceATSet = new HashSet<EntityID>();

		this.predictor = new ParticleFilter();
		this.predictor.setHp(hp, time);
		this.predictor.setDmg(dmg, time);
		this.predictor.setBury(buriedness);
		this.predictor.step(time);

	}

	public void updatePredictor(int hp, int dmg, int buriedness, int time) {
		if(lastUpdateTime>=time){
			return;
		}
		this.hp = hp;
		this.dmg = dmg;
		this.buriedness = buriedness;

		this.lastUpdateTime = time;

		this.predictor.setHp(hp, time);
		this.predictor.setDmg(dmg, time);
		this.predictor.setBury(buriedness);
		this.predictor.step(time);
	}

	public EntityID getID() {
		return id;
	}

	public void setID(EntityID id) {
		this.id = id;
	}

	public EntityID getPosition() {
		return position;
	}

	public void setPosition(EntityID position) {
		this.position = position;
	}

	public int getHp() {
		return hp;
	}

	public void setHp(int hp) {
		this.hp = hp;
	}

	public int getDmg() {
		return dmg;
	}

	public void setDmg(int dmg) {
		this.dmg = dmg;
	}

	public int getBuriedness() {
		return buriedness;
	}

	public void setBuriedness(int buriedness) {
		this.buriedness = buriedness;
	}

	public int getTimeToDie() {
		return timeToDie;
	}

	public void setTimeToDie(int timeToDie) {
		this.timeToDie = timeToDie;
	}

	public boolean isLocked() {
		return this.lifeTimeLocker;
	}

	public void lock() {
		this.lifeTimeLocker = true;
	}

	public void unlock() {
		this.lifeTimeLocker = false;
	}

	public void setSecondDeadTime(int secondDeadTime) {
		this.secondDeadTime = secondDeadTime;
	}

	@Deprecated
	public int getSecondDeadTime() {
		return this.secondDeadTime;
	}

	public int getRescuePriority() {
		return rescuePriority;
	}

	public void setRescuePriority(int rescuePriority) {
		this.rescuePriority = rescuePriority;
	}

	public int getDiscoverTime() {
		return discoverTime;
	}

	public void setDiscoverTime(int discoverTime) {
		this.discoverTime = discoverTime;
	}

	public ParticleFilter getPredictor() {
		return this.predictor;
	}

	public Building getNearestBurningBuilding() {
		return nearestBurningBuilding;
	}

	public void setNearestBurningBuilding(Building nearestBurningBuilding) {
		this.nearestBurningBuilding = nearestBurningBuilding;
	}

	public	Set<EntityID> getResuceATSet() {
		return resuceATSet;
	}

	public void setDeadTimeFromFire(int deadTimeFromFire) {
		this.deadTimeFromFire = deadTimeFromFire;
	}

	public int getDeadTimeFromFire() {
		return this.deadTimeFromFire;
	}

	public int getLastUpdateTime() {
		return lastUpdateTime;
	}

	public int getTimeCostToRefuge() {
		return timeCostToRefuge;
	}

	public void setTimeCostToRefuge(int timeToRefuge) {
		this.timeCostToRefuge = timeToRefuge;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + buriedness;
		result = prime * result + deadTimeFromFire;
		result = prime * result + discoverTime;
		result = prime * result + dmg;
		result = prime * result + hp;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + lastUpdateTime;
		result = prime * result + (lifeTimeLocker ? 1231 : 1237);
		result = prime
				* result
				+ ((nearestBurningBuilding == null) ? 0
						: nearestBurningBuilding.hashCode());
		result = prime * result
				+ ((position == null) ? 0 : position.hashCode());
		result = prime * result
				+ ((predictor == null) ? 0 : predictor.hashCode());
		result = prime * result + rescuePriority;
		result = prime * result
				+ ((resuceATSet == null) ? 0 : resuceATSet.hashCode());
		result = prime * result + secondDeadTime;
		result = prime * result + timeToDie;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CivilianInfo other = (CivilianInfo) obj;
		if (buriedness != other.buriedness)
			return false;
		if (deadTimeFromFire != other.deadTimeFromFire)
			return false;
		if (discoverTime != other.discoverTime)
			return false;
		if (dmg != other.dmg)
			return false;
		if (hp != other.hp)
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (lastUpdateTime != other.lastUpdateTime)
			return false;
		if (lifeTimeLocker != other.lifeTimeLocker)
			return false;
		if (nearestBurningBuilding == null) {
			if (other.nearestBurningBuilding != null)
				return false;
		} else if (!nearestBurningBuilding.equals(other.nearestBurningBuilding))
			return false;
		if (position == null) {
			if (other.position != null)
				return false;
		} else if (!position.equals(other.position))
			return false;
		if (predictor == null) {
			if (other.predictor != null)
				return false;
		} else if (!predictor.equals(other.predictor))
			return false;
		if (rescuePriority != other.rescuePriority)
			return false;
		if (resuceATSet == null) {
			if (other.resuceATSet != null)
				return false;
		} else if (!resuceATSet.equals(other.resuceATSet))
			return false;
		if (secondDeadTime != other.secondDeadTime)
			return false;
		if (timeToDie != other.timeToDie)
			return false;
		return true;
	}

}
