package ant.utils.civilian;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import rescuecore2.worldmodel.EntityID;
import ant.component.at.AmbulanceMemComponent;

/**
 * 
 * @author Fun
 * 
 */
public class SimulationElement implements Comparable<SimulationElement>,
		Prioritized {
	private static Log log = LogFactory.getLog(SimulationElement.class);

	private AmbulanceMemComponent amc;

	private int time;
	private int cid;
	private double workLeft;
	private int atNum;
	private int timeSaved;
	private int timeDead;
	private CivilianInfo ci;

	public SimulationElement(int cid,AmbulanceMemComponent amc) {
		this.amc = amc;
		ci = amc.getCivilianInfoMap().get(new EntityID(cid));
		if (ci == null) {
			log.fatal("CivilianInfo " + ci + " not be found!");
		}
		time = amc.getCurrentTime();
		this.cid = cid;
		this.workLeft = ci.getBuriedness();
		this.atNum = 0;
		this.timeSaved = 10000;
		this.timeDead = ci.getTimeToDie();
	}
	
	public int getCid(){
		return this.cid;
	}
	
	public int getTimeSaved(){
		return this.timeSaved;
	}
	
	public int getTimeDead(){
		return this.timeDead;
	}
	
	public int getATnum(){
		return this.atNum;
	}
	
	public int getNumATNeeded(int reachTime) {
		
		if (this.timeSaved + ci.getTimeCostToRefuge() < this.timeDead)
			return 0;
		if (reachTime >= this.timeDead - ci.getTimeCostToRefuge())
			return 100;
		double workLeft = this.workLeft - this.atNum * (reachTime - time);
		double tmp = workLeft / (this.timeDead - ci.getTimeCostToRefuge() - reachTime)
				- this.atNum;
		if (tmp < 0)
			return 0;
		int result = ((int) tmp) + 1;
		return result;
	}

	public int getTimeToRefuge(){
		return ci.getTimeCostToRefuge();
	}
	
	public void addATSkillful(int atNum, int timeReady) {
		CivilianInfo ci = amc.getCivilianInfoMap().get(new EntityID(this.cid));
		if (ci.getResuceATSet().size() <= 0) {
			addAT(atNum, timeReady);
			return;
		}

		if (atNum <= ci.getResuceATSet().size()) {
			addAT(atNum, amc.getCurrentTime());
			return;
		}

		addAT(ci.getResuceATSet().size(), amc.getCurrentTime());
		addAT(atNum - ci.getResuceATSet().size(), timeReady);
	}

	public void addAT(int atNum, int timeReady) {
		if (this.atNum == 0) {
			time = timeReady;
			this.atNum = atNum;
			this.timeSaved = time + (int) (this.workLeft / atNum);
		} else {
			if (timeReady < time) {
				this.workLeft -= (time - timeReady) * atNum;
				if (this.workLeft < 0)
					this.workLeft = 0;
				this.atNum += atNum;
				this.timeSaved = time + (int) (this.workLeft / this.atNum);
			}

			int t = (int) (this.workLeft / this.atNum) + 1;
			this.workLeft -= (timeReady - time) * this.atNum;
			if (this.workLeft < 0) {
				this.workLeft = 0;
				time += t;
			} else
				time = timeReady;
			this.atNum += atNum;
			this.timeSaved = time + (int) (this.workLeft / this.atNum);
		}
	}

	@Override
	public double getPriority() {
		return this.timeSaved;
	}

	@Override
	public void setPriority(double key) {
		this.timeSaved = (int) key;
	}

	@Override
	public int getHashKey() {
		return this.cid;
	}

	@Override
	public int compareTo(SimulationElement other) {
		if(this.timeSaved < other.timeSaved){
			return -1;
		}else{
			return 1;
		}
	}

}
