package ant.utils.geo;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Edge;

/**
 * 
 * @author Fun
 *
 */
public class Converter {
	/**
	 * Generate apexes list from apexes array
	 * 
	 * @param polygonApexes
	 * @return
	 */
	public static List<Point> generatePoints(int[] polygonApexes) {
		List<Point> ret = new ArrayList<Point>();

		for (int i = 0; i < polygonApexes.length - 1; i += 2) {
			Point p = new Point(polygonApexes[i], polygonApexes[i + 1]);
			ret.add(p);
		}

		return ret;
	}
	
	public static List<Point2D> generatePoint2Ds(int[] polygonApexes) {
		List<Point2D> ret = new ArrayList<Point2D>();

		for (int i = 0; i < polygonApexes.length - 1; i += 2) {
			Point2D p = new Point2D(polygonApexes[i],polygonApexes[i + 1]);
			ret.add(p);
		}

		return ret;
	}
	
	public static int [] generateApexes(List<Point> points){
		int [] ret = new int[points.size()*2];
		for(int i=0;i<points.size();i++){
			Point p=points.get(i);
			ret[i*2]=p.x();
			ret[i*2+1]=p.y();
		}
		
		return ret;
	}

	/**
	 * Generate edges list from apexes array
	 * 
	 * @param polygonApexes
	 * @return
	 */
	public static List<Segment> generateSegments(int[] polygonApexes) {
		List<Segment> ret = new ArrayList<Segment>();

		for (int i = 0; i < polygonApexes.length - 3; i += 2) {
			Segment s = new Segment(polygonApexes[i], polygonApexes[i + 1],
					polygonApexes[i + 2], polygonApexes[i + 3]);
			ret.add(s);
		}
		ret.add(new Segment(polygonApexes[polygonApexes.length - 2],
				polygonApexes[polygonApexes.length - 1], polygonApexes[0],
				polygonApexes[1]));
		return ret;
	}
	
	public static List<Line2D> generateLine2Ds(int[] polygonApexes) {
		List<Line2D> ret = new ArrayList<Line2D>();

		for (int i = 0; i < polygonApexes.length - 3; i += 2) {
			Line2D s = new Line2D(new Point2D(polygonApexes[i], polygonApexes[i + 1]),new Point2D(polygonApexes[i + 2], polygonApexes[i + 3]));
			ret.add(s);
		}
		ret.add(new Line2D(new Point2D(polygonApexes[polygonApexes.length - 2],	polygonApexes[polygonApexes.length - 1]),new Point2D(polygonApexes[0],polygonApexes[1])));
		return ret;
	}
	
	
	
	public static Rectangle generateMinBoundingRectangle(int[] polygonApexes) {
		List<Point> points = generatePoints(polygonApexes);
		int minX, minY, maxX, maxY;
		Point point = points.get(0);
		minX = maxX = point.x();
		minY = maxY = point.y();

		for (int i = 1; i < points.size(); i++) {
			point = points.get(i);
			int x = point.x();
			int y = point.y();
			if (x < minX) {
				minX = x;
			}
			if (x > maxX) {
				maxX = x;
			}
			if (y < minY) {
				minY = y;
			}
			if (y > maxY) {
				maxY = y;
			}
		}
		return new Rectangle(minX, minY, maxX - minX, maxY - minY);
	}
	
	public static Segment generateSegment(Edge e){
		return new Segment(e.getStartX(),e.getStartY(),e.getEndX(),e.getEndY());
	}
	
}
