package ant.utils.geo;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author Fun
 *
 */
public class Rectangle {
	private int x, y; // Left bottom point
	private int height, width;
	private List<Segment> edges;
	private List<Point> points;

	/**
	 * Construct a rectangle from the left bottom point (x,y) , height and width
	 * 
	 * @param x
	 *            left
	 * @param y
	 *            bottom
	 * @param height
	 * @param width
	 */
	public Rectangle(int x, int y, int width, int height) {
		this.x = x;
		this.y = y;
		this.height = height;
		this.width = width;
	}

	public void setX(int x) {
		this.x = x;
	}

	public void setY(int y) {
		this.y = y;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getX() {
		return this.x;
	}

	public int getY() {
		return this.y;
	}

	public int getHeight() {
		return this.height;
	}

	public int getWidth() {
		return this.width;
	}

	public List<Segment> getEdges() {
		if (edges == null) {
			edges = new ArrayList<Segment>();
			edges.add(new Segment(x, y, x + width, y));
			edges.add(new Segment(x + width, y, x + width, y + height));
			edges.add(new Segment(x + width, y + height, x, y + height));
			edges.add(new Segment(x, y + height, x, y));
		}
		return edges;
	}

	public List<Point> getPoints() {
		if (points == null) {
			points = new ArrayList<Point>();
			points.add(new Point(this.x, this.y));
			points.add(new Point(this.x + this.width, this.y));
			points.add(new Point(this.x + this.width, this.y + this.height));
			points.add(new Point(this.x, this.y + this.height));
		}
		return points;
	}

	public boolean containsPoint(Point p) {
		int maxX = x + width;
		int maxY = y + height;
		if (p.x() >= x && p.x() <= maxX && p.y() >= y && p.y() <= maxY) {
			return true;
		}
		return false;
	}

	@Override
	public String toString() {
		return "Rectangle [(" + x + ", " + y + "), width=" + width
				+ ", height=" + height + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + x;
		result = prime * result + y;
		result = prime * result + width;
		result = prime * result + height;
		return result;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Rectangle r = (Rectangle) obj;
		if (r.x != this.x) {
			return false;
		}
		if (r.y != this.y) {
			return false;
		}
		if (r.width != this.width) {
			return false;
		}
		if (r.height != this.height) {
			return false;
		}
		return true;
	}
}
